﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Data;
using System.Dynamic;
using System.Collections.Generic;
using System.Runtime.CompilerServices;

using Dodoni.BasicComponents.Utilities;

/* Hint: Use sn.exe to display the public key, i.e. call "sn -p  BasicComponentsTestsKey.snk BasicComponentsTests.publickey" 
 *       and "sn -tp BasicComponentsTests.publickey" to display the public key.
 */
[assembly: InternalsVisibleTo("Dodoni.BasicComponents.Tests.Unit, PublicKey="
    + "002400000480000094000000060200000024000052534131000400000100010043a9e1b570eaa2"
    + "3770f86b4152405324608a3ced74c26fbca8a1a34641083b90b797a46d15e25008859ff8538fa0"
    + "f606396ccf2d3cbf66d91e85cb61212566a9fe57a02e454de0b38f0b7a543b7683af938ce18cf2"
    + "a3e7dffd45795ce67c8f5996d0b65ab490356ee6a760076a954612cf9961c5a816ad887159747d"
    + "0ddd4dcd")]
namespace Dodoni.BasicComponents.Containers
{
    /// <summary>Represents a collection of <see cref="InfoOutputProperty"/> and <see cref="System.Data.DataTable"/> objects.
    /// </summary>    
    public class InfoOutputPackage : IIdentifierNameable
    {
        #region nested enumerations

        /// <summary>The type of a specific <see cref="DataTable"/> object that represents output of a specific <see cref="IInfoOutputQueriable"/> object.
        /// </summary>
        [Flags]
        public enum DataTableType
        {
            /// <summary>'None', i.e. ignoring all <see cref="DataTable"/> objects. 
            /// </summary>
            None = 0,

            /// <summary>A 'single' <see cref="DataTable"/> object, i.e. neither a parent nor a child table.
            /// </summary>
            Single = 1,

            /// <summary>A 'parent' <see cref="DataTable"/> object, i.e. linked to a specific child table.
            /// </summary>
            Parent = 2,

            /// <summary>A 'child' <see cref="DataTable"/> object, i.e. linked to a secific parent table.
            /// </summary>
            Child = 4,

            /// <summary>An arbitrary <see cref="DataTable"/> object.
            /// </summary>
            Arbitrary = Single | Parent | Child
        }
        #endregion

        #region public (readonly) members

        /// <summary>The (group) name of the collection of general properites.
        /// </summary>
        public static IdentifierString GeneralPropertyGroupName = new IdentifierString("General properties");

        /// <summary>The separator used to separate the parent and child <see cref="DataTable"/> objects, for example "parentTableName->childTableName".
        /// </summary>
        public const string ParentChildTableNameSeparator = "->";
        #endregion

        #region private members

        /// <summary>The name of the <see cref="InfoOutputPackage"/>.
        /// </summary>
        private IdentifierString m_Name;

        /// <summary>A collection of <see cref="DataTable"/> objects, where the key is the name of the table.
        /// </summary>
        /// <remarks>The table name of an <see cref="DataTable"/> object can be changed.</remarks>
        private IdentifierStringDictionary<DataTable> m_Tables = new IdentifierStringDictionary<DataTable>(isReadOnlyExceptAdding: false);

        /// <summary>The collection of <see cref="InfoOutputParentChildDataTable"/> objects, where the key is the name of the parent table.
        /// </summary>
        private IdentifierStringDictionary<InfoOutputParentChildDataTable> m_ParentChildTables = new IdentifierStringDictionary<InfoOutputParentChildDataTable>(isReadOnlyExceptAdding: false);

        /// <summary>A collection of <see cref="InfoOutputProperty"/> objects, where the first key is the property group name ('general properties' etc.) and the second key is the 
        /// name of the property itself. 
        /// </summary>
        private IdentifierStringDictionary<IdentifierStringDictionary<InfoOutputProperty>> m_Properties = new IdentifierStringDictionary<IdentifierStringDictionary<InfoOutputProperty>>(isReadOnlyExceptAdding: false);

        /// <summary>A collection of 'general properties', i.e. the elements of <see cref="m_Properties"/>, where the key is equal to <see cref="GeneralPropertyGroupName"/>,
        /// </summary>
        private IdentifierStringDictionary<InfoOutputProperty> m_GeneralProperties = new IdentifierStringDictionary<InfoOutputProperty>(isReadOnlyExceptAdding: false);
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="InfoOutputPackage"/> class.
        /// </summary>
        /// <param name="categoryName">The name of the category, i.e. the name of the current instance.</param>        
        internal InfoOutputPackage(IdentifierString categoryName)
        {
            m_Name = categoryName;
            m_Properties.Add(GeneralPropertyGroupName, m_GeneralProperties);
        }

        /// <summary>Initializes a new instance of the <see cref="InfoOutputPackage"/> class.
        /// </summary>
        /// <param name="categoryName">The name of the category, i.e. the name of the current instance.</param>
        internal InfoOutputPackage(string categoryName)
            : this(new IdentifierString(categoryName))
        {
        }
        #endregion

        #region public properties

        #region IIdentifierNameable Members

        /// <summary>Gets the name of the current instance.
        /// </summary>
        /// <value>The language independent name of the current instance.</value>
        IdentifierString IIdentifierNameable.Name
        {
            get { return m_Name; }
        }

        /// <summary>Gets the long name of the current instance.
        /// </summary>
        /// <value>The language dependent long name of the current instance.</value>
        IdentifierString IIdentifierNameable.LongName
        {
            get { return m_Name; }
        }
        #endregion

        /// <summary>Gets the name of the category.
        /// </summary>
        /// <value>The name of the category.</value>
        public IdentifierString CategoryName
        {
            get { return m_Name; }
        }

        /// <summary>Gets the collection of general properties, i.e. name/value pairs, where the key represents the name of the property.
        /// </summary>
        /// <value>The general properties.</value>
        public IIdentifierStringDictionary<InfoOutputProperty> GeneralProperties
        {
            get { return m_GeneralProperties; }
        }

        /// <summary>Gets the name of the property groups, i.e. 'general properties' etc.
        /// </summary>
        /// <value>The property group names.</value>
        public IEnumerable<string> PropertyGroupNames
        {
            get { return m_Properties.Names; }
        }

        /// <summary>Gets the collection of properties, where the first component is the property group name (i.e. 'General Properties' etc.)  and the second component is the collection of 
        /// the corresponding properties.
        /// </summary>
        /// <value>The collection of properties.</value>
        public IEnumerable<Tuple<IdentifierString, IIdentifierStringDictionary<InfoOutputProperty>>> Properties
        {
            get
            {
                foreach (var property in m_Properties.NamedValues)
                {
                    yield return Tuple.Create(property.Item1, (IIdentifierStringDictionary<InfoOutputProperty>)property.Item2);
                }
            }
        }

        /// <summary>Gets the collection of <see cref="InfoOutputParentChildDataTable"/> objects (as well as the unique identifier) that
        /// represents parent/child <see cref="DataTable"/> objects with a 1:n relation.
        /// </summary>
        /// <value>The parent/child data tables as well a unique identifier in its <see cref="IdentifierString"/> representation.</value>
        public IEnumerable<Tuple<IdentifierString, InfoOutputParentChildDataTable>> ParentChildDataTables
        {
            get { return m_ParentChildTables.NamedValues; }
        }
        #endregion

        #region public methods

        /// <summary>Gets the <see cref="System.Data.DataTable"/> objects with homogeneous informations, and the table name in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <param name="dataTableType">The type of the tables to take into account.</param>
        /// <returns>The tables and table names.</returns>
        /// <remarks>The table name of an <see cref="DataTable"/> object can be changed.</remarks>
        public IEnumerable<Tuple<IdentifierString, DataTable>> GetDataTables(DataTableType dataTableType = DataTableType.Single)
        {
            if (dataTableType.HasFlag(DataTableType.Single) == true)
            {
                foreach (var namedTable in m_Tables.NamedValues)
                {
                    yield return namedTable;
                }
            }
            if (dataTableType.HasFlag(DataTableType.Parent) == true)
            {
                foreach (var namedParentChildTables in m_ParentChildTables.NamedValues)
                {
                    yield return Tuple.Create(namedParentChildTables.Item1, namedParentChildTables.Item2.ParentDataTable);
                }
            }
            if (dataTableType.HasFlag(DataTableType.Child) == true)
            {
                foreach (var namedParentChildTables in m_ParentChildTables.NamedValues)
                {
                    yield return Tuple.Create(IdentifierString.Create(namedParentChildTables.Item2.ParentDataTable.TableName + ParentChildTableNameSeparator + namedParentChildTables.Item2.ChildDataTable.TableName), namedParentChildTables.Item2.ChildDataTable);
                }
            }
        }

        /// <summary>Gets the table names.
        /// </summary>
        /// <param name="dataTableType">The type of the tables to take into account.</param>
        /// <returns>The table names.</returns>
        public IEnumerable<string> GetDataTableNames(DataTableType dataTableType = DataTableType.Single)
        {
            if (dataTableType.HasFlag(DataTableType.Single) == true)
            {
                foreach (var tableName in m_Tables.Names)
                {
                    yield return tableName;
                }
            }
            if (dataTableType.HasFlag(DataTableType.Parent) == true)
            {
                foreach (var namedParentChildTables in m_ParentChildTables.NamedValues)
                {
                    yield return (String)namedParentChildTables.Item1;
                }
            }
            if (dataTableType.HasFlag(DataTableType.Child) == true)
            {
                foreach (var namedParentChildTables in m_ParentChildTables.NamedValues)
                {
                    yield return namedParentChildTables.Item2.ParentDataTable.TableName + ParentChildTableNameSeparator + namedParentChildTables.Item2.ChildDataTable.TableName;
                }
            }
        }

        /// <summary>Gets a specific <see cref="DataTable"/> object.
        /// </summary>
        /// <param name="tableName">The name of the table.</param>
        /// <param name="value">The <see cref="DataTable"/>  (output).</param>
        /// <param name="dataTableType">The type of the tables to take into account.</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetDataTable(string tableName, out DataTable value, DataTableType dataTableType = DataTableType.Single)
        {
            return TryGetDataTable(IdentifierString.Create(tableName), out value, dataTableType);
        }

        /// <summary>Gets a specific <see cref="DataTable"/> object.
        /// </summary>
        /// <param name="tableName">The name of the table.</param>
        /// <param name="dataTableType">The type of the tables to take into account.</param>
        /// <returns>A <see cref="DataTable"/> with the desired name.</returns>
        /// <exception cref="ArgumentException">Thrown, if no table available with the desired name.</exception>
        public DataTable GetDataTable(string tableName, DataTableType dataTableType = DataTableType.Single)
        {
            DataTable value;
            if (TryGetDataTable(IdentifierString.Create(tableName), out value, dataTableType) == true)
            {
                return value;
            }
            throw new ArgumentException(String.Format(ExceptionMessages.TableNameUnknown, tableName), tableName);
        }

        /// <summary>Gets a specific <see cref="DataTable"/> object.
        /// </summary>
        /// <param name="tableName">The name of the table.</param>
        /// <param name="value">The <see cref="DataTable"/> and its name in its <see cref="IdentifierString"/> representation (output).</param>
        /// <param name="dataTableType">The type of the tables to take into account.</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetDataTable(IdentifierString tableName, out DataTable value, DataTableType dataTableType = DataTableType.Single)
        {
            if (dataTableType.HasFlag(DataTableType.Single) == true)
            {
                if (m_Tables.TryGetValue(tableName, out value) == true)
                {
                    return true;
                }
            }
            if (dataTableType.HasFlag(DataTableType.Parent) == true)
            {
                InfoOutputParentChildDataTable parentChildDataTable;
                if (m_ParentChildTables.TryGetValue(tableName, out parentChildDataTable) == true)
                {
                    value = parentChildDataTable.ParentDataTable;
                    return true;
                }
            }
            if (dataTableType.HasFlag(DataTableType.Child) == true)
            {
                InfoOutputParentChildDataTable parentChildDataTable;
                int index = tableName.IDString.IndexOf(ParentChildTableNameSeparator);
                if (index >= 0)
                {
                    string parentTableName = tableName.IDString.Substring(0, index);
                    string childTableName = tableName.IDString.Substring(index + ParentChildTableNameSeparator.Length, tableName.IDString.Length - index - ParentChildTableNameSeparator.Length);

                    if (m_ParentChildTables.TryGetValue(parentTableName, out parentChildDataTable) == true)
                    {
                        if (parentChildDataTable.ChildDataTable.TableName.ToIDString() == childTableName.ToIDString())
                        {
                            value = parentChildDataTable.ChildDataTable;
                            return true;
                        }
                    }
                }
            }
            value = null;
            return false;
        }

        /// <summary>Gets a specific <see cref="DataTable"/> object.
        /// </summary>
        /// <param name="tableName">The name of the table.</param>
        /// <param name="dataTableType">The type of the tables to take into account.</param>
        /// <returns>A <see cref="DataTable"/> with the desired name.</returns>
        /// <exception cref="ArgumentException">Thrown, if no table available with the desired name.</exception>
        public DataTable GetDataTable(IdentifierString tableName, DataTableType dataTableType = DataTableType.Single)
        {
            DataTable value;
            if (TryGetDataTable(tableName, out value, dataTableType) == true)
            {
                return value;
            }
            throw new ArgumentException(String.Format(ExceptionMessages.TableNameUnknown, (String)tableName), (String)tableName);
        }

        /// <summary>Gets a specific <see cref="InfoOutputParentChildDataTable"/> object, i.e. one parent and one child <see cref="DataTable"/> object, 
        /// connected with respect to a specified relation.
        /// </summary>
        /// <param name="parentDataTableName">The name of the parent data table.</param>
        /// <param name="value">The value.</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetParentChildDataTable(string parentDataTableName, out InfoOutputParentChildDataTable value)
        {
            return m_ParentChildTables.TryGetValue(parentDataTableName, out value);
        }

        /// <summary>Gets a specific <see cref="InfoOutputParentChildDataTable"/> object, i.e. one parent and one child <see cref="DataTable"/> object, 
        /// connected with respect to a specified relation.
        /// </summary>
        /// <param name="parentDataTableName">The name of the parent data table.</param>
        /// <returns>The requested <see cref="InfoOutputParentChildDataTable"/> object.</returns>
        /// <exception cref="ArgumentException">Thrown, if no data available with the desired name.</exception>
        public InfoOutputParentChildDataTable GetParentChildDataTable(string parentDataTableName)
        {
            InfoOutputParentChildDataTable value;
            if (m_ParentChildTables.TryGetValue(parentDataTableName, out value) == true)
            {
                return value;
            }
            throw new ArgumentException(String.Format(ExceptionMessages.TableNameUnknown, parentDataTableName), parentDataTableName);
        }

        /// <summary>Gets a specific <see cref="InfoOutputProperty"/> object with repsect to a specific property group name.
        /// </summary>
        /// <param name="propertyName">The name of the property.</param>
        /// <param name="value">The property (output).</param>
        /// <param name="propertyGroupName">The name of the property group (i.e. 'General Properties' etc.).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetProperty(string propertyName, out InfoOutputProperty value, string propertyGroupName = "General properties")
        {
            if (propertyGroupName != null)
            {
                IdentifierStringDictionaryBase<InfoOutputProperty> propertyCollection;
                if (m_Properties.TryGetValue(propertyGroupName, out propertyCollection) == true)
                {
                    return propertyCollection.TryGetValue(propertyName, out value);
                }
            }
            value = default(InfoOutputProperty);
            return false;
        }

        /// <summary>Gets a specific <see cref="InfoOutputProperty"/> object with repsect to a specific property group name.
        /// </summary>
        /// <param name="propertyName">The name of the property.</param>
        /// <param name="propertyGroupName">The name of the property group (i.e. 'General Properties' etc.).</param>
        /// <returns>The property.</returns>
        /// <exception cref="ArgumentException">Thrown, if no property available with the desired name.</exception>
        public InfoOutputProperty GetProperty(string propertyName, string propertyGroupName = "General properties")
        {
            InfoOutputProperty value;
            if (TryGetProperty(propertyName, out value, propertyGroupName) == true)
            {
                return value;
            }
            throw new ArgumentException(String.Format(ExceptionMessages.PropertyNameUnknown, propertyGroupName + "." + propertyName), propertyName);
        }

        /// <summary>Gets a specific <see cref="InfoOutputProperty"/> object with repsect to a specific property group name.
        /// </summary>
        /// <param name="propertyName">The name of the property.</param>
        /// <param name="value">The property (output).</param>
        /// <param name="propertyGroupName">The name of the property group (i.e. 'General Properties' etc.).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetProperty(string propertyName, out InfoOutputProperty value, IdentifierString propertyGroupName)
        {
            if (propertyGroupName != null)
            {
                IdentifierStringDictionaryBase<InfoOutputProperty> propertyCollection;
                if (m_Properties.TryGetValue(propertyGroupName, out propertyCollection) == true)
                {
                    return propertyCollection.TryGetValue(propertyName, out value);
                }
            }
            value = default(InfoOutputProperty);
            return false;
        }

        /// <summary>Gets a specific <see cref="InfoOutputProperty"/> object with repsect to a specific property group name.
        /// </summary>
        /// <param name="propertyName">The name of the property.</param>
        /// <param name="propertyGroupName">The name of the property group (i.e. 'General Properties' etc.).</param>
        /// <returns>The property.</returns>
        /// <exception cref="ArgumentException">Thrown, if no property available with the desired name.</exception>
        public InfoOutputProperty GetProperty(string propertyName, IdentifierString propertyGroupName)
        {
            InfoOutputProperty value;
            if (TryGetProperty(propertyName, out value, propertyGroupName) == true)
            {
                return value;
            }
            throw new ArgumentException(String.Format(ExceptionMessages.PropertyNameUnknown, propertyGroupName + "." + propertyName), propertyName);
        }

        /// <summary>Gets the collection of properties with respect to a specific property group name.
        /// </summary>
        /// <param name="propertyGroupName">The name of the property group (i.e. 'General Properties' etc.).</param>
        /// <param name="value">The property collection (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetProperties(string propertyGroupName, out IIdentifierStringDictionary<InfoOutputProperty> value)
        {
            if (propertyGroupName != null)
            {
                IdentifierStringDictionaryBase<InfoOutputProperty> propertyCollection;
                if (m_Properties.TryGetValue(propertyGroupName, out propertyCollection) == true)
                {
                    value = propertyCollection;
                    return true;
                }
            }
            value = null;
            return false;
        }

        /// <summary>Gets the collection of properties with respect to a specific property group name.
        /// </summary>
        /// <param name="propertyGroupName">The name of the property group (i.e. 'General Properties' etc.).</param>
        /// <returns>The property collection.</returns>
        public IIdentifierStringDictionary<InfoOutputProperty> GetProperties(string propertyGroupName)
        {
            IIdentifierStringDictionary<InfoOutputProperty> value;
            if (TryGetProperties(propertyGroupName, out value) == true)
            {
                return value;
            }
            throw new ArgumentException(String.Format(ExceptionMessages.PropertyGroupNameUnknown, propertyGroupName), propertyGroupName);
        }

        /// <summary>Gets the collection of properties with respect to a specific property group name.
        /// </summary>
        /// <param name="propertyGroupName">The name of the property group (i.e. 'General Properties' etc.).</param>
        /// <param name="value">The property collection (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetProperties(IdentifierString propertyGroupName, out IIdentifierStringDictionary<InfoOutputProperty> value)
        {
            if (propertyGroupName != null)
            {
                IdentifierStringDictionaryBase<InfoOutputProperty> propertyCollection;
                if (m_Properties.TryGetValue(propertyGroupName, out propertyCollection) == true)
                {
                    value = propertyCollection;
                    return true;
                }
            }
            value = null;
            return false;
        }

        /// <summary>Gets the collection of properties with respect to a specific property group name.
        /// </summary>
        /// <param name="propertyGroupName">The name of the property group (i.e. 'General Properties' etc.).</param>
        /// <returns>The property collection.</returns>
        public IIdentifierStringDictionary<InfoOutputProperty> GetProperties(IdentifierString propertyGroupName)
        {
            IIdentifierStringDictionary<InfoOutputProperty> value;
            if (TryGetProperties(propertyGroupName, out value) == true)
            {
                return value;
            }
            throw new ArgumentException(String.Format(ExceptionMessages.PropertyGroupNameUnknown, (String)propertyGroupName), (String)propertyGroupName);
        }

        /// <summary>Gets a specific <see cref="InfoOutputProperty"/> object which represents the value of a property with respect to <see cref="GeneralPropertyGroupName"/>.
        /// </summary>
        /// <param name="propertyName">The name of the general property.</param>
        /// <param name="value">The general property (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetGeneralProperty(string propertyName, out InfoOutputProperty value)
        {
            if (propertyName != null)
            {
                return m_GeneralProperties.TryGetValue(propertyName, out value);
            }
            value = default(InfoOutputProperty);
            return false;
        }

        /// <summary>Gets a specific <see cref="InfoOutputProperty"/> object which represents the value of a property with respect to <see cref="GeneralPropertyGroupName"/>.
        /// </summary>
        /// <param name="propertyName">The name of the general property.</param>
        /// <param name="value">The general property (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetGeneralProperty(IdentifierString propertyName, out InfoOutputProperty value)
        {
            if (propertyName != null)
            {
                return m_GeneralProperties.TryGetValue(propertyName, out value);
            }
            value = default(InfoOutputProperty);
            return false;
        }

        /// <summary>Adds a specified <see cref="DataTable"/> object, i.e. with respect to <see cref="DataTableType.Single"/>. 
        /// </summary>
        /// <param name="value">Homogeneous informations in some <see cref="System.Data.DataTable"/> representation.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="value"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the table name of <paramref name="value"/> is empty.</exception>
        public void Add(DataTable value)
        {
            if (value == null)
            {
                throw new ArgumentNullException("value");
            }
            if ((value.TableName == null) || (value.TableName.Length == 0))
            {
                throw new ArgumentException(String.Format(ExceptionMessages.ArgumentIsNotWellDefined, "Table name"), "value");
            }
            m_Tables.Add(value.TableName, value);
        }

        /// <summary>Adds a specified <see cref="InfoOutputParentChildDataTable"/> object, i.e. a parent and child <see cref="DataTable"/> object with a 1:n relation. 
        /// </summary>
        /// <param name="value">Homogeneous informations in some <see cref="InfoOutputParentChildDataTable"/> representation.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="value"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the table name(s) of <paramref name="value"/> is empty.</exception>
        public void Add(InfoOutputParentChildDataTable value)
        {
            if (value == null)
            {
                throw new ArgumentNullException("value");
            }
            if ((value.ParentDataTable == null) || (value.ParentDataTable.TableName == null) || (value.ParentDataTable.TableName.Length == 0))
            {
                throw new ArgumentException(String.Format(ExceptionMessages.ArgumentIsNotWellDefined, "Parent DataTable name"), "value");
            }
            m_ParentChildTables.Add(value.ParentDataTable.TableName, value);
        }

        /// <summary>Adds a specified <see cref="InfoOutputParentChildDataTable"/> object, i.e. a parent and child <see cref="DataTable"/> object with a 1:n relation. 
        /// </summary>
        /// <param name="parentDataTable">The parent data table.</param>
        /// <param name="childDataTable">The child data table.</param>
        /// <param name="parentDataColumn">The parent data column, i.e. the column with respect to the relation between parent und child data table.</param>
        /// <param name="childDataColumn">The child data column, i.e. the column with respect to the relation between parent und child data table.</param>
        public void Add(DataTable parentDataTable, DataTable childDataTable, DataColumn parentDataColumn, DataColumn childDataColumn)
        {
            Add(new InfoOutputParentChildDataTable(parentDataTable, childDataTable, parentDataColumn, childDataColumn));
        }

        /// <summary>Adds a specified collection of <see cref="InfoOutputProperty"/> objects to the current instance.
        /// </summary>
        /// <param name="propertyGroupName">The name of the property group (i.e. 'General Properties' etc.).</param>
        /// <param name="properties">The properties to add, if <c>null</c> an empty property group will be added.</param>
        public void Add(string propertyGroupName, params InfoOutputProperty[] properties)
        {
            if (propertyGroupName == null)
            {
                throw new ArgumentNullException("propertyCategory");
            }

            if (properties != null)
            {
                IdentifierStringDictionary<InfoOutputProperty> propertyCollection;
                if (m_Properties.TryGetValue(propertyGroupName, out propertyCollection) == false)
                {
                    propertyCollection = new IdentifierStringDictionary<InfoOutputProperty>(isReadOnlyExceptAdding: false);
                    m_Properties.Add(propertyGroupName, propertyCollection);
                }
                foreach (var property in properties)
                {
                    propertyCollection.Add(property.Name, property);
                }
            }
        }

        /// <summary>Adds a specified collection of <see cref="InfoOutputProperty"/> objects to the current instance with respect to <see cref="GeneralPropertyGroupName"/>.
        /// </summary>
        /// <param name="generalProperties">The general properties to add.</param>
        public void Add(params InfoOutputProperty[] generalProperties)
        {
            if (generalProperties != null)
            {
                foreach (var property in generalProperties)
                {
                    m_GeneralProperties.Add(property.Name, property);
                }
            }
        }

        /// <summary>Adds a specific property to the current instance with respect to <see cref="GeneralPropertyGroupName"/>.
        /// </summary>
        /// <param name="generalPropertyName">The name of the general property.</param>
        /// <param name="generalPropertyValue">The value of the general property.</param>
        public void Add(string generalPropertyName, object generalPropertyValue)
        {
            InfoOutputProperty property = new InfoOutputProperty(generalPropertyName, generalPropertyValue);
            m_GeneralProperties.Add(property.Name, property);
        }

        /// <summary>Adds a specific property to the current instance with respect to <see cref="GeneralPropertyGroupName"/>.
        /// </summary>
        /// <param name="generalPropertyName">The name of the general property.</param>
        /// <param name="generalPropertyValue">The value of the general property.</param>
        /// <param name="annotation">A description of the property.</param>
        public void Add(string generalPropertyName, object generalPropertyValue, string annotation)
        {
            InfoOutputProperty property = new InfoOutputProperty(generalPropertyName, generalPropertyValue, annotation);
            m_GeneralProperties.Add(property.Name, property);
        }
        #endregion
    }
}