/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

namespace Dodoni.BasicComponents.Containers
{
    /// <summary>Serves as property (i.e. name/value pair) representation of a specific information with respect to <see cref="InfoOutput"/>.
    /// </summary>
    public class InfoOutputProperty : IIdentifierNameable, IAnnotatable
    {
        #region private members

        /// <summary>The property name in its identifier string representation.
        /// </summary>
        private IdentifierString m_IdPropertyName;

        /// <summary>The annotation of the property, perhaps <c>null</c>.
        /// </summary>
        private string m_Annotation;
        #endregion

        #region public (readonly) members

        /// <summary>The value of the property in its <see cref="System.Object"/> representation.
        /// </summary>
        public readonly object Value;
        #endregion

        #region public properties

        #region IIdentifierNameable Members

        /// <summary>Gets the name of the property in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <value>The language independent name of the current instance.</value>
        public IdentifierString Name
        {
            get { return m_IdPropertyName; }
        }

        /// <summary>Gets the long name of the current instance, i.e. <see cref="InfoOutputProperty.Name"/> in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <value>The language dependent long name of the current instance.</value>
        IdentifierString IIdentifierNameable.LongName
        {
            get { return m_IdPropertyName; }
        }
        #endregion

        #region IAnnotatable Members

        /// <summary>Gets a value indicating whether the annotation is readonly.
        /// </summary>
        /// <value><c>true</c> if the annotation of this instance is readonly; otherwise, <c>false</c>.
        /// </value>
        bool IAnnotatable.HasReadOnlyAnnotation
        {
            get { return true; }
        }

        /// <summary>Gets the annotation of the current instance.
        /// </summary>
        /// <value>The annotation of the current instance.</value>
        public string Annotation
        {
            get { return m_Annotation; }
        }
        #endregion

        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="InfoOutputProperty"/> class.
        /// </summary>
        /// <param name="propertyName">The name of the property.</param>
        /// <param name="propertyValue">The value of the property.</param>
        /// <param name="annotation">A annotation of the property in its <see cref="System.String"/> representation.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="propertyName"/> or <paramref name="propertyValue"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="propertyName"/> represents the empty string.</exception>
        public InfoOutputProperty(string propertyName, object propertyValue, string annotation = null)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            if (propertyName.Length == 0)
            {
                throw new ArgumentException(String.Format(ExceptionMessages.ArgumentIsInvalid, "Property name ''"), "propertyName");
            }

            if (propertyValue == null)
            {
                throw new ArgumentNullException("propertyValue");
            }
            Value = propertyValue;
            m_Annotation = annotation;
            m_IdPropertyName = new IdentifierString(propertyName);
        }
        #endregion

        #region public methods

        #region IAnnotatable Members

        /// <summary>Sets the annotation of the current instance.
        /// </summary>
        /// <param name="annotation">The annotation.</param>
        /// <returns>A value indicating whether the <see cref="Annotation"/> has been changed.
        /// </returns>
        public bool TrySetAnnotation(string annotation)
        {
            return false;
        }
        #endregion

        /// <summary>Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return String.Format("{0} = {1}", Name, Value);
        }
        #endregion

        #region public (static) methods

        /// <summary>Creates a new <see cref="InfoOutputProperty"/> object.
        /// </summary>
        /// <param name="propertyName">The name of the property.</param>
        /// <param name="propertyValue">The value of the property.</param>
        /// <param name="annotation">A annotation of the property in its <see cref="System.String"/> representation.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="propertyName"/> or <paramref name="propertyValue"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="propertyName"/> represents the empty string.</exception>
        public static InfoOutputProperty Create(string propertyName, object propertyValue, string annotation = null)
        {
            return new InfoOutputProperty(propertyName, propertyValue, annotation);
        }
        #endregion
    }
}