/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Resources;
using System.Threading;
using System.Reflection;
using System.Collections.Generic;

namespace Dodoni.BasicComponents
{
    /// <summary>Represents a <see cref="System.String"/> object and a 'normalized' representation; each character which comes next to 
    /// <see cref="IdentifierString.IgnoringStartCharacter"/> will be ignored.
    /// </summary>
    public class IdentifierString : IComparable<String>, IComparable<IdentifierString>, IEquatable<IdentifierString>, IEquatable<string>
    {
        #region nested classes

        /// <summary>Represents a comparer where the identifier string is taken into account only.
        /// </summary>
        private class Compararer : IEqualityComparer<IdentifierString>
        {
            #region IEqualityComparer<IdentifierString> Members

            /// <summary>Determines whether the specified objects are equal.
            /// </summary>
            /// <param name="x">The first object of type <see cref="IdentifierString"/> to compare.</param>
            /// <param name="y">The second object of type <see cref="IdentifierString"/> to compare.</param>
            /// <returns><c>true</c> if the specified objects are equal; otherwise, <c>false</c>.</returns>
            public bool Equals(IdentifierString x, IdentifierString y)
            {
                return x.IDString == y.IDString;
            }

            /// <summary>Returns a hash code for a specific <see cref="IdentifierString"/> object.
            /// </summary>
            /// <param name="obj">The <see cref="IdentifierString"/> object.</param>
            /// <returns>A hash code for <paramref name="obj"/>, suitable for use in hashing algorithms and data structures like a hash table. 
            /// </returns>
            public int GetHashCode(IdentifierString obj)
            {
                return obj.IDString.GetHashCode();
            }
            #endregion
        }
        #endregion

        #region public readonly static members

        /// <summary>A comparer implementation where the identifier string is taken into account only.
        /// </summary>
        public static readonly IEqualityComparer<IdentifierString> EqualityComparer = new Compararer();

        /// <summary>Represents the empty <see cref="IdentifierString"/>. This field is read-only.
        /// </summary>
        public static readonly IdentifierString Empty = new IdentifierString(String.Empty);

        /// <summary>The character which represents the start token for some comment, i.e. each character which comes next to this
        /// character will be ignored.
        /// </summary>
        public const char IgnoringStartCharacter = '@';
        #endregion

        #region public (readonly) members

        /// <summary>The (raw) <see cref="System.String"/> representation.
        /// </summary>
        public readonly string String;

        /// <summary>The <see cref="IdentifierString.String"/> object in its 'normalized' <see cref="System.String"/> representation.
        /// </summary>
        public readonly string IDString;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="IdentifierString"/> class.
        /// </summary>
        /// <param name="rawString">The (raw) string.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="rawString"/> is <c>null</c>.</exception>
        public IdentifierString(string rawString)
        {
            if (rawString == null)
            {
                throw new ArgumentNullException("rawString");
            }
            String = rawString;
            IDString = GetIDString(rawString);
        }
        #endregion

        #region public methods

        #region IComparable<IdentifierString> Member

        /// <summary>Compares this instance to a specified <see cref="IdentifierString"/> object and returns an indication of their relative values.
        /// </summary>
        /// <param name="other">A <see cref="IdentifierString"/> object to compare to this instance.</param>
        /// <returns>A 32-bit signed integer that indicates the relative order of the objects being compared. The return value has this meanings:
        /// <para>Less than zero: This instance is less than <paramref name="other"/>.</para>
        /// <para>Zero: This instance is equal to <paramref name="other"/>.</para>
        /// <para>Greater than zero: This instance is greater than <paramref name="other"/>.</para>
        /// </returns>
        /// <remarks>The <see cref="IDString"/> is used for the compare operation.</remarks>
        public int CompareTo(IdentifierString other)
        {
            return this.IDString.CompareTo(other.IDString);
        }
        #endregion

        #region IComparable<string> Members

        /// <summary>Compares this instance to a specified <see cref="String"/> object and returns an indication of their relative values.
        /// </summary>
        /// <param name="other">A <see cref="String"/> object to compare to this instance.</param>
        /// <returns>A 32-bit signed integer that indicates the relative order of the objects being compared. The return value has this meanings:
        /// <para>Less than zero: This instance is less than <paramref name="other"/>.</para>
        /// <para>Zero: This instance is equal to <paramref name="other"/>.</para>
        /// <para>Greater than zero: This instance is greater than <paramref name="other"/>.</para>
        /// </returns>
        /// <remarks>The <see cref="IDString"/> is used for the compare operation.</remarks>
        public int CompareTo(string other)
        {
            return this.IDString.CompareTo(GetIDString(other));
        }
        #endregion

        #region IEquatable<IdentifierString> Members

        /// <summary>Indicates whether the current object is equal to another object of the same type.
        /// </summary>
        /// <param name="other">An object to compare with this object.</param>
        /// <returns><c>true</c> if the current object is equal to the <paramref name="other"/> parameter; otherwise, <c>false</c>.
        /// </returns>
        public bool Equals(IdentifierString other)
        {
            return IDString.Equals(other.IDString);
        }
        #endregion

        #region IEquatable<string> Members

        /// <summary>Indicates whether the current object is equal to another <see cref="System.String"/> object.
        /// </summary>
        /// <param name="other">An object to compare with this object.</param>
        /// <returns><c>true</c> if the current object is equal to the <paramref name="other"/> parameter; otherwise, <c>false</c>.
        /// </returns>
        /// <remarks>The <see cref="IDString"/> is used for the compare operation.</remarks>
        public bool Equals(string other)
        {
            return IDString.Equals(GetIDString(other));
        }
        #endregion

        /// <summary>Determines whether the specified <see cref="System.Object"/> is equal to this instance.
        /// </summary>
        /// <param name="obj">The <see cref="System.Object"/> to compare with this instance.</param>
        /// <returns><c>true</c> if the specified <see cref="System.Object"/> is equal to this instance; otherwise, <c>false</c>.
        /// </returns>
        /// <remarks>The <see cref="IDString"/> is used for the compare operation.</remarks>
        public override bool Equals(object obj)
        {
            if (obj is IdentifierString)
            {
                return (IDString == (((IdentifierString)obj).IDString));
            }
            else if (obj is string)
            {
                return (IDString == GetIDString((string)obj));
            }
            return false;
        }

        /// <summary>Returns a hash code for this instance.
        /// </summary>
        /// <returns>A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table. 
        /// </returns>
        /// <remarks>The <see cref="IDString"/> is used for the compare operation.</remarks>
        public override int GetHashCode()
        {
            return IDString.GetHashCode();
        }

        /// <summary>Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return String;
        }
        #endregion

        #region public static methods

        /// <summary>Creates a new <see cref="IdentifierString"/>.
        /// </summary>
        /// <param name="rawString">The (raw) string.</param>
        /// <returns>A <see cref="IdentifierString"/> object that represents <paramref name="rawString"/>.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="rawString"/> is <c>null</c>.</exception>
        public static IdentifierString Create(string rawString)
        {
            return new IdentifierString(rawString);
        }

        /// <summary>Creates a new <see cref="IdentifierString"/>.
        /// </summary>
        /// <param name="resourceManager">The resource manager with respect to the resource which contains the <see cref="System.String"/> representations.</param>
        /// <param name="resourcePropertyName">The property name with respect to a given <paramref name="resourceManager"/> which contains some language dependend <see cref="System.String"/> representation.</param>
        /// <exception cref="ArgumentException">Thrown if no resource file is found or the property name is not valid for the given resource file.</exception>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="resourceManager"/> is <c>null</c>.</exception>
        /// <remarks>Use this method if a <see cref="IdentifierString"/> should be language depended, i.e. if a resource file is available.</remarks>
        public static IdentifierString Create(ResourceManager resourceManager, string resourcePropertyName)
        {
            if (resourceManager == null)
            {
                throw new ArgumentNullException("resourceManager");
            }
            try
            {
                string rawString = resourceManager.GetString(resourcePropertyName, Thread.CurrentThread.CurrentUICulture);
                if (rawString == null)
                {
                    throw new ArgumentException(String.Format("Resource file name {0} or resource property name {1} invalid.", resourceManager.BaseName, resourcePropertyName));
                }
                return new IdentifierString(rawString);
            }
            catch (MissingManifestResourceException e)
            {
                throw new ArgumentException(String.Format("Resource file name {0} or resource property name {1} invalid.", resourceManager.BaseName, resourcePropertyName), e);
            }
            catch (InvalidOperationException e)
            {
                throw new ArgumentException(String.Format("Resource file name {0} or resource property name {1} invalid.", resourceManager.BaseName, resourcePropertyName), e);
            }
        }

        /// <summary>Creates a new <see cref="IdentifierString"/>.
        /// </summary>
        /// <param name="fullResourceName">The resource name (no language dependend suffix) and the corresponding namespace.</param>
        /// <param name="resourcePropertyName">The property name with respect to a given resource which contains some language dependend <see cref="System.String"/> representation.</param>
        /// <param name="assembly">The assembly to take into account.</param>
        /// <exception cref="ArgumentException">Thrown if no resource file is found or the property name is not valid for the given resource file.</exception>
        /// <remarks>Use this method if a <see cref="IdentifierString"/> should be language depended, i.e. if a resource file is available.</remarks>
        public static IdentifierString Create(string fullResourceName, string resourcePropertyName, Assembly assembly)
        {
            var resourceManager = new ResourceManager(fullResourceName, assembly);

            return Create(resourceManager, resourcePropertyName);
        }

        /// <summary>Performs an explicit conversion from <see cref="Dodoni.BasicComponents.IdentifierString"/> to <see cref="System.String"/>.
        /// </summary>
        /// <param name="identifierString">The identifier string.</param>
        /// <returns>The result of the conversion, i.e. the 'raw' <see cref="System.String"/> representation.</returns>
        public static explicit operator string(IdentifierString identifierString)
        {
            return (identifierString != null) ? identifierString.String : null;
        }
        #endregion

        #region internal static methods

        /// <summary>Gets the ID string, i.e. a 'normalized' <see cref="String"/> representation.
        /// </summary>
        /// <param name="rawString">The (raw) string.</param>
        /// <param name="ignorCommentCharacter">A value indicating whether all character which comes next to <see cref="IdentifierString.IgnoringStartCharacter"/> will be ignored.</param>
        /// <returns>A 'normalized' representation of <paramref name="rawString"/>, especially without white spaces.
        /// </returns>
        internal static string GetIDString(string rawString, bool ignorCommentCharacter = true)
        {
            if (ignorCommentCharacter == true)
            {
                int splitIndex = rawString.IndexOf(IgnoringStartCharacter);
                if (splitIndex >= 0)
                {
                    rawString = rawString.Remove(splitIndex);
                }
            }
            StringBuilder strBuilder = new StringBuilder(rawString.Length);
            for (int i = 0; i < rawString.Length; i++)
            {
                char c = rawString[i];
                if (Char.IsWhiteSpace(c) == false)
                {
                    strBuilder.Append(Char.ToLower(c));
                }
            }
            return strBuilder.ToString();
        }
        #endregion
    }
}