﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes;

namespace Dodoni.BasicComponents.LogFileFactory
{
    /// <summary>Serves as logfile generator.
    /// </summary>
    /// <remarks>Implement a logfile generator and store a (static) instance in <see cref="LoggingFactory"/> for a custom logfile.</remarks>
    public interface ILogFileGenerator
    {
        /// <summary>Gets a value indicating whether the log file generator is already in use.
        /// </summary>
        /// <value><c>true</c> if at least one <see cref="ILogFile"/> object has been created; otherwise, <c>false</c>.</value>
        /// <remarks>This flag will be take into account to thrown an exception if at least one <see cref="ILogFile"/> object has been created.</remarks>
        bool IsInUse
        {
            get;
        }

        /// <summary>Gets the standard <see cref="LogFileOutputType"/> used for creating <see cref="ILogFile"/> objects.
        /// </summary>
        /// <value>The standard log file output type.</value>
        LogFileOutputType StandardLogFileOutputType
        {
            get;
        }

        /// <summary>Creates a new <see cref="ILogFile"/> object.
        /// </summary>
        /// <param name="objectName">The object name.</param>
        /// <param name="objectTypeName">The name of the object type.</param>
        /// <param name="objectTypeID">The object type ID.</param>
        /// <param name="logFileOutputType">The type of the logfile output.</param>
        /// <returns>A new <see cref="ILogFile"/> instance with respect to the <paramref name="logFileOutputType"/>.</returns>
        /// <remarks>Fatal error messages must be written into a global logfile (if available), even if the logfile is some 'local' logfile.</remarks>
        ILogFile CreateLogFile(string objectName, string objectTypeName, System.Guid objectTypeID, LogFileOutputType logFileOutputType);

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        void AddGlobalLogFileMessage(ILogFileMessageType messageType, string message, ILogging loggingObject = null);

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        void AddGlobalLogFileMessage(WarningLogFileMessageType messageType, string message, ILogging loggingObject = null);

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        void AddGlobalLogFileMessage(InfoLogFileMessageType messageType, string message, ILogging loggingObject = null);

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        void AddGlobalLogFileMessage(ErrorLogFileMessageType messageType, string message, ILogging loggingObject = null);

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        void AddGlobalLogFileMessage(FatalErrorLogFileMessageType messageType, string message, ILogging loggingObject = null);
    }
}