﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes
{
    /// <summary>Represents a pool of logfile error message types.
    /// </summary>
    public class ErrorMessageTypePool
    {
        #region private members

        /// <summary>Represents the mapping of the logfile message type ID and the corresponding logfile message type.
        /// </summary>
        private Dictionary<Guid, ErrorLogFileMessageType> m_LogFileMessageIDTypeMapping;
        #endregion

        #region public (readonly) members

        /// <summary>Represents the (generic) error logfile message.
        /// </summary>
        /// <remarks>The MessageTypeID is {218C68FC-6D59-4B0F-9406-476343180E37}.</remarks>
        public readonly ErrorLogFileMessageType Generic = new ErrorLogFileMessageType("Error message [generic]", LogFileMessageStrings.ErrorMsgString, new Guid(0x218c68fc, 0x6d59, 0x4b0f,  0x94, 0x6, 0x47, 0x63, 0x43, 0x18, 0xe, 0x37 ));

        /// <summary>Represents the error logfile message which indicates the failure of a specific calibration.
        /// </summary>
        /// <remarks>The MessageTypeID is {46CE269C-40A6-4A4A-B2F8-44B0918927AF}.</remarks>
        public readonly ErrorLogFileMessageType CalibrationFailure = new ErrorLogFileMessageType("Calibration failure", LogFileMessageStrings.CalibrationFailedErrorMsgString, new Guid(0x46ce269c, 0x40a6, 0x4a4a, 0xb2, 0xf8, 0x44, 0xb0, 0x91, 0x89, 0x27, 0xaf));

        /// <summary>Represents the error logfile message which occures if some object has been created but it can not been added to a specific pool.
        /// </summary>
        /// <example>Holiday calendar pool, currency pool etc.</example>
        /// <remarks>The MessageTypeID is {18BDAEAD-F23B-411D-80D3-4195D9E4B21C}.</remarks>
        public readonly ErrorLogFileMessageType PoolElementAddingFailure = new ErrorLogFileMessageType("Pool element adding failure", LogFileMessageStrings.PoolElementAddingFailureErrorMsgString, new Guid(0x18bdaead, 0xf23b, 0x411d, 0x80, 0xd3, 0x41, 0x95, 0xd9, 0xe4, 0xb2, 0x1c));
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="ErrorMessageTypePool"/> class.
        /// </summary>
        internal ErrorMessageTypePool()
        {
            m_LogFileMessageIDTypeMapping = new Dictionary<Guid, ErrorLogFileMessageType>();
            m_LogFileMessageIDTypeMapping.Add(Generic.TypeID, Generic);
            m_LogFileMessageIDTypeMapping.Add(CalibrationFailure.TypeID, CalibrationFailure);
            m_LogFileMessageIDTypeMapping.Add(PoolElementAddingFailure.TypeID, PoolElementAddingFailure);
        }
        #endregion

        #region public properties

        /// <summary>Gets the message type ID's, i.e. the <see cref="System.Guid"/> representations of the message types.
        /// </summary>
        /// <value>The unique message type ID's.</value>
        public IEnumerable<Guid> MessageTypeIDs
        {
            get { return m_LogFileMessageIDTypeMapping.Keys; }
        }

        /// <summary>Gets the message types, i.e. the list of available error message types.
        /// </summary>
        /// <value>The error message types.</value>
        public IEnumerable<ErrorLogFileMessageType> MessageTypes
        {
            get { return m_LogFileMessageIDTypeMapping.Values; }
        }
        #endregion

        #region public methods

        /// <summary>Gets the error logfile message type for a specific <see cref="System.Guid"/> object, taken into account the <see cref="ErrorLogFileMessageType.TypeID"/>.
        /// </summary>
        /// <param name="errorLogFileMessageTypeID">The (unique) logfile message type ID.</param>
        /// <param name="errorLogFileMessageType">The error logfile message type with respect to <paramref name="errorLogFileMessageTypeID"/> (output).</param>
        /// <returns>A value indicating whether <paramref name="errorLogFileMessageType"/> contains valid data.</returns>
        public bool TryGetMessageType(Guid errorLogFileMessageTypeID, out ErrorLogFileMessageType errorLogFileMessageType)
        {
            return m_LogFileMessageIDTypeMapping.TryGetValue(errorLogFileMessageTypeID, out errorLogFileMessageType);
        }
        #endregion

        #region internal methods

        /// <summary>Register a specific error message type, i.e. add some error message type to the pool.
        /// </summary>
        /// <param name="errorLogFileMessageType">The error message type to register.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="errorLogFileMessageType"/> is <c>null</c>.</exception>
        internal void RegisterMessageType(ErrorLogFileMessageType errorLogFileMessageType)
        {
            if (errorLogFileMessageType == null)
            {
                throw new ArgumentNullException("errorMsgType");
            }
            m_LogFileMessageIDTypeMapping.Add(errorLogFileMessageType.TypeID, errorLogFileMessageType);
        }
        #endregion
    }
}