﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes
{
    /// <summary>Represents a pool of logfile fatal error message types.
    /// </summary>
    public class FatalErrorMessageTypePool
    {
        #region private members

        /// <summary>Represents the mapping of the logfile message type ID and the corresponding logfile message type.
        /// </summary>
        private Dictionary<Guid, FatalErrorLogFileMessageType> m_LogFileMessageIDTypeMapping;
        #endregion

        #region public (readonly) members

        /// <summary>Represents the (generic) fatal error logfile message.
        /// </summary>
        /// <remarks>The MessageTypeID is {2888F890-80BE-48F3-BD0B-DB90DB5F6BC4}.</remarks>
        public readonly FatalErrorLogFileMessageType Generic = new FatalErrorLogFileMessageType("Fatal error [generic]", LogFileMessageStrings.FatalErrorMsgString, new Guid(0x2888f890, 0x80be, 0x48f3, 0xbd, 0xb, 0xdb, 0x90, 0xdb, 0x5f, 0x6b, 0xc4));
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="FatalErrorMessageTypePool"/> class.
        /// </summary>
        internal FatalErrorMessageTypePool()
        {
            m_LogFileMessageIDTypeMapping = new Dictionary<Guid, FatalErrorLogFileMessageType>();
            m_LogFileMessageIDTypeMapping.Add(Generic.TypeID, Generic);
        }
        #endregion

        #region public properties

        /// <summary>Gets the message type ID's, i.e. the <see cref="System.Guid"/> representations of the message types.
        /// </summary>
        /// <value>The unique message type ID's.</value>
        public IEnumerable<Guid> MessageTypeIDs
        {
            get { return m_LogFileMessageIDTypeMapping.Keys; }
        }

        /// <summary>Gets the message types, i.e. the list of available fatal error message types.
        /// </summary>
        /// <value>The fatal error message types.</value>
        public IEnumerable<FatalErrorLogFileMessageType> MessageTypes
        {
            get { return m_LogFileMessageIDTypeMapping.Values; }
        }
        #endregion

        #region public methods

        /// <summary>Gets the fatal error logfile message type for a specific <see cref="System.Guid"/> object, taken into account the <see cref="FatalErrorLogFileMessageType.TypeID"/>.
        /// </summary>
        /// <param name="fatalErrorLogFileMessageTypeID">The (unique) logfile message type ID.</param>
        /// <param name="fatalErrorLogFileMessageType">The fatal error logfile message type with respect to <paramref name="fatalErrorLogFileMessageTypeID"/> (output).</param>
        /// <returns>A value indicating whether <paramref name="fatalErrorLogFileMessageType"/> contains valid data.</returns>
        public bool TryGetMessageType(Guid fatalErrorLogFileMessageTypeID, out FatalErrorLogFileMessageType fatalErrorLogFileMessageType)
        {
            return m_LogFileMessageIDTypeMapping.TryGetValue(fatalErrorLogFileMessageTypeID, out fatalErrorLogFileMessageType);
        }
        #endregion

        #region internal methods

        /// <summary>Register some fatal error message type, i.e. add some fatal error message type to the pool.
        /// </summary>
        /// <param name="fatalErrorLogFileMessageType">The fatal error message type to register.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="fatalErrorLogFileMessageType"/> is <c>null</c>.</exception>
        internal void RegisterMessageType(FatalErrorLogFileMessageType fatalErrorLogFileMessageType)
        {
            if (fatalErrorLogFileMessageType == null)
            {
                throw new ArgumentNullException("fatalErrorLogFileMessageType");
            }
            m_LogFileMessageIDTypeMapping.Add(fatalErrorLogFileMessageType.TypeID, fatalErrorLogFileMessageType);
        }
        #endregion
    }
}