﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes
{
    /// <summary>Represents a pool of logfile info message types.
    /// </summary>
    public class InfoMessageTypePool
    {
        #region private members

        /// <summary>Represents the mapping of the logfile message type ID and the corresponding logfile message type.
        /// </summary>
        private Dictionary<Guid, InfoLogFileMessageType> m_LogFileMessageIDTypeMapping;
        #endregion

        #region public (readonly) members

        /// <summary>Represents the (generic) info logfile message type.
        /// </summary>
        /// <remarks>The MessageTypeID is {CD052F69-A7BF-4C8F-A91E-1F50EE05A026}.</remarks>
        public readonly InfoLogFileMessageType Generic = new InfoLogFileMessageType("Info message [generic]", LogFileMessageStrings.InfoMsgString, new Guid(0xcd052f69, 0xa7bf, 0x4c8f, 0xa9, 0x1e, 0x1f, 0x50, 0xee, 0x5, 0xa0, 0x26));

        /// <summary>Represents the info logfile message type which indicates that the creation of some
        /// new object (instrument, market data object) was started.
        /// </summary>
        /// <remarks>The MessageTypeID is {44C876E9-4CF5-4148-86CB-313F553F6E7E}.</remarks>
        public readonly InfoLogFileMessageType CreatingObject = new InfoLogFileMessageType("Object creating", LogFileMessageStrings.CreatingObjectInfoMsgString, new Guid(0x44c876e9, 0x4cf5, 0x4148, 0x86, 0xcb, 0x31, 0x3f, 0x55, 0x3f, 0x6e, 0x7e));

        /// <summary>Represents the info logfile message type which indicates that some new object (instrument, market data object) has been created.
        /// </summary>
        /// <remarks>The MessageTypeID is {CDF8E301-E05A-48B2-85DF-96F1B132186B}.</remarks>
        public readonly InfoLogFileMessageType CreateObject = new InfoLogFileMessageType("Object created", LogFileMessageStrings.CreateObjectInfoMsgString, new Guid(0xcdf8e301, 0xe05a, 0x48b2, 0x85, 0xdf, 0x96, 0xf1, 0xb1, 0x32, 0x18, 0x6b));

        /// <summary>Represents the info logfile message type which indicates that some object instance has been cleared, i.e. each contained element has been removed.
        /// </summary>
        /// <remarks>The MessageTypeID is {515EA3C1-C45A-49E4-BDA0-035CAEDBE557}.</remarks>
        public readonly InfoLogFileMessageType ClearObject = new InfoLogFileMessageType("Object cleared", LogFileMessageStrings.ClearObjectInfoMsgString, new Guid(0x515ea3c1, 0xc45a, 0x49e4, 0xbd, 0xa0, 0x3, 0x5c, 0xae, 0xdb, 0xe5, 0x57));

        /// <summary>Represents the info logfile message type which indicates that some standard value is taken into account.
        /// </summary>
        /// <remarks>The MessageTypeID is {A496C8FD-5BE9-4550-9130-901C1147BF14}.</remarks>
        public readonly InfoLogFileMessageType StandardValueUsed = new InfoLogFileMessageType("Standard value set", LogFileMessageStrings.StandardValueInfoMsgString, new Guid(0xa496c8fd, 0x5be9, 0x4550, 0x91, 0x30, 0x90, 0x1c, 0x11, 0x47, 0xbf, 0x14));

        /// <summary>Represents the info logfile message type which indicates that the preparation of some calibration procedure has been started.
        /// </summary>
        /// <remarks>The MessageTypeID is {9B7CDA9D-9934-4251-94B2-6B856D631C12}.</remarks>
        public readonly InfoLogFileMessageType CalibrationPrepared = new InfoLogFileMessageType("Calibration prepared", LogFileMessageStrings.CalibrationPreparationInfoMsgString, new Guid(0x9b7cda9d, 0x9934, 0x4251, 0x94, 0xb2, 0x6b, 0x85, 0x6d, 0x63, 0x1c, 0x12));

        /// <summary>Represents the info logfile message type which indicates that some calibration procedure has been started.
        /// </summary>
        /// <remarks>The MessageTypeID is {62126CF9-A320-4E3B-A237-DF62263A3FA2}.</remarks>
        public readonly InfoLogFileMessageType CalibrationStarted = new InfoLogFileMessageType("Calibration started", LogFileMessageStrings.CalibrationStartInfoMsgString, new Guid(0x62126cf9, 0xa320, 0x4e3b, 0xa2, 0x37, 0xdf, 0x62, 0x26, 0x3a, 0x3f, 0xa2));

        /// <summary>Represents the info logfile message type which indicates that some calibration procedure has been terminated.
        /// </summary>
        /// <remarks>The MessageTypeID is {663F38B6-48AE-4720-9AF3-160637482751}.</remarks>
        public readonly InfoLogFileMessageType CalibrationTerminated = new InfoLogFileMessageType("Calibration terminated", LogFileMessageStrings.CalibrationTerminatedInfoMsgString, new Guid(0x663f38b6, 0x48ae, 0x4720, 0x9a, 0xf3, 0x16, 0x6, 0x37, 0x48, 0x27, 0x51));

        /// <summary>Represents the info logfile message type which indicates that for a specific pool 
        /// a new element has been created and added.
        /// </summary>
        /// <remarks>The MessageTypeID is {D1865739-07A3-465E-AF03-EDADEA6B87C1}.</remarks>
        public readonly InfoLogFileMessageType PoolElementAdded = new InfoLogFileMessageType("Pool element added", LogFileMessageStrings.PoolElementAddedInfoMsgString, new Guid(0xd1865739, 0x7a3, 0x465e, 0xaf, 0x3, 0xed, 0xad, 0xea, 0x6b, 0x87, 0xc1));

        /// <summary>Represents the info logfile message which indicates that for a specific pool 
        /// some element has been changed, i.e. overwritten.
        /// </summary>
        /// <remarks>The MessageTypeID is {5E746E7E-AE18-445B-BE46-1D508509A9EE}.</remarks>
        public readonly InfoLogFileMessageType PoolElementReplaced = new InfoLogFileMessageType("Pool element replaced", LogFileMessageStrings.PoolElementReplacedInfoMsgString, new Guid(0x5e746e7e, 0xae18, 0x445b, 0xbe, 0x46, 0x1d, 0x50, 0x85, 0x9, 0xa9, 0xee));
        #endregion

        #region internal constructors

        /// <summary>Initializes the <see cref="InfoMessageTypePool"/> class.
        /// </summary>
        internal InfoMessageTypePool()
        {
            m_LogFileMessageIDTypeMapping = new Dictionary<Guid, InfoLogFileMessageType>();

            m_LogFileMessageIDTypeMapping.Add(Generic.TypeID, Generic);
            m_LogFileMessageIDTypeMapping.Add(CreatingObject.TypeID, CreatingObject);
            m_LogFileMessageIDTypeMapping.Add(CreateObject.TypeID, CreateObject);
            m_LogFileMessageIDTypeMapping.Add(ClearObject.TypeID, ClearObject);
            m_LogFileMessageIDTypeMapping.Add(StandardValueUsed.TypeID, StandardValueUsed);
            m_LogFileMessageIDTypeMapping.Add(CalibrationPrepared.TypeID, CalibrationPrepared);
            m_LogFileMessageIDTypeMapping.Add(CalibrationStarted.TypeID, CalibrationStarted);
            m_LogFileMessageIDTypeMapping.Add(CalibrationTerminated.TypeID, CalibrationTerminated);
            m_LogFileMessageIDTypeMapping.Add(PoolElementAdded.TypeID, PoolElementAdded);
            m_LogFileMessageIDTypeMapping.Add(PoolElementReplaced.TypeID, PoolElementReplaced);
        }
        #endregion

        #region public properties

        /// <summary>Gets the message type ID's, i.e. the <see cref="System.Guid"/> representations of the message types.
        /// </summary>
        /// <value>The unique message type ID's.</value>
        public IEnumerable<Guid> MessageTypeIDs
        {
            get { return m_LogFileMessageIDTypeMapping.Keys; }
        }

        /// <summary>Gets the message types, i.e. the list of available error message types.
        /// </summary>
        /// <value>The error message types.</value>
        public IEnumerable<InfoLogFileMessageType> MessageTypes
        {
            get { return m_LogFileMessageIDTypeMapping.Values; }
        }
        #endregion

        #region public methods

        /// <summary>Gets the info logfile message type for a specific <see cref="System.Guid"/> object, taken into account the <see cref="InfoLogFileMessageType.TypeID"/>.
        /// </summary>
        /// <param name="infoLogFileMessageTypeID">The (unique) logfile message type ID.</param>
        /// <param name="infoLogFileMessageType">The info logfile message type with respect to <paramref name="infoLogFileMessageTypeID"/> (output).</param>
        /// <returns>A value indicating whether <paramref name="infoLogFileMessageType"/> contains valid data.</returns>
        public bool TryGetMessageType(Guid infoLogFileMessageTypeID, out InfoLogFileMessageType infoLogFileMessageType)
        {
            return m_LogFileMessageIDTypeMapping.TryGetValue(infoLogFileMessageTypeID, out infoLogFileMessageType);
        }
        #endregion

        #region internal methods

        /// <summary>Register a specific info message type, i.e. add some info message type to the pool.
        /// </summary>
        /// <param name="infoLogFileMessageType">The info message type to register.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="infoLogFileMessageType"/> is <c>null</c>.</exception>
        internal void RegisterMessageType(InfoLogFileMessageType infoLogFileMessageType)
        {
            if (infoLogFileMessageType == null)
            {
                throw new ArgumentNullException("infoLogFileMessageType");
            }
            m_LogFileMessageIDTypeMapping.Add(infoLogFileMessageType.TypeID, infoLogFileMessageType);
        }
        #endregion
    }
}