﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes
{
    /// <summary>Represents the warning message types available for the logfile.
    /// </summary>
    public class WarningMessageTypePool
    {
        #region private members

        /// <summary>Represents the mapping of the logfile message type ID and the corresponding logfile message type.
        /// </summary>
        private Dictionary<Guid, WarningLogFileMessageType> m_LogFileMessageIDTypeMapping;
        #endregion

        #region public (readonly) members

        /// <summary>Represents the (generic) warning logfile message type.
        /// </summary>
        /// <remarks>The MessageTypeID is {6F4E4E1D-2498-4CFF-A01E-6DD63CCA4CD4}.</remarks>
        public readonly WarningLogFileMessageType Generic = new WarningLogFileMessageType("Warning message [generic]", LogFileMessageStrings.WarningMsgString, new Guid(0x6f4e4e1d, 0x2498, 0x4cff, 0xa0, 0x1e, 0x6d, 0xd6, 0x3c, 0xca, 0x4c, 0xd4));

        /// <summary>Represents the warning logfile message type for the case that some <see cref="System.DateTime"/> object is not a business day.
        /// </summary>
        /// <remarks>The MessageTypeID is {15C49FEF-4700-4125-AD62-7B6731EF24F6}.</remarks>
        public readonly WarningLogFileMessageType NoBusinessDay = new WarningLogFileMessageType("No business day", LogFileMessageStrings.NoBusinessDayWarningMsgString, new Guid(0x15c49fef, 0x4700, 0x4125, 0xad, 0x62, 0x7b, 0x67, 0x31, 0xef, 0x24, 0xf6));
        #endregion

        #region internal constructors

        /// <summary>Initializes the <see cref="WarningMessageTypePool"/> class.
        /// </summary>
        internal WarningMessageTypePool()
        {
            m_LogFileMessageIDTypeMapping = new Dictionary<Guid, WarningLogFileMessageType>();
            m_LogFileMessageIDTypeMapping.Add(Generic.TypeID, Generic);
            m_LogFileMessageIDTypeMapping.Add(NoBusinessDay.TypeID, NoBusinessDay);
        }
        #endregion

        #region public properties

        /// <summary>Gets the message type ID's, i.e. the <see cref="System.Guid"/> representations of the message types.
        /// </summary>
        /// <value>The unique message type ID's.</value>
        public IEnumerable<Guid> MessageTypeIDs
        {
            get { return m_LogFileMessageIDTypeMapping.Keys; }
        }

        /// <summary>Gets the message types, i.e. the list of available warning message types.
        /// </summary>
        /// <value>The warning message types.</value>
        public IEnumerable<WarningLogFileMessageType> MessageTypes
        {
            get { return m_LogFileMessageIDTypeMapping.Values; }
        }
        #endregion

        #region public methods

        /// <summary>Gets the error logfile message type for a specific <see cref="System.Guid"/> object, taken into account the <see cref="WarningLogFileMessageType.TypeID"/>.
        /// </summary>
        /// <param name="warningLogFileMessageTypeID">The (unique) logfile message type ID.</param>
        /// <param name="warningLogFileMessageType">The error logfile message type with respect to <paramref name="warningLogFileMessageTypeID"/> (output).</param>
        /// <returns>A value indicating whether <paramref name="warningLogFileMessageType"/> contains valid data.</returns>
        public bool TryGetMessageType(Guid warningLogFileMessageTypeID, out WarningLogFileMessageType warningLogFileMessageType)
        {
            return m_LogFileMessageIDTypeMapping.TryGetValue(warningLogFileMessageTypeID, out warningLogFileMessageType);
        }
        #endregion

        #region internal methods

        /// <summary>Register a specific warning message type, i.e. add some warning message type to the pool.
        /// </summary>
        /// <param name="warningLogFileMessageType">The error message type to register.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="warningLogFileMessageType"/> is <c>null</c>.</exception>
        internal void RegisterMessageType(WarningLogFileMessageType warningLogFileMessageType)
        {
            if (warningLogFileMessageType == null)
            {
                throw new ArgumentNullException("warningLogFileMessageType");
            }
            m_LogFileMessageIDTypeMapping.Add(warningLogFileMessageType.TypeID, warningLogFileMessageType);
        }
        #endregion
    }
}