﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.BasicComponents.LogFileFactory
{
    /// <summary>Provide a basis class for classes which contains some logfile functionality as well as some name.
    /// </summary>
    public class LogObject : ILogging, IIdentifierNameable
    {
        #region public (readonly) members

        /// <summary>The logfile.
        /// </summary>
        /// <remarks>The <see cref="System.Guid"/> in the logfile will be used for the global logfile (if desired) as the unique object name ID.
        /// </remarks>
        public readonly ILogFile LogFile;
        #endregion

        #region private members

        /// <summary>The name of the instance in its <see cref="IdentifierString"/> representation.
        /// </summary>
        private IdentifierString m_ObjectIdentifier;

        /// <summary>The name of the instance and some additional time stamp.
        /// </summary>
        private IdentifierString m_TimeStampObjectIdentifier;
        #endregion

        #region protected constructors

        /// <summary>Initializes a new instance of the <see cref="LogObject"/> class.
        /// </summary>
        /// <param name="objectName">The name of the object; if <c>null</c> or empty a standard name will be used.</param>
        /// <param name="objectTypeName">The name of the object type.</param>
        /// <param name="objectTypeID">The unique object type ID.</param>
        protected LogObject(string objectName, string objectTypeName, System.Guid objectTypeID)
        {
            if ((objectName == null) || (objectName.Length == 0))
            {
                objectName = LogFileStrings.UnknownObjectName;
            }
            m_ObjectIdentifier = new IdentifierString(objectName);
            m_TimeStampObjectIdentifier = new IdentifierString(objectName.ToTimeStampString());

            LogFile = LoggingFactory.CreateLogFile(objectName, objectTypeName, objectTypeID);
            LogFile.AddMessage(LoggingFactory.Infos.CreateObject, objectName);
        }

        /// <summary>Initializes a new instance of the <see cref="LogObject"/> class.
        /// </summary>
        /// <param name="logObject">The <see cref="LogObject"/> instance.</param>
        /// <param name="objectTypeName">The name of the object type.</param>
        /// <param name="objectTypeID">The unique object type ID.</param>
        /// <remarks>This copy constructor creates a new a some new <see cref="LogFile"/> instance. The object type (ID) is not stored in the <paramref name="logObject"/>.</remarks>
        protected LogObject(LogObject logObject, string objectTypeName, System.Guid objectTypeID)
        {
            string objectName = logObject.m_ObjectIdentifier.String;
            m_TimeStampObjectIdentifier = new IdentifierString(objectName.ToTimeStampString());

            m_ObjectIdentifier = new IdentifierString(objectName);
            LogFile = LoggingFactory.CreateLogFile(objectName, objectTypeName, objectTypeID);
            LogFile.AddMessage(LoggingFactory.Infos.CreateObject, objectName);
        }

        /// <summary>Initializes a new instance of the <see cref="LogObject"/> class.
        /// </summary>
        /// <param name="objectName">The name of the object.</param>
        /// <param name="sharedLogFile">A shared log file.</param>
        /// <exception cref="ArgumentNullException">Thrown if <paramref name="sharedLogFile"/> is <c>null</c>.</exception>
        /// <remarks>Call this constructor to share the some log-file instance for different <see cref="LogObject"/> instances.
        /// </remarks>
        protected LogObject(string objectName, ILogFile sharedLogFile)
        {
            if (sharedLogFile == null)
            {
                throw new ArgumentNullException("sharedLogFile");
            }
            LogFile = sharedLogFile;
            m_ObjectIdentifier = new IdentifierString(objectName);
            m_TimeStampObjectIdentifier = new IdentifierString(objectName.ToTimeStampString());
        }
        #endregion

        #region public properties

        #region ILogging Members

        /// <summary>Gets the logfile.
        /// </summary>
        /// <value>The logfile.</value>
        ILogFile ILogging.LogFile
        {
            get { return this.LogFile; }
        }
        #endregion

        #region IIdentifierNameable Members

        /// <summary>Gets the name with respect to the current instance.
        /// </summary>
        /// <value>The name of the current instance.</value>
        /// <remarks>The return value of this property is equal to <see cref="m_ObjectIdentifier"/>.</remarks>
        public IdentifierString Name
        {
            get { return m_ObjectIdentifier; }
        }

        /// <summary>Gets the long name of the current instance, i.e. the <see cref="LogObject.Name"/> of the current instance and some additional time stamp.
        /// </summary>
        /// <value>The long name of the current instance, i.e. the <see cref="LogObject.Name"/> of the current instance and some additional time stamp.</value>
        public IdentifierString LongName
        {
            get { return m_TimeStampObjectIdentifier; }
        }
        #endregion

        #endregion

        #region public methods

        /// <summary>Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        /// <remarks>The string representation of the current instance, i.e. the string component of
        /// the <see cref="m_ObjectIdentifier"/> is taken into account.
        /// </remarks>
        public override string ToString()
        {
            return m_ObjectIdentifier.String;
        }
        #endregion
    }
}