﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Data;
using System.Collections.Generic;

using Dodoni.BasicComponents.Containers;
using Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes;

namespace Dodoni.BasicComponents.LogFileFactory
{
    /// <summary>Serves as factory for logfiles as well as collection of logfile message types.
    /// </summary>
    public static class LoggingFactory
    {
        #region private static members

        /// <summary>The (custom) logfile generator.
        /// </summary>
        private static ILogFileGenerator sm_LogFileGenerator = new DiagnosticsLogFileGenerator();
        #endregion

        #region public (readonly) members

        /// <summary>The <see cref="System.Guid"/> instance which is used for unknown object types.
        /// </summary>
        public static readonly System.Guid UnknownObjectTypeGuid = Guid.Empty;

        /// <summary>The <c>fatal error</c> message types.
        /// </summary>
        public static readonly FatalErrorMessageTypePool FatalErrors;

        /// <summary>The <c>error</c> message types.
        /// </summary>
        public static readonly ErrorMessageTypePool Errors;

        /// <summary>The <c>warning</c> message types.
        /// </summary>
        public static readonly WarningMessageTypePool Warnings;

        /// <summary>The <c>info</c> message types.
        /// </summary>
        public static readonly InfoMessageTypePool Infos;
        #endregion

        #region static constructor

        /// <summary>Initializes the <see cref="LoggingFactory"/> class.
        /// </summary>
        static LoggingFactory()
        {
            FatalErrors = new FatalErrorMessageTypePool();
            Errors = new ErrorMessageTypePool();
            Warnings = new WarningMessageTypePool();
            Infos = new InfoMessageTypePool();
        }
        #endregion

        #region public static methods

        #region register methods

        /// <summary>Register a specific logfile message type.
        /// </summary>
        /// <param name="messageType">The logfile message type.</param>
        /// <remarks>This method is thread safe. Call this method if an additional (fatal) error/warning/info message type will be used,
        /// for example if .NET extensions are used for some new message type.</remarks>
        /// <exception cref="ArgumentNullException">Thrown if <paramref name="messageType"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown if the message type ID of <paramref name="messageType"/> is already used by an other logfile message type.</exception>
        public static void RegisterLogFileMsgType(InfoLogFileMessageType messageType)
        {
            if (messageType == null)
            {
                throw new ArgumentNullException("messageType");
            }
            Infos.RegisterMessageType(messageType);
        }

        /// <summary>Register a specific logfile message type.
        /// </summary>
        /// <param name="messageType">The logfile message type.</param>
        /// <remarks>This method is thread safe. Call this method if an additional (fatal) error/warning/info message type will be used,
        /// for example if .NET extensions are used for some new message type.</remarks>
        /// <exception cref="ArgumentNullException">Thrown if <paramref name="messageType"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown if the message type ID of <paramref name="messageType"/> is already used by an other logfile message type.</exception>
        public static void RegisterLogFileMsgType(WarningLogFileMessageType messageType)
        {
            if (messageType == null)
            {
                throw new ArgumentNullException("messageType");
            }
            Warnings.RegisterMessageType(messageType);
        }

        /// <summary>Register a specific logfile message type.
        /// </summary>
        /// <param name="messageType">The logfile message type.</param>
        /// <remarks>This method is thread safe. Call this method if an additional (fatal) error/warning/info message type will be used,
        /// for example if .NET extensions are used for some new message type.</remarks>
        /// <exception cref="ArgumentNullException">Thrown if <paramref name="messageType"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown if the message type ID of <paramref name="messageType"/> is already used by an other logfile message type.</exception>
        public static void RegisterLogFileMsgType(ErrorLogFileMessageType messageType)
        {
            if (messageType == null)
            {
                throw new ArgumentNullException("messageType");
            }
            Errors.RegisterMessageType(messageType);
        }

        /// <summary>Register a specific logfile message type.
        /// </summary>
        /// <param name="messageType">The logfile message type.</param>
        /// <remarks>This method is thread safe. Call this method if an additional (fatal) error/warning/info message type will be used,
        /// for example if .NET extensions are used for some new message type.</remarks>
        /// <exception cref="ArgumentNullException">Thrown if <paramref name="messageType"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown if the message type ID of <paramref name="messageType"/> is already used by an other logfile message type.</exception>
        public static void RegisterLogFileMsgType(FatalErrorLogFileMessageType messageType)
        {
            if (messageType == null)
            {
                throw new ArgumentNullException("messageType");
            }
            FatalErrors.RegisterMessageType(messageType);
        }
        #endregion

        #region logfile creation methods

        /// <summary>Creates a new <see cref="ILogFile"/> object.
        /// </summary>
        /// <param name="objectName">The object name.</param>
        /// <param name="objectTypeName">The name of the object type.</param>
        /// <param name="objectTypeID">The (unique) object type ID.</param>
        /// <returns>A new <see cref="ILogFile"/> instance with respect to the <see cref="ILogFileGenerator.StandardLogFileOutputType"/>.</returns>
        /// <remarks>Fatal error messages are written into the global logfile, even if the logfile is some 'local' logfile.</remarks>
        public static ILogFile CreateLogFile(string objectName, string objectTypeName, System.Guid objectTypeID)
        {
            return sm_LogFileGenerator.CreateLogFile(objectName, objectTypeName, objectTypeID, sm_LogFileGenerator.StandardLogFileOutputType);
        }

        /// <summary>Creates a new <see cref="ILogFile"/> object with respect to a global logfile.
        /// </summary>
        /// <param name="objectName">The object name.</param>
        /// <param name="objectTypeName">The name of the object type.</param>
        /// <param name="objectTypeID">The (unique) object type ID.</param>
        /// <returns>A new <see cref="ILogFile"/> object that represents the global log file.</returns>
        public static ILogFile CreateGlobalLogFile(string objectName, string objectTypeName, System.Guid objectTypeID)
        {
            return sm_LogFileGenerator.CreateLogFile(objectName, objectTypeName, objectTypeID, LogFileOutputType.WriteIntoGlobalLogFile);
        }

        /// <summary>Creates a new <see cref="ILogFile"/> object.
        /// </summary>
        /// <param name="objectTypeName">The name of the object type.</param>
        /// <param name="objectTypeID">The (unique) object type ID.</param>
        /// <returns>A new <see cref="ILogFile"/> instance with respect to the <see cref="ILogFileGenerator.StandardLogFileOutputType"/>.</returns>
        /// <remarks>Fatal error messages are written into the global logfile, even if the logfile is some 'local' logfile.
        /// <para>The object name is set to 'unknown'.</para></remarks>
        public static ILogFile CreateLogFile(string objectTypeName, System.Guid objectTypeID)
        {
            return sm_LogFileGenerator.CreateLogFile(LogFileStrings.UnknownObjectName, objectTypeName, objectTypeID, sm_LogFileGenerator.StandardLogFileOutputType);
        }
        #endregion

        #region add global logfile message methods

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public static void AddGlobalLogFileMessage(InfoLogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            sm_LogFileGenerator.AddGlobalLogFileMessage(messageType, message, loggingObject);
        }

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public static void AddGlobalLogFileMessage(WarningLogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            sm_LogFileGenerator.AddGlobalLogFileMessage(messageType, message, loggingObject);
        }

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public static void AddGlobalLogFileMessage(ErrorLogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            sm_LogFileGenerator.AddGlobalLogFileMessage(messageType, message, loggingObject);
        }

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public static void AddGlobalLogFileMessage(FatalErrorLogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            sm_LogFileGenerator.AddGlobalLogFileMessage(messageType, message, loggingObject);
        }

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public static void AddGlobalLogFileMessage(ILogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            sm_LogFileGenerator.AddGlobalLogFileMessage(messageType, message, loggingObject);
        }
        #endregion

        #region extension methods

        /// <summary>Adds a specific 'default value message' into a logfile.
        /// </summary>
        /// <param name="logFile">The logfile object.</param>
        /// <param name="name">The name of the property.</param>
        /// <param name="value">The (default) value in its <see cref="System.String"/> representation.</param>
        public static void AddDefaultValueMessage(this ILogFile logFile, string name, string value)
        {
            logFile.AddMessage(LoggingFactory.Infos.StandardValueUsed, name + " := " + value);
        }

        /// <summary>Adds a specific 'default value message' into a logfile.
        /// </summary>
        /// <param name="logFile">The logfile object.</param>
        /// <param name="name">The name of the property.</param>
        /// <param name="value">The (default) value in its <see cref="IdentifierString"/> representation.</param>
        public static void AddDefaultValueMessage(this ILogFile logFile, string name, IdentifierString value)
        {
            logFile.AddMessage(LoggingFactory.Infos.StandardValueUsed, name + " := " + value.String);
        }

        /// <summary>Adds a specific 'default value message' into a logfile.
        /// </summary>
        /// <param name="logFile">The logfile object.</param>
        /// <param name="name">The name of the property.</param>
        /// <param name="value">The (default) value.</param>
        public static void AddDefaultValueMessage(this ILogFile logFile, string name, double value)
        {
            logFile.AddMessage(LoggingFactory.Infos.StandardValueUsed, name + " := " + value);
        }

        /// <summary>Adds a specific 'default value message' into a logfile.
        /// </summary>
        /// <param name="logFile">The logfile object.</param>
        /// <param name="name">The name of the property.</param>
        /// <param name="value">The (default) value.</param>
        public static void AddDefaultValueMessage(this ILogFile logFile, string name, int value)
        {
            logFile.AddMessage(LoggingFactory.Infos.StandardValueUsed, name + " := " + value);
        }

        /// <summary>Adds a specific 'default value message' into a logfile.
        /// </summary>
        /// <param name="logFile">The logfile object.</param>
        /// <param name="name">The name of the property.</param>
        /// <param name="value">The (default) value.</param>
        public static void AddDefaultValueMessage(this ILogFile logFile, string name, bool value)
        {
            logFile.AddMessage(LoggingFactory.Infos.StandardValueUsed, name + " := " + value);
        }

        /// <summary>Adds a log file message if some pool element has been added, overwritten or the adding has been failure.
        /// </summary>
        /// <param name="logFile">The log file.</param>
        /// <param name="state">The state of the element to add to the pool.</param>
        /// <param name="objectName">The object name, i.e. the name of the element to add into the pool; perhaps <c>null</c> if no object name is available.</param>
        public static void AddPoolAddingLogFileMessage(this ILogFile logFile, ItemAddedState state, IdentifierString objectName)
        {
            string message = ((objectName != null) && (objectName.String != null) && (objectName.String.Length > 0)) ? objectName.String : "UnknownObjectName"; // ToDo: LogFileStrings.UnknownObjectName;

            switch (state)
            {
                case ItemAddedState.Added:
                    logFile.AddMessage(LoggingFactory.Infos.PoolElementAdded, message);
                    break;

                case ItemAddedState.Replaced:
                    logFile.AddMessage(LoggingFactory.Infos.PoolElementReplaced, message);
                    break;

                case ItemAddedState.Rejected:
                    logFile.AddMessage(LoggingFactory.Errors.PoolElementAddingFailure, message);
                    break;

                default:
                    logFile.AddMessage(LoggingFactory.Errors.Generic, message);
                    break;
            }
        }
        #endregion

        /// <summary>Sets the logfile generator.
        /// </summary>
        /// <param name="logFileGenerator">The logfile generator.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="logFileGenerator"/> is <c>null</c>.</exception>
        public static void SetLogFileGenerator(ILogFileGenerator logFileGenerator)
        {
            if (logFileGenerator == null)
            {
                throw new ArgumentNullException("logFileGenerator");
            }
            lock (sm_LogFileGenerator)
            {
                if (sm_LogFileGenerator.IsInUse)
                {
                    throw new InvalidOperationException("Set a custom logfile generator at program start only!");
                }
                sm_LogFileGenerator = logFileGenerator;
            }
        }

        /// <summary>Gets the message types, i.e. for (fatal) error, warning and info message types.
        /// </summary>
        /// <value>The message types.</value>
        public static IEnumerable<ILogFileMessageType> GetMessageTypes()
        {
            foreach (var fatalErrorMsgType in FatalErrors.MessageTypes)
            {
                yield return fatalErrorMsgType;
            }
            foreach (var errorMsgType in Errors.MessageTypes)
            {
                yield return errorMsgType;
            }
            foreach (var warningMsgType in Warnings.MessageTypes)
            {
                yield return warningMsgType;
            }
            foreach (var infoMsgType in Infos.MessageTypes)
            {
                yield return infoMsgType;
            }
        }
        #endregion
    }
}