﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

namespace Dodoni.BasicComponents.Utilities
{
    /// <summary>Represents an attribute class that contains a language depending description, more precisely it contains 
    /// the property name of a resource file which contains a specific language depending <see cref="System.String"/> representation.
    /// </summary>
    /// <example>
    /// <code>
    ///   [LanguageResource("YourNamespace.ResourceFileName")]
    ///   public enum Example {
    ///    [String("Number 1")]   // or use LanguageStringAttribute
    ///    [LanguageDescriptionAttribute("Example_1", "Test")]
    ///     No_1,
    ///     
    ///     // ...
    ///   }
    ///   // Moreover a resource file is given with respect to the 'LanguageResource' attribute
    ///   // which contains some element with name 'Example_1' (no code generation is necessary),
    ///   // for example: "Example_1": "This is my {0}."
    ///   ...
    ///   Example example = Example.No_1;
    ///   string name = example.ToFormatString();
    ///   string description = example.GetDescription();  // = "This is my Test."
    /// </code></example>
    [AttributeUsage(AttributeTargets.All, Inherited = false, AllowMultiple = false)]
    public sealed class LanguageDescriptionAttribute : System.Attribute
    {
        #region private/public (readonly) members

        /// <summary>The property name with respect to a given resource which contains some language dependend <see cref="System.String"/> representation.
        /// </summary>
        public readonly string ResourcePropertyName;

        /// <summary>The first argument used in the <see cref="System.String.Format(string,object)"/> method needed to generate the <see cref="System.String"/> representation.
        /// </summary>
        private object m_Arg0;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="LanguageDescriptionAttribute"/> class.
        /// </summary>
        /// <param name="resourcePropertyName">The property name of the language dependend <see cref="System.String"/> representation with respect 
        /// to a resource file which is represented by an instance of <see cref="LanguageResourceAttribute"/>. The resource string represents some composite format string.</param>
        public LanguageDescriptionAttribute(string resourcePropertyName)
        {
            ResourcePropertyName = resourcePropertyName;
        }
        #endregion

        #region public properties

        /// <summary>Gets or sets the first argument used in the <see cref="System.String.Format(string,object)"/> method 
        /// needed to generate the <see cref="System.String"/> representation of the current instance.
        /// </summary>
        /// <value>The first argument needed for the <see cref="System.String"/> representation.</value>
        public object Arg0
        {
            get { return m_Arg0; }
            set { m_Arg0 = value; }
        }
        #endregion
    }
}