﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2013 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.IO;
using System.Xml;
using System.Data;
using System.Collections;
using System.Globalization;
using System.Collections.Generic;

using Dodoni.BasicComponents.Containers;

namespace Dodoni.BasicComponents.Utilities
{
    /// <summary>Represents a configuration file, i.e. a collection of names and values.
    /// </summary>
    public class ConfigurationFile
    {
        #region nested classes

        /// <summary>Represents a collection of properties in a configuration file, i.e. a specified section in the xml file representation of the configuration file.
        /// </summary>
        public class PropertyCollection : IInfoOutputQueriable, IIdentifierNameable
        {
            #region private members

            /// <summary>The configuration file in its <see cref="ConfigurationFile"/> representation.
            /// </summary>
            private ConfigurationFile m_ConfigurationFile;

            /// <summary>The name of the collection of properties configuration file.
            /// </summary>
            private IdentifierString m_PropertyCollectionName;

            /// <summary>The parent node of the section in the xml file that contains the collection of properties.
            /// </summary>
            private XmlNode m_PropertyCollectionParentNode;

            /// <summary>A value indicating whether the configuration file already contains settings with respect to the current <see cref="ConfigurationFile.PropertyCollection"/> object.
            /// </summary>
            private bool m_ContainsSettings;
            #endregion

            #region internal protected constructors

            /// <summary>Initializes a new instance of the <see cref="PropertyCollection" /> class.
            /// </summary>
            /// <param name="configurationFile">The configuration file.</param>
            /// <param name="propertyCollectionName">The name of the property collection.</param>
            internal protected PropertyCollection(ConfigurationFile configurationFile, IdentifierString propertyCollectionName)
            {
                m_ConfigurationFile = configurationFile;
                m_PropertyCollectionName = propertyCollectionName;

                m_PropertyCollectionParentNode = configurationFile.GetSectionNode(propertyCollectionName.IDString, out m_ContainsSettings);
            }


            /// <summary>Initializes a new instance of the <see cref="PropertyCollection" /> class.
            /// </summary>
            /// <param name="configurationFile">The configuration file.</param>
            /// <param name="propertyCollectionName">The name of the property collection.</param>
            internal protected PropertyCollection(ConfigurationFile configurationFile, string propertyCollectionName)
                : this(configurationFile, IdentifierString.Create(propertyCollectionName))
            {
            }
            #endregion

            #region public properties

            #region IIdentifierNameable Members

            /// <summary>Gets the name of the current instance.
            /// </summary>
            /// <value>The language independent name of the current instance.</value>
            public IdentifierString Name
            {
                get { return m_PropertyCollectionName; }
            }

            /// <summary>Gets the long name of the current instance.
            /// </summary>
            /// <value>The (perhaps) language dependent long name of the current instance.</value>
            public IdentifierString LongName
            {
                get { return m_PropertyCollectionName; }
            }
            #endregion

            #region IInfoOutputQueriable Members

            /// <summary>Gets the info-output level of detail.
            /// </summary>
            /// <value>The info-output level of detail.</value>
            public InfoOutputDetailLevel InfoOutputDetailLevel
            {
                get { return InfoOutputDetailLevel.Full; }
            }
            #endregion

            /// <summary>Gets a value indicating whether the configuration file already contains settings with respect to the current <see cref="ConfigurationFile"/> object; will be set to <c>true</c>
            /// after calling <see cref="Save()"/>.
            /// </summary>
            /// <value><c>true</c> if the configuration file already contains some settings; otherwise, <c>false</c>.</value>
            public bool ContainsSettings
            {
                get { return m_ContainsSettings; }
            }
            #endregion

            #region public methods

            #region IInfoOutputQueriable Members

            /// <summary>Sets the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel" /> property.
            /// </summary>
            /// <param name="infoOutputDetailLevel">The info-output level of detail.</param>
            /// <returns>A value indicating whether the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel" /> has been set to <paramref name="infoOutputDetailLevel" />.
            /// </returns>
            public bool TrySetInfoOutputDetailLevel(InfoOutputDetailLevel infoOutputDetailLevel)
            {
                return (infoOutputDetailLevel == Containers.InfoOutputDetailLevel.Full);
            }

            /// <summary>Gets informations of the current object as a specific <see cref="InfoOutput" /> instance.
            /// </summary>
            /// <param name="infoOutput">The <see cref="InfoOutput" /> object which is to be filled with informations concering the current instance.</param>
            /// <param name="categoryName">The name of the category, i.e. all informations will be added to these category.</param>
            public void FillInfoOutput(InfoOutput infoOutput, string categoryName = InfoOutput.GeneralCategoryName)
            {
                var infoPackage = infoOutput.AcquirePackage(categoryName);

                if (m_PropertyCollectionParentNode.HasChildNodes == true)
                {
                    for (int j = 0; j < m_PropertyCollectionParentNode.ChildNodes.Count; j++)
                    {
                        var node = m_PropertyCollectionParentNode.ChildNodes[j];

                        var InfoProperty = new InfoOutputProperty(node.Name, node.InnerText);
                        infoPackage.Add(InfoProperty);
                    }
                }
            }
            #endregion

            #region setValue methods

            /// <summary>Sets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the XML file.</param>
            /// <param name="value">The value in its <see cref="System.String"/> representation.</param>
            public void SetValue(string key, string value)
            {
                var keyValuePairs = m_PropertyCollectionParentNode.SelectNodes(key);
                if (keyValuePairs.Count == 0)
                {
                    var newConfigNode = m_ConfigurationFile.m_XmlFileRepresenation.CreateElement(key);
                    newConfigNode.AppendChild(m_ConfigurationFile.m_XmlFileRepresenation.CreateTextNode(value));
                    m_PropertyCollectionParentNode.AppendChild(newConfigNode);
                }
                else if (keyValuePairs.Count == 1)
                {
                    keyValuePairs[0].InnerText = value;
                }
                else
                {
                    throw new ConfigurationFileErrorException(String.Format("Configuration file {0} is invalid for key {1}.", m_ConfigurationFile.m_FilePath, key));
                }
            }

            /// <summary>Sets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the XML file.</param>
            /// <param name="value">The value in its <see cref="System.Boolean"/> representation.</param>
            public void SetValue(string key, bool value)
            {
                string boolValueString = (value == true) ? Boolean.TrueString : Boolean.FalseString;
                SetValue(key, boolValueString);
            }

            /// <summary>Sets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the XML file.</param>
            /// <param name="value">The value in its <see cref="System.Enum"/> representation.</param>
            public void SetValue(string key, Enum value)
            {
                SetValue(key, value.ToString());
            }

            /// <summary>Sets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the XML file.</param>
            /// <param name="value">The value in its <see cref="System.Enum"/> representation.</param>
            public void SetValue(string key, double value)
            {
                SetValue(key, value.ToString(CultureInfo.InvariantCulture));
            }

            /// <summary>Sets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the XML file.</param>
            /// <param name="value">The value in its <see cref="System.Enum"/> representation.</param>
            public void SetValue(string key, int value)
            {
                SetValue(key, value.ToString(CultureInfo.InvariantCulture));
            }
            #endregion

            #region getValue methods

            /// <summary>Gets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the xml file.</param>
            /// <param name="value">The value in its <see cref="System.String"/> representation (output).</param>
            /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
            public bool TryGetValue(string key, out string value)
            {
                var keyValuePairs = m_PropertyCollectionParentNode.SelectNodes(key);
                if (keyValuePairs.Count == 0)
                {
                    value = String.Empty;
                    return false;
                }
                else if (keyValuePairs.Count == 1)
                {
                    value = keyValuePairs[0].InnerText;
                    return true;
                }
                else
                {
                    throw new ConfigurationFileErrorException(String.Format("Configuration file {0} is invalid for key {1}.", m_ConfigurationFile.m_FilePath, key));
                }
            }

            /// <summary>Gets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the xml file.</param>
            /// <param name="value">The value in its <see cref="System.String"/> representation (output).</param>
            /// <param name="defaultValue">The default value to take into account if the configuration file does not contain an entry for <paramref name="key"/>.</param>
            /// <returns>A value indicating whether <paramref name="value"/> contains some data from the configuration file.</returns>
            public bool TryGetValue(string key, out string value, string defaultValue)
            {
                var keyValuePairs = m_PropertyCollectionParentNode.SelectNodes(key);
                if (keyValuePairs.Count == 0)
                {
                    value = defaultValue;
                    return false;
                }
                else if (keyValuePairs.Count == 1)
                {
                    value = keyValuePairs[0].InnerText;
                    return true;
                }
                else
                {
                    throw new ConfigurationFileErrorException(String.Format("Configuration file {0} is invalid for key {1}.", m_ConfigurationFile.m_FilePath, key));
                }
            }

            /// <summary>Gets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the xml file.</param>
            /// <param name="value">The value in its <see cref="System.Boolean"/> representation (output).</param>
            /// <param name="defaultValue">The default value to take into account if the configuration file does not contain an entry for <paramref name="key"/>.</param>
            /// <returns>A value indicating whether <paramref name="value"/> contains some data from the configuration file.</returns>
            public bool TryGetValue(string key, out bool value, bool defaultValue = false)
            {
                string tempValue;

                if (TryGetValue(key, out tempValue) && Boolean.TryParse(tempValue, out value))
                {
                    return true;
                }
                value = defaultValue;
                return false;
            }

            /// <summary>Gets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the xml file.</param>
            /// <param name="value">The value in its <see cref="System.Double"/> representation (output).</param>
            /// <param name="defaultValue">The default value to take into account if the configuration file does not contain an entry for <paramref name="key"/>.</param>
            /// <returns>A value indicating whether <paramref name="value"/> contains some data from the configuration file.</returns>
            public bool TryGetValue(string key, out double value, double defaultValue = Double.NaN)
            {
                string tempValue;

                if (TryGetValue(key, out tempValue) && Double.TryParse(tempValue, NumberStyles.Any, CultureInfo.InvariantCulture, out value))
                {
                    return true;
                }
                value = defaultValue;
                return false;
            }

            /// <summary>Gets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the xml file.</param>
            /// <param name="value">The value in its <see cref="System.Int32"/> representation (output).</param>
            /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
            public bool TryGetValue(string key, out int value)
            {
                string tempValue;

                if (TryGetValue(key, out tempValue) && Int32.TryParse(tempValue, out value))
                {
                    return true;
                }
                value = default(Int32);
                return false;
            }

            /// <summary>Gets a specific entry in the configuration file.
            /// </summary>
            /// <param name="key">The key, i.e. the key in the xml file.</param>
            /// <param name="value">The value in its <see cref="System.Int32"/> representation (output).</param>
            /// <param name="defaultValue">The default value to take into account if the configuration file does not contain an entry for <paramref name="key"/>.</param>
            /// <returns>A value indicating whether <paramref name="value"/> contains some data from the configuration file.</returns>
            public bool TryGetValue(string key, out int value, int defaultValue)
            {
                string tempValue;

                if (TryGetValue(key, out tempValue) && Int32.TryParse(tempValue, out value))
                {
                    return true;
                }
                value = defaultValue;
                return false;
            }

            /// <summary>Gets a specific entry in the configuration file.
            /// </summary>
            /// <typeparam name="TEnum">The type of the enumeration.</typeparam>
            /// <param name="key">The key, i.e. the key in the xml file.</param>
            /// <param name="value">The value in its <typeparamref name="TEnum"/> representation (output).</param>
            /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
            public bool TryGetEnumValue<TEnum>(string key, out TEnum value)
                where TEnum : struct
            {
                string tempValue;

                if (TryGetValue(key, out tempValue) && Enum.TryParse<TEnum>(tempValue, true, out value))
                {
                    return true;
                }
                value = default(TEnum);
                return false;
            }
            #endregion

            #endregion
        }

        /// <summary>Represents a collection of homogenious entries in a configuration file.
        /// </summary>
        public class Table : IInfoOutputQueriable, IIdentifierNameable
        {
            #region nested classes

            /// <summary>The collection of each data entry in the table.
            /// </summary>
            public class DataCollection : IEnumerable<IList<string>>
            {
                #region private nested classes

                /// <summary>The Enumerator implementation of the <see cref="DataCollection"/> class.
                /// </summary>
                private class Enumerator : IEnumerator<IList<string>>
                {
                    #region private members

                    private DataCollection m_DataCollection;
                    private int m_CurrentIndex = -1;
                    #endregion

                    #region internal constructors

                    /// <summary>Initializes a new instance of the <see cref="Enumerator" /> class.
                    /// </summary>
                    /// <param name="dataCollection">The data collection.</param>
                    internal Enumerator(DataCollection dataCollection)
                    {
                        m_DataCollection = dataCollection;
                    }
                    #endregion

                    #region IEnumerator<IList<string>> Members

                    /// <summary>Gets the element in the collection at the current position of the enumerator.
                    /// </summary>
                    /// <value>The element in the collection at the current position of the enumerator.</value>
                    public IList<string> Current
                    {
                        get { return m_DataCollection[m_CurrentIndex]; }
                    }

                    #endregion

                    #region IDisposable Members

                    /// <summary>Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
                    /// </summary>
                    public void Dispose()
                    {
                    }
                    #endregion

                    #region IEnumerator Members

                    /// <summary> Gets the element in the collection at the current position of the enumerator.
                    /// </summary>
                    /// <value>The element in the collection at the current position of the enumerator.</value>
                    object IEnumerator.Current
                    {
                        get { return m_DataCollection[m_CurrentIndex]; }
                    }

                    /// <summary>Advances the enumerator to the next element of the collection.
                    /// </summary>
                    /// <returns><c>true</c> if the enumerator was successfully advanced to the next element; <c>false</c> if the enumerator has passed the end of the collection.</returns>
                    public bool MoveNext()
                    {
                        m_CurrentIndex++;
                        return m_CurrentIndex < m_DataCollection.Count;
                    }

                    /// <summary>Sets the enumerator to its initial position, which is before the first element in the collection.
                    /// </summary>
                    public void Reset()
                    {
                        m_CurrentIndex = -1;
                    }
                    #endregion
                }
                #endregion

                #region private members

                /// <summary>The reference to the <see cref="Table"/> object.
                /// </summary>
                private Table m_Table;

                /// <summary>The name of the entries in the data table, for example 'Plain Vanilla option'.
                /// </summary>
                private IdentifierString m_TableEntryName;

                /// <summary>The parent node of the section in the XML file that represents the data.
                /// </summary>
                private XmlNode m_TableParentNode;
                #endregion

                #region public constructors

                /// <summary>Initializes a new instance of the <see cref="DataCollection" /> class.
                /// </summary>
                /// <param name="table">The reference to the <see cref="Table"/> object.</param>
                /// <param name="tableParentNode">The parent node of the section in the XML file that represents the data.</param>
                /// <param name="tableEntryName">The name of each data table entry.</param>
                public DataCollection(Table table, XmlNode tableParentNode, IdentifierString tableEntryName)
                {
                    m_Table = table;

                    m_TableEntryName = tableEntryName;
                    m_TableParentNode = tableParentNode;
                }
                #endregion

                #region public properties

                /// <summary>Gets the number of entries.
                /// </summary>
                /// <value>The number of entries.
                /// </value>
                public int Count
                {
                    get { return m_TableParentNode.ChildNodes.Count; }
                }

                /// <summary>Gets the data at a specified null-based index.
                /// </summary>
                /// <param name="index">The null-based index.</param>
                /// <returns>For each field the value in its <see cref="System.String"/> representation.</returns>
                public string[] this[int index]
                {
                    get
                    {
                        var entryNode = m_TableParentNode.ChildNodes[index];

                        var value = new string[entryNode.ChildNodes.Count];
                        for (int j = 0; j < entryNode.ChildNodes.Count; j++)
                        {
                            value[j] = entryNode.ChildNodes[j].InnerText;
                        }
                        return value;
                    }
                }
                #endregion

                #region public methods

                #region IEnumerable<IList<string>> Members

                /// <summary>Returns an enumerator that iterates through a collection.
                /// </summary>
                /// <returns>An <see cref="T:System.Collections.IEnumerator" /> object that can be used to iterate through the collection.</returns>
                public IEnumerator<IList<string>> GetEnumerator()
                {
                    return new Enumerator(this);
                }
                #endregion

                #region IEnumerable Members

                /// <summary>Returns an enumerator that iterates through a collection.
                /// </summary>
                /// <returns>An <see cref="T:System.Collections.IEnumerator" /> object that can be used to iterate through the collection.</returns>
                IEnumerator IEnumerable.GetEnumerator()
                {
                    return new Enumerator(this);
                }
                #endregion

                /// <summary>Append a new data entry into the configuration file.
                /// </summary>
                /// <param name="values">The values.</param>
                /// <exception cref="System.ArgumentNullException">Thrown if <paramref name="values"/> is <c>null</c>.</exception>
                /// <exception cref="System.ArgumentException">Thrown if the length of <paramref name="values"/> is different from the number of fields.</exception>
                public void Append(params string[] values)
                {
                    if (values == null)
                    {
                        throw new ArgumentNullException("values");
                    }
                    if (values.Length != m_Table.m_FieldNames.Length)
                    {
                        throw new ArgumentException(String.Format(ExceptionMessages.ArgumentHasWrongDimension, "value"));
                    }
                    var dataRow = m_Table.m_ConfigurationFile.m_XmlFileRepresenation.CreateElement(m_TableEntryName.IDString);
                    for (int j = 0; j < m_Table.m_FieldNames.Length; j++)
                    {
                        var dataEntry = m_Table.m_ConfigurationFile.m_XmlFileRepresenation.CreateElement(m_Table.m_FieldNames[j]);
                        dataEntry.AppendChild(m_Table.m_ConfigurationFile.m_XmlFileRepresenation.CreateTextNode(values[j]));

                        dataRow.AppendChild(dataEntry);
                    }
                    m_TableParentNode.AppendChild(dataRow);
                }

                /// <summary>Removes all data.
                /// </summary>
                public void Clear()
                {
                    m_TableParentNode.RemoveAll();
                }

                /// <summary>Removes an item at the specified null-based index.
                /// </summary>
                /// <param name="index">The null-based index of the item to remove.</param>
                public void RemoveAt(int index)
                {
                    m_TableParentNode.RemoveChild(m_TableParentNode.ChildNodes[index]);
                }

                /// <summary>Changes the value at a specified null-based index and with respect to a specified field.
                /// </summary>
                /// <param name="index">The null-based index of the data entry.</param>
                /// <param name="fieldIndex">The null-based index of the field.</param>
                /// <param name="value">The value to set in its <see cref="System.String"/> representation.</param>
                public void ChangeAt(int index, int fieldIndex, string value)
                {
                    var entryNode = m_TableParentNode.ChildNodes[index];

                    entryNode.ChildNodes[fieldIndex].InnerText = value;
                }
                #endregion
            }
            #endregion

            #region private members

            /// <summary>The configuration file in its <see cref="ConfigurationFile"/> representation.
            /// </summary>
            private ConfigurationFile m_ConfigurationFile;

            /// <summary>The name of the data table in the configuration file.
            /// </summary>
            private IdentifierString m_TableName;

            /// <summary>The name of the fields of each entry in the data table.
            /// </summary>
            private string[] m_FieldNames;

            /// <summary>The data collection.
            /// </summary>
            private DataCollection m_Items;

            /// <summary>A value indicating whether the configuration file already contains settings with respect to the current <see cref="ConfigurationFile.Table"/> object.
            /// </summary>
            private bool m_ContainsSettings;
            #endregion

            #region internal protected constructors

            /// <summary>Initializes a new instance of the <see cref="Table" /> class.
            /// </summary>
            /// <param name="configurationFile">The configuration file.</param>
            /// <param name="tableName">The name of the table, i.e. of the section in the XML representation of the configuration file.</param>
            /// <param name="tableEntryName">The name of each data table entry.</param>
            /// <param name="fieldNames">The name of each field of the data table entries.</param>
            internal protected Table(ConfigurationFile configurationFile, IdentifierString tableName, IdentifierString tableEntryName, params string[] fieldNames)
            {
                m_ConfigurationFile = configurationFile;
                m_TableName = tableName;
                m_FieldNames = fieldNames;

                m_Items = new DataCollection(this, configurationFile.GetSectionNode(tableName.IDString, out m_ContainsSettings), tableEntryName);
            }

            /// <summary>Initializes a new instance of the <see cref="Table" /> class.
            /// </summary>
            /// <param name="configurationFile">The configuration file.</param>
            /// <param name="tableName">The name of the table, i.e. of the section in the XML representation of the configuration file.</param>
            /// <param name="tableEntryName">The name of each data table entry.</param>
            /// <param name="fieldNames">The name of each field of the data table entries.</param>
            internal protected Table(ConfigurationFile configurationFile, string tableName, string tableEntryName, params string[] fieldNames)
            {
                m_ConfigurationFile = configurationFile;
                m_TableName = new IdentifierString(tableName);
                m_FieldNames = fieldNames;

                m_Items = new DataCollection(this, configurationFile.GetSectionNode(m_TableName.IDString, out m_ContainsSettings), IdentifierString.Create(tableEntryName));
            }
            #endregion

            #region public properties

            #region IIdentifierNameable Members

            /// <summary>Gets the name of the current instance.
            /// </summary>
            /// <value>The language independent name of the current instance.
            /// </value>
            public IdentifierString Name
            {
                get { return m_TableName; }
            }

            /// <summary>Gets the long name of the current instance.
            /// </summary>
            /// <value>The (perhaps) language dependent long name of the current instance.
            /// </value>
            public IdentifierString LongName
            {
                get { return m_TableName; }
            }
            #endregion

            #region IInfoOutputQueriable Members

            /// <summary>Gets the info-output level of detail.
            /// </summary>
            /// <value>The info-output level of detail.
            /// </value>
            public InfoOutputDetailLevel InfoOutputDetailLevel
            {
                get { return InfoOutputDetailLevel.Full; }
            }
            #endregion

            /// <summary>Gets the name of the fields of each entry in the table.
            /// </summary>
            /// <value>The data field names.
            /// </value>
            public string[] FieldNames
            {
                get { return m_FieldNames; }
            }

            /// <summary>Gets an object representing the collection of the items contained in this <see cref="ConfigurationFile.Table"/> object.
            /// </summary>
            /// <value>The collection of data.
            /// </value>
            public DataCollection Items
            {
                get { return m_Items; }
            }

            /// <summary>Gets a value indicating whether the configuration file already contains settings with respect to the current <see cref="ConfigurationFile"/> object; will be set to <c>true</c>
            /// after calling <see cref="Save()"/>.
            /// </summary>
            /// <value><c>true</c> if the configuration file already contains some settings; otherwise, <c>false</c>.</value>
            public bool ContainsSettings
            {
                get { return m_ContainsSettings; }
            }
            #endregion

            #region public methods

            #region IInfoOutputQueriable Members

            /// <summary>Sets the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel" /> property.
            /// </summary>
            /// <param name="infoOutputDetailLevel">The info-output level of detail.</param>
            /// <returns>A value indicating whether the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel" /> has been set to <paramref name="infoOutputDetailLevel" />.
            /// </returns>
            public bool TrySetInfoOutputDetailLevel(InfoOutputDetailLevel infoOutputDetailLevel)
            {
                return (infoOutputDetailLevel == Containers.InfoOutputDetailLevel.Full);
            }

            /// <summary>Gets informations of the current object as a specific <see cref="InfoOutput" /> instance.
            /// </summary>
            /// <param name="infoOutput">The <see cref="InfoOutput" /> object which is to be filled with informations concering the current instance.</param>
            /// <param name="categoryName">The name of the category, i.e. all informations will be added to these category.</param>
            public void FillInfoOutput(InfoOutput infoOutput, string categoryName = InfoOutput.GeneralCategoryName)
            {
                var infoPackage = infoOutput.AcquirePackage(categoryName);

                var dataTable = new DataTable(m_TableName.String);
                foreach (var fieldName in m_FieldNames)
                {
                    dataTable.Columns.Add(fieldName, typeof(String));
                }

                for (int j = 0; j < m_Items.Count; j++)
                {
                    var values = m_Items[j];
                    var rowData = dataTable.NewRow();

                    for (int k = 0; k < m_FieldNames.Length; k++)
                    {
                        rowData[k] = values[k];
                    }
                    dataTable.Rows.Add(rowData);
                }
            }
            #endregion

            #endregion
        }
        #endregion

        #region private static members

        /// <summary>The configuration of the Dodoni.net framework.
        /// </summary>
        public static readonly ConfigurationFile Dodoni = Create("Dodoni.net.config", "Dodoni.net");
        #endregion

        #region private members

        /// <summary>The file path of the configuration file used to store the user settings.
        /// </summary>
        private string m_FilePath;

        /// <summary>The configuration file in its <see cref="XmlDocument"/> representation.
        /// </summary>
        private XmlDocument m_XmlFileRepresenation;

        /// <summary>The name of the main node in the xml representation of the configuration file.
        /// </summary>
        private string m_XmlRootNodeName;
        #endregion

        #region protected constructors

        /// <summary>Initializes a new instance of the <see cref="ConfigurationFile"/> class, the path of the assembly will be used as well as the assembly file name. 
        /// </summary>
        /// <param name="configurationFileName">The name of the configuration file (the path will be ignored).</param>
        /// <param name="xmlRootNodeName">The name of the root node in the XML file representation of the configuration file.</param>
        protected ConfigurationFile(string configurationFileName, string xmlRootNodeName = "Dodoni.net")
        {
            try
            {
                m_XmlRootNodeName = xmlRootNodeName;
                m_FilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, configurationFileName);
                m_XmlFileRepresenation = new XmlDocument();


                if (File.Exists(m_FilePath) == false) // no config file available, create an internal representation
                {
                    XmlNode mainNode = m_XmlFileRepresenation.CreateElement(xmlRootNodeName);
                    m_XmlFileRepresenation.AppendChild(mainNode);
                }
                else
                {

                    XmlReaderSettings xmlSettings = new XmlReaderSettings();
                    xmlSettings.ConformanceLevel = ConformanceLevel.Fragment;
                    xmlSettings.IgnoreWhitespace = true;
                    xmlSettings.IgnoreComments = true;

                    var reader = XmlReader.Create(m_FilePath, xmlSettings);
                    m_XmlFileRepresenation.Load(reader);
                    reader.Close();
                }
            }
            catch (Exception e)
            {
                throw new ConfigurationFileErrorException(e.Message, e);
            }
        }
        #endregion

        #region public properties

        /// <summary>Gets the physical path to the configuration file represented by this <see cref="ConfigurationFile"/> file path.
        /// </summary>
        /// <value>The file path of the configuration.</value>
        public string FilePath
        {
            get { return m_FilePath; }
        }
        #endregion

        #region public methods

        /// <summary>Gets a specified property collection, i.e. a collection of settings grouped by a specified name.
        /// </summary>
        /// <param name="propertyCollectionName">The name of the property collection.</param>
        /// <returns>A perhaps empty collection of properties (i.e. settings) in the configuration file in its <see cref="ConfigurationFile.PropertyCollection"/> representation.</returns>
        /// <exception cref="System.ArgumentNullException">Thrown if <paramref name="propertyCollectionName"/> is <c>null</c>.</exception>
        public PropertyCollection GetPropertyCollection(string propertyCollectionName)
        {
            if (propertyCollectionName == null)
            {
                throw new ArgumentNullException("propertyCollectionName");
            }
            return new PropertyCollection(this, IdentifierString.Create(propertyCollectionName));
        }

        /// <summary>Gets a specified property collection, i.e. a collection of settings grouped by a specified name.
        /// </summary>
        /// <typeparam name="T">The type of the property collection.</typeparam>
        /// <param name="propertyCollectionName">The name of the property collection.</param>
        /// <param name="propertyCollectionFactory">A factory for <typeparamref name="T"/> objects, where the first argument is the current instance and the second argument is the <paramref name="propertyCollectionName"/>.</param>
        /// <returns>A perhaps empty collection of properties (i.e. settings) in the configuration file in its <typeparamref name="T"/> representation.</returns>
        /// <exception cref="System.ArgumentNullException">Thrown if <paramref name="propertyCollectionName"/> is <c>null</c>.</exception>
        /// <remarks>The argument <paramref name="propertyCollectionFactory"/> is a workaround because C# does not allow to restrict a generic type to a class that contains a specified constructor, only <code>where T: PropertyCollection, new()</code> is possible.</remarks>
        public T GetPropertyCollection<T>(string propertyCollectionName, Func<ConfigurationFile, string, T> propertyCollectionFactory)
            where T : PropertyCollection
        {
            if (propertyCollectionName == null)
            {
                throw new ArgumentNullException("propertyCollectionName");
            }
            return propertyCollectionFactory(this, propertyCollectionName);
        }

        /// <summary>Gets a specified data table, i. e. a collection of homogeneous data grouped by a specified name.
        /// </summary>
        /// <param name="tableName">The name of the table, i.e. of the section in the XML representation of the configuration file.</param>
        /// <param name="tableEntryName">The name of each data table entry.</param>
        /// <param name="fieldNames">The name of each field of the data table entries.</param>
        /// <returns>A <see cref="Table"/> object that contains homogeneous data.</returns>
        public Table GetTable(string tableName, string tableEntryName, params string[] fieldNames)
        {
            return new Table(this, IdentifierString.Create(tableName), IdentifierString.Create(tableEntryName), fieldNames);
        }

        /// <summary>Gets a specified data table, i.e. a collection of homogenious data grouped by a specified name.
        /// </summary>
        /// <typeparam name="T">The type of the table.</typeparam>
        /// <param name="tableName">The name of the table, i.e. of the section in the XML representation of the configuration file.</param>
        /// <param name="tableFactory">A factory for <typeparamref name="T"/> objects, where the first argument is the current instance and the second argument is the <paramref name="tableName"/>.</param>
        /// <returns>A typed <see cref="Table"/> object as object of type <typeparamref name="T"/> that contains homogeneous data.</returns>
        /// <remarks>The argument <paramref name="tableFactory"/> is a workaround because in C# does not allow to restrict a generic type to a class that contains a specified constructor, only <code>where T: Table, new()</code> is possible.</remarks>
        public T GetTable<T>(string tableName, Func<ConfigurationFile, string, T> tableFactory)
            where T : Table
        {
            return tableFactory(this, tableName);
        }

        /// <summary>Writes the configuration settings into the configuration file.
        /// </summary>
        /// <exception cref="ConfigurationFileErrorException">Thrown if some error occured while saving the settings.</exception>
        public void Save()
        {
            try
            {
                var settings = new XmlWriterSettings();
                settings.OmitXmlDeclaration = false;
                settings.Indent = true;

                var writer = XmlWriter.Create(m_FilePath, settings);

                m_XmlFileRepresenation.WriteTo(writer);
                writer.Close();
            }
            catch (Exception e)
            {
                throw new ConfigurationFileErrorException(String.Format("An error occured while writing the configuration settings into file {0}.", m_FilePath), e);
            }
        }
        #endregion

        #region public static methods

        /// <summary>Creates a new <see cref="ConfigurationFile"/> object.
        /// </summary>
        /// <param name="configurationFileName">The name of the configuration file (the path will be ignored).</param>
        /// <param name="xmlRootNodeName">The name of the root node in the XML file representation of the configuration file.</param>
        /// <returns>A new <see cref="ConfigurationFile"/> object.</returns>
        public static ConfigurationFile Create(string configurationFileName, string xmlRootNodeName)
        {
            return new ConfigurationFile(configurationFileName, xmlRootNodeName);
        }
        #endregion

        #region protected methods

        /// <summary>Gets a specified section node in the XML representation of the configuration file in its <see cref="XmlNode"/> representation.
        /// </summary>
        /// <param name="sectionName">The name of the section.</param>
        /// <param name="sectionExists">A value indicating whether a section of the specified name already exists in the internal <see cref="XmlDocument"/> representation.</param>
        /// <returns>A <see cref="XmlNode"/> object that represents the root node of a specified section in the XML file representation of the configuration file. If no section with
        /// the specified <paramref name="sectionName"/> exists an empty node will be inserted into the internal <see cref="XmlDocument"/> representation and returned.</returns>
        /// <exception cref="ConfigurationFileErrorException">Thrown if the configuration file exists and is in an invalid format.</exception>
        protected XmlNode GetSectionNode(string sectionName, out bool sectionExists)
        {
            var sectionNode = m_XmlFileRepresenation.SelectSingleNode(String.Format(@"/{0}/{1}", m_XmlRootNodeName, sectionName));

            if (sectionNode == null)
            {
                sectionExists = false;

                var rootNode = m_XmlFileRepresenation.SelectSingleNode(@"/" + m_XmlRootNodeName);
                if (rootNode == null)
                {
                    throw new ConfigurationFileErrorException(String.Format("No node {0} in configuration file {1} available.", m_XmlRootNodeName, m_FilePath));
                }
                sectionNode = m_XmlFileRepresenation.CreateElement(sectionName);
                rootNode.AppendChild(sectionNode);
            }
            else
            {
                sectionExists = true;
            }
            return sectionNode;
        }
        #endregion
    }
}