/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Linq;
using System.Collections.Generic;

namespace Dodoni.BasicComponents.Utilities
{
    /// <summary>Serves as collection of general extension methods.
    /// </summary>
    [System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)]
    public static class Extensions
    {
        /// <summary>Gets a raw distance between two dates normalized by the number of days per year, i.e. a generic day count fraction.
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="endDate">The end date.</param>
        /// <returns>The number of days in the time period over the number of days per year.</returns>
        public static double GetRawDistanceTo(this DateTime startDate, DateTime endDate)
        {
            if (startDate > endDate)
            {
                DateTime temp = startDate;
                startDate = endDate;
                endDate = temp;
            }
            if (startDate.Year == endDate.Year)
            {
                return endDate.Subtract(startDate).Days / ((DateTime.IsLeapYear(startDate.Year) == true) ? 366.0 : 365.0);
            }
            int numberOfYearsInBetween;
            DateTime adjEndDate;
            if (endDate.AddYears(-1) < startDate)
            {
                numberOfYearsInBetween = 0;
                adjEndDate = endDate;
            }
            else
            {
                numberOfYearsInBetween = endDate.Year - startDate.Year;
                adjEndDate = endDate.AddMonths(-12 * numberOfYearsInBetween);
            }

            // compute denominator:
            double denominator = 365.0;
            if (DateTime.IsLeapYear(adjEndDate.Year))
            {
                DateTime temp = new DateTime(adjEndDate.Year, 2, 29);
                if ((startDate <= temp) && (temp < adjEndDate))
                {
                    denominator += 1.0;
                }
            }
            else if (DateTime.IsLeapYear(startDate.Year))
            {
                DateTime temp = new DateTime(startDate.Year, 2, 29);
                if ((startDate <= temp) && (temp < adjEndDate))
                {
                    denominator += 1.0;
                }
            }
            return numberOfYearsInBetween + adjEndDate.Subtract(startDate).Days / denominator;
        }

        /// <summary>Shrinks a specific enumeration.
        /// </summary>
        /// <typeparam name="T">The data type.</typeparam>
        /// <param name="source">The source.</param>
        /// <param name="elementsToSkipAtTheBeginning">The number of elements to skip at the beginning of the enumeration.</param>
        /// <param name="elementsToSkipAtTheEnd">The number of elements to skip at the end of the enumeration.</param>
        /// <returns>The subset of the <paramref name="source"/> enumeration.</returns>
        public static IEnumerable<T> Shrink<T>(this IEnumerable<T> source, int elementsToSkipAtTheBeginning, int elementsToSkipAtTheEnd)
        {
            int i = 0;
            var buffer = new Queue<T>(elementsToSkipAtTheEnd + 1);

            foreach (T item in source)
            {
                if (i >= elementsToSkipAtTheBeginning) // Read past left many elements at the start 
                {
                    buffer.Enqueue(item);
                    if (buffer.Count > elementsToSkipAtTheEnd) // Build a buffer to drop right many elements at the end 
                    {
                        yield return buffer.Dequeue();
                    }
                }
                else
                {
                    i++;
                }
            }
        }

        /// <summary>Searches an entire one-dimensional sorted IList for a specific element, using the generic IComparable interface 
        /// implemented by each element of the list and by the specified object.
        /// </summary>
        /// <typeparam name="T">The type of the elements of the list.</typeparam>
        /// <param name="list">The sorted one-dimensional, zero-based list to search.</param>
        /// <param name="value">The object to search for.</param>
        /// <returns>The index of the specified value in the specified list, if <paramref name="value"/> is found. If <paramref name="value"/> is not 
        /// found and <paramref name="value"/> is less than one or more elements in <paramref name="list"/>, a negative number which is the bitwise 
        /// complement of the index of the first element that is larger than <paramref name="value"/>. 
        /// If <paramref name="value"/> is not found and <paramref name="value"/> is greater than any of the elements in <paramref name="list"/>, 
        /// a negative number which is the bitwise complement of (the index of the last element plus 1).</returns>
        public static int BinarySearch<T>(this IList<T> list, T value) where T : IComparable<T>
        {
            /* do a binary search: */
            int a = 0;
            int b = list.Count;
            int mid;

            while (a < b)
            {
                mid = (a + b) >> 1;
                if (list[mid].CompareTo(value) < 0) // Keys[mid] < key
                {
                    a = mid + 1;
                }
                else
                {
                    b = mid;
                }
            }
            if ((a < list.Count) && (list[a].CompareTo(value) == 0))
            {
                return a;
            }
            return ~b;
        }

        /// <summary>Adds a collection of items to the strongly typed list.
        /// </summary>
        /// <typeparam name="T">The type of elements in the list.</typeparam>
        /// <param name="list">The strongly typed list.</param>
        /// <param name="values">The items to add to the list.</param>
        public static void AddRange<T>(this IList<T> list, IEnumerable<T> values)
        {
            foreach (var value in values)
            {
                list.Add(value);
            }
        }
    }
}