/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.BasicComponents.Utilities
{
    /// <summary>Represents a two sided array, i.e. two arrays and some center element where the index can be negative as well.
    /// </summary>
    /// <typeparam name="T">The type of the items.</typeparam>
    public class TwoSideArray<T>
    {
        #region public members

        /// <summary>The center element.
        /// </summary>
        public T CenterElement;

        /// <summary>The left array.
        /// </summary>
        public readonly T[] LeftArray;

        /// <summary>The right array.
        /// </summary>
        public readonly T[] RightArray;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="TwoSideArray&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="leftArrayLength">The length of the left array.</param>
        /// <param name="rightArrayLength">The lenght of the right array.</param>
        public TwoSideArray(int leftArrayLength, int rightArrayLength)
        {
            LeftArray = new T[leftArrayLength];
            RightArray = new T[rightArrayLength];
        }

        /// <summary>Initializes a new instance of the <see cref="TwoSideArray&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="leftArray">The left array.</param>
        /// <param name="rightArray">The right array.</param>
        /// <param name="centerElement">The center element.</param>
        public TwoSideArray(T[] leftArray, T[] rightArray, T centerElement)
        {
            LeftArray = leftArray;
            RightArray = rightArray;
            CenterElement = centerElement;
        }
        #endregion

        #region public properties

        /// <summary>Gets the number of elements.
        /// </summary>
        /// <value>The number of elements.</value>
        public int Count
        {
            get
            {
                return ((LeftArray == null) ? 0 : LeftArray.Length) + ((RightArray == null) ? 0 : RightArray.Length) + 1;
            }
        }

        /// <summary>Gets the number of elements with negative index.
        /// </summary>
        /// <value>The number of elements with negative index.</value>
        public int LeftCount
        {
            get { return (LeftArray == null) ? 0 : LeftArray.Length; }
        }

        /// <summary>Gets the number of elements with strict positive index.
        /// </summary>
        /// <value>The number of elements with strict positive index.</value>
        public int RightCount
        {
            get { return (RightArray == null) ? 0 : RightArray.Length; }
        }

        /// <summary>Gets or sets the  <typeparamref name="T"/> at the specified index.
        /// </summary>
        /// <value>The element with the given index.</value>
        public T this[int index]
        {
            get
            {
                if (index == 0)
                {
                    return CenterElement;
                }
                else if (index > 0)
                {
                    return RightArray[index - 1];
                }
                else
                {
                    return LeftArray[-index - 1];
                }
            }
            set
            {
                if (index == 0)
                {
                    CenterElement = value;
                }
                else if (index > 0)
                {
                    RightArray[index - 1] = value;
                }
                else
                {
                    LeftArray[-index - 1] = value;
                }
            }
        }
        #endregion
    }
}