/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.BasicComponents.Utilities
{
    /// <summary>Represents a two sided list, i.e. two instances of <see cref="List&lt;T&gt;"/> and some center element where
    /// the index can be negative as well.
    /// </summary>
    /// <typeparam name="T">The type of the items.</typeparam>
    public class TwoSideList<T>
    {
        #region public members

        /// <summary>The center element.
        /// </summary>
        public T CenterElement;

        /// <summary>The left list.
        /// </summary>
        public readonly List<T> LeftList;

        /// <summary>The right list.
        /// </summary>
        public readonly List<T> RightList;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="TwoSideList&lt;T&gt;"/> class.
        /// </summary>
        public TwoSideList()
        {
            LeftList = new List<T>();
            RightList = new List<T>();
            CenterElement = default(T);
        }
        /// <summary>Initializes a new instance of the <see cref="TwoSideList&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="capacity">The number of elements that the new list can initialize store for each direction.</param>
        public TwoSideList(int capacity)
        {
            LeftList = new List<T>(capacity);
            RightList = new List<T>(capacity);
            CenterElement = default(T);
        }
        #endregion

        #region public properties

        /// <summary>Gets the number of elements.
        /// </summary>
        /// <value>The number of elements.</value>
        public int Count
        {
            get { return LeftList.Count + RightList.Count + 1; }
        }

        /// <summary>Gets the number of elements with negative index.
        /// </summary>
        /// <value>The number of elements with negative index.</value>
        public int LeftCount
        {
            get { return LeftList.Count; }
        }

        /// <summary>Gets the number of elements with strict positive index.
        /// </summary>
        /// <value>The number of elements with strict positive index.</value>
        public int RightCount
        {
            get { return RightList.Count; }
        }

        /// <summary>Gets or sets the <typeparamref name="T"/> at the specified index.
        /// </summary>
        /// <value>The element with the given index.</value>
        public T this[int index]
        {
            get
            {
                if (index == 0)
                {
                    return CenterElement;
                }
                else if (index > 0)
                {
                    return RightList[index - 1];
                }
                else
                {
                    return LeftList[-index - 1];
                }
            }
            set
            {
                if (index == 0)
                {
                    CenterElement = value;
                }
                else if (index > 0)
                {
                    RightList[index - 1] = value;
                }
                else
                {
                    LeftList[-index - 1] = value;
                }
            }
        }
        #endregion

        #region public methods

        /// <summary>Adds an element on the right side.
        /// </summary>
        /// <param name="value">The value.</param>
        public void AddRight(T value)
        {
            RightList.Add(value);
        }

        /// <summary>Adds an element on the left side.
        /// </summary>
        /// <param name="value">The value.</param>
        public void AddLeft(T value)
        {
            LeftList.Add(value);
        }

        /// <summary>Adds an element on the left and a second element on the right side.
        /// </summary>
        /// <param name="leftValue">The left value.</param>
        /// <param name="rightValue">The right value.</param>
        public void Add(T leftValue, T rightValue)
        {
            LeftList.Add(leftValue);
            RightList.Add(rightValue);
        }

        /// <summary>Removes all elements from the list, the <see cref="CenterElement"/> will be set to the default value.
        /// </summary>
        public void Clear()
        {
            LeftList.Clear();
            RightList.Clear();
            CenterElement = default(T);
        }

        /// <summary>Removes a sub-list.
        /// </summary>
        /// <param name="index">The position where to start removing elements.</param>
        /// <remarks>The element at position <paramref name="index"/> will be removed as well but the center element will not be removed.</remarks>
        public void RemoveSubList(int index)
        {
            if (index >= 1)
            {
                RightList.RemoveRange(index - 1, RightList.Count - index + 1);
            }
            else if (index <= -1)
            {
                LeftList.RemoveRange(-index - 1, LeftList.Count + index + 1);
            }
        }

        /// <summary>Remove two symmetric sub-lists from the internal list.
        /// </summary>
        /// <param name="index">The position where to start removing elements in both directions, i.e. also with the negative index.</param>
        /// <remarks>The center element will not be removed.</remarks>
        public void SymmetricClear(int index)
        {
            RemoveSubList(index);
            RemoveSubList(-index);
        }
        #endregion
    }
}