﻿using System;
using System.Text;
using System.Reflection;
using System.Collections.Generic;

using NUnit.Framework;

namespace Dodoni.BasicComponents
{
    /// <summary>A unit test class for <see cref="IdentifierString"/>.
    /// </summary>
    [TestFixture]
    public class IdentifierStringTests
    {
        /// <summary>Data points for the unit tests.
        /// </summary>
        [Datapoints]
        public string[] TestInputStrings = new string[] { "Hello", " StranGe ExAmple   ", " Contains a @ ignoring Part" };

        /// <summary>A test function that compares the <see cref="IdentifierString.String"/> component of the result of <see cref="IdentifierString.Create(String)"/>.
        /// </summary>
        /// <param name="rawString">The (raw) string.</param>
        [Theory]
        public void CreateAndGetString_RawString_RawString(string rawString)
        {
            Assume.That(rawString != null);

            IdentifierString identifierString = IdentifierString.Create(rawString);

            Assert.That(identifierString != null);
            Assert.That(identifierString.String, Is.EqualTo(rawString));
        }

        /// <summary>A test function that compares the <see cref="IdentifierString.IDString"/> component of the result of <see cref="IdentifierString.Create(String)"/>.
        /// </summary>
        /// <param name="rawString">The (raw) string.</param>
        [Theory]
        public void CreateAndGetIDString_RawString_IDStringOfRawString(string rawString)
        {
            Assume.That(rawString != null);

            IdentifierString identifierString = IdentifierString.Create(rawString);

            Assert.That(identifierString != null);
            Assert.That(rawString.ToIDString(), Is.EqualTo(identifierString.IDString));
        }

        /// <summary>A test function that compares the <see cref="IdentifierString.IDString"/> component of the result of <see cref="IdentifierString.Create(String)"/>.
        /// </summary>
        /// <param name="rawString">The (raw) string.</param>
        [Theory]
        public void CreateAndGetIDString_TrimLowerRawString_IDStringOfRawString(string rawString)
        {
            Assume.That(rawString != null);

            IdentifierString identifierString = IdentifierString.Create(rawString);

            string modifiedRawString = rawString.Trim().ToLower();
            IdentifierString modifiedIdentifierString = IdentifierString.Create(modifiedRawString);

            Assert.That(identifierString.IDString, Is.EqualTo(modifiedIdentifierString.IDString));
        }

        /// <summary>A test function that compares the <see cref="IdentifierString.IDString"/> component of the result of <see cref="IdentifierString.Create(String)"/>.
        /// </summary>
        /// <param name="rawString">The (raw) string.</param>
        [Theory]
        public void CreateAndGetIDString_TrimUpperRawString_IDStringOfRawString(string rawString)
        {
            Assume.That(rawString != null);

            IdentifierString identifierString = IdentifierString.Create(rawString);

            string modifiedRawString = rawString.Trim().ToUpper();
            IdentifierString modifiedIdentifierString = IdentifierString.Create(modifiedRawString);

            Assert.That(identifierString.IDString, Is.EqualTo(modifiedIdentifierString.IDString));
        }

        /// <summary>A test function that compares the <see cref="IdentifierString.IDString"/> component of the result of <see cref="IdentifierString.Create(String)"/>.
        /// </summary>
        /// <param name="rawString">The (raw) string.</param>
        [Theory]
        public void CreateAndGetIDString_NoWhiteSpacesLowerRawString_IDStringOfRawString(string rawString)
        {
            Assume.That(rawString != null);

            IdentifierString identifierString = IdentifierString.Create(rawString);

            string modifiedRawString = rawString.Replace(" ", String.Empty).ToLower();
            IdentifierString modifiedIdentifierString = IdentifierString.Create(modifiedRawString);

            Assert.That(identifierString.IDString, Is.EqualTo(modifiedIdentifierString.IDString));
        }

        /// <summary>A test function that compares the <see cref="IdentifierString.IDString"/> component of the result of <see cref="IdentifierString.Create(String)"/>.
        /// </summary>
        /// <param name="rawString">The (raw) string.</param>
        [Theory]
        public void CreateAndGetIDString_NoWhiteSpacesUpperRawString_IDStringOfRawString(string rawString)
        {
            Assume.That(rawString != null);

            IdentifierString identifierString = IdentifierString.Create(rawString);

            string modifiedRawString = rawString.Replace(" ", String.Empty).ToUpper();
            IdentifierString modifiedIdentifierString = IdentifierString.Create(modifiedRawString);

            Assert.That(identifierString.IDString, Is.EqualTo(modifiedIdentifierString.IDString));
        }

        /// <summary>A test function for <see cref="IdentifierString.Create(String,String,Assembly)"/>.
        /// </summary>
        /// <param name="fullResourceName">The resource name (no language dependend suffix) and the corresponding namespace.</param>
        /// <param name="resourcePropertyName">The property name with respect to a given resource which contains some language dependend <see cref="System.String"/> representation.</param>
        [TestCase("Dodoni.BasicComponents.IdentifierStringTestsResources", "ExampleTwoResourcePropertyName", Result = "A    simple EXample")]
        [TestCase("Dodoni.BasicComponents.IdentifierStringTestsResources", "ExampleResourcePropertyName", Result = "  a sTranGe NAme@ should be ignored")]
        [TestCase("Dodoni.BasicComponents.WrongResourceName", "ExampleResourcePropertyName", ExpectedException = typeof(ArgumentException))]
        [TestCase("Dodoni.BasicComponents.IdentifierStringTestsResources", "InvalidResourcePropertyName", ExpectedException = typeof(ArgumentException))]
        public string CreateAndGetString_ResourceNameAndResourcePropertyName_PropertyValue(string fullResourceName, string resourcePropertyName)
        {
            IdentifierString identifierString = IdentifierString.Create(fullResourceName, resourcePropertyName, Assembly.GetAssembly(typeof(IdentifierStringTests)));

            return identifierString.String;
        }
    }
}