﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

using NUnit.Framework;

using Dodoni.BasicComponents.Utilities;
using Dodoni.BasicComponents.LogFileFactory;

namespace Dodoni.BasicComponents.Utilities
{
    /// <summary>A unit test class for <see cref="EnumString"/>.
    /// </summary>
    [TestFixture]
    public class EnumStringTests
    {
        /// <summary>Prepares the unit tests.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            EnumString.FlagsEnumSeparatorChar = ',';
        }

        /// <summary>A test function that compares the result of <see cref="EnumString.Parse(Type, string, EnumStringRepresentationUsage)"/> with respect
        /// to the example enumeration <see cref="FileAccess"/>.
        /// </summary>
        /// <param name="stringRepresentation">The <see cref="System.String"/> representation to search for.</param>
        /// <param name="enumStringRepresentationUsage">The method how to compute the string representation of the items of the enumeration.</param>
        /// <returns>The item of the enumeration corresponds to <paramref name="stringRepresentation"/>.</returns>
        [TestCase("Read", EnumStringRepresentationUsage.ToStringMethod, Result = FileAccess.Read)]
        [TestCase("Write", EnumStringRepresentationUsage.ToStringMethod, Result = FileAccess.Write)]
        [TestCase("Delete", EnumStringRepresentationUsage.ToStringMethod, Result = FileAccess.Delete)]
        [TestCase("ReadWrite", EnumStringRepresentationUsage.ToStringMethod, Result = FileAccess.ReadWrite)]
        [TestCase("Read, Write", EnumStringRepresentationUsage.ToStringMethod, Result = FileAccess.ReadWrite)]
        [TestCase("Read,Delete,Write", EnumStringRepresentationUsage.ToStringMethod, Result = FileAccess.ReadWrite | FileAccess.Delete)]
        [TestCase("Read", EnumStringRepresentationUsage.StringAttribute, Result = FileAccess.Read)]
        [TestCase("Write", EnumStringRepresentationUsage.StringAttribute, Result = FileAccess.Write)]
        [TestCase("Delete", EnumStringRepresentationUsage.StringAttribute, Result = FileAccess.Delete)]
        [TestCase("ReadWrite", EnumStringRepresentationUsage.StringAttribute, Result = FileAccess.ReadWrite)]
        [TestCase("Read, Write", EnumStringRepresentationUsage.StringAttribute, Result = FileAccess.ReadWrite)]
        [TestCase("Read,Delete,Write", EnumStringRepresentationUsage.StringAttribute, Result = FileAccess.ReadWrite | FileAccess.Delete)]
        [TestCase("Read", EnumStringRepresentationUsage.LanguageStringAttribute, Result = FileAccess.Read)]
        [TestCase("Write", EnumStringRepresentationUsage.LanguageStringAttribute, Result = FileAccess.Write)]
        [TestCase("Delete", EnumStringRepresentationUsage.LanguageStringAttribute, Result = FileAccess.Delete)]
        [TestCase("ReadWrite", EnumStringRepresentationUsage.LanguageStringAttribute, Result = FileAccess.ReadWrite)]
        [TestCase("Read, Write", EnumStringRepresentationUsage.LanguageStringAttribute, Result = FileAccess.ReadWrite)]
        [TestCase("Read,Delete,Write", EnumStringRepresentationUsage.LanguageStringAttribute, Result = FileAccess.ReadWrite | FileAccess.Delete)]
        public Enum Parse_FileAccessEnumName_ValidEnumValue(string stringRepresentation, EnumStringRepresentationUsage enumStringRepresentationUsage)
        {
            return EnumString.Parse(typeof(FileAccess), stringRepresentation, enumStringRepresentationUsage);
        }

        /// <summary>A test function that compares the result of <see cref="EnumString.Parse(Type, string, EnumStringRepresentationUsage)"/> with respect
        /// to the example enumeration <see cref="StringAttributeFileAccess"/>.
        /// </summary>
        /// <param name="stringRepresentation">The <see cref="System.String"/> representation to search for.</param>
        /// <param name="enumStringRepresentationUsage">The method how to compute the string representation of the items of the enumeration.</param>
        /// <returns>The item of the enumeration corresponds to <paramref name="stringRepresentation"/>.</returns>
        [TestCase("Read", EnumStringRepresentationUsage.ToStringMethod, Result = StringAttributeFileAccess.Read)]
        [TestCase("Write", EnumStringRepresentationUsage.ToStringMethod, Result = StringAttributeFileAccess.Write)]
        [TestCase("Delete", EnumStringRepresentationUsage.ToStringMethod, Result = StringAttributeFileAccess.Delete)]
        [TestCase("ReadWrite", EnumStringRepresentationUsage.ToStringMethod, Result = StringAttributeFileAccess.ReadWrite)]
        [TestCase("Read, Write", EnumStringRepresentationUsage.ToStringMethod, Result = StringAttributeFileAccess.ReadWrite)]
        [TestCase("Read,Delete,Write", EnumStringRepresentationUsage.ToStringMethod, Result = StringAttributeFileAccess.ReadWrite | StringAttributeFileAccess.Delete)]
        [TestCase("Lesen", EnumStringRepresentationUsage.StringAttribute, Result = StringAttributeFileAccess.Read)]
        [TestCase("Schreiben", EnumStringRepresentationUsage.StringAttribute, Result = StringAttributeFileAccess.Write)]
        [TestCase("Löschen", EnumStringRepresentationUsage.StringAttribute, Result = StringAttributeFileAccess.Delete)]
        [TestCase("Lesen oder schreiben", EnumStringRepresentationUsage.StringAttribute, Result = StringAttributeFileAccess.ReadWrite)]
        [TestCase("Lesen, Schreiben", EnumStringRepresentationUsage.StringAttribute, Result = StringAttributeFileAccess.ReadWrite)]
        [TestCase("Schreiben,Löschen,Lesen", EnumStringRepresentationUsage.StringAttribute, Result = StringAttributeFileAccess.ReadWrite | StringAttributeFileAccess.Delete)]
        [TestCase("Lesen", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAttributeFileAccess.Read)]
        [TestCase("Schreiben", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAttributeFileAccess.Write)]
        [TestCase("Löschen", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAttributeFileAccess.Delete)]
        [TestCase("Lesen oder schreiben", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAttributeFileAccess.ReadWrite)]
        [TestCase("Lesen, schreiben", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAttributeFileAccess.ReadWrite)]
        [TestCase("Lesen,löschen,schreiben", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAttributeFileAccess.ReadWrite | StringAttributeFileAccess.Delete)]
        public Enum Parse_StringAttributeFileAccessEnumName_ValidEnumValue(string stringRepresentation, EnumStringRepresentationUsage enumStringRepresentationUsage)
        {
            return EnumString.Parse(typeof(StringAttributeFileAccess), stringRepresentation, enumStringRepresentationUsage);
        }

        /// <summary>A test function that compares the result of <see cref="EnumString.Parse(Type, string, EnumStringRepresentationUsage)"/> with respect
        /// to the example enumeration <see cref="LanguageAttributeFileAccess"/>.
        /// </summary>
        /// <param name="stringRepresentation">The <see cref="System.String"/> representation to search for.</param>
        /// <param name="enumStringRepresentationUsage">The method how to compute the string representation of the items of the enumeration.</param>
        /// <returns>The item of the enumeration corresponds to <paramref name="stringRepresentation"/>.</returns>
        [TestCase("Read", EnumStringRepresentationUsage.ToStringMethod, Result = LanguageAttributeFileAccess.Read)]
        [TestCase("Write", EnumStringRepresentationUsage.ToStringMethod, Result = LanguageAttributeFileAccess.Write)]
        [TestCase("Delete", EnumStringRepresentationUsage.ToStringMethod, Result = LanguageAttributeFileAccess.Delete)]
        [TestCase("ReadWrite", EnumStringRepresentationUsage.ToStringMethod, Result = LanguageAttributeFileAccess.ReadWrite)]
        [TestCase("Read, Write", EnumStringRepresentationUsage.ToStringMethod, Result = LanguageAttributeFileAccess.ReadWrite)]
        [TestCase("Read,Delete,Write", EnumStringRepresentationUsage.ToStringMethod, Result = LanguageAttributeFileAccess.ReadWrite | LanguageAttributeFileAccess.Delete)]
        [TestCase("Read", EnumStringRepresentationUsage.StringAttribute, Result = LanguageAttributeFileAccess.Read)]
        [TestCase("Write", EnumStringRepresentationUsage.StringAttribute, Result = LanguageAttributeFileAccess.Write)]
        [TestCase("Delete", EnumStringRepresentationUsage.StringAttribute, Result = LanguageAttributeFileAccess.Delete)]
        [TestCase("ReadWrite", EnumStringRepresentationUsage.StringAttribute, Result = LanguageAttributeFileAccess.ReadWrite)]
        [TestCase("Read, Write", EnumStringRepresentationUsage.StringAttribute, Result = LanguageAttributeFileAccess.ReadWrite)]
        [TestCase("Write,delete,read", EnumStringRepresentationUsage.StringAttribute, Result = LanguageAttributeFileAccess.ReadWrite | LanguageAttributeFileAccess.Delete)]
        [TestCase("ResourceRead", EnumStringRepresentationUsage.LanguageStringAttribute, Result = LanguageAttributeFileAccess.Read)]
        [TestCase("ResourceWrite", EnumStringRepresentationUsage.LanguageStringAttribute, Result = LanguageAttributeFileAccess.Write)]
        [TestCase("ResourceDelete", EnumStringRepresentationUsage.LanguageStringAttribute, Result = LanguageAttributeFileAccess.Delete)]
        [TestCase("ResourceReadwrite", EnumStringRepresentationUsage.LanguageStringAttribute, Result = LanguageAttributeFileAccess.ReadWrite)]
        [TestCase("ResourceRead, ResourceWrite", EnumStringRepresentationUsage.LanguageStringAttribute, Result = LanguageAttributeFileAccess.ReadWrite)]
        [TestCase("ResourceRead, ResourceDelete, ResourceWrite", EnumStringRepresentationUsage.LanguageStringAttribute, Result = LanguageAttributeFileAccess.ReadWrite | LanguageAttributeFileAccess.Delete)]
        public Enum Parse_LanguageAttributeFileAccessEnumName_ValidEnumValue(string stringRepresentation, EnumStringRepresentationUsage enumStringRepresentationUsage)
        {
            return EnumString.Parse(typeof(LanguageAttributeFileAccess), stringRepresentation, enumStringRepresentationUsage);
        }

        /// <summary>A test function that compares the result of <see cref="EnumString.Parse(Type, string, EnumStringRepresentationUsage)"/> with respect
        /// to the example enumeration <see cref="StringAndLanguageAttributeFileAccess"/>.
        /// </summary>
        /// <param name="stringRepresentation">The <see cref="System.String"/> representation to search for.</param>
        /// <param name="enumStringRepresentationUsage">The method how to compute the string representation of the items of the enumeration.</param>
        /// <returns>The item of the enumeration corresponds to <paramref name="stringRepresentation"/>.</returns>
        [TestCase("Read", EnumStringRepresentationUsage.ToStringMethod, Result = StringAndLanguageAttributeFileAccess.Read)]
        [TestCase("Write", EnumStringRepresentationUsage.ToStringMethod, Result = StringAndLanguageAttributeFileAccess.Write)]
        [TestCase("Delete", EnumStringRepresentationUsage.ToStringMethod, Result = StringAndLanguageAttributeFileAccess.Delete)]
        [TestCase("ReadWrite", EnumStringRepresentationUsage.ToStringMethod, Result = StringAndLanguageAttributeFileAccess.ReadWrite)]
        [TestCase("Read, Write", EnumStringRepresentationUsage.ToStringMethod, Result = StringAndLanguageAttributeFileAccess.ReadWrite)]
        [TestCase("Read,Delete,Write", EnumStringRepresentationUsage.ToStringMethod, Result = StringAndLanguageAttributeFileAccess.ReadWrite | StringAndLanguageAttributeFileAccess.Delete)]
        [TestCase("Lesen", EnumStringRepresentationUsage.StringAttribute, Result = StringAndLanguageAttributeFileAccess.Read)]
        [TestCase("Schreiben", EnumStringRepresentationUsage.StringAttribute, Result = StringAndLanguageAttributeFileAccess.Write)]
        [TestCase("Löschen", EnumStringRepresentationUsage.StringAttribute, Result = StringAndLanguageAttributeFileAccess.Delete)]
        [TestCase("Lesen oder schreiben", EnumStringRepresentationUsage.StringAttribute, Result = StringAndLanguageAttributeFileAccess.ReadWrite)]
        [TestCase("Lesen, schreiben", EnumStringRepresentationUsage.StringAttribute, Result = StringAndLanguageAttributeFileAccess.ReadWrite)]
        [TestCase("Schreiben, löschen, lesen", EnumStringRepresentationUsage.StringAttribute, Result = StringAndLanguageAttributeFileAccess.ReadWrite | StringAndLanguageAttributeFileAccess.Delete)]
        [TestCase("ResourceRead", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAndLanguageAttributeFileAccess.Read)]
        [TestCase("ResourceWrite", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAndLanguageAttributeFileAccess.Write)]
        [TestCase("ResourceDelete", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAndLanguageAttributeFileAccess.Delete)]
        [TestCase("ResourceReadwrite", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAndLanguageAttributeFileAccess.ReadWrite)]
        [TestCase("ResourceRead, ResourceWrite", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAndLanguageAttributeFileAccess.ReadWrite)]
        [TestCase("ResourceRead, ResourceDelete, ResourceWrite", EnumStringRepresentationUsage.LanguageStringAttribute, Result = StringAndLanguageAttributeFileAccess.ReadWrite | StringAndLanguageAttributeFileAccess.Delete)]
        public Enum Parse_StringAndLanguageAttributeFileAccessEnumName_ValidEnumValue(string stringRepresentation, EnumStringRepresentationUsage enumStringRepresentationUsage)
        {
            return EnumString.Parse(typeof(StringAndLanguageAttributeFileAccess), stringRepresentation, enumStringRepresentationUsage);
        }

        /// <summary>A test function that compares the result of <see cref="EnumString.Parse(Type, string, EnumStringRepresentationUsage)"/> with respect
        /// to the example enumeration <see cref="PartialAttributeWeekdays"/>.
        /// </summary>
        /// <param name="stringRepresentation">The <see cref="System.String"/> representation to search for.</param>
        /// <param name="enumStringRepresentationUsage">The method how to compute the string representation of the items of the enumeration.</param>
        /// <returns>The item of the enumeration corresponds to <paramref name="stringRepresentation"/>.</returns>
        [TestCase("Sunday", EnumStringRepresentationUsage.ToStringMethod, Result = PartialAttributeWeekdays.Sunday)]
        [TestCase("Monday", EnumStringRepresentationUsage.ToStringMethod, Result = PartialAttributeWeekdays.Monday)]
        [TestCase("Tuesday", EnumStringRepresentationUsage.ToStringMethod, Result = PartialAttributeWeekdays.Tuesday)]
        [TestCase("Sunday, Monday", EnumStringRepresentationUsage.ToStringMethod, Result = PartialAttributeWeekdays.Sunday | PartialAttributeWeekdays.Monday)]
        [TestCase("Sunday", EnumStringRepresentationUsage.StringAttribute, Result = PartialAttributeWeekdays.Sunday)]
        [TestCase("Montag", EnumStringRepresentationUsage.StringAttribute, Result = PartialAttributeWeekdays.Monday)]
        [TestCase("Dienstag", EnumStringRepresentationUsage.StringAttribute, Result = PartialAttributeWeekdays.Tuesday)]
        [TestCase("Sunday, Montag", EnumStringRepresentationUsage.StringAttribute, Result = PartialAttributeWeekdays.Sunday | PartialAttributeWeekdays.Monday)]
        [TestCase("Sunday", EnumStringRepresentationUsage.LanguageStringAttribute, Result = PartialAttributeWeekdays.Sunday)]
        [TestCase("ResourceMonday", EnumStringRepresentationUsage.LanguageStringAttribute, Result = PartialAttributeWeekdays.Monday)]
        [TestCase("Dienstag", EnumStringRepresentationUsage.LanguageStringAttribute, Result = PartialAttributeWeekdays.Tuesday)]
        [TestCase("Sunday, ResourceMonday", EnumStringRepresentationUsage.LanguageStringAttribute, Result = PartialAttributeWeekdays.Sunday | PartialAttributeWeekdays.Monday)]
        public Enum Parse_PartialAttributeWeekdaysEnumName_ValidEnumValue(string stringRepresentation, EnumStringRepresentationUsage enumStringRepresentationUsage)
        {
            return EnumString.Parse(typeof(PartialAttributeWeekdays), stringRepresentation, enumStringRepresentationUsage);
        }

        /// <summary>A test function that compares the result of <see cref="EnumString.CompareTo(EnumString)"/> with respect to 
        /// the example enumeration <see cref="StringAndLanguageAttributeFileAccess"/>.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="valueToCompare">The value to compare.</param>
        /// <param name="enumStringRepresentationUsage">The method how to compute the string representation of <paramref name="value"/>.</param>
        /// <param name="enumStringRepresentationUsageOfValueToCompare">The method how to compute the string representation of <paramref name="valueToCompare"/>.</param>
        [Test]
        public void CompareTo_StringAndLanguageAttributeFileAccessEnum_RelationOfUnderlyingEnumValues(
            [Values(StringAndLanguageAttributeFileAccess.Delete, StringAndLanguageAttributeFileAccess.Read, StringAndLanguageAttributeFileAccess.ReadWrite, StringAndLanguageAttributeFileAccess.Write)]
            Enum value,

            [Values(StringAndLanguageAttributeFileAccess.Delete, StringAndLanguageAttributeFileAccess.Read, StringAndLanguageAttributeFileAccess.ReadWrite, StringAndLanguageAttributeFileAccess.Write)]
            Enum valueToCompare,

            [Values(EnumStringRepresentationUsage.ToStringMethod, EnumStringRepresentationUsage.StringAttribute, EnumStringRepresentationUsage.LanguageStringAttribute)]
            EnumStringRepresentationUsage enumStringRepresentationUsage,

            [Values(EnumStringRepresentationUsage.ToStringMethod, EnumStringRepresentationUsage.StringAttribute, EnumStringRepresentationUsage.LanguageStringAttribute)]
            EnumStringRepresentationUsage enumStringRepresentationUsageOfValueToCompare)
        {
            EnumString enumString = EnumString.Create(value, enumStringRepresentationUsage);
            EnumString other = EnumString.Create(valueToCompare, enumStringRepresentationUsageOfValueToCompare);

            Assert.AreEqual(value.CompareTo(valueToCompare), enumString.CompareTo(other));
        }
    }
}