﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;

using Dodoni.BasicComponents;
using Dodoni.MathLibrary.Basics.LowLevel;

namespace Dodoni.MathLibrary.Basics
{
    /// <summary>Provides the Basic Linear Algebra Subprograms, see http://www.netlib.org/blas for further information.
    /// </summary>
    public static class BLAS
    {
        #region public nested classes, enumerations

        /// <summary>Serves as interface for a BLAS Library.
        /// </summary>
        public interface ILibrary : IIdentifierNameable, IAnnotatable
        {
            /// <summary>Gets Level 1 operations, i.e. vector operations.
            /// </summary>
            /// <value>Level 1 operations.</value>
            ILevel1BLAS Level1
            {
                get;
            }

            /// <summary>Gets Level 2 operations, i.e. matrix-vector operations.
            /// </summary>
            /// <value>Level 2 operations.</value>
            ILevel2BLAS Level2
            {
                get;
            }

            /// <summary>Gets Level 3 operations, i.e. matrix-matrix operations.
            /// </summary>
            /// <value>Level 3 operations.</value>
            ILevel3BLAS Level3
            {
                get;
            }
        }

        /// <summary>The catalog of BLAS Libraries via Managed Extensibility Framework.
        /// </summary>
        internal class Catalog
        {
            #region private members

            /// <summary>The collection of BLAS Libraries.
            /// </summary>
            [ImportMany]
            private IEnumerable<ILibrary> m_Libraries = null;
            #endregion

            #region internal constructors

            /// <summary>Initializes a new instance of the <see cref="Catalog" /> class.
            /// </summary>
            internal Catalog()
            {
                var catalog = new DirectoryCatalog(AppDomain.CurrentDomain.BaseDirectory);

                var container = new CompositionContainer(catalog);
                container.ComposeParts(this);
            }
            #endregion

            #region internal properties

            /// <summary>Gets the collection of (external) BLAS Libraries.
            /// </summary>
            /// <value>The collection of (external) BLAS Libraries.
            /// </value>
            internal IEnumerable<ILibrary> Libraries
            {
                get { return m_Libraries; }
            }
            #endregion
        }

        /// <summary>Represents the states of the property 'transpose' for a specific matrix.
        /// </summary>
        public enum MatrixTransposeState
        {
            /// <summary>The matrix will not be transposed or is not transpose, respectively.
            /// </summary>
            NoTranspose,

            /// <summary>The matrix will be transposed or is transpose, respectively.
            /// </summary>
            Transpose,

            /// <summary>The conjugate matrix will be tranposed or is transpose, respecitively.
            /// </summary>
            Hermite
        }

        /// <summary>Represents the type of a triangular matrix.
        /// </summary>
        public enum TriangularMatrixType
        {
            /// <summary>A lower triangular matrix.
            /// </summary>
            LowerTriangularMatrix,

            /// <summary>A upper triangular matrix.
            /// </summary>
            UpperTriangularMatrix
        }

        /// <summary>Represents the side of a specific matrix in the calculation of a specified BLAS level 3 method.
        /// </summary>
        public enum Side
        {
            /// <summary>The matrix A or op(A) is on the left side.
            /// </summary>
            Left,

            /// <summary>The matrix A or op(A) is on the right side.
            /// </summary>
            Right
        }

        /// <summary>Represents the kind of operation for the BLAS method ?SYR2K.
        /// </summary>
        public enum Xsyr2kOperation
        {
            /// <summary>Calculate C := alpha*A*B^t + alpha*B*A^t + beta*C.
            /// </summary>
            ATimesBTransPlusBTimesATrans,

            /// <summary>Calculate C := alpha*A^t*B + alpha*B^t*A + beta*C.
            /// </summary>
            ATransTimesBPlusBTransTimesA
        }

        /// <summary>Represents the kind of operation for the BLAS method ZHER2K.
        /// </summary>
        public enum Zher2kOperation
        {
            /// <summary>Calculate C := \alpha*A*B^h + conjg(\alpha)*B*A^h + \beta * C.
            /// </summary>
            ATimesBHermitePlusBTimesAHermite,

            /// <summary>Calculate C := \alpha*B^h*A + conjg(\alpha)*A^h*B + beta*C.
            /// </summary>
            BHermiteTimesAPlusAHermiteTimesB
        }

        /// <summary>Represents the kind of operation for the BLAS method ZHERK.
        /// </summary>
        public enum ZherkOperation
        {
            /// <summary>Calculate C := \alpha * A * A^h + \beta*C.
            /// </summary>
            ATimesAHermite,

            /// <summary>Calculate C := alpha*A^h * A + beta*C.
            /// </summary>
            AHermiteTimesA
        }

        /// <summary>Represents the kind of operation for the BLAS method ?SYRK.
        /// </summary>
        public enum XsyrkOperation
        {
            /// <summary>Calculate C := alpha*A*A^t + beta*C.
            /// </summary>
            ATimesATranspose,

            /// <summary>Calculate C := alpha*A^t*A + beta*C.
            /// </summary>
            AtransposeTimesA
        }
        #endregion

        #region public (readonly) members

        /// <summary>Level 1 operations, i.e. vector operations.
        /// </summary>
        public static readonly ILevel1BLAS Level1;

        /// <summary>Level 2 operations, i.e. matrix-vector operations.
        /// </summary>
        public static readonly ILevel2BLAS Level2;

        /// <summary>Level 3 operations, i.e. matrix-matrix operations.
        /// </summary>
        public static readonly ILevel3BLAS Level3;

        /// <summary>Provides Matrix storage conversion methods.
        /// </summary>
        public static readonly MatrixStorageConversion MatrixStorageConversion = new MatrixStorageConversion();
        #endregion

        #region static constructor

        /// <summary>Initializes the <see cref="BLAS"/> class.
        /// </summary>
        /// <remarks>This constructor loads the BLAS dll's with respect to <see cref="LowLevelMathConfiguration"/>.</remarks>
        static BLAS()
        {
            Level1 = LowLevelMathConfiguration.BLAS.Level1;
            Level2 = LowLevelMathConfiguration.BLAS.Level2;
            Level3 = LowLevelMathConfiguration.BLAS.Level3;
        }
        #endregion
    }
}