﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;

using Dodoni.MathLibrary.Basics.LowLevel;

namespace Dodoni.MathLibrary.Basics
{
    public static partial class Extensions
    {
        /// <summary>Compute a singular value decomposition of a general band matrix, i.e. A = U * \Sigma *V', where the diagonal elements of \Sigma are the singular values in descending order.
        /// </summary>
        /// <param name="singularValueDecomposition">The <see cref="LapackEigenvalues.ISingularValueDecomposition"/> object.</param>
        /// <param name="computeSingularVectors">If set to <c>true</c> the left/right singular vectors will be computed as well.</param>
        /// <param name="m">The number of rows of the general band matrix.</param>
        /// <param name="n">The number of columns of the general band matrix.</param>
        /// <param name="kl">The number of sub-diagonals of the general band matrix.</param>
        /// <param name="ku">The number of super-diagonals of the general band matrix.</param>
        /// <param name="a">The general band matrix, matrix entries are provided columnwise.</param>
        /// <param name="s">The singular values in descending order; at least min(<paramref name="n"/>, <paramref name="m"/>) elements (output).</param>
        /// <param name="u">The left singular vectors U, i.e. a m x m unitary matrix (output, if <paramref name="computeSingularVectors"/> is <c>true</c>).</param>
        /// <param name="vt">The right singular vectors V', i.e. a n x n unitary matrix (output, if <paramref name="computeSingularVectors"/> is <c>true</c>).</param>
        /// <remarks>The singular values are the roots of the non-negative eigenvalues of A^t*A.</remarks>
        public static void dgbsvd(this LapackEigenvalues.ISingularValueDecomposition singularValueDecomposition, bool computeSingularVectors, int m, int n, int kl, int ku, double[] a, double[] s, double[] u, double[] vt)
        {
            int ncvt, nru;
            char vect;

            if (computeSingularVectors)
            {
                ncvt = n;
                nru = m;
                vect = 'B';
            }
            else
            {
                ncvt = nru = 0;
                vect = 'N';
            }
            /* reduces the general band matrix to bidiagonal form, i.e.
             *           A = Q * B * P^H, where Q,P orthogonal: 
             * The value of 'vect' indicates whether Q and P^H will be compute. 
             */

            int ncc = 0;   // do not generate C = Q^H*c
            int ldc = 1;   // dummy needed for ncc = 0

            int ldab = ku + kl + 1;

            // working array and off-diagonal elements are used for both MKL functions,
            // therefore we adjust the minimum size for both:
            double[] work = new double[2 * Math.Max(n, m) + 2 * n];
            double[] e = new double[Math.Max(n, m)]; // off-diagonal elements of B

            singularValueDecomposition.dgbbrd(vect, m, n, ncc, kl, ku, a, ldab, s, e, u, m, vt, n, null, ldc, work);
            singularValueDecomposition.dbdsqr(LAPACK.BidiagonalMatrixType.UpperBidiagonalMatrix, n, ncvt, nru, ncc, s, e, vt, n, u, m, null, ldc, work);
        }
    }
}