﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.ComponentModel;

using Dodoni.BasicComponents.Utilities;
using Dodoni.MathLibrary.Basics.LowLevel;

namespace Dodoni.MathLibrary.Basics
{
    /// <summary>Provides the Linear Algebra PACKage, see http://www.netlib.org/lapack/index.html for further information.
    /// </summary>
    public static class LAPACK
    {
        #region nested enumerations

        /// <summary>Represents the calculation type of left singular vectors within a singular 
        /// value decomposition, i.e. 'U' with respect to the decomposition A=U * \Sigma * V^t.
        /// </summary>
        public enum LeftSingularVectorsJob
        {
            /// <summary>All m columns of U are returned in the array u. 
            /// </summary>
            All,

            /// <summary>The first min(m, n) columns of U (the left singular vectors) are returned in the array u.
            /// </summary>
            OutOfPlace,

            /// <summary>The first min(m, n) columns of U (the left singular vectors) are overwritten on the array a; the parameter u is not referenced.
            /// </summary>
            InPlace,

            /// <summary>No columns of U (no left singular vectors) are compute, the parameter u is not referenced.
            /// </summary>
            NoLeftSingularVectors,
        }

        /// <summary>Represents the calculation type of right singular vectors within a singular 
        /// value decomposition, i.e. 'V^t' with respect to the decomposition A=U * \Sigma * V^t.
        /// </summary>
        public enum RightSingularVectorsJob
        {
            /// <summary>All n rows of V^t are returned in the array vt. 
            /// </summary>
            All,

            /// <summary>The first min(m, n) rows of V^t (the right singular vectors) are returned in the array vt.
            /// </summary>
            OutOfPlace,

            /// <summary>The first min(m, n) rows of V^t (the right singular vectors) are overwritten on the array a; the parameter vt is not referenced.
            /// </summary>
            InPlace,

            /// <summary>No rows of V^t (no right singular vectors) are compute, the
            /// parameter vt is not referenced.
            /// </summary>
            NoLeftSingularVectors,
        }

        /// <summary>Represents matrix balance approaches needed for example for 
        /// the Lapack function 'dgebal' to improve the accuracy of computed eigenvalues and eigenvectors. 
        /// </summary>
        public enum MatrixBalancesType
        {
            /// <summary>Neither permuted nor scale the matrix.
            /// </summary>
            NeitherPermutedNorScaled,

            /// <summary>Permute the matrix.
            /// </summary>
            Permuted,

            /// <summary>Scale the matrix.
            /// </summary>
            Scaled,

            /// <summary>Permuted and scale the matrix.
            /// </summary>
            PermutedAndScaled
        }

        /// <summary>Represents the type of a bidiagonal matrix.
        /// </summary>
        public enum BidiagonalMatrixType
        {
            /// <summary>A lower bidiagonal matrix.
            /// </summary>
            LowerBidiagonalMatrix,

            /// <summary>A upper bidiagonal matrix.
            /// </summary>
            UpperBidiagonalMatrix
        }
        #endregion

        #region public static (readonly) members

        /// <summary>Provides routines that are related to Least Square and Eigenvalue calculations.
        /// </summary>
        public static readonly LapackEigenvalues EigenValues = new LapackEigenvalues();

        /// <summary>Provides routines that are related to Linear Equations.
        /// </summary>
        public static readonly LapackLinearEquations LinearEquations = new LapackLinearEquations();
        #endregion
    }
}