﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;
using System.Collections.Generic;

using Dodoni.MathLibrary.Basics;
using Dodoni.MathLibrary.Basics.LowLevel;

namespace Dodoni.MathLibrary.Basics.LowLevel.BuildIn
{
    /// <summary>Serves as (primitive) managed code implementation of BLAS level 3 operations (fall-back solution).
    /// </summary>
    /// <remarks>This implementation is based on the C code of the BLAS implementation, see http://www.netlib.org/clapack/cblas. </remarks>
    internal partial class BuildInLevel3BLAS
    {
        /// <summary>Solves a triangular matrix equation, i.e. op(A) * X = \alpha * B or X * op(A) = \alpha *B, where A is a unit or non-unit upper or lower triangular matrix.
        /// </summary>
        /// <param name="m">The number of rows of matrix B.</param>
        /// <param name="n">The number of column of matrix B.</param>
        /// <param name="alpha">The scalar \alpha.</param>
        /// <param name="a">The triangular matrix A supplied column-by-column of dimension (<paramref name="lda" />, k), where k is <paramref name="m" /> if to calculate op(A) * X = \alpha * B; <paramref name="n" /> otherwise.</param>
        /// <param name="b">The matrix B supplied column-by-column of dimension (<paramref name="ldb" />, <paramref name="n" />).</param>
        /// <param name="lda">The leading dimension of <paramref name="a" />, must be at least max(1,<paramref name="m" />) if to calculate op(A) * X = \alpha * B; max(1,<paramref name="n" />) otherwise.</param>
        /// <param name="ldb">The leading dimension of <paramref name="b" />, must be at least max(1,<paramref name="m" />).</param>
        /// <param name="isUnitTriangular">A value indicating whether the matrix A is unit triangular.</param>
        /// <param name="side">A value indicating whether to calculate op(A) * X = \alpha * B or X * op(A) = \alpha *B.</param>
        /// <param name="triangularMatrixType">A value whether matrix A is in its upper or lower triangular representation.</param>
        /// <param name="transpose">A value indicating whether 'op(A)=A' or 'op(A)=A^t'.</param>
        public void ztrsm(int m, int n, Complex alpha, Complex[] a, Complex[] b, int lda, int ldb, bool isUnitTriangular = true, BLAS.Side side = BLAS.Side.Left, BLAS.TriangularMatrixType triangularMatrixType = BLAS.TriangularMatrixType.UpperTriangularMatrix, BLAS.MatrixTransposeState transpose = BLAS.MatrixTransposeState.NoTranspose)
        {
            if (side == BLAS.Side.Left)
            {
                if (transpose == BLAS.MatrixTransposeState.NoTranspose)  // B := \alpha * Inv(A) * B
                {
                    if (triangularMatrixType == BLAS.TriangularMatrixType.UpperTriangularMatrix)
                    {
                        for (int j = 0; j < n; j++)
                        {
                            for (int i = 0; i < m; i++)
                            {
                                b[i + j * ldb] *= alpha;
                            }
                            for (int k = m - 1; k >= 0; k--)
                            {
                                if (isUnitTriangular == false)
                                {
                                    b[k + j * ldb] = b[k + j * ldb] / a[k + k * lda];
                                }
                                for (int i = 0; i <= k - 1; i++)
                                {
                                    b[i + j * ldb] += -b[k + j * ldb] * a[i + k * lda];
                                }
                            }
                        }
                    }
                    else
                    {
                        for (int j = 0; j < n; j++)
                        {
                            for (int i = 0; i < m; i++)
                            {
                                b[i + j * ldb] *= alpha;
                            }
                            for (int k = 0; k < m; k++)
                            {
                                if (isUnitTriangular == false)
                                {
                                    b[k + j * ldb] = b[k + j * ldb] / a[k + k * lda];
                                }
                                for (int i = k + 1; i < m; i++)
                                {
                                    b[i + j * ldb] += -b[k + j * ldb] * a[i + k * lda];
                                }
                            }
                        }
                    }
                }
                else  // B:= \alpha * Inv(A') * B or B := \alpha * Inv( Conj(A') ) * B
                {
                    if (triangularMatrixType == BLAS.TriangularMatrixType.UpperTriangularMatrix)
                    {
                        Complex temp = 0.0;

                        for (int j = 0; j < n; j++)
                        {
                            for (int i = 0; i < m; i++)
                            {
                                temp = alpha * b[i + j * ldb];
                                if (transpose == BLAS.MatrixTransposeState.Transpose)
                                {
                                    for (int k = 0; k <= i - 1; k++)
                                    {
                                        temp += -a[k + i * lda] * b[k + j * ldb];
                                    }
                                    if (isUnitTriangular == false)
                                    {
                                        temp = temp / a[i + i * lda];
                                    }
                                }
                                else
                                {
                                    for (int k = 0; k <= i - 1; k++)
                                    {
                                        temp += -Complex.Conjugate(a[k + i * lda]) * b[k + j * ldb];
                                    }
                                    if (isUnitTriangular == false)
                                    {
                                        temp = temp / Complex.Conjugate(a[i + i * lda]);
                                    }
                                }
                                b[i + j * ldb] = temp;
                            }
                        }
                    }
                    else
                    {
                        Complex temp = 0.0;
                        for (int j = 0; j < n; j++)
                        {
                            for (int i = m - 1; i >= 0; i--)
                            {
                                temp = alpha * b[i + j * ldb];
                                if (transpose == BLAS.MatrixTransposeState.Transpose)
                                {
                                    for (int k = i + 1; k < m; k++)
                                    {
                                        temp += -a[k + i * lda] * b[k + j * ldb];
                                    }
                                    if (isUnitTriangular == false)
                                    {
                                        temp = temp / a[i + i * lda];
                                    }
                                }
                                else
                                {
                                    for (int k = i + 1; k < m; k++)
                                    {
                                        temp = temp - Complex.Conjugate(a[k + i * lda]) * b[k + j * ldb];
                                    }
                                    if (isUnitTriangular == false)
                                    {
                                        temp = temp / Complex.Conjugate(a[i + i * lda]);
                                    }
                                }
                                b[i + j * ldb] = temp;
                            }
                        }
                    }
                }
            }
            else
            {
                if (transpose == BLAS.MatrixTransposeState.NoTranspose)  // B := \alpha * B * Inv(A)
                {
                    if (triangularMatrixType == BLAS.TriangularMatrixType.UpperTriangularMatrix)
                    {
                        for (int j = 0; j < n; j++)
                        {
                            for (int i = 0; i < m; i++)
                            {
                                b[i + j * ldb] = alpha * b[i + j * ldb];
                            }
                            for (int k = 0; k <= j - 1; k++)
                            {
                                for (int i = 0; i < m; i++)
                                {
                                    b[i + j * ldb] = b[i + j * ldb] - a[k + j * lda] * b[i + k * ldb];
                                }
                            }
                            if (isUnitTriangular == false)
                            {
                                Complex temp = 1.0 / a[j + j * lda];
                                for (int i = 0; i < m; i++)
                                {
                                    b[i + j * ldb] = temp * b[i + j * ldb];
                                }
                            }
                        }
                    }
                    else
                    {
                        for (int j = n - 1; j >= 0; j--)
                        {
                            for (int i = 0; i < m; i++)
                            {
                                b[i + j * ldb] = alpha * b[i + j * ldb];
                            }
                            for (int k = j + 1; k < n; k++)
                            {
                                for (int i = 0; i < m; i++)
                                {
                                    b[i + j * ldb] = b[i + j * ldb] - a[k + j * lda] * b[i + k * ldb];
                                }
                            }

                            if (isUnitTriangular == false)
                            {
                                Complex temp = 1.0 / a[j + j * lda];
                                for (int i = 0; i < m; i++)
                                {
                                    b[i + j * ldb] = temp * b[i + j * ldb];
                                }
                            }
                        }
                    }
                }
                else  // B := \alpha * B * Inv(A') or B:=\alpha * B *Inv( Conj(A') )
                {
                    if (triangularMatrixType == BLAS.TriangularMatrixType.UpperTriangularMatrix)
                    {
                        Complex temp = 0.0;

                        for (int k = n - 1; k >= 0; k--)
                        {
                            if (isUnitTriangular == false)
                            {
                                if (transpose == BLAS.MatrixTransposeState.Hermite)
                                {
                                    temp = 1.0 / Complex.Conjugate(a[k + k * lda]);
                                }
                                else
                                {
                                    temp = 1.0 / a[k + k * lda];
                                }
                                for (int i = 0; i < m; i++)
                                {
                                    b[i + k * ldb] = temp * b[i + k * ldb];
                                }
                            }
                            for (int j = 0; j <= k - 1; j++)
                            {
                                if (transpose == BLAS.MatrixTransposeState.Hermite)
                                {
                                    temp = Complex.Conjugate(a[j + k * lda]);
                                }
                                else
                                {
                                    temp = a[j + k * lda];
                                }
                                for (int i = 0; i < m; i++)
                                {
                                    b[i + j * ldb] = b[i + j * ldb] - temp * b[i + k * ldb];
                                }
                            }
                            for (int i = 0; i < m; i++)
                            {
                                b[i + k * ldb] = alpha * b[i + k * ldb];
                            }
                        }
                    }
                    else
                    {
                        Complex temp = 0.0;

                        for (int k = 0; k < n; k++)
                        {
                            if (isUnitTriangular == false)
                            {
                                if (transpose == BLAS.MatrixTransposeState.Hermite)
                                {
                                    temp = 1.0 / Complex.Conjugate(a[k + k * lda]);
                                }
                                else
                                {
                                    temp = 1.0 / a[k + k * lda];
                                }
                                for (int i = 0; i < m; i++)
                                {
                                    b[i + k * ldb] = temp * b[i + k * ldb];
                                }
                            }
                            for (int j = k + 1; j < n; j++)
                            {
                                if (transpose == BLAS.MatrixTransposeState.Hermite)
                                {
                                    temp = Complex.Conjugate(a[j + k * lda]);
                                }
                                else
                                {
                                    temp = a[j + k * lda];
                                }
                                for (int i = 0; i < m; i++)
                                {
                                    b[i + j * ldb] = b[i + j * ldb] - temp * b[i + k * ldb];
                                }
                            }

                            for (int i = 0; i < m; i++)
                            {
                                b[i + k * ldb] = alpha * b[i + k * ldb];
                            }
                        }
                    }
                }
            }
        }
    }
}