﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

using Dodoni.MathLibrary.ProbabilityTheory.Distributions;

namespace Dodoni.MathLibrary.Basics.LowLevel.BuildIn
{
    /// <summary>Provides a managed .net implementation of vector operations with respect to special mathematical functions.
    /// </summary>
    internal class BuildInVectorUnitSpecial : IVectorUnitSpecial
    {
        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="BuildInVectorUnitSpecial"/> class.
        /// </summary>
        internal BuildInVectorUnitSpecial()
        {
        }
        #endregion

        #region IVectorUnitSpecial Members

        /// <summary>Computes the cumulative normal distribution function values of vector elements.
        /// </summary>
        /// <param name="n">The number of elements to be calculated.</param>
        /// <param name="a">The input vector a.</param>
        /// <param name="y">The output vector N(a[j]), j=0,...,<paramref name="n"/>-1, where N(x) =\int_{-\infty}^x  1/\sqrt(2*PI) * exp(-1/2 *t^2) dt.</param>
        public void CdfNorm(int n, double[] a, double[] y)
        {
            for (int j = 0; j < n; j++)
            {
                y[j] = NormalDistribution.GetStandardCdfValue(a[j]);
            }
        }

        /// <summary>Computes the cumulative normal distribution function values of vector elements [in-place].
        /// </summary>
        /// <param name="n">The number of elements to be calculated.</param>
        /// <param name="a">The input vector a; overwritten by the result of the operation, i.e. N(a[j]), j=0,...,<paramref name="n" />-1, where N(x) =\int_{-\infty}^x  1/\sqrt(2*PI) * exp(-1/2 *t^2) dt.</param>
        public void CdfNorm(int n, double[] a)
        {
            for (int j = 0; j < n; j++)
            {
                a[j] = NormalDistribution.GetStandardCdfValue(a[j]);
            }
        }

        /// <summary>Computes the cumulative normal distribution function values of vector elements.
        /// </summary>
        /// <param name="n">The number of elements to be calculated.</param>
        /// <param name="a">The input vector a.</param>
        /// <param name="y">The output vector y with y[<paramref name="startIndexY"/> + j] := N(a[<paramref name="startIndexA"/> + j]) for j = 0,...,<paramref name="n"/>-1, where N(x) =\int_{-\infty}^x  1/\sqrt(2*PI) * exp(-1/2 *t^2) dt.</param>
        /// <param name="startIndexA">The null-based start index of <paramref name="a"/>.</param>
        /// <param name="startIndexY">The null-based start index of <paramref name="y"/>.</param>
        public void CdfNorm(int n, double[] a, double[] y, int startIndexA, int startIndexY)
        {
            for (int j = 0; j < n; j++)
            {
                y[j + startIndexY] = NormalDistribution.GetStandardCdfValue(a[j + startIndexA]);
            }
        }

        /// <summary>Computes the inverse cumulative normal distribution function values of vector elements.
        /// </summary>
        /// <param name="n">The number of elements to be calculated.</param>
        /// <param name="a">The input vector a.</param>
        /// <param name="y">The output vector N^{-1}(a[j]), j=0,...,<paramref name="n"/>-1, where N(x) =\int_{-\infty}^x  1/\sqrt(2*PI) * exp(-1/2 *t^2) dt.</param>
        public void CdfNormInv(int n, double[] a, double[] y)
        {
            for (int j = 0; j < n; j++)
            {
                y[j] = NormalDistribution.GetStandardInverseCdfValue(a[j]);
            }
        }

        /// <summary>Computes the inverse cumulative normal distribution function values of vector elements [in-place].
        /// </summary>
        /// <param name="n">The number of elements to be calculated.</param>
        /// <param name="a">The input vector a; overwritten by the result of the operation, i.e. N^{-1}(a[j]), j=0,...,<paramref name="n" />-1, where N(x) =\int_{-\infty}^x  1/\sqrt(2*PI) * exp(-1/2 *t^2) dt.</param>
        public void CdfNormInv(int n, double[] a)
        {
            for (int j = 0; j < n; j++)
            {
                a[j] = NormalDistribution.GetStandardInverseCdfValue(a[j]);
            }
        }

        /// <summary>Computes the inverse cumulative normal distribution function values of vector elements.
        /// </summary>
        /// <param name="n">The number of elements to be calculated.</param>
        /// <param name="a">The input vector a.</param>
        /// <param name="y">The output vector y with y[<paramref name="startIndexY"/> + j] := N^{-1}(a[<paramref name="startIndexA"/> + j]) for j = 0,...,<paramref name="n"/>-1, where N(x) =\int_{-\infty}^x  1/\sqrt(2*PI) * exp(-1/2 *t^2) dt.</param>
        /// <param name="startIndexA">The null-based start index of <paramref name="a"/>.</param>
        /// <param name="startIndexY">The null-based start index of <paramref name="y"/>.</param>
        public void CdfNormInv(int n, double[] a, double[] y, int startIndexA, int startIndexY)
        {
            for (int j = 0; j < n; j++)
            {
                y[j + startIndexY] = NormalDistribution.GetStandardInverseCdfValue(a[j + startIndexA]);
            }
        }
        #endregion
    }
}