﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    /// <summary>Serves as interface for level 3 Sparse Basic Linear Algebra Subprograms, namely (C-)BLAS, i.e. <para>matrix-matrix operations.</para> 
    /// </summary>
    /// <remarks>The function names are almost identically to the BLAS naming convention, see http://www.netlib.org/blas, here we restrict to double precision and complex numbers.</remarks>
    public interface ILevel3SparseBLAS
    {
        /// <summary>Perform the matrix-matrix operation 'C = \alpha*A*B + \beta*C', where A,B,C are general band matrices.
        /// </summary>
        /// <param name="m">The number of rows of matrix A.</param>
        /// <param name="n">The number of columns of matrix B.</param>
        /// <param name="k">The number of columns of matrix A and the number of rows of matrix B.</param>
        /// <param name="alpha">The scalar \alpha.</param>
        /// <param name="klA">Specifies the number of sub-diagonals of the matrix A.</param>
        /// <param name="kuA">Specifies the number of super-diagonals of the matrix A.</param>
        /// <param name="a">The general band matrix A given column-by-column, i.e. at least (<paramref name="klA"/> + <paramref name="kuA"/> + 1) * <paramref name="k"/> elements.</param>
        /// <param name="klB">Specifies the number of sub-diagonals of the matrix B.</param>
        /// <param name="kuB">Specifies the number of super-diagonals of the matrix B.</param>
        /// <param name="b">The general band matrix B given column-by-column, i.e. at least (<paramref name="klB"/> + <paramref name="kuB"/> + 1) * <paramref name="n"/> elements.</param>
        /// <param name="beta">The scalar \beta.</param>
        /// <param name="c">The general band matrix C given column-by-column where the number of sub-diagonals is klC and the number of
        /// super-diagonals is kuC, where klC = max { <paramref name="klA"/> - <paramref name="kuB"/>, 0} and 
        /// kuC = max { <paramref name="kuA"/> - <paramref name="klB"/>, 0}, i.e. at least (klC + kuC + 1) * <paramref name="n"/> elements (output).</param>
        /// <remarks>The general band matrix storage is given in the same way as in BLAS level 3, i.e. perhaps for the first/last
        /// column the array contains more elements than necessary.</remarks>
        void dgbmm(int m, int n, int k, double alpha, int klA, int kuA, double[] a, int klB, int kuB, double[] b, double beta, double[] c);


        /// <summary>Perform the matrix-matrix operation 'C = \alpha*A*B', where A,B,C are general band matrices.
        /// </summary>
        /// <param name="m">The number of rows of matrix A.</param>
        /// <param name="n">The number of columns of matrix B.</param>
        /// <param name="k">The number of columns of matrix A and the number of rows of matrix B.</param>
        /// <param name="alpha">The scalar \alpha.</param>
        /// <param name="klA">Specifies the number of sub-diagonals of the matrix A.</param>
        /// <param name="kuA">Specifies the number of super-diagonals of the matrix A.</param>
        /// <param name="a">The general band matrix A given column-by-column, i.e. at least (<paramref name="klA"/> + <paramref name="kuA"/> + 1) * <paramref name="k"/> elements.</param>
        /// <param name="klB">Specifies the number of sub-diagonals of the matrix B.</param>
        /// <param name="kuB">Specifies the number of super-diagonals of the matrix B.</param>
        /// <param name="b">The general band matrix B given column-by-column, i.e. at least (<paramref name="klB"/> + <paramref name="kuB"/> + 1) * <paramref name="n"/> elements.</param>
        /// <param name="c">The general band matrix C given column-by-column where the number of sub-diagonals is klC and the number of
        /// super-diagonals is kuC, where klC = max { <paramref name="klA"/> - <paramref name="kuB"/>, 0} and 
        /// kuC = max { <paramref name="kuA"/> - <paramref name="klB"/>, 0}, i.e. at least (klC + kuC + 1) * <paramref name="n"/> elements (output).</param>
        /// <remarks>The general band matrix storage is given in the same way as in BLAS level 3, i.e. perhaps for the first/last
        /// column the array contains more elements than necessary.</remarks>
        void dgbmm(int m, int n, int k, double alpha, int klA, int kuA, double[] a, int klB, int kuB, double[] b, double[] c);
    }
}