﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    public partial class LapackEigenvalues
    {
        public interface IDriverRoutines
        {
            /// <summary>Computes a singular value decomposition of a dense matrix, i.e. 'A = U * \Sigma * V^t', where the diagonal elements
            /// of '\Sigma' are the singular values in descending order.
            /// </summary>
            /// <param name="m">The number of rows of 'A'.</param>
            /// <param name="n">The number of columns of 'A'.</param>
            /// <param name="a">The dense matrix 'A', matrix entries are given columnwise.</param>
            /// <param name="s">The singular values in descending order, must have at least min(n,m) elements (output).</param>
            /// <param name="u">The left singular vectors 'U', i.e. a 'm x m' unitary matrix (output).</param>
            /// <param name="vt">The right singular vectors 'V^t', i.e. a 'n x n' unitary matrix (output).</param>
            /// <remarks>The singular values are the roots of the non-negative eigenvalues of A^t*A.
            /// <para>The input matrix <paramref name="a"/> will be changed in any case.</para></remarks>
            void dgesvd(int m, int n, double[] a, double[] s, double[] u, double[] vt);

            /// <summary>Computes a singular value decomposition of a dense matrix, i.e. 'A = U * \Sigma * V^t', where the diagonal elements
            /// of '\Sigma' are the singular values in descending order.
            /// </summary>
            /// <param name="uJob">A value indicating whether the left singular vectors 'U' are calculated.</param>
            /// <param name="vtJob">A value indicating whether the right singular vectors 'V^t' are calculated.</param>
            /// <param name="m">The number of rows of 'A'.</param>
            /// <param name="n">The number of columns of 'A'.</param>
            /// <param name="a">The dense matrix 'A', matrix entries are given columnwise.</param>
            /// <param name="s">The singular values in descending order, must have at least min(n,m) elements (output).</param>
            /// <param name="u">The left singular vectors 'U', i.e. a 'm x m' unitary matrix (output, if <paramref name="uJob"/> indicate to calculate left singular vectors).</param>
            /// <param name="vt">The right singular vectors 'V^t', i.e. a 'n x n' unitary matrix (output, if <paramref name="vtJob"/> is indicate to calculate right singular vectors).</param>
            /// <exception cref="ArgumentException">Thrown, if <paramref name="uJob"/> and <paramref name="vtJob"/> indicate a in Place calcuation.</exception>
            /// <remarks>The singular values are the roots of the non-negative eigenvalues of A^t*A.
            /// <para>The input matrix <paramref name="a"/> will be changed in any case.</para></remarks>
            void dgesvd(LAPACK.LeftSingularVectorsJob uJob, LAPACK.RightSingularVectorsJob vtJob, int m, int n, double[] a, double[] s, double[] u, double[] vt);

            /// <summary>Gets a optimal working array for the <c>dgesvd</c> algorithm.
            /// </summary>
            /// <param name="uJob">A value indicating whether the left singular vectors 'U' are calculated.</param>
            /// <param name="vtJob">A value indicating whether the right singular vectors 'V^t' are calculated.</param>
            /// <param name="m">The number of rows of 'A'.</param>
            /// <param name="n">The number of columns of 'A'.</param>
            /// <param name="a">The dense matrix 'A', matrix entries are given columnwise.</param>
            /// <param name="s">The singular values in descending order, must have at least min(n,m) elements (output).</param>
            /// <param name="u">The left singular vectors 'U', i.e. a 'm x m' unitary matrix (output, if <paramref name="uJob"/> indicate to calculate left singular vectors).</param>
            /// <param name="vt">The right singular vectors 'V^t', i.e. a 'n x n' unitary matrix (output, if <paramref name="vtJob"/> is indicate to calculate right singular vectors).</param>
            /// <returns>A working array for the <c>dgesvd</c> algorithm with optimal length.
            /// </returns>
            /// <exception cref="InvalidOperationException">Thrown, if the execution is not successful.</exception>
            /// <exception cref="ArgumentException">Thrown, if <paramref name="uJob"/> and <paramref name="vtJob"/> indicate a in Place calcuation.</exception>
            /// <remarks>The singular values are the roots of the non-negative eigenvalues of A^t*A. Use this method
            /// if a singular value decomposition is needed several times with the same parameters.
            /// <para>The input matrix <paramref name="a"/> will not be changed.</para></remarks>
            double[] dgesvdQuery(LAPACK.LeftSingularVectorsJob uJob, LAPACK.RightSingularVectorsJob vtJob, int m, int n, double[] a, double[] s, double[] u, double[] vt);

            /// <summary>Computes a singular value decomposition of a dense matrix, i.e. 'A = U * \Sigma * V^t', where the diagonal elements
            /// of '\Sigma' are the singular values in descending order.
            /// </summary>
            /// <param name="uJob">A value indicating whether the left singular vectors 'U' are calculated.</param>
            /// <param name="vtJob">A value indicating whether the right singular vectors 'V^t' are calculated.</param>
            /// <param name="m">The number of rows of 'A'.</param>
            /// <param name="n">The number of columns of 'A'.</param>
            /// <param name="a">The dense matrix 'A', matrix entries are given columnwise.</param>
            /// <param name="s">The singular values in descending order, must have at least min(n,m) elements (output).</param>
            /// <param name="u">The left singular vectors 'U', i.e. a 'm x m' unitary matrix (output, if <paramref name="uJob"/> indicate to calculate left singular vectors).</param>
            /// <param name="vt">The right singular vectors 'V^t', i.e. a 'n x n' unitary matrix (output, if <paramref name="vtJob"/> is indicate to calculate right singular vectors).</param>
            /// <param name="work">A working array, where the minimum length is given by a previous call
            /// of <see cref="dgesvdQuery(LAPACK.LeftSingularVectorsJob, LAPACK.RightSingularVectorsJob, int, int, double[], double[], double[], double[])"/>.</param>
            /// <exception cref="ArgumentException">Thrown, if <paramref name="uJob"/> and <paramref name="vtJob"/> indicate a in Place calcuation.</exception>
            /// <remarks>The singular values are the roots of the non-negative eigenvalues of A^t*A. Use this method
            /// for better performance and re-using a specific working array.
            /// <para>The input matrix <paramref name="a"/> will be changed in any case.</para></remarks>
            void dgesvd(LAPACK.LeftSingularVectorsJob uJob, LAPACK.RightSingularVectorsJob vtJob, int m, int n, double[] a, double[] s, double[] u, double[] vt, double[] work);

        }
    }
}