﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    public partial class LapackEigenvalues
    {
        /// <summary>The job for <c>?hseqr</c> function calls.
        /// </summary>
        public enum JobXhseqr
        {
            /// <summary>Eigenvalues are required only.
            /// </summary>
            EigenvaluesOnly,

            /// <summary>Calculate Eigenvalues and Schur Factorization.
            /// </summary>
            EigenvaluesAndSchurForm
        }

        /// <summary>Represents the kind of operation for the Lapack method <c>?hseqr</c>.
        /// </summary>
        public enum XhseqrOperation
        {
            /// <summary>Do not compute Schur vectors and input argument z of <c>?hseqr</c> is not referenced, the leading dimension of z can be set to <c>1</c>.
            /// </summary>
            NoSchurVectors,

            /// <summary>Compute the Schur vectors of H. The second dimension of argument z of <c>?hseqr</c> should be at least n and z need not be set.
            /// </summary>
            SchurVectorOfH,

            /// <summary>Compute the Schur vectors of the input matrix A. The second dimension of argument z of <c>?hseqr</c> should be at least n and z must contain the 
            /// matrix Q from the reduction to Hessenberg form on input.
            /// </summary>
            SchurVectorOfA
        }

        /// <summary>Provides methods of non-symmetric Eigenvalue problems.
        /// </summary>
        public interface INonsymmetricEigenvalueProblems
        {
            #region double precision methods

            /// <summary>Reduces a general matrix to upper Hessenberg form H by an orthogonal or unitary similarity transformation A = Q * H * Q', where H has real subdiagonal elements.
            /// </summary>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the upper Hessenberg matrix H and details of the matrix Q.</param>
            /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q (output).</param>
            /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength"/>.</param>
            /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace"/>, at least <paramref name="n"/>.</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, at least <paramref name="n"/>.</param>
            void dgehrd(int n, int ilo, int ihi, double[] a, double[] tau, double[] workspace, int workspaceLength, int lda);

            /// <summary>Gets the ideal length of the workspace array required for optimum performance.
            /// </summary>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="ilo">If the input matrix is an output by <c>?gebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>?gebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the upper Hessenberg matrix H and details of the matrix Q.</param>
            /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q (output).</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, at least <paramref name="n"/>.</param>
            /// <returns>The ideal length of the working array required for optimum performance.</returns>
            int dgehrd_workspaceQuery(int n, int ilo, int ihi, double[] a, double[] tau, int lda);

            /// <summary>Generates the real orthogonal matrix Q determined by <c>dgehrd</c>, i.e. explicitly generates the orthogonal matrix Q with A = Q * H * Q'.
            /// </summary>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="a">Contains on input details of the vectors which define the elementary reflectors, as returned by <c>dgehrd</c> of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the n-by-n orthogonal matrix Q.</param>
            /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q as returned by <c>dgehrd</c>.</param>
            /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength"/>.</param>
            /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace"/>, at least max( 1, <paramref name="ihi"/> - <paramref name="ilo"/>).</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, at least <paramref name="n"/>.</param>
            void dorghr(int n, int ilo, int ihi, double[] a, double[] tau, double[] workspace, int workspaceLength, int lda);

            /// <summary>Gets the ideal length of the workspace array required for optimum performance.
            /// </summary>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="a">Contains on input details of the vectors which define the elementary reflectors, as returned by <c>dgehrd</c> of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the n-by-n orthogonal matrix Q.</param>
            /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q as returned by <c>dgehrd</c>.</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, at least <paramref name="n"/>.</param>
            /// <returns>The ideal length of the working array required for optimum performance.</returns>
            int dorghr_workspaceQuery(int n, int ilo, int ihi, double[] a, double[] tau, int lda);

            /// <summary>Multiplies an arbitrary real matrix C by the real orthogonal matrix Q determined by <c>dgehrd</c>, i.e. \op(Q) * C or C * \op(Q), where \op(Q) = Q or \op(Q) = Q'.
            /// </summary>
            /// <param name="side">A value indicating whether to calculate \op(Q) * C [left] or C * \op(Q) [right].</param>
            /// <param name="transpose">A value indicating whether \op(Q) = Q or \op(Q) = Q'.</param>
            /// <param name="m">The number of rows in matrix C.</param>
            /// <param name="n">The number of columns in matrix C.</param>
            /// <param name="ilo">The same parameter value as supplied to <c>dgehrd</c>.</param>
            /// <param name="ihi">The same parameter value as supplied to <c>dgehrd</c>.</param>
            /// <param name="a">Contains details of the vectors which define the elementary reflectors, as returned by <c>dgehrd</c> of dimension (<paramref name="lda"/>; k) where k must be at least max(1, <paramref name="m"/>) if to calculate \op(Q) * C; otherwise max( 1, <paramref name="n"/>).</param>
            /// <param name="tau">Contains further details of the elementary reflectors as returned by <c>dgehrd</c>.</param>
            /// <param name="c">The m-by-n matrix C supplied column-by-column of dimension (<paramref name="ldc"/>, n).</param>
            /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength"/>.</param>
            /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace"/>, at least max( 1, <paramref name="n"/> ) if to calculate \op(Q) * C; otherwise max( 1, <paramref name="m"/>).</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, must be at least max(1,<paramref name="m"/>) if to calculate \op(Q) * C; max(1,<paramref name="n"/>) otherwise.</param>
            /// <param name="ldc">The leading dimension of <paramref name="a"/>, at least <paramref name="m"/>.</param>
            void dormhr(BLAS.Side side, BLAS.MatrixTransposeState transpose, int m, int n, int ilo, int ihi, double[] a, double[] tau, double[] c, double[] workspace, int workspaceLength, int lda, int ldc);

            /// <summary>Gets the ideal length of the workspace array required for optimum performance.
            /// </summary>
            /// <param name="side">A value indicating whether to calculate \op(Q) * C [left] or C * \op(Q) [right].</param>
            /// <param name="transpose">A value indicating whether \op(Q) = Q or \op(Q) = Q'.</param>
            /// <param name="m">The number of rows in matrix C.</param>
            /// <param name="n">The number of columns in matrix C.</param>
            /// <param name="ilo">The same parameter value as supplied to <c>dgehrd</c>.</param>
            /// <param name="ihi">The same parameter value as supplied to <c>dgehrd</c>.</param>
            /// <param name="a">Contains details of the vectors which define the elementary reflectors, as returned by <c>dgehrd</c> of dimension (<paramref name="lda"/>; k) where k must be at least max(1, <paramref name="m"/>) if to calculate \op(Q) * C; otherwise max( 1, <paramref name="n"/>).</param>
            /// <param name="tau">Contains further details of the elementary reflectors as returned by <c>dgehrd</c>.</param>
            /// <param name="c">The m-by-n matrix C supplied column-by-column of dimension (<paramref name="ldc"/>, n).</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, must be at least max(1,<paramref name="m"/>) if to calculate \op(Q) * C; max(1,<paramref name="n"/>) otherwise.</param>
            /// <param name="ldc">The leading dimension of <paramref name="a"/>, at least <paramref name="m"/>.</param>
            /// <returns>The ideal length of the working array required for optimum performance.</returns>
            int dormhr_workspaceQuery(BLAS.Side side, BLAS.MatrixTransposeState transpose, int m, int n, int ilo, int ihi, double[] a, double[] tau, double[] c, int lda, int ldc);

            /// <summary>Balances a general matrix to improve the accuracy of computed eigenvalues and eigenvectors.
            /// </summary>
            /// <param name="job">The job, i.e. the matrix balance type.</param>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the balanced matrix.</param>
            /// <param name="ilo">A null-based index such that a[i,j] is zero for i &gt; j and 1 &lt; j &lt;= <paramref name="ilo"/> or <paramref name="ihi"/> &lt; i &lt;= n.</param>
            /// <param name="ihi">A null-based index such that a[i,j] is zero for i &gt; j and 1 &lt; j &lt;= <paramref name="ilo"/> or <paramref name="ihi"/> &lt; i &lt;= n.</param>
            /// <param name="scale">Contains details of the permutations and scaling factors; at least <paramref name="n"/> elements (output).</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, at least <paramref name="n"/>.</param>
            void dgebal(LAPACK.MatrixBalancesType job, int n, double[] a, out int ilo, out int ihi, double[] scale, int lda);

            /// <summary>Transforms eigenvectors of a balanced matrix to those of the original nonsymmetric matrix. Is intended to be used after a matrix has been balanced by a call to <c>dgebal</c>.
            /// </summary>
            /// <param name="job">The job, i.e. the matrix balance type.</param>
            /// <param name="side">A value indicating whether to transform left eigenvectors, right eigenvectors respectively.</param>
            /// <param name="n">The number of rows of the matrix of eigenvectors.</param>
            /// <param name="ilo">The returned value of <c>dgebal</c>.</param>
            /// <param name="ihi">The returned value of <c>dgebal</c>.</param>
            /// <param name="scale">The returned value of <c>dgebal</c>.</param>
            /// <param name="m">The number of columns of the matrix of eigenvectors.</param>
            /// <param name="v">The matrix of left or right eigenvectors to be transformed of dimension (<paramref name="ldv"/>; <paramref name="m"/>).</param>
            /// <param name="ldv">The leading dimension of <paramref name="v"/>, at least max( 1, <paramref name="n"/>).</param>
            void dgebak(LAPACK.MatrixBalancesType job, BLAS.Side side, int n, int ilo, int ihi, double[] scale, int m, double[] v, int ldv);

            /// <summary>Computes all eigenvalues and (optionally) the Schur factorization of a matrix reduced to Hessenberg form, i.e. A = Q * H * Q' where Q is orthogonal and H = Z * T * Z'.
            /// </summary>
            /// <param name="job">A value indicating whether eigenvalues or Schur form is required.</param>
            /// <param name="operation">A value indicating </param>
            /// <param name="n">The order of the Hessenberg matrix H.</param>
            /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="h">On input the Hessenberg matrix H of dimension (<paramref name="ldh"/>, <paramref name="n"/>), overwritten by the upper triangular matrix T from the Schur decomposition if <paramref name="job"/> indicates
            /// to calculate the Schur decomposition, otherwise the content is unspecified on exit.</param>
            /// <param name="wr">The real part of the eigenvalues, i.e. an array with at least <paramref name="n"/> elements (output).</param>
            /// <param name="wi">The imaginary part of the eigenvalues, i.e. an array with at least <paramref name="n"/> elements (output).</param>
            /// <param name="z">Contains the matrix Q reduced to Hessenberg form of dimension (<paramref name="ldz"/>, k), where the minimal length of k is depending on <paramref name="operation"/>; perhaps this parameter will not referenced.</param>
            /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength"/>.</param>
            /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace"/>, at least max( 1, <paramref name="n"/> ).</param>
            /// <param name="ldh">The leading dimension of <paramref name="h"/>, at least max( 1, <paramref name="n"/>).</param>
            /// <param name="ldz">The leading dimension of <paramref name="z"/>, at least max( 1, <paramref name="n"/>) or <c>1</c> depending on <paramref name="operation"/>.</param>
            void dhseqr(JobXhseqr job, XhseqrOperation operation, int n, int ilo, int ihi, double[] h, double[] wr, double[] wi, double[] z, double[] workspace, int workspaceLength, int ldh, int ldz);

            /// <summary>Gets the ideal length of the workspace array required for optimum performance.
            /// </summary>
            /// <param name="job">A value indicating whether eigenvalues or Schur form is required.</param>
            /// <param name="operation">A value indicating </param>
            /// <param name="n">The order of the Hessenberg matrix H.</param>
            /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="h">On input the Hessenberg matrix H of dimension (<paramref name="ldh"/>, <paramref name="n"/>), overwritten by the upper triangular matrix T from the Schur decomposition if <paramref name="job"/> indicates
            /// to calculate the Schur decomposition, otherwise the content is unspecified on exit.</param>
            /// <param name="wr">The real part of the eigenvalues, i.e. an array with at least <paramref name="n"/> elements (output).</param>
            /// <param name="wi">The imaginary part of the eigenvalues, i.e. an array with at least <paramref name="n"/> elements (output).</param>
            /// <param name="z">Contains the matrix Q reduced to Hessenberg form of dimension (<paramref name="ldz"/>, k), where the minimal length of k is depending on <paramref name="operation"/>; perhaps this parameter will not referenced.</param>
            /// <param name="ldh">The leading dimension of <paramref name="h"/>, at least max( 1, <paramref name="n"/>).</param>
            /// <param name="ldz">The leading dimension of <paramref name="z"/>, at least max( 1, <paramref name="n"/>) or <c>1</c> depending on <paramref name="operation"/>.</param>
            /// <returns>The ideal length of the working array required for optimum performance.</returns>
            int dhseqr_workspaceQuery(JobXhseqr job, XhseqrOperation operation, int n, int ilo, int ihi, double[] h, double[] wr, double[] wi, double[] z, int ldh, int ldz);

            /// <summary>Computes selected (let/right) eigenvectors of an upper Hessenberg matrix  that correspond to specified eigenvalues.
            /// </summary>
            /// <param name="job">A value indicating whether to calculate left/right eigenvectors or both.</param>
            /// <param name="eigsrc">A value indicating whether the eigenvalues of H were found using <c>dhseqr</c> and the routine performs inverse iteration on just one diagonal block; otherwise the routine performs inverse iteration using the whole matrix.</param>
            /// <param name="initv">A value indicating whether initial estimates for the selected eigenvectors are supplied in <paramref name="vl"/> and/or <paramref name="vr"/>.</param>
            /// <param name="select">Specified which eigenvectors are to be computed; of dimension at least max( 1, <paramref name="n"/>).</param>
            /// <param name="n">The order of matrix H.</param>
            /// <param name="h">On input the Hessenberg matrix H of dimension (<paramref name="ldh"/>, <paramref name="n"/>)</param>
            /// <param name="ldh">The leading dimension of <paramref name="h"/>, at least max( 1, <paramref name="n"/>).</param>
            /// <param name="wr">Contains the real parts of the eigenvalues of the matrix H, if <paramref name="eigsrc"/> indicates to use <c>dhseqr</c> this array must be exactly as returned by <c>dhseqr</c>; of dimension at least max( 1, <paramref name="n"/>). May be modified after function call.</param>
            /// <param name="wi">Contains the imaginary parts of the eigenvalues of the matrix H, if <paramref name="eigsrc"/> indicates to use <c>dhseqr</c> this array must be exactly as returned by <c>dhseqr</c>; of dimension at least max( 1, <paramref name="n"/>).</param>
            /// <param name="vl">Depending on <paramref name="initv"/> this parameter must contain starting vectors for inverse iteration for left eigenvectors or must not be set; of dimension (<paramref name="ldvl"/>, k) with k = <paramref name="mm"/> if to calculate left or left and right eigenvectors; otherwise k = 1.</param>
            /// <param name="ldvl">The leading dimension of <paramref name="vl"/>, at least 1 if left eigenvalues are not calculated otherwise at least max( 1, <paramref name="n"/>).</param>
            /// <param name="vr">Depending on <paramref name="initv"/> this parameter must contain starting vectors for inverse iteration for right eigenvectors or must not be set; of dimension (<paramref name="ldvr"/>, k) with k = <paramref name="mm"/> if to calculate left or left and right eigenvectors; otherwise k = 1.</param>
            /// <param name="ldvr">The leading dimension of <paramref name="vr"/>, at least 1 if left eigenvalues are not calculated otherwise at least max( 1, <paramref name="n"/>).</param>
            /// <param name="mm">The number of columns in <paramref name="vl"/> and/or <paramref name="vr"/>. Must be at least <paramref name="m"/>.</param>
            /// <param name="m">The number of columns in <paramref name="vl"/> and/or <paramref name="vr"/> required to store the selected eigenvectors (output).</param>
            /// <param name="ifaill">An array of dimension at least <paramref name="mm"/> indicating for which column of <paramref name="vl"/> the calculation converged (output).</param>
            /// <param name="ifailr">An array of dimension at least <paramref name="mm"/> indicating for which column of <paramref name="vr"/> the calculation converged (output).</param>
            /// <param name="workspace">A workspace array with dimension at least <paramref name="n"/> * (<paramref name="n"/> + 2).</param>
            void dhsein(char job, char eigsrc, char initv, bool[] select, int n, double[] h, int ldh, double[] wr, double[] wi, double[] vl, int ldvl, double[] vr, int ldvr, int mm, out int m, double[] workspace, int[] ifaill, int[] ifailr);

            /// <summary>Computes selected eigenvectors of an upper (quasi-) triangular matrix computed by <c>dhseqr</c>.
            /// </summary>
            void dtrevc(char side, char howmny, bool select, int n, double[] t, int ldt, double[] vl, int ldvl, double[] vr, int ldvr, int mm, int m, double[] work);

            /// <summary>Estimates condition numbers for specified eigenvalues and right eigenvectors of an upper (quasi-) triangular matrix.
            /// </summary>
            void dtrsna(char job, char howmny, bool select, int n, double[] t, int ldt, double[] vl, int ldvl, double[] vr, int ldvr, double[] s, double[] sep, int mm, int m, double[] work, int ldwork, int iwork);

            /// <summary>Reorders the Schur factorization of a general matrix.
            /// </summary>
            void dtrexc(char compq, int n, double[] t, int ldt, double[] q, int ldq, int ifst, int ilst, double[] work);

            /// <summary>Reorders the Schur factorization of a matrix and (optionally) computes the reciprocal condition numbers and invariant subspace for the selected cluster of eigenvalues.
            /// </summary>
            void dtrsen(char job, char compq, bool select, int n, double[] t, int ldt, double[] q, int ldq, double[] wr, double[] wi, int m, out double s, out double sep, double[] work, int lwork, int[] iwork, int liwork);

            /// <summary>Solves Sylvester equation for real quasi-triangular or complex triangular matrices.
            /// </summary>
            void dtrsyl(char trana, char tranb, int isgn, int m, int n, double[] a, int lda, double[] b, int ldb, double[] c, int ldc, out double scale);
            #endregion

            #region (double precision) complex methods

            /// <summary>Reduces a general matrix to upper Hessenberg form H by an orthogonal or unitary similarity transformation A = Q * H * Q', where H has real subdiagonal elements.
            /// </summary>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the upper Hessenberg matrix H and details of the matrix Q.</param>
            /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q (output).</param>
            /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength"/>.</param>
            /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace"/>, at least <paramref name="n"/>.</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, at least <paramref name="n"/>.</param>
            void zgehrd(int n, int ilo, int ihi, Complex[] a, Complex[] tau, Complex[] workspace, int workspaceLength, int lda);

            /// <summary>Gets the ideal length of the workspace array required for optimum performance.
            /// </summary>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="ilo">If the input matrix is an output by <c>?gebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>?gebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the upper Hessenberg matrix H and details of the matrix Q.</param>
            /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q (output).</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, at least <paramref name="n"/>.</param>
            /// <returns>The ideal length of the working array required for optimum performance.</returns>
            int zgehrd_workspaceQuery(int n, int ilo, int ihi, Complex[] a, Complex[] tau, int lda);

            /// <summary>Balances a general matrix to improve the accuracy of computed eigenvalues and eigenvectors.
            /// </summary>
            /// <param name="job">The job, i.e. the matrix balance type.</param>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the balanced matrix.</param>
            /// <param name="ilo">A null-based index such that a[i,j] is zero for i &gt; j and 1 &lt; j &lt;= <paramref name="ilo"/> or <paramref name="ihi"/> &lt; i &lt;= n.</param>
            /// <param name="ihi">A null-based index such that a[i,j] is zero for i &gt; j and 1 &lt; j &lt;= <paramref name="ilo"/> or <paramref name="ihi"/> &lt; i &lt;= n.</param>
            /// <param name="scale">Contains details of the permutations and scaling factors; at least <paramref name="n"/> elements (output).</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, at least <paramref name="n"/>.</param>
            void zgebal(LAPACK.MatrixBalancesType job, int n, Complex[] a, out int ilo, out int ihi, double[] scale, int lda);

            /// <summary>Transforms eigenvectors of a balanced matrix to those of the original nonsymmetric matrix. Is intended to be used after a matrix has been balanced by a call to <c>dgebal</c>.
            /// </summary>
            /// <param name="job">The job, i.e. the matrix balance type.</param>
            /// <param name="side">A value indicating whether to transform left eigenvectors, right eigenvectors respectively.</param>
            /// <param name="n">The number of rows of the matrix of eigenvectors.</param>
            /// <param name="ilo">The returned value of <c>dgebal</c>.</param>
            /// <param name="ihi">The returned value of <c>dgebal</c>.</param>
            /// <param name="scale">The returned value of <c>dgebal</c>.</param>
            /// <param name="m">The number of columns of the matrix of eigenvectors.</param>
            /// <param name="v">The matrix of left or right eigenvectors to be transformed of dimension (<paramref name="ldv"/>; <paramref name="m"/>).</param>
            /// <param name="ldv">The leading dimension of <paramref name="v"/>, at least max( 1, <paramref name="n"/>).</param>
            void zgebak(LAPACK.MatrixBalancesType job, BLAS.Side side, int n, int ilo, int ihi, Complex[] scale, int m, Complex[] v, int ldv);

            /// <summary>Computes all eigenvalues and (optionally) the Schur factorization of a matrix reduced to Hessenberg form, i.e. A = Q * H * Q' where Q is orthogonal and H = Z * T * Z'.
            /// </summary>
            /// <param name="job">A value indicating whether eigenvalues or Schur form is required.</param>
            /// <param name="operation">A value indicating </param>
            /// <param name="n">The order of the Hessenberg matrix H.</param>
            /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="h">On input the Hessenberg matrix H of dimension (<paramref name="ldh"/>, <paramref name="n"/>), overwritten by the upper triangular matrix T from the Schur decomposition if <paramref name="job"/> indicates
            /// to calculate the Schur decomposition, otherwise the content is unspecified on exit.</param>
            /// <param name="w">The eigenvalues, i.e. an array with at least <paramref name="n"/> elements (output). The eigenvalues are stored in the same order as on the diagonal of the Schur form (if computed).</param>
            /// <param name="z">Contains the matrix Q reduced to Hessenberg form of dimension (<paramref name="ldz"/>, k), where the minimal length of k is depending on <paramref name="operation"/>; perhaps this parameter will not referenced.</param>
            /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength"/>.</param>
            /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace"/>, at least max( 1, <paramref name="n"/> ).</param>
            /// <param name="ldh">The leading dimension of <paramref name="h"/>, at least max( 1, <paramref name="n"/>).</param>
            /// <param name="ldz">The leading dimension of <paramref name="z"/>, at least max( 1, <paramref name="n"/>) or <c>1</c> depending on <paramref name="operation"/>.</param>
            void zhseqr(JobXhseqr job, XhseqrOperation operation, int n, int ilo, int ihi, Complex[] h, Complex[] w, Complex[] z, Complex[] workspace, int workspaceLength, int ldh, int ldz);

            /// <summary>Gets the ideal length of the workspace array required for optimum performance.
            /// </summary>
            /// <param name="job">A value indicating whether eigenvalues or Schur form is required.</param>
            /// <param name="operation">A value indicating </param>
            /// <param name="n">The order of the Hessenberg matrix H.</param>
            /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
            /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n"/>.</param>
            /// <param name="h">On input the Hessenberg matrix H of dimension (<paramref name="ldh"/>, <paramref name="n"/>), overwritten by the upper triangular matrix T from the Schur decomposition if <paramref name="job"/> indicates
            /// to calculate the Schur decomposition, otherwise the content is unspecified on exit.</param>
            /// <param name="w">The eigenvalues, i.e. an array with at least <paramref name="n"/> elements (output). The eigenvalues are stored in the same order as on the diagonal of the Schur form (if computed).</param>
            /// <param name="z">Contains the matrix Q reduced to Hessenberg form of dimension (<paramref name="ldz"/>, k), where the minimal length of k is depending on <paramref name="operation"/>; perhaps this parameter will not referenced.</param>
            /// <param name="ldh">The leading dimension of <paramref name="h"/>, at least max( 1, <paramref name="n"/>).</param>
            /// <param name="ldz">The leading dimension of <paramref name="z"/>, at least max( 1, <paramref name="n"/>) or <c>1</c> depending on <paramref name="operation"/>.</param>
            /// <returns>The ideal length of the working array required for optimum performance.</returns>
            int zhseqr_workspaceQuery(JobXhseqr job, XhseqrOperation operation, int n, int ilo, int ihi, Complex[] h, Complex[] w, Complex[] z, int ldh, int ldz);
            #endregion
        }
    }
}