﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    public partial class LapackLinearEquations
    {
        public interface IDriverRoutines
        {
            /// <summary>Computes the solution to the system of linear equations with a dense matrix A, i.e. Ax=b.
            /// </summary>
            /// <param name="n">The number of rows/columns of 'A'.</param>
            /// <param name="a">The dense matrix 'A', matrix entries are given columnwise.
            /// (Output: Overwritten by the LU decomposition. The unit diagonal elements of L 
            /// are not stored.)</param>
            /// <param name="b">The vector b, whose single unique column is the right-hand side 
            /// for the systems of equations (output: Overwritten by the solution vector x).</param>
            /// <param name="ipivot">The pivot indices, i.e. row i was interchanged with row ipivot[i], at least <paramref name="n"/> elements (output).</param>
            void dgesv(int n, double[] a, double[] b, int[] ipivot);

            /// <summary>Computes the solution to the system of linear equations with a general band matrix A, i.e. Ax=b.
            /// </summary>
            /// <param name="n">The number of rows/columns of 'A'.</param>
            /// <param name="kl">The number of sub-diagonals of the matrix A.</param>
            /// <param name="ku">The number of super-diagonals of the matrix A.</param>
            /// <param name="a">The general band matrix 'A', matrix entries are given columnwise in band matrix storage.
            /// (Output: Overwritten by the LU decomposition. U is stored as an upper triangular band matrix
            /// with <paramref name="kl"/> + <paramref name="ku"/> superdiagonals stored in the first 
            /// 1 + <paramref name="kl"/> + <paramref name="ku"/> rows of <paramref name="a"/>. 
            /// The multipliers used to form L are stored in the next <paramref name="kl"/> rows.)</param>
            /// <param name="b">The vector b, whose single unique column is the right-hand side 
            /// for the systems of equations (output: Overwritten by the solution vector x).</param>
            /// <param name="ipivot">The pivot indices, i.e. row i was interchanged with row ipivot[i], at least <paramref name="n"/> elements (output).</param>
            void dgbsv(int n, int kl, int ku, double[] a, double[] b, int[] ipivot);

            /// <summary>Computes the solution to the system 'Ax=b' of linear equations with a symmetric or Hermitian
            /// positive-definite n-x-n matrix 'A' and multiple right-hand sides; the Cholesky decomposition will be returned as well.
            /// </summary>
            /// <param name="outputMatrixType">Type of the output matrix.</param>
            /// <param name="n">The order of the matrix 'A'.</param>
            /// <param name="nrhs">The number of right-hand sides, that is, the number of columns in 'b'.</param>
            /// <param name="a">The columnwise given matrix 'A'; will be overwritten with the Cholesky decomposition 'A=L*L^t' or 'A=U^t*U'.</param>
            /// <param name="b">Contains the matrix B (columnwise) whose columns are the right-hand sides for the systems of equations (will be overwritten, i.e. output).</param>
            void dposv(BLAS.TriangularMatrixType outputMatrixType, int n, int nrhs, double[] a, double[] b);

        }
    }
}