﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    public partial class LapackLinearEquations
    {
        public enum MatrixNormType
        {
            One,
            Infinity
        }

        /// <summary>Provides methods for the estimation of conditional numbers. For more informations see the documentation of the Linear Algebra PACKage http://www.netlib.org/lapack/index.html.
        /// </summary>
        public interface IMatrixConditionalNumbers
        {
            /// <summary>Estimates the reciprocal of the conditional number of a general matrix in the 1-norm or the infinity-norm.
            /// </summary>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="a">The LU-factored matrix A as returned by <c>?getrf</c>, at least <paramref name="n"/> elements.</param>
            /// <param name="normOfOriginalMatrix">The 1- or infinity-norm of original matrix.</param>
            /// <param name="matrixNormType">The type of the matrix norm.</param>
            /// <param name="work">A workspace array of length at least <paramref name="workLength"/>.</param>
            /// <param name="workLength">The length of the workspace array <paramref name="work"/>, at least 4 * <paramref name="n"/>.</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, i.e. at least <paramref name="n"/>.</param>
            /// <returns>The reciprocal of the conditional number , i.e. \norm{A} * \norm{A^{-1}}, where \norm{.} denotes the 1-norm or infinity-norm.</returns>
            double dgecon(int n, double[] a, double normOfOriginalMatrix, MatrixNormType matrixNormType, double[] work, int workLength, int lda);

            /// <summary>Estimates the reciprocal of the conditional number of a band matrix in the 1-norm or the infinity-norm.
            /// </summary>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="kl">The number of subdiagonals within the band of the input matrix.</param>
            /// <param name="ku">The number of superdiagonals within the band of the input matrix.</param>
            /// <param name="a">The LU-factored matrix A as returned by <c>?gbtrf</c> of dimension (<paramref name="lda"/>; <paramref name="n"/>).</param>          
            /// <param name="normOfOriginalMatrix">The 1- or infinity-norm of original matrix.</param>
            /// <param name="matrixNormType">The type of the matrix norm.</param>
            /// <param name="work">A workspace array of length at least <paramref name="workLength"/>.</param>
            /// <param name="workLength">The length of the workspace array <paramref name="work"/>, at least 3 * <paramref name="n"/>.</param>
            /// <param name="lda">The leading dimension of <paramref name="a"/>, i.e. at least 2 * <paramref name="kl"/> + <paramref name="ku"/> + 1.</param>
            /// <returns>The reciprocal of the conditional number , i.e. \norm{A} * \norm{A^{-1}}, where \norm{.} denotes the 1-norm or infinity-norm.</returns>
            double dgbcon(int n, int kl, int ku, double[] a, double normOfOriginalMatrix, MatrixNormType matrixNormType, double[] work, int workLength, int lda);

            /// <summary>Estimates the reciprocal of the conditional number of a tridiagonal matrix in the 1-norm or the infinity-norm using the factorization computed by <c>?gttrf</c>.
            /// </summary>
            /// <param name="n">The order of the matrix.</param>
            /// <param name="dl">The (<paramref name="n"/> - 1) subdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) multipliers that defines the matrix L from the LU factorization.</param>
            /// <param name="d">The <paramref name="n"/> diagonal elements of the input matrix; overwritten by the <paramref name="n"/> diagonal elements of the upper triangular matrix U from the LU factorization.</param>
            /// <param name="du">The (<paramref name="n"/> - 1) superdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) elements of the first superdiagonal of U.</param>
            /// <param name="du2">The (<paramref name="n"/> - 2) elements of the second superdiagonal of U (output).</param>
            /// <param name="iPivot">The <paramref name="n"/> pivot indices, i.e. row j was interchanged with row <paramref name="iPivot"/>[j].
            /// <paramref name="iPivot"/>[j] is always j or j+1; <paramref name="iPivot"/>[j] = j indicates a row interchange was not required (output).</param>
            /// <param name="normOfOriginalMatrix">The 1- or infinity-norm of original matrix.</param>
            /// <param name="matrixNormType">The type of the matrix norm.</param>
            /// <param name="work">A workspace array of length at least <paramref name="workLength"/>.</param>
            /// <param name="workLength">The length of the workspace array <paramref name="work"/>, at least 3 * <paramref name="n"/>.</param>
            /// <returns>The reciprocal of the conditional number , i.e. \norm{A} * \norm{A^{-1}}, where \norm{.} denotes the 1-norm or infinity-norm.</returns>
            double dgtcon(int n, double[] dl, double[] d, double[] du, double[] du2, int[] iPivot, double normOfOriginalMatrix, MatrixNormType matrixNormType, double[] work, int workLength);


        }
    }
}