﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    public partial class LapackLinearEquations
    {
        public interface ISolver
        {
            /// <summary>Solves a system of linear equations with a tridiagonal matrix using the LU factorization, i.e. op(A)*x=b.
            /// </summary>
            /// <param name="transpose">A value indicating whether 'op(A)=A' or 'op(A)=A^t'.</param>
            /// <param name="n">The order of matrix 'A'.</param>
            /// <param name="nrhs">The number of right-hand sides, that is, the number of columns in 'b'.</param>
            /// <param name="dl">Contains the (n - 1) multipliers that define the matrix L from the LU factorization of 'A'.</param>
            /// <param name="d">Contains the n diagonal elements of the upper triangular matrix U from the LU factorization of 'A'.</param>
            /// <param name="du">Contains the (n - 1) elements of the first superdiagonal of U.</param>
            /// <param name="du2">Contains the (n - 2) elements of the second superdiagonal of U.</param>
            /// <param name="ipiv">Contains the n pivot indices.</param>
            /// <param name="b">Contains the matrix B (columnwise) whose columns are the right-hand sides for the systems of equations (will be overwritten, i.e. output).</param>
            void dgttrs(BLAS.MatrixTransposeState transpose, int n, int nrhs, double[] dl, double[] d, double[] du, double[] du2, int[] ipiv, double[] b);

            /// <summary>Solves a system of linear equations with an LU-factored square matrix, i.e. op(A)x=b, where A=L*U.
            /// </summary>
            /// <param name="transpose">A value indicating whether 'op(A)=A' or 'op(A)=A^t'.</param>
            /// <param name="n">The number of rows/columns of 'A'.</param>
            /// <param name="nrhs">The number of right-hand sides, that is, the number of columns in 'b'.</param>
            /// <param name="a">The dense matrix 'A', matrix given in its LU-decomposition, i.e. output of <c>dgesv</c> etc. (this parameter will not be changed).</param>
            /// <param name="iPivot">The pivot indices, i.e. row i was interchanged with row iPivot[i]; the pivot indices are output of <c>dgesv</c> etc. (this parameter will not be changed).</param>
            /// <param name="b">Contains the matrix B (columnwise) whose columns are the right-hand sides for the systems of equations (will be overwritten, i.e. output).</param>
            void dgetrs(BLAS.MatrixTransposeState transpose, int n, int nrhs, double[] a, int[] iPivot, double[] b);

            /// <summary>Solves a system of linear equations with a Cholesky-factored symmetric (Hermitian) positive-definite matrix.
            /// </summary>
            /// <param name="inputMatrixType">Type of the input matrix.</param>
            /// <param name="n">The order of the matrix 'A'.</param>
            /// <param name="nrhs">The number of right-hand sides, that is, the number of columns in 'b'.</param>
            /// <param name="a">The columnwise given matrix 'A' in its Cholesky decomposition 'A=L*L^t' or 'A=U^t*U'.</param>
            /// <param name="b">Contains the matrix B (columnwise) whose columns are the right-hand sides for the systems of equations (will be overwritten, i.e. output).</param>
            void dpotrs(BLAS.TriangularMatrixType inputMatrixType, int n, int nrhs, double[] a, double[] b);

        }
    }
}