﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Linq;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Utilities;
using Dodoni.BasicComponents.LogFileFactory;
using Dodoni.MathLibrary.Basics.LowLevel.BuildIn;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    /// <summary>Represents the configuration for the dll-Wrapper of the low level mathematical functions, i.e. the kind of interfaces for BLAS, FFT etc.
    /// </summary>
    /// <remarks>A restart is necessary if some config file changed.</remarks>
    public static class LowLevelMathConfiguration
    {
        #region private const members

        /// <summary>The key in the XML file for the BLAS device.
        /// </summary>
        private const string sm_BLAS_ConfigurationKey = "BLAS-Device";

        /// <summary>The key in the XML file for the Lapack device.
        /// </summary>
        private const string sm_Lapack_ConfigurationKey = "Lapack-Device";

        /// <summary>The key in the XML file for the FFT device.
        /// </summary>
        private const string sm_FFT_ConfigurationKey = "FFT-Device";

        /// <summary>The key in the XML file for the Vector unit device.
        /// </summary>
        private const string sm_VectorUnit_ConfigurationKey = "VectorUnit-Device";
        #endregion

        #region private (static) members

        /// <summary>The settings in the configuration file concerning the Low-Level Math operations. 
        /// </summary>
        private static ConfigurationFile.PropertyCollection sm_ConfigFileProperties;

        /// <summary>The catalog of external BLAS Libraries (via Managed Extensibility Framework).
        /// </summary>
        private static BLAS.Catalog sm_BlasCatalog;

        /// <summary>The catalog of external Fast-Fourier Transformation Libraries (via Managed Extensibility Framework).
        /// </summary>
        private static FFT.Catalog sm_FFTCatalog;

        /// <summary>The catalog of external Vector Unit Libraries (via Managed Extensibility Framework).
        /// </summary>
        private static VectorUnit.Catalog sm_VectorUnitCatalog;
        #endregion

        #region internal (static) members

        /// <summary>The Fast-Fourier Transformation Library.
        /// </summary>
        public static readonly FFT.ILibrary FFT;

        public static readonly BLAS.ILibrary BLAS;

        public static readonly VectorUnit.ILibrary VectorUnit;
        #endregion

        #region static constructor

        /// <summary>Initializes the <see cref="LowLevelMathConfiguration" /> class.
        /// </summary>
        static LowLevelMathConfiguration()
        {
            sm_ConfigFileProperties = ConfigurationFile.Dodoni.GetPropertyCollection("BasicLowLevelMath");

            sm_BlasCatalog = new BLAS.Catalog();
            sm_FFTCatalog = new FFT.Catalog();
            sm_VectorUnitCatalog = new VectorUnit.Catalog();

            if (sm_ConfigFileProperties.ContainsSettings == false)
            {
                FFT = new BuildInNaiveFourierTransformationLibrary();
                BLAS = new BuildInBLAS();
                VectorUnit = new BuildInVectorUnit();

                WriteDefaultConfigFile();
            }
            else  // configuration file exists
            {
                try
                {
                    BLAS = GetBlasDevice();
                    FFT = GetFourierTransformationDevice();
                    VectorUnit = GetVectorUnitDevice();
                }
                catch (Exception e)
                {
                    throw new ConfigurationFileErrorException("Corrupt configuration file detected. Delete config file and start again. A new configuration file will be create with some standard values.", e);
                }
            }
            VectorUnit.Initialize();
        }
        #endregion

        #region public (static) properties

        /// <summary>Gets the collection of available BLAS libraries.
        /// </summary>
        /// <value>The BLAS libraries.</value>
        public static IEnumerable<BLAS.ILibrary> BlasLibraries
        {
            get
            {
                yield return new BuildInBLAS();

                if (sm_BlasCatalog.Libraries != null)
                {
                    foreach (var library in sm_BlasCatalog.Libraries)
                    {
                        yield return library;
                    }
                }
            }
        }

        /// <summary>Gets the collection of available Fast-Fourier Transformation Libraries.
        /// </summary>
        /// <value>The Fast-Fourier Transformation Libraries</value>
        public static IEnumerable<FFT.ILibrary> FastFourierTransformationLibraries
        {
            get
            {
                yield return new BuildInNaiveFourierTransformationLibrary();

                if (sm_FFTCatalog.Libraries != null)
                {
                    foreach (var lib in sm_FFTCatalog.Libraries)
                    {
                        yield return lib;
                    }
                }
            }
        }


        /// <summary>Gets the collection of available Vector Unit Libraries.
        /// </summary>
        /// <value>The Vector Unit libraries.</value>
        public static IEnumerable<VectorUnit.ILibrary> VectorUnitLibraries
        {
            get
            {
                yield return new BuildInVectorUnit();

                if (sm_VectorUnitCatalog.Libraries != null)
                {
                    foreach (var library in sm_VectorUnitCatalog.Libraries)
                    {
                        yield return library;
                    }
                }
            }
        }
        #endregion

        #region public static methods

        /// <summary>Writes configuration parameters into the config file.
        /// </summary>
        /// <exception cref="ConfigurationFileErrorException">Thrown, if the configuration is invalid and it is not allowed to write the configuration into some file.</exception>
        public static void WriteConfigFile()
        {
            ConfigurationFile.Dodoni.Save();
        }

        /// <summary>Stores the specified BLAS device in the configuration file.
        /// </summary>
        /// <param name="device">The BLAS device.</param>
        /// <remarks>The config file will not change. Use <see cref="WriteConfigFile()"/> to write the changes into the file.</remarks>
        public static void Setup(BLAS.ILibrary device)
        {
            sm_ConfigFileProperties.SetValue(sm_BLAS_ConfigurationKey, device.Name.IDString);
        }

        /// <summary>Stores the specified FFT device in the configuration file.
        /// </summary>
        /// <param name="device">The FFT device.</param>
        /// <remarks>The config file will not change. Use <see cref="WriteConfigFile()"/> to write the changes into the file.</remarks>
        public static void Setup(FFT.ILibrary device)
        {
            sm_ConfigFileProperties.SetValue(sm_FFT_ConfigurationKey, device.Name.IDString);
        }

        /// <summary>Stores the specified Vector unit device in the configuration file.
        /// </summary>
        /// <param name="device">The device of the Vector unit.</param>
        /// <remarks>The config file will not change. Use <see cref="WriteConfigFile()"/> to write the changes into the file.</remarks>
        public static void Setup(VectorUnit.ILibrary device)
        {
            sm_ConfigFileProperties.SetValue(sm_VectorUnit_ConfigurationKey, device.Name.IDString);
        }
        #endregion

        #region private static methods

        /// <summary>Sets default values and stores these in the config file.
        /// </summary>
        private static void WriteDefaultConfigFile()
        {
            sm_ConfigFileProperties.SetValue(sm_BLAS_ConfigurationKey, BLAS.Name.String);
            sm_ConfigFileProperties.SetValue(sm_FFT_ConfigurationKey, FFT.Name.String);
            sm_ConfigFileProperties.SetValue(sm_VectorUnit_ConfigurationKey, VectorUnit.Name.String);

            ConfigurationFile.Dodoni.Save();
        }

        /// <summary>Gets the BLAS device.
        /// </summary>
        /// <returns>The BLAS library with respect to the configuration file.</returns>
        private static BLAS.ILibrary GetBlasDevice()
        {
            string deviceName;
            if (sm_ConfigFileProperties.TryGetValue(sm_BLAS_ConfigurationKey, out deviceName) == false)
            {
                LoggingFactory.AddGlobalLogFileMessage(LoggingFactory.Warnings.Generic, "Configuration file error. BLAS configuration set to build-in implementation");
                return new BuildInBLAS();
            }

            var deviceIdentifierName = new IdentifierString(deviceName);
            var devices = sm_BlasCatalog.Libraries.Where(library => library.Name.IDString == deviceIdentifierName.IDString);
            if (devices.Count() > 0)
            {
                return devices.First();
            }
            LoggingFactory.AddGlobalLogFileMessage(LoggingFactory.Warnings.Generic, "Configuration file error. BLAS configuration set to build-in implementation");
            return new BuildInBLAS();
        }

        /// <summary>Gets the FFT device.
        /// </summary>
        /// <returns>The FFT library with respect to the configuration file.</returns>
        private static FFT.ILibrary GetFourierTransformationDevice()
        {
            string deviceName;
            if (sm_ConfigFileProperties.TryGetValue(sm_FFT_ConfigurationKey, out deviceName) == false)
            {
                LoggingFactory.AddGlobalLogFileMessage(LoggingFactory.Warnings.Generic, "Configuration file error. FFT configuration set to build-in implementation");
                return new BuildInNaiveFourierTransformationLibrary();
            }

            var deviceIdentifierName = new IdentifierString(deviceName);
            var devices = sm_FFTCatalog.Libraries.Where(library => library.Name.IDString == deviceIdentifierName.IDString);
            if (devices.Count() > 0)
            {
                return devices.First();
            }
            LoggingFactory.AddGlobalLogFileMessage(LoggingFactory.Warnings.Generic, "Configuration file error. FFT configuration set to build-in implementation");
            return new BuildInNaiveFourierTransformationLibrary();
        }

        /// <summary>Gets the Vector Unit device.
        /// </summary>
        /// <returns>The Vector Unit library with respect to the configuration file.</returns>
        private static VectorUnit.ILibrary GetVectorUnitDevice()
        {
            string deviceName;
            if (sm_ConfigFileProperties.TryGetValue(sm_FFT_ConfigurationKey, out deviceName) == false)
            {
                LoggingFactory.AddGlobalLogFileMessage(LoggingFactory.Warnings.Generic, "Configuration file error. Vector unit configuration set to build-in implementation");
                return new BuildInVectorUnit();
            }

            var deviceIdentifierName = new IdentifierString(deviceName);
            var devices = sm_VectorUnitCatalog.Libraries.Where(library => library.Name.IDString == deviceIdentifierName.IDString);
            if (devices.Count() > 0)
            {
                return devices.First();
            }
            LoggingFactory.AddGlobalLogFileMessage(LoggingFactory.Warnings.Generic, "Configuration file error. Vector unit configuration set to build-in implementation");
            return new BuildInVectorUnit();
        }
        #endregion
    }
}