﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;
using System.Security;
using System.Runtime.InteropServices;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.Basics.LowLevel.Native
{
    /// <summary>Serves as native wrapper for the Lapack functions, see http://www.netlib.org/lapack/index.html.
    /// </summary>
    internal partial class LapackNativeWrapper : LapackEigenvalues.INonsymmetricEigenvalueProblems
    {
        #region private function import

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DGEHRD", CallingConvention = sm_CallingConvention), SuppressUnmanagedCodeSecurity]
        private static extern void _dgehrd(ref int n, ref int ilo, ref int ihi, IntPtr a, ref int lda, IntPtr tau, IntPtr work, ref int lWork, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DORGHR", CallingConvention = sm_CallingConvention), SuppressUnmanagedCodeSecurity]
        private static extern void _dorghr(ref int n, ref int ilo, ref int ihi, IntPtr a, ref int lda, IntPtr tau, IntPtr work, ref int lWork, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DORMHR", CallingConvention = sm_CallingConvention), SuppressUnmanagedCodeSecurity]
        private static extern void _dormhr(ref char side, ref char trans, ref int m, ref int n, ref int ilo, ref int ihi, IntPtr a, ref int lda, IntPtr tau, IntPtr c, ref int ldc, IntPtr work, ref int lWork, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DGEBAL", CallingConvention = sm_CallingConvention), SuppressUnmanagedCodeSecurity]
        private static extern void _dgebal(ref char job, ref int n, IntPtr a, ref int lda, out int ilo, out int ihi, IntPtr scale, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DGEBAK", CallingConvention = sm_CallingConvention), SuppressUnmanagedCodeSecurity]
        private static extern void _dgebak(ref char job, ref char side, ref int n, ref int ilo, ref int ihi, IntPtr scale, ref int m, IntPtr v, ref int ldv, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DHSEQR", CallingConvention = sm_CallingConvention), SuppressUnmanagedCodeSecurity]
        private static extern void _dhseqr(ref char job, ref char compz, ref int n, ref int ilo, ref int ihi, IntPtr h, ref int ldh, IntPtr wr, IntPtr wi, IntPtr z, ref int ldz, IntPtr work, ref int lwork, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZGEBAL", CallingConvention = sm_CallingConvention), SuppressUnmanagedCodeSecurity]
        private static extern void _zgebal(ref char job, ref int n, IntPtr a, ref int lda, ref int ilo, ref int ihi, IntPtr scale, ref int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZHSEQR", CallingConvention = sm_CallingConvention), SuppressUnmanagedCodeSecurity]
        private static extern void _zhseqr(ref char job, ref char compz, ref int n, ref int ilo, ref int ihi, IntPtr h, ref int ldh, IntPtr w, IntPtr z, ref int ldz, IntPtr work, ref int lwork, ref int info);
        #endregion

        #region INonsymmetricEigenvalueProblems Members

        #region double precision methods

        /// <summary>Reduces a general matrix to upper Hessenberg form H by an orthogonal or unitary similarity transformation A = Q * H * Q', where H has real subdiagonal elements.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
        /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n" />.</param>
        /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda" />; <paramref name="n" />); overwritten by the upper Hessenberg matrix H and details of the matrix Q.</param>
        /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q (output).</param>
        /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength" />.</param>
        /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace" />, at least <paramref name="n" />.</param>
        /// <param name="lda">The leading dimension of <paramref name="a" />, at least <paramref name="n" />.</param>
        public void dgehrd(int n, int ilo, int ihi, double[] a, double[] tau, double[] workspace, int workspaceLength, int lda)
        {
            int info;
            unsafe
            {
                fixed (double* aPtr = a, tauPtr = tau, workPtr = workspace)
                {
                    _dgehrd(ref n, ref ilo, ref ihi, (IntPtr)aPtr, ref lda, (IntPtr)tauPtr, (IntPtr)workPtr, ref workspaceLength, out info);
                }
                CheckForError(info, "DGEHRD");
            }
        }

        /// <summary>Gets the ideal length of the workspace array required for optimum performance.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="ilo">If the input matrix is an output by <c>?gebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
        /// <param name="ihi">If the input matrix is an output by <c>?gebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n" />.</param>
        /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda" />; <paramref name="n" />); overwritten by the upper Hessenberg matrix H and details of the matrix Q.</param>
        /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q (output).</param>
        /// <param name="lda">The leading dimension of <paramref name="a" />, at least <paramref name="n" />.</param>
        /// <returns>The ideal length of the working array required for optimum performance.</returns>
        public int dgehrd_workspaceQuery(int n, int ilo, int ihi, double[] a, double[] tau, int lda)
        {
            int info;
            int workspaceLength = -1;

            unsafe
            {
                double* work = stackalloc double[1];
                fixed (double* aPtr = a, tauPtr = tau)
                {
                    _dgehrd(ref n, ref ilo, ref ihi, (IntPtr)aPtr, ref lda, (IntPtr)tauPtr, (IntPtr)work, ref workspaceLength, out info);
                }
                CheckForError(info, "DGEHRD");
                return (int)work[0];
            }
        }

        /// <summary>Generates the real orthogonal matrix Q determined by <c>dgehrd</c>, i.e. explicitly generates the orthogonal matrix Q with A = Q * H * Q'.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
        /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n" />.</param>
        /// <param name="a">Contains on input details of the vectors which define the elementary reflectors, as returned by <c>dgehrd</c> of dimension (<paramref name="lda" />; <paramref name="n" />); overwritten by the n-by-n orthogonal matrix Q.</param>
        /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q as returned by <c>dgehrd</c>.</param>
        /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength" />.</param>
        /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace" />, at least max( 1, <paramref name="ihi" /> - <paramref name="ilo" />).</param>
        /// <param name="lda">The leading dimension of <paramref name="a" />, at least <paramref name="n" />.</param>
        public void dorghr(int n, int ilo, int ihi, double[] a, double[] tau, double[] workspace, int workspaceLength, int lda)
        {
            int info;
            unsafe
            {
                fixed (double* aPtr = a, tauPtr = tau, workPtr = workspace)
                {
                    _dorghr(ref n, ref ilo, ref ihi, (IntPtr)aPtr, ref lda, (IntPtr)tauPtr, (IntPtr)workPtr, ref workspaceLength, out info);
                }
                CheckForError(info, "DORGHR");
            }
        }

        /// <summary>Gets the ideal length of the workspace array required for optimum performance.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
        /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n" />.</param>
        /// <param name="a">Contains on input details of the vectors which define the elementary reflectors, as returned by <c>dgehrd</c> of dimension (<paramref name="lda" />; <paramref name="n" />); overwritten by the n-by-n orthogonal matrix Q.</param>
        /// <param name="tau">A array of dimension at least max(1, n-1) containing additional informations on the matrix Q as returned by <c>dgehrd</c>.</param>
        /// <param name="lda">The leading dimension of <paramref name="a" />, at least <paramref name="n" />.</param>
        /// <returns>The ideal length of the working array required for optimum performance.</returns>
        public int dorghr_workspaceQuery(int n, int ilo, int ihi, double[] a, double[] tau, int lda)
        {
            int info;
            int workspaceLength = -1;

            unsafe
            {
                double* work = stackalloc double[1];
                fixed (double* aPtr = a, tauPtr = tau)
                {
                    _dorghr(ref n, ref ilo, ref ihi, (IntPtr)aPtr, ref lda, (IntPtr)tauPtr, (IntPtr)work, ref workspaceLength, out info);
                }
                CheckForError(info, "DORGHR");
                return (int)work[0];
            }
        }

        /// <summary>Multiplies an arbitrary real matrix C by the real orthogonal matrix Q determined by <c>dgehrd</c>, i.e. \op(Q) * C or C * \op(Q), where \op(Q) = Q or \op(Q) = Q'.
        /// </summary>
        /// <param name="side">A value indicating whether to calculate \op(Q) * C [left] or C * \op(Q) [right].</param>
        /// <param name="transpose">A value indicating whether \op(Q) = Q or \op(Q) = Q'.</param>
        /// <param name="m">The number of rows in matrix C.</param>
        /// <param name="n">The number of columns in matrix C.</param>
        /// <param name="ilo">The same parameter value as supplied to <c>dgehrd</c>.</param>
        /// <param name="ihi">The same parameter value as supplied to <c>dgehrd</c>.</param>
        /// <param name="a">Contains details of the vectors which define the elementary reflectors, as returned by <c>dgehrd</c> of dimension (<paramref name="lda" />; k) where k must be at least max(1, <paramref name="m" />) if to calculate \op(Q) * C; otherwise max( 1, <paramref name="n" />).</param>
        /// <param name="tau">Contains further details of the elementary reflectors as returned by <c>dgehrd</c>.</param>
        /// <param name="c">The m-by-n matrix C supplied column-by-column of dimension (<paramref name="ldc" />, n).</param>
        /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength" />.</param>
        /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace" />, at least max( 1, <paramref name="n" /> ) if to calculate \op(Q) * C; otherwise max( 1, <paramref name="m" />).</param>
        /// <param name="lda">The leading dimension of <paramref name="a" />, must be at least max(1,<paramref name="m" />) if to calculate \op(Q) * C; max(1,<paramref name="n" />) otherwise.</param>
        /// <param name="ldc">The leading dimension of <paramref name="a" />, at least <paramref name="m" />.</param>
        public void dormhr(BLAS.Side side, BLAS.MatrixTransposeState transpose, int m, int n, int ilo, int ihi, double[] a, double[] tau, double[] c, double[] workspace, int workspaceLength, int lda, int ldc)
        {
            char cSide = (side == BLAS.Side.Left) ? 'L' : 'R';
            char transposeAsChar = transpose.AsBlasTransposeChar();

            int info;
            unsafe
            {
                fixed (double* aPtr = a, tauPtr = tau, cPtr = c, workPtr = workspace)
                {
                    _dormhr(ref cSide, ref transposeAsChar, ref m, ref n, ref ilo, ref ihi, (IntPtr)aPtr, ref lda, (IntPtr)tauPtr, (IntPtr)cPtr, ref ldc, (IntPtr)workPtr, ref workspaceLength, out info);
                }
                CheckForError(info, "DORMHR");
            }
        }

        /// <summary>Gets the ideal length of the workspace array required for optimum performance.
        /// </summary>
        /// <param name="side">A value indicating whether to calculate \op(Q) * C [left] or C * \op(Q) [right].</param>
        /// <param name="transpose">A value indicating whether \op(Q) = Q or \op(Q) = Q'.</param>
        /// <param name="m">The number of rows in matrix C.</param>
        /// <param name="n">The number of columns in matrix C.</param>
        /// <param name="ilo">The same parameter value as supplied to <c>dgehrd</c>.</param>
        /// <param name="ihi">The same parameter value as supplied to <c>dgehrd</c>.</param>
        /// <param name="a">Contains details of the vectors which define the elementary reflectors, as returned by <c>dgehrd</c> of dimension (<paramref name="lda" />; k) where k must be at least max(1, <paramref name="m" />) if to calculate \op(Q) * C; otherwise max( 1, <paramref name="n" />).</param>
        /// <param name="tau">Contains further details of the elementary reflectors as returned by <c>dgehrd</c>.</param>
        /// <param name="c">The m-by-n matrix C supplied column-by-column of dimension (<paramref name="ldc" />, n).</param>
        /// <param name="lda">The leading dimension of <paramref name="a" />, must be at least max(1,<paramref name="m" />) if to calculate \op(Q) * C; max(1,<paramref name="n" />) otherwise.</param>
        /// <param name="ldc">The leading dimension of <paramref name="a" />, at least <paramref name="m" />.</param>
        /// <returns>The ideal length of the working array required for optimum performance.</returns>
        public int dormhr_workspaceQuery(BLAS.Side side, BLAS.MatrixTransposeState transpose, int m, int n, int ilo, int ihi, double[] a, double[] tau, double[] c, int lda, int ldc)
        {
            char cSide = (side == BLAS.Side.Left) ? 'L' : 'R';
            char transposeAsChar = transpose.AsBlasTransposeChar();

            int info;
            int workspaceLength = -1;
            unsafe
            {
                double* work = stackalloc double[1];
                fixed (double* aPtr = a, tauPtr = tau, cPtr = c)
                {
                    _dormhr(ref cSide, ref transposeAsChar, ref m, ref n, ref ilo, ref ihi, (IntPtr)aPtr, ref lda, (IntPtr)tauPtr, (IntPtr)cPtr, ref ldc, (IntPtr)work, ref workspaceLength, out info);
                }
                return (int)work[0];
            }
        }

        /// <summary>Balances a general matrix to improve the accuracy of computed eigenvalues and eigenvectors.
        /// </summary>
        /// <param name="job">The job, i.e. the matrix balance type.</param>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda" />; <paramref name="n" />); overwritten by the balanced matrix.</param>
        /// <param name="ilo">A null-based index such that a[i,j] is zero for i &gt; j and 1 &lt; j &lt;= <paramref name="ilo" /> or <paramref name="ihi" /> &lt; i &lt;= n.</param>
        /// <param name="ihi">A null-based index such that a[i,j] is zero for i &gt; j and 1 &lt; j &lt;= <paramref name="ilo" /> or <paramref name="ihi" /> &lt; i &lt;= n.</param>
        /// <param name="scale">Contains details of the permutations and scaling factors; at least <paramref name="n" /> elements (output).</param>
        /// <param name="lda">The leading dimension of <paramref name="a" />, at least <paramref name="n" />.</param>
        public void dgebal(LAPACK.MatrixBalancesType job, int n, double[] a, out int ilo, out int ihi, double[] scale, int lda)
        {
            char laPackJob;
            switch (job)
            {
                case LAPACK.MatrixBalancesType.NeitherPermutedNorScaled:
                    laPackJob = 'N';
                    break;
                case LAPACK.MatrixBalancesType.Permuted:
                    laPackJob = 'P';
                    break;
                case LAPACK.MatrixBalancesType.PermutedAndScaled:
                    laPackJob = 'B';
                    break;
                case LAPACK.MatrixBalancesType.Scaled:
                    laPackJob = 'S';
                    break;
                default:
                    throw new NotImplementedException();
            }
            int info;
            unsafe
            {
                fixed (double* ptrA = a, ptrScale = scale)
                {
                    _dgebal(ref laPackJob, ref n, (IntPtr)ptrA, ref lda, out ilo, out ihi, (IntPtr)ptrScale, out info);
                }
            }
            if (info != 0)
            {
                throw new ArithmeticException("Return value " + info + " in dgebal.");
            }
        }

        /// <summary>Transforms eigenvectors of a balanced matrix to those of the original nonsymmetric matrix. Is intended to be used after a matrix has been balanced by a call to <c>dgebal</c>.
        /// </summary>
        /// <param name="job">The job, i.e. the matrix balance type.</param>
        /// <param name="side">A value indicating whether to transform left eigenvectors, right eigenvectors respectively.</param>
        /// <param name="n">The number of rows of the matrix of eigenvectors.</param>
        /// <param name="ilo">The returned value of <c>dgebal</c>.</param>
        /// <param name="ihi">The returned value of <c>dgebal</c>.</param>
        /// <param name="scale">The returned value of <c>dgebal</c>.</param>
        /// <param name="m">The number of columns of the matrix of eigenvectors.</param>
        /// <param name="v">The matrix of left or right eigenvectors to be transformed of dimension (<paramref name="ldv" />; <paramref name="m" />).</param>
        /// <param name="ldv">The leading dimension of <paramref name="v" />, at least max( 1, <paramref name="n" />).</param>
        public void dgebak(LAPACK.MatrixBalancesType job, BLAS.Side side, int n, int ilo, int ihi, double[] scale, int m, double[] v, int ldv)
        {
            int info;
            char jobAsChar;
            switch (job)
            {
                case LAPACK.MatrixBalancesType.NeitherPermutedNorScaled:
                    jobAsChar = 'N';
                    break;
                case LAPACK.MatrixBalancesType.Permuted:
                    jobAsChar = 'P';
                    break;
                case LAPACK.MatrixBalancesType.PermutedAndScaled:
                    jobAsChar = 'B';
                    break;
                case LAPACK.MatrixBalancesType.Scaled:
                    jobAsChar = 'S';
                    break;
                default:
                    throw new NotImplementedException();
            }

            char cSide = (side == BLAS.Side.Left) ? 'L' : 'R';
            unsafe
            {
                fixed (double* scalePtr = scale, vPtr = v)
                {
                    _dgebak(ref jobAsChar, ref cSide, ref n, ref ilo, ref ihi, (IntPtr)scalePtr, ref m, (IntPtr)vPtr, ref ldv, out info);
                }
            }
        }

        /// <summary>Computes all eigenvalues and (optionally) the Schur factorization of a matrix reduced to Hessenberg form, i.e. A = Q * H * Q' where Q is orthogonal and H = Z * T * Z'.
        /// </summary>
        /// <param name="job">A value indicating whether eigenvalues or Schur form is required.</param>
        /// <param name="operation">A value indicating</param>
        /// <param name="n">The order of the Hessenberg matrix H.</param>
        /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
        /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n" />.</param>
        /// <param name="h">On input the Hessenberg matrix H of dimension (<paramref name="ldh" />, <paramref name="n" />), overwritten by the upper triangular matrix T from the Schur decomposition if <paramref name="job" /> indicates
        /// to calculate the Schur decomposition, otherwise the content is unspecified on exit.</param>
        /// <param name="wr">The real part of the eigenvalues, i.e. an array with at least <paramref name="n" /> elements (output).</param>
        /// <param name="wi">The imaginary part of the eigenvalues, i.e. an array with at least <paramref name="n" /> elements (output).</param>
        /// <param name="z">Contains the matrix Q reduced to Hessenberg form of dimension (<paramref name="ldz" />, k), where the minimal length of k is depending on <paramref name="operation" />; perhaps this parameter will not referenced.</param>
        /// <param name="workspace">A workspace array of length at least <paramref name="workspaceLength" />.</param>
        /// <param name="workspaceLength">The length of the workspace array <paramref name="workspace" />, at least max( 1, <paramref name="n" /> ).</param>
        /// <param name="ldh">The leading dimension of <paramref name="h" />, at least max( 1, <paramref name="n" />).</param>
        /// <param name="ldz">The leading dimension of <paramref name="z" />, at least max( 1, <paramref name="n" />) or <c>1</c> depending on <paramref name="operation" />.</param>
        public void dhseqr(LapackEigenvalues.JobXhseqr job, LapackEigenvalues.XhseqrOperation operation, int n, int ilo, int ihi, double[] h, double[] wr, double[] wi, double[] z, double[] workspace, int workspaceLength, int ldh, int ldz)
        {
            //char job = 'E';  // eigen values only.
            char jobChar = 'E';
            char compz = 'N';  // do not compute Schur vectors
            //            int ldh = n;
            //           int ldz = 1;
            int info;
            unsafe
            {
                fixed (double* ptrH = h, ptrWReal = wr, PtrWImaginary = wi, ptrWork = workspace)
                {
                    _dhseqr(ref jobChar, ref compz, ref n, ref ilo, ref ihi, (IntPtr)ptrH, ref ldh, (IntPtr)ptrWReal, (IntPtr)PtrWImaginary, IntPtr.Zero, ref ldz, (IntPtr)ptrWork, ref workspaceLength, out info);
                }
            }
            if (info != 0)
            {
                throw new ArithmeticException("Return value " + info + " in dhseqr.");
            }
        }

        /// <summary>Gets the ideal length of the workspace array required for optimum performance.
        /// </summary>
        /// <param name="job">A value indicating whether eigenvalues or Schur form is required.</param>
        /// <param name="operation">A value indicating</param>
        /// <param name="n">The order of the Hessenberg matrix H.</param>
        /// <param name="ilo">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <c>1</c>.</param>
        /// <param name="ihi">If the input matrix is an output by <c>dgebal</c> this parameter must contain the value returned by that routine; otherwise <paramref name="n" />.</param>
        /// <param name="h">On input the Hessenberg matrix H of dimension (<paramref name="ldh" />, <paramref name="n" />), overwritten by the upper triangular matrix T from the Schur decomposition if <paramref name="job" /> indicates
        /// to calculate the Schur decomposition, otherwise the content is unspecified on exit.</param>
        /// <param name="wr">The real part of the eigenvalues, i.e. an array with at least <paramref name="n" /> elements (output).</param>
        /// <param name="wi">The imaginary part of the eigenvalues, i.e. an array with at least <paramref name="n" /> elements (output).</param>
        /// <param name="z">Contains the matrix Q reduced to Hessenberg form of dimension (<paramref name="ldz" />, k), where the minimal length of k is depending on <paramref name="operation" />; perhaps this parameter will not referenced.</param>
        /// <param name="ldh">The leading dimension of <paramref name="h" />, at least max( 1, <paramref name="n" />).</param>
        /// <param name="ldz">The leading dimension of <paramref name="z" />, at least max( 1, <paramref name="n" />) or <c>1</c> depending on <paramref name="operation" />.</param>
        /// <returns>The ideal length of the working array required for optimum performance.</returns>
        public int dhseqr_workspaceQuery(LapackEigenvalues.JobXhseqr job, LapackEigenvalues.XhseqrOperation operation, int n, int ilo, int ihi, double[] h, double[] wr, double[] wi, double[] z, int ldh, int ldz)
        {
            //char job = 'E';  // eigen values only.
            char jobChar = 'E';
            char compz = 'N';  // do not compute Schur vectors
            //            int ldh = n;
            //           int ldz = 1;
            int info;

            int workspaceLength = -1;
            unsafe
            {
                double* work = stackalloc double[1];
                fixed (double* ptrH = h, ptrWReal = wr, PtrWImaginary = wi)
                {
                    _dhseqr(ref jobChar, ref compz, ref n, ref ilo, ref ihi, (IntPtr)ptrH, ref ldh, (IntPtr)ptrWReal, (IntPtr)PtrWImaginary, IntPtr.Zero, ref ldz, (IntPtr)work, ref workspaceLength, out info);
                }

                if (info != 0)
                {
                    throw new ArithmeticException("Return value " + info + " in dhseqr.");
                }
                return (int)work[0];
            }
        }

        public void dhsein(char job, char eigsrc, char initv, bool[] select, int n, double[] h, int ldh, double[] wr, double[] wi, double[] vl, int ldvl, double[] vr, int ldvr, int mm, out int m, double[] workspace, int[] ifaill, int[] ifailr)
        {
            throw new NotImplementedException();
        }

        public void dtrevc(char side, char howmny, bool select, int n, double[] t, int ldt, double[] vl, int ldvl, double[] vr, int ldvr, int mm, int m, double[] work)
        {
            throw new NotImplementedException();
        }

        public void dtrsna(char job, char howmny, bool select, int n, double[] t, int ldt, double[] vl, int ldvl, double[] vr, int ldvr, double[] s, double[] sep, int mm, int m, double[] work, int ldwork, int iwork)
        {
            throw new NotImplementedException();
        }

        public void dtrexc(char compq, int n, double[] t, int ldt, double[] q, int ldq, int ifst, int ilst, double[] work)
        {
            throw new NotImplementedException();
        }

        public void dtrsen(char job, char compq, bool select, int n, double[] t, int ldt, double[] q, int ldq, double[] wr, double[] wi, int m, out double s, out double sep, double[] work, int lwork, int[] iwork, int liwork)
        {
            throw new NotImplementedException();
        }

        public void dtrsyl(char trana, char tranb, int isgn, int m, int n, double[] a, int lda, double[] b, int ldb, double[] c, int ldc, out double scale)
        {
            throw new NotImplementedException();
        }
        #endregion

        #region (double precision) complex methods

        public void zgehrd(int n, int ilo, int ihi, Complex[] a, Complex[] tau, Complex[] workspace, int workspaceLength, int lda)
        {
            throw new NotImplementedException();
        }

        public int zgehrd_workspaceQuery(int n, int ilo, int ihi, Complex[] a, Complex[] tau, int lda)
        {
            throw new NotImplementedException();
        }


        // fehlt: zunghr, zunmhr etc.!!


        public void zgebal(LAPACK.MatrixBalancesType job, int n, Complex[] a, out int ilo, out int ihi, double[] scale, int lda)
        {
            char laPackJob;
            switch (job)
            {
                case LAPACK.MatrixBalancesType.NeitherPermutedNorScaled:
                    laPackJob = 'N';
                    break;
                case LAPACK.MatrixBalancesType.Permuted:
                    laPackJob = 'P';
                    break;
                case LAPACK.MatrixBalancesType.PermutedAndScaled:
                    laPackJob = 'B';
                    break;
                case LAPACK.MatrixBalancesType.Scaled:
                    laPackJob = 'S';
                    break;
                default:
                    throw new NotImplementedException();
            }
            int info = 0;
            ilo = 1;
            ihi = n;
            unsafe
            {
                fixed (Complex* ptrA = a)
                {
                    fixed (double* ptrScale = scale)
                    {
                        _zgebal(ref laPackJob, ref n, (IntPtr)ptrA, ref lda, ref ilo, ref ihi, (IntPtr)ptrScale, ref info);
                    }
                }
            }
            if (info != 0)
            {
                throw new ArithmeticException("Return value " + info + " in zgebal.");
            }
        }

        public void zgebak(LAPACK.MatrixBalancesType job, BLAS.Side side, int n, int ilo, int ihi, Complex[] scale, int m, Complex[] v, int ldv)
        {
            throw new NotImplementedException();
        }

        public void zhseqr(LapackEigenvalues.JobXhseqr job, LapackEigenvalues.XhseqrOperation operation, int n, int ilo, int ihi, Complex[] h, Complex[] w, Complex[] z, Complex[] workspace, int workspaceLength, int ldh, int ldz)
        {
            char jobChar = 'E';  // eigen values only.
            char compz = 'N'; // do not compute Schur vectors
            int info = 0;

            unsafe
            {
                fixed (Complex* ptrH = h, ptrW = w, ptrWork = workspace)
                {
                    _zhseqr(ref jobChar, ref compz, ref n, ref ilo, ref ihi, (IntPtr)ptrH, ref ldh, (IntPtr)ptrW, IntPtr.Zero, ref ldz, (IntPtr)ptrWork, ref workspaceLength, ref info);
                }
            }
            if (info != 0)
            {
                throw new ArithmeticException("Return value " + info + " in zhseqr.");
            }
        }

        public int zhseqr_workspaceQuery(LapackEigenvalues.JobXhseqr job, LapackEigenvalues.XhseqrOperation operation, int n, int ilo, int ihi, Complex[] h, Complex[] w, Complex[] z, int ldh, int ldz)
        {
            char jobAsChar = 'E';  // eigen values only.
            char compz = 'N'; // do not compute Schur vectors
            int info = 0;

            int lwork = -1;
            unsafe
            {
                Complex* work = stackalloc Complex[1];
                fixed (Complex* hPtr = h, wPtr = w, zPtr = z)
                {
                    _zhseqr(ref jobAsChar, ref compz, ref n, ref ilo, ref ihi, (IntPtr)hPtr, ref ldh, (IntPtr)wPtr, IntPtr.Zero, ref ldz, (IntPtr)work, ref lwork, ref info);
                }
                if (info == 0)
                {
                    return (int)work[0].Real;
                }
                throw new NotImplementedException();
            }
        }
        #endregion

        #endregion
    }
}