﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;
using System.Collections.Generic;

using Dodoni.MathLibrary.Basics.LowLevel.Interdisciplinary;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    /// <summary>Serves as wrapper for the factory of 1-dimensional (Fast) Fourier transformations.
    /// </summary>
    /// <remarks>This wrapper class is needed because the most implementations do not support a general Fractional Fourier Transformation
    /// and will be computed 'on top' of a ordinary (Fast) Fourier Transformation.</remarks>
    [System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)]
    public class OneDimFourierTransformationFactory
    {
        #region nested classes

        /// <summary>Serves as factory for real (Fast) Fourier transformations.
        /// </summary>
        public class RealFactory
        {
            /// <summary>The factory of one-dimensional (real) Fourier Transformations.
            /// </summary>
            private IOneDimFourierTransformationFactory m_Factory;

            /// <summary>Initializes a new instance of the <see cref="RealFactory"/> class.
            /// </summary>
            /// <param name="fourierTransformationFactory">The (Fast) Fourier Transformation factory.</param>
            internal RealFactory(IOneDimFourierTransformationFactory fourierTransformationFactory)
            {
                m_Factory = fourierTransformationFactory;
            }

            /// <summary>Creates a specific one-dimensional real (Fast) Fourier transformation.
            /// </summary>
            /// <param name="length">The length, i.e. the number of real (input) coefficients.</param>
            /// <returns>A specific one-dimensional real Fourier transformation.</returns>
            public FFT.IOneDimensionalRealData Create(int length)
            {
                return m_Factory.CreateRealTransformation(length);
            }
        }
        #endregion

        #region public / private members

        /// <summary>The factory of real-valued (Fast) Fourier Transformations.
        /// </summary>
        public readonly RealFactory Real;

        /// <summary>The (Fast) Fourier Transformation factory 
        /// </summary>
        private IOneDimFourierTransformationFactory m_Factory;
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="OneDimFourierTransformationFactory"/> class.
        /// </summary>
        /// <param name="fourierTransformationFactory">The (Fast) Fourier Transformation factory.</param>
        internal OneDimFourierTransformationFactory(IOneDimFourierTransformationFactory fourierTransformationFactory)
        {
            Real = new RealFactory(fourierTransformationFactory);
            m_Factory = fourierTransformationFactory;
        }
        #endregion

        #region public methods

        /// <summary>Creates a specific one-dimensional Fourier transformation.
        /// </summary>
        /// <param name="length">The length, i.e. the number of complex coefficients.</param>
        /// <returns>A specific one-dimensional Fourier transformation.</returns>
        public FFT.IOneDimensional Create(int length)
        {
            return m_Factory.Create(length);
        }

        /// <summary>Creates a specific one-dimensional Fractional Fourier transformation.
        /// </summary>
        /// <param name="length">The length, i.e. the number of complex coefficients.</param>
        /// <param name="alpha">The parameter of the fractional Fast Fourier transformation.</param>
        /// <returns>A specific one-dimensional Fourier transformation.</returns>
        public FFT.IOneDimensional Create(int length, double alpha)
        {
            if (m_Factory is IOneDimFractionalFourierTransformationFactory)
            {
                var fractionalFourierTransformationFactory = m_Factory as IOneDimFractionalFourierTransformationFactory;
                return fractionalFourierTransformationFactory.Create(length, alpha);
            }
            return new OneDimFractionalFourierTransformationRealAlpha(length, alpha, m_Factory);
        }

        /// <summary>Creates a specific one-dimensional Fractional Fourier transformation.
        /// </summary>
        /// <param name="length">The length, i.e. the number of complex coefficients.</param>
        /// <param name="alpha">The parameter of the fractional Fast Fourier transformation.</param>
        /// <returns>A specific one-dimensional Fourier transformation.</returns>
        public FFT.IOneDimensional Create(int length, Complex alpha)
        {
            if (m_Factory is IOneDimFractionalFourierTransformationFactory)
            {
                var fractionalFourierTransformationFactory = m_Factory as IOneDimFractionalFourierTransformationFactory;
                if (fractionalFourierTransformationFactory.FourierExponentialFactorRestriction.HasFlag(FFT.FourierExponentialFactorRestriction.Arbitrary) == true)
                {
                    return fractionalFourierTransformationFactory.Create(length, alpha);
                }
            }
            return new OneDimFractionalFourierTransformation(length, alpha, m_Factory);
        }

        /// <summary>Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return String.Format("1-dimensional (Fast) Fourier Transformation factory ({0})", m_Factory.ToString());
        }
        #endregion
    }
}