﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;

using Dodoni.BasicComponents;
using Dodoni.MathLibrary.Basics.LowLevel;

namespace Dodoni.MathLibrary.Basics
{
    /// <summary>Provides functions for Vector units, i.e. methods applied to arrays of floating point numbers, complex numbers etc.
    /// </summary>
    public static class VectorUnit
    {
        #region nested interfaces, classes etc.

        /// <summary>Serves as interface for a Vector Unit Library.
        /// </summary>
        public interface ILibrary : IIdentifierNameable, IAnnotatable
        {
            /// <summary>Gets basic mathematical functions.
            /// </summary>
            /// <value>Provides basic mathematical functions.</value>
            IVectorUnitBasics Basics
            {
                get;
            }

            /// <summary>Gets special mathematical functions.
            /// </summary>
            /// <value>Provides special mathematical functions.</value>
            IVectorUnitSpecial Special
            {
                get;
            }

            /// <summary>Initializes the Library.
            /// </summary>
            /// <remarks>Call this method before using the Library the first time.</remarks>
            void Initialize();
        }

        /// <summary>The catalog of Vecotr unit Libraries via Managed Extensibility Framework.
        /// </summary>
        internal class Catalog
        {
            #region private members

            /// <summary>The available Vector Unit Libraries.
            /// </summary>
            [ImportMany]
            private IEnumerable<ILibrary> m_Libraries = null;
            #endregion

            #region internal constructors

            /// <summary>Initializes a new instance of the <see cref="Catalog" /> class.
            /// </summary>
            internal Catalog()
            {
                var catalog = new DirectoryCatalog(AppDomain.CurrentDomain.BaseDirectory);

                var container = new CompositionContainer(catalog);
                container.ComposeParts(this);
            }
            #endregion

            #region internal properties

            /// <summary>Gets the collection of (external) Vector Unit Libraries.
            /// </summary>
            /// <value>The collection of (external) Vector Unit Libraries.
            /// </value>
            internal IEnumerable<ILibrary> Libraries
            {
                get { return m_Libraries; }
            }
            #endregion
        }
        #endregion

        #region public (readonly) members

        /// <summary>Provide basic mathematical functions.
        /// </summary>
        public static readonly IVectorUnitBasics Basics;

        /// <summary>Provides special mathematical functions.
        /// </summary>
        public static readonly IVectorUnitSpecial Special;
        #endregion

        #region constructor

        /// <summary>Initializes the <see cref="VectorUnit"/> class.
        /// </summary>
        static VectorUnit()
        {
            Basics = LowLevelMathConfiguration.VectorUnit.Basics;
            Special = LowLevelMathConfiguration.VectorUnit.Special;
        }
        #endregion
    }
}