/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Globalization;

using Dodoni.BasicComponents;
using Dodoni.MathLibrary.Basics;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.MathLibrary
{
    /// <summary>Represents a dense matrix.
    /// </summary>
    /// <remarks>This class does not implement all BLAS, Lapack etc. functionality, one has to apply the specified BLAS, Lapack etc. function to the <see cref="DenseMatrix.Data"/> property.</remarks>
    public partial class DenseMatrix : IMatrix, IInfoOutputQueriable
    {
        #region private members

        /// <summary>The number of rows, i.e. the first dimension.
        /// </summary>
        private int m_RowCount;

        /// <summary>The number of columns, i.e. the second dimension.
        /// </summary>
        private int m_ColumnCount;

        /// <summary>The matrix entries, provided column-by-column.
        /// </summary>
        private double[] m_Data;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="DenseMatrix"/> class.
        /// </summary>
        /// <param name="rowCount">The number of rows.</param>
        /// <param name="columnCount">The number of columns.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if one of the argument is less or equal <c>0</c>.</exception>
        public DenseMatrix(int rowCount, int columnCount)
        {
            if (rowCount < 1)
            {
                throw new ArgumentOutOfRangeException("rowCount");
            }
            if (columnCount < 1)
            {
                throw new ArgumentOutOfRangeException("columnCount");
            }
            m_RowCount = rowCount;
            m_ColumnCount = columnCount;
            m_Data = new double[m_RowCount * m_ColumnCount];
        }

        /// <summary>Initializes a new instance of the <see cref="DenseMatrix"/> class.
        /// </summary>
        /// <param name="rowCount">The number of rows.</param>
        /// <param name="columnCount">The number of columns.</param>
        /// <param name="data">The matrix entries provided column-by-column with at least <paramref name="columnCount"/> * <paramref name="rowCount"/> elements.</param>
        /// <param name="createDeepCopyOfArgument">A value indicating whether a deep copy of <paramref name="data"/> will be created and stored in the current object.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="data"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if <paramref name="rowCount"/> or <paramref name="columnCount"/> is less or equal <c>0</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="data"/> contains less elements than <paramref name="rowCount"/> * <paramref name="columnCount"/>.</exception>
        public DenseMatrix(int rowCount, int columnCount, double[] data, bool createDeepCopyOfArgument = false)
        {
            if (rowCount < 1)
            {
                throw new ArgumentOutOfRangeException("rowCount");
            }
            if (columnCount < 1)
            {
                throw new ArgumentOutOfRangeException("columnCount");
            }
            m_RowCount = rowCount;
            m_ColumnCount = columnCount;

            if (data == null)
            {
                throw new ArgumentNullException("columnwiseDenseMatrix");
            }
            if (data.Length < rowCount * columnCount)
            {
                throw new ArgumentException("columnwiseDenseMatrix");
            }
            if (createDeepCopyOfArgument == false)
            {
                m_Data = data;
            }
            else
            {
                m_Data = new double[rowCount * columnCount];
                BLAS.Level1.dcopy(rowCount * columnCount, data, m_Data);
            }
        }

        /// <summary>Initializes a new instance of the <see cref="DenseMatrix"/> class.
        /// </summary>
        /// <param name="data">The dense matrix entries, where the first null-based index corresponds to the row and the second index corresponds to the column.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="data"/> is <c>null</c>.</exception>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="data"/> does not represents a matrix.</exception>
        /// <remarks>This constructor creates a deep copy of its argument.</remarks>
        public DenseMatrix(double[][] data)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }
            m_RowCount = data.GetLength(0);
            m_ColumnCount = data[0].Length;

            m_Data = new double[m_RowCount * m_ColumnCount];
            for (int j = 0; j < m_RowCount; j++)
            {
                if (data[j].Length != m_ColumnCount)
                {
                    throw new IndexOutOfRangeException(String.Format(ExceptionMessages.ArgumentHasWrongDimension, "data"));
                }
                for (int k = 0; k < m_ColumnCount; k++)
                {
                    m_Data[j + m_RowCount * k] = data[j][k];
                }
            }
        }

        /// <summary>Initializes a new instance of the <see cref="DenseMatrix"/> class.
        /// </summary>
        /// <param name="data">The dense matrix entries, where the first null-based index corresponds to the row and the second index corresponds to the column.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="data"/> is <c>null</c>.</exception>
        /// <remarks>This constructor creates a deep copy of its argument.</remarks>
        [CLSCompliant(false)]
        public DenseMatrix(double[,] data)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }
            m_RowCount = data.GetLength(0);
            m_ColumnCount = data.GetLength(1);


            m_Data = new double[m_RowCount * m_ColumnCount];
            for (int j = 0; j < m_RowCount; j++)
            {
                for (int k = 0; k < m_ColumnCount; k++)
                {
                    m_Data[j + m_RowCount * k] = data[j, k];
                }
            }
        }
        #endregion

        #region public properties

        #region IMaxtrix Members

        /// <summary>Gets the number of rows of the current matrix object.
        /// </summary>
        /// <value>The number of rows.</value>
        public int RowCount
        {
            get { return m_RowCount; }
        }

        /// <summary>Gets the number of columns of the current matrix object.
        /// </summary>
        /// <value>The number of columns.</value>
        public int ColumnCount
        {
            get { return m_ColumnCount; }
        }

        /// <summary>Gets a value indicating whether this instance represents a quadratic matrix.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is quadratic; otherwise, <c>false</c>.
        /// </value>
        public bool IsQuadratic
        {
            get { return (m_ColumnCount == m_RowCount); }
        }

        /// <summary>Gets or sets a specified value.
        /// </summary>
        /// <param name="rowIndex">The null-based row index.</param>
        /// <param name="columnIndex">The null-based column index.</param>
        /// <value>The value of the matrix at the specified coordinate.</value>
        /// <exception cref="IndexOutOfRangeException">Thrown, if the row-/column position is invalid.</exception>
        public double this[int rowIndex, int columnIndex]
        {
            get { return m_Data[rowIndex + m_RowCount * columnIndex]; }
            set { m_Data[rowIndex + m_RowCount * columnIndex] = value; }
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Gets the info-output level of detail.
        /// </summary>
        /// <value>The info-output level of detail.</value>
        InfoOutputDetailLevel IInfoOutputQueriable.InfoOutputDetailLevel
        {
            get { return InfoOutputDetailLevel.Full; }
        }
        #endregion

        /// <summary>Gets the reference to the internal data, i.e. the matrix entries provided column-by-column.
        /// </summary>
        /// <value>The reference to the internal data.</value>
        /// <remarks>The BLAS and Lapack functions assumes an array of floating point numbers as argument.</remarks>
        public double[] Data
        {
            get { return m_Data; }
        }
        #endregion

        #region public methods

        #region IMatrix Members

        /// <summary>Creates a <see cref="DenseMatrix"/> object from the current object.
        /// </summary>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at
        /// least <see cref="IMatrix.RowCount"/> * <see cref="IMatrix.ColumnCount"/> elements.</param>
        /// <returns>A <see cref="DenseMatrix"/> object that represents the current object.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        public DenseMatrix ToDenseMatrix(double[] matrixData = null)
        {
            double[] dataOfNewDenseMatrix = matrixData;

            if ((dataOfNewDenseMatrix == null) || (dataOfNewDenseMatrix.Length < m_RowCount * m_ColumnCount))
            {
                dataOfNewDenseMatrix = new double[m_RowCount * m_ColumnCount];
            }
            BLAS.Level1.dcopy(m_RowCount * m_ColumnCount, m_Data, dataOfNewDenseMatrix);
            return new DenseMatrix(m_RowCount, m_ColumnCount, dataOfNewDenseMatrix);
        }

        /// <summary>Gets a specific column.
        /// </summary>
        /// <param name="columnIndex">The null-based index of the column.</param>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at
        /// least <see cref="IMatrix.RowCount"/> elements.</param>
        /// <returns>The specified column vector in its <see cref="DenseMatrix"/> representation.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        public DenseMatrix GetColumn(int columnIndex, double[] matrixData = null)
        {
            double[] column = matrixData;
            if ((column == null) || (column.Length < m_RowCount))
            {
                column = new double[m_RowCount];
            }
            // set 'column[j] = m_Data[j + m_RowCount * columnIndex]' for j=0,..,m_RowCount-1:
            BLAS.Level1.dcopy(m_RowCount, m_Data, column, 1, 1, m_RowCount * columnIndex, 0);
            return new DenseMatrix(m_RowCount, 1, column);
        }

        /// <summary>Gets a specific transposed row.
        /// </summary>
        /// <param name="rowIndex">The null-based index of the row.</param>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at
        /// least <see cref="IMatrix.ColumnCount"/> elements.</param>
        /// <returns>The specified transposed row vector in its <see cref="DenseMatrix"/> representation.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        public DenseMatrix GetTransposedRow(int rowIndex, double[] matrixData = null)
        {
            double[] row = matrixData;
            if ((row == null) || (row.Length < m_ColumnCount))
            {
                row = new double[m_ColumnCount];
            }
            // set 'row[j] = m_Data[rowIndex + m_RowCount * j]' for j =0,...,m_ColumnCount-1:
            BLAS.Level1.dcopy(m_ColumnCount, m_Data, row, m_RowCount, 1, rowIndex, 0);
            return new DenseMatrix(m_ColumnCount, 1, row);
        }

        /// <summary>Gets a specific sub-matrix.
        /// </summary>
        /// <param name="startRowIndex">The null-based index of the upper row.</param>
        /// <param name="endRowIndex">The null-based index of the lower row.</param>
        /// <param name="startColumnIndex">The null-based index of the left column.</param>
        /// <param name="endColumnIndex">The null-based index of the right column.</param>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at
        /// least (<paramref name="endRowIndex"/> - <paramref name="startRowIndex"/> + 1) * (<paramref name="endColumnIndex"/> - <paramref name="startColumnIndex"/> + 1) elements.</param>
        /// <returns>The specified sub-matrix in its <see cref="DenseMatrix"/> representation.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        public DenseMatrix GetSubMatrix(int startRowIndex, int endRowIndex, int startColumnIndex, int endColumnIndex, double[] matrixData = null)
        {
            int subColumnCount = endColumnIndex - startColumnIndex + 1;
            int subRowCount = endRowIndex - startRowIndex + 1;

            if ((subRowCount > m_RowCount) || (subColumnCount > m_ColumnCount))
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "sub-matrix range and given matrix"));
            }
            double[] subMatrix = matrixData;
            if ((subMatrix == null) || (subMatrix.Length < subRowCount * subColumnCount))
            {
                subMatrix = new double[subRowCount * subColumnCount];
            }

            int sourceArrayOffset = startRowIndex + startColumnIndex * m_RowCount;
            for (int i = 0; i < subRowCount; i++)
            {
                BLAS.Level1.dcopy(subColumnCount, m_Data, subMatrix, m_RowCount, subRowCount, i + sourceArrayOffset, i);
            }
            return new DenseMatrix(subRowCount, subColumnCount, subMatrix);
        }

        /// <summary>Determines whether the matrix is symmetric.
        /// </summary>
        /// <param name="tolerance">A tolerance taken into account, i.e. if abs(a_{i,j} - a{j,i}) is less than <paramref name="tolerance"/> the values are assumed to be equal.</param>
        /// <returns><c>true</c> if this instance is symmetric; otherwise, <c>false</c>.</returns>
        public bool IsSymmetric(double tolerance)
        {
            if (m_RowCount != m_ColumnCount)
            {
                return false;
            }
            for (int j = 0; j < m_ColumnCount; j++)
            {
                for (int k = j; k <= m_RowCount; k++)
                {
                    if (Math.Abs(m_Data[k + j * m_RowCount] - m_Data[j + k * m_RowCount]) > tolerance)
                    {
                        return false;
                    }
                }
            }
            return true;
        }

        /// <summary>Gets the trace of a quadratic matrix.
        /// </summary>
        /// <returns>The trace of the matrix, i.e. the sum of the diagonal elements.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if the current object does not represent a quadratic matrix.</exception>
        public double GetTrace()
        {
            if (IsQuadratic == false)
            {
                throw new InvalidOperationException();
            }
            double trace = 0.0;
            for (int j = 0; j < m_RowCount; j++)
            {
                trace += m_Data[j + j * m_RowCount];  // = j * (1 + m_RowCount);
            }
            return trace;
        }

        /// <summary>Gets a specific norm of the current matrix object.
        /// </summary>
        /// <param name="normType">The type of the (matrix) norm.</param>
        /// <returns>The norm of the current instance with respect to the <paramref name="normType"/>.</returns>
        public double GetNorm(Matrix.NormType normType)
        {
            switch (normType)
            {
                case Matrix.NormType.FrobeniusNorm:
                    return BLAS.Level1.dnrm2(m_RowCount * m_ColumnCount, m_Data);

                case Matrix.NormType.MaximalNorm:
                    return m_Data[BLAS.Level1.idamax(m_RowCount * m_ColumnCount, m_Data)];

                default: throw new NotImplementedException();
            }
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Gets informations of the current object as a specific <see cref="InfoOutput"/> instance.
        /// </summary>
        /// <param name="infoOutput">The <see cref="InfoOutput"/> object which is to be filled with informations concering the current instance.</param>
        /// <param name="categoryName">The name of the category, i.e. all informations will be added to these category.</param>
        void IInfoOutputQueriable.FillInfoOutput(InfoOutput infoOutput, string categoryName)
        {
            InfoOutputPackage infoOutputCollection = infoOutput.AcquirePackage(categoryName);
            infoOutputCollection.Add(new InfoOutputProperty("RowCount", m_RowCount),
                                     new InfoOutputProperty("ColumnCount", m_ColumnCount));

            var dataTable = new System.Data.DataTable("Data");
            for (int j = 0; j < m_ColumnCount; j++)
            {
                dataTable.Columns.Add(j.ToString(), typeof(double));
            }

            for (int k = 0; k < m_RowCount; k++)
            {
                var row = dataTable.NewRow();
                for (int j = 0; j < m_ColumnCount; j++)
                {
                    row[j] = this[k, j];
                }
                dataTable.Rows.Add(row);
            }
            infoOutputCollection.Add(dataTable);
        }

        /// <summary>Sets the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel"/> property.
        /// </summary>
        /// <param name="infoOutputDetailLevel">The info-output level of detail.</param>
        /// <returns>A value indicating whether the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel"/> has been set to <paramref name="infoOutputDetailLevel"/>.</returns>
        bool IInfoOutputQueriable.TrySetInfoOutputDetailLevel(InfoOutputDetailLevel infoOutputDetailLevel)
        {
            return (infoOutputDetailLevel == InfoOutputDetailLevel.Full);
        }
        #endregion

        /// <summary>Overwrites values of a specified sub-matrix by entries of another matrix.
        /// </summary>
        /// <param name="startRowIndex">The null-based index of the upper row.</param>
        /// <param name="endRowIndex">The null-based index of the lower row.</param>
        /// <param name="startColumnIndex">The null-based index of the left column.</param>
        /// <param name="endColumnIndex">The null-based index of the right column.</param>
        /// <param name="denseSubMatrix">The values to store in the current object at the specified position.</param>
        public void SetRange(int startRowIndex, int endRowIndex, int startColumnIndex, int endColumnIndex, double[,] denseSubMatrix)
        {
            if ((endRowIndex < m_RowCount) && (startRowIndex <= endRowIndex) && (endColumnIndex < m_ColumnCount) && (startColumnIndex <= endColumnIndex))
            {
                int subColumnCount = endColumnIndex - startColumnIndex + 1;
                int subRowCount = endRowIndex - startRowIndex + 1;

                if ((denseSubMatrix.GetLength(0) > subRowCount) || (denseSubMatrix.GetLength(1) > subColumnCount))
                {
                    throw new ArgumentException();
                }
                for (int k = startColumnIndex; k < subColumnCount; k++)
                {
                    int offset = k * m_RowCount;
                    for (int i = startRowIndex; i < subRowCount; i++)
                    {
                        m_Data[i + offset] = denseSubMatrix[i - startRowIndex, k - startColumnIndex];
                    }
                }
            }
            else
            {
                throw new ArgumentException();
            }
        }

        /// <summary>Overwrites values of a specified sub-matrix by entries of another matrix.
        /// </summary>
        /// <param name="startRowIndex">The null-based index of the upper row.</param>
        /// <param name="endRowIndex">The null-based index of the lower row.</param>
        /// <param name="startColumnIndex">The null-based index of the left column.</param>
        /// <param name="endColumnIndex">The null-based index of the right column.</param>
        /// <param name="denseSubMatrix">The values to store in the current object at the specified position.</param>
        public void SetRange(int startRowIndex, int endRowIndex, int startColumnIndex, int endColumnIndex, DenseMatrix denseSubMatrix)
        {
            if ((endRowIndex < m_RowCount) && (startRowIndex <= endRowIndex) && (endColumnIndex < m_ColumnCount) && (startColumnIndex <= endColumnIndex))
            {
                int subColumnCount = endColumnIndex - startColumnIndex + 1;
                int subRowCount = endRowIndex - startRowIndex + 1;

                if ((denseSubMatrix.RowCount > subRowCount) || (denseSubMatrix.ColumnCount > subColumnCount))
                {
                    throw new ArgumentException();
                }
                /* set
                 * m_Data[i + k * m_RowCount] = denseSubMatrix.m_Data[i-startRowIndex + (k-startColumnIndex) * subRowCount]
                 *                            = denseSubMatrix.m_Data[ {i-startRowIndex - startColumnIndex * subRowCount} + k * subRowCount]
                 * for  k = startColumnIndex, ..., subColumnCount-1 and i = startRowIndex, ..., endRowIndex */

                for (int i = startRowIndex; i < endRowIndex; i++)
                {
                    BLAS.Level1.dcopy(subColumnCount, m_Data, denseSubMatrix.m_Data, subRowCount, m_RowCount, i - startRowIndex - startColumnIndex * subRowCount, i);
                }
            }
            else
            {
                throw new ArgumentException();
            }
        }

        /// <summary>Gets a singular value decomposition, i.e. A = U * \Sigma *V', where the diagonal elements of '\Sigma' are the singular values in descending order.
        /// </summary>
        /// <param name="u">The left singular vectors 'U', i.e. a 'm x m' unitary matrix (output).</param>
        /// <param name="vt">The right singular vectors 'V^t', i.e. a 'n x n' unitary matrix (output).</param>
        /// <returns>The singular values in descending order.</returns>
        /// <remarks>The singular values are the roots of the non-negative eigenvalues of A^t*A.
        /// <para>This method changes the current instance</para></remarks>
        public double[] GetSingularValueDecomposition(out DenseMatrix u, out DenseMatrix vt)
        {
            var uArray = new double[m_RowCount * m_RowCount];
            var vtArray = new double[m_ColumnCount * m_ColumnCount];
            var s = new double[Math.Min(m_RowCount, m_ColumnCount)];
            LAPACK.EigenValues.DriverRoutines.dgesvd(m_RowCount, m_ColumnCount, m_Data, s, uArray, vtArray);

            u = new DenseMatrix(m_RowCount, m_RowCount, uArray);
            vt = new DenseMatrix(m_ColumnCount, m_ColumnCount, vtArray);
            return s;
        }

        /// <summary>Solves the system of linear equations Ax = b (in-place).
        /// </summary>
        /// <param name="b">The vector b, whose single unique column is the right-hand side for the systems of equations (output: Overwritten by the solution vector x).</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="InvalidOperationException">Thrown, if the current instance does not representate a square matrix.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="b"/> contains less elements than the number of rows/columns of the current instance.</exception>
        /// <remarks>After calling this method the current instance contains the LU decomposition of the specified matrix.</remarks>
        public void SolveSystemOfLinearEquationsInPlace(double[] b)
        {
            if (m_RowCount != m_ColumnCount)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.CurrentCulture, ExceptionMessages.ObjectIsInvalid, "General band matrix", "not quadratic"));
            }
            if (b == null)
            {
                throw new ArgumentNullException("b");
            }
            if (b.Length < m_RowCount)
            {
                throw new ArgumentException("b");
            }
            int[] ipivot = new int[m_RowCount];
            LAPACK.LinearEquations.DriverRoutines.dgesv(m_RowCount, m_Data, b, ipivot);
        }

        /// <summary>Solves the system of linear equations Ax = b.
        /// </summary>
        /// <param name="b">The vector b, whose single unique column is the right-hand side for the systems of equations (output: Overwritten by the solution vector x).</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="InvalidOperationException">Thrown, if the current instance does not representate a square matrix.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="b"/> contains less elements than the number of rows/columns of the current instance.</exception>
        /// <remarks>The current instance will not be changed.</remarks>
        public void SolveSystemOfLinearEquationsOutOfPlace(double[] b)
        {
            if (m_RowCount != m_ColumnCount)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.CurrentCulture, ExceptionMessages.ObjectIsInvalid, "General band matrix", "not quadratic"));
            }
            if (b == null)
            {
                throw new ArgumentNullException("b");
            }
            if (b.Length < m_RowCount)
            {
                throw new ArgumentException("b");
            }
            int[] ipivot = new int[m_RowCount];
            double[] data = new double[m_RowCount * m_ColumnCount]; // create a deep copy of the current matrix
            BLAS.Level1.dcopy(m_RowCount * m_ColumnCount, m_Data, data);

            LAPACK.LinearEquations.DriverRoutines.dgesv(m_RowCount, data, b, ipivot);
        }

        /// <summary>Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.</returns>
        public override string ToString()
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.AppendLine(string.Format("Row count: {0:g}, Column count: {1:g} ", m_RowCount, m_ColumnCount));
            strBuilder.AppendLine("{");
            for (int j = 0; j < m_RowCount; j++)
            {
                strBuilder.Append("[");
                for (int k = 0; k < m_ColumnCount; k++)
                {
                    strBuilder.Append(string.Format("{0:g} ", m_Data[j + m_RowCount * k]));
                }
                strBuilder.AppendLine("]");
            }
            strBuilder.AppendLine("}");
            return strBuilder.ToString();
        }
        #endregion

        #region public static methods

        #region multiplication operator methods

        /// <summary>The multiplication of two <see cref="DenseMatrix"/> objects, i.e. A * B.
        /// </summary>
        /// <param name="a">The first dense matrix A.</param>
        /// <param name="b">The second dense matrix B.</param>
        /// <returns>The result of the operator, i.e. 'A * B'.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of columns of <paramref name="a"/> does not coincide with the number of rows of <paramref name="b"/>.</exception>
        public static DenseMatrix operator *(DenseMatrix a, DenseMatrix b)
        {
            Matrix.TestMultiplicationInput(a, b);

            var length = a.m_RowCount * b.m_ColumnCount;
            var c = new double[length];
            BLAS.Level3.dgemm(a.m_RowCount, b.m_ColumnCount, a.m_ColumnCount, 1, a.m_Data, b.m_Data, 0.0, c);

            return new DenseMatrix(a.m_RowCount, b.m_ColumnCount, c);
        }

        /// <summary>The multiplication of two <see cref="DenseMatrix"/> objects, i.e. op(A) * op(B).
        /// </summary>
        /// <param name="a">The first dense matrix A.</param>
        /// <param name="transA">A value indicating  whether <paramref name="a"/> will be transposed, i.e.  op(A) = A or op(A) = A'.</param>
        /// <param name="b">The second dense matrix B.</param>
        /// <param name="transB">A value indicating  whether <paramref name="b"/> will be transposed, i.e.  op(B) = B or op(B) = B'.</param>
        /// <returns>The result of the operator, i.e. op(A) * op(B).</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the matrix operation op(A) * op(B) can not be place because of unsuited arguments.</exception>
        [CLSCompliant(false)]
        public static DenseMatrix Multiply(DenseMatrix a, BLAS.MatrixTransposeState transA, DenseMatrix b, BLAS.MatrixTransposeState transB)
        {
            if (a == null)
            {
                throw new ArgumentNullException("a", string.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'a'"));
            }
            if (b == null)
            {
                throw new ArgumentNullException("b", string.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'b'"));
            }

            // prepare the dgemm (BLAS level 1) parameters:
            int m;  // number of rows of op(A) and C
            int n;  // number of columns of op(B) and C
            int k;  // number of columns of op(A) = number of rows of op(B)

            if (transA == BLAS.MatrixTransposeState.NoTranspose)
            {
                m = a.m_RowCount;
                k = a.m_ColumnCount;
            }
            else if (transA == BLAS.MatrixTransposeState.Transpose)
            {
                m = a.m_ColumnCount;
                k = a.m_RowCount;
            }
            else
            {
                throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, transA), "transA");
            }

            if (transB == BLAS.MatrixTransposeState.NoTranspose)
            {
                n = b.m_ColumnCount;
                if (b.m_RowCount != k)
                {
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "op(A).ColumnCount, op(B).RowCount"));
                }
            }
            else if (transB == BLAS.MatrixTransposeState.Transpose)
            {
                n = b.m_RowCount;
                if (b.m_ColumnCount != k)
                {
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "op(A).ColumnCount, op(B).RowCount"));
                }
            }
            else
            {
                throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, transB), "transB");
            }
            var c = new double[n * m];
            BLAS.Level3.dgemm(m, n, k, 1, a.m_Data, b.m_Data, 0.0, c, transA, transB);

            return new DenseMatrix(m, n, c);
        }

        /// <summary>The multiplication of a <see cref="DenseMatrix"/> and a specific scalar.
        /// </summary>
        /// <param name="alpha">The scalar \alpha.</param>
        /// <param name="a">The dense matrix A.</param>
        /// <returns>The result of the operator, i.e. \alpha * A.</returns>
        public static DenseMatrix operator *(double alpha, DenseMatrix a)
        {
            var length = a.m_RowCount * a.m_ColumnCount;
            var c = new double[length];
            BLAS.Level1.dcopy(length, a.m_Data, c);
            BLAS.Level1.dscal(length, alpha, c);

            return new DenseMatrix(a.m_RowCount, a.m_ColumnCount, c);
        }

        /// <summary>The multiplication of a <see cref="DenseMatrix"/> and a specific scalar.
        /// </summary>
        /// <param name="a">The dense matrix A.</param>
        /// <param name="alpha">The scalar \alpha.</param>
        /// <returns>The result of the operator, i.e. \alpha * A.</returns>
        public static DenseMatrix operator *(DenseMatrix a, double alpha)
        {
            int length = a.m_RowCount * a.m_ColumnCount;
            double[] c = new double[length];
            BLAS.Level1.dcopy(length, a.m_Data, c);
            BLAS.Level1.dscal(length, alpha, c);

            return new DenseMatrix(a.m_RowCount, a.m_ColumnCount, c);
        }
        #endregion

        #region additive operator methods

        /// <summary>The addition of two <see cref="DenseMatrix"/> objects.
        /// </summary>
        /// <param name="a">The first dense matrix A.</param>
        /// <param name="b">The second dense matrix B.</param>
        /// <returns>The result of the operator, i.e. 'A + B'.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of columns of <paramref name="a"/> does not coincide with the number of rows of <paramref name="b"/>.</exception>
        public static DenseMatrix operator +(DenseMatrix a, DenseMatrix b)
        {
            Matrix.TestAdditionInput(a, b);
            int length = a.m_RowCount * b.m_ColumnCount;
            var c = new double[length];

            BLAS.Level1.dcopy(length, a.m_Data, c);
            BLAS.Level1.daxpy(length, 1.0, b.m_Data, c);
            return new DenseMatrix(a.m_RowCount, b.m_ColumnCount, c);
        }

        /// <summary>Computes the sum of two specific <see cref="DenseMatrix"/> objects, i.e. op(A) + op(B).
        /// </summary>
        /// <param name="a">The first matrix, i.e. 'A'.</param>
        /// <param name="b">The second matrix, i.e. 'B'.</param>
        /// <param name="opA">The operation for matrix A, i.e. op(A) = A or op(A) = A^t.</param>
        /// <param name="opB">The operation for matrix B, i.e. op(B) = B or op(B) = B^t.</param>
        /// <returns>The sum op(A) + op(B) in its <see cref="DenseMatrix"/> representation.</returns>
        [CLSCompliant(false)]
        public static DenseMatrix Add(DenseMatrix a, DenseMatrix b, BLAS.MatrixTransposeState opA = BLAS.MatrixTransposeState.NoTranspose, BLAS.MatrixTransposeState opB = BLAS.MatrixTransposeState.NoTranspose)
        {
            int n = a.RowCount * a.ColumnCount;

            var y = new double[n];
            int rowCount, columnCount;

            if (opA == BLAS.MatrixTransposeState.NoTranspose)
            {
                rowCount = a.RowCount;
                columnCount = a.ColumnCount;

                if (opB == BLAS.MatrixTransposeState.NoTranspose)
                {
                    if ((rowCount != b.RowCount) || (columnCount != b.ColumnCount))
                    {
                        throw new ArgumentException(String.Format(ExceptionMessages.ArgumentHasWrongDimension, "Matrix 'a' resp. 'b'"));
                    }
                    BLAS.Level1.dcopy(n, b.Data, y);
                    BLAS.Level1.daxpy(n, 1.0, a.Data, y);
                }
                else
                {
                    if ((rowCount != b.ColumnCount) || (columnCount != b.RowCount))
                    {
                        throw new ArgumentException(String.Format(ExceptionMessages.ArgumentHasWrongDimension, "Matrix 'a' resp. 'b'"));
                    }
                    BLAS.Level1.dcopy(n, a.Data, y);
                    for (int i = 0; i < rowCount; i++)
                    {
                        BLAS.Level1.daxpy(columnCount, 1.0, b.Data, y, 1, rowCount, i * columnCount, i);
                    }
                }
            }
            else  // A^t
            {
                rowCount = a.ColumnCount;
                columnCount = a.RowCount;
                if (opB == BLAS.MatrixTransposeState.NoTranspose)
                {
                    if ((rowCount != b.RowCount) || (columnCount != b.ColumnCount))
                    {
                        throw new ArgumentException(String.Format(ExceptionMessages.ArgumentHasWrongDimension, "Matrix 'a' resp. 'b'"));
                    }
                    BLAS.Level1.dcopy(n, b.Data, y);
                    for (int i = 0; i < rowCount; i++)
                    {
                        BLAS.Level1.daxpy(columnCount, 1.0, a.Data, y, 1, rowCount, i * columnCount, i);
                    }
                }
                else
                {
                    if ((rowCount != b.ColumnCount) || (columnCount != b.RowCount))
                    {
                        throw new ArgumentException(String.Format(ExceptionMessages.ArgumentHasWrongDimension, "Matrix 'a' resp. 'b'"));
                    }
                    double[] aData = a.Data;
                    double[] bData = b.Data;

                    for (int i = 0; i < rowCount; i++)
                    {
                        int offset = columnCount * i;
                        for (int j = 0; j < columnCount; j++)
                        {
                            y[i + rowCount * j] = aData[j + offset] + bData[j + offset];
                        }
                    }
                }
            }
            return new DenseMatrix(rowCount, columnCount, y);
        }

        /// <summary>The subtraction of two <see cref="DenseMatrix"/> objects.
        /// </summary>
        /// <param name="a">The first dense matrix A.</param>
        /// <param name="b">The second dense matrix B.</param>
        /// <returns>The result of the operator, i.e. 'a - b'.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of columns of <paramref name="a"/> does not coincide with the number of rows of <paramref name="b"/>.</exception>
        public static DenseMatrix operator -(DenseMatrix a, DenseMatrix b)
        {
            Matrix.TestAdditionInput(a, b);
            int length = a.m_RowCount * b.m_ColumnCount;

            var c = new double[length];
            BLAS.Level1.dcopy(length, a.m_Data, c);
            BLAS.Level1.daxpy(length, -1.0, b.m_Data, c);
            return new DenseMatrix(a.m_RowCount, b.m_ColumnCount, c);
        }
        #endregion

        /// <summary>Calculates the bilinear form x^t*A*x, where A is some quadratic matrix.
        /// </summary>
        /// <param name="quadraticMatrix">The quadratic matrix 'A'.</param>
        /// <param name="x">The vector 'x'.</param>
        /// <returns>The value of the bilinear form x^t*A*x.</returns>
        public static double GetBilinearForm(DenseMatrix quadraticMatrix, double[] x)
        {
            int n = quadraticMatrix.RowCount;
            double value = 0.0;

            for (int j = 0; j < n; j++)
            {
                value += x[j] * BLAS.Level1.ddot(n, x, quadraticMatrix.Data, 1, n, 0, j);
            }
            return value;
        }
        #endregion
    }
}