﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Globalization;

using Dodoni.BasicComponents;
using Dodoni.MathLibrary.Basics;

namespace Dodoni.MathLibrary
{
    public partial class DenseMatrix
    {
        #region scaling operation and "this = \alpha * op(A) + this" methods

        /// <summary>Scales the current matrix, i.e. C = \alpha * C, where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="alpha">The (real) scalar.</param>
        /// <returns>The result of the operation, i.e. \alpha * C, which is a reference to the current instance.</returns>
        /// <remarks>The current instance will be changed and a reference to the current object will be returned.</remarks>
        public DenseMatrix Scale(double alpha)
        {
            BLAS.Level1.dscal(m_RowCount * m_ColumnCount, alpha, m_Data);
            return this;
        }

        /// <summary>Gets 'C = C + \alpha * A, where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="a">The dense matrix 'A' to add.</param>
        /// <param name="alpha">The scalar factor \alpha.</param>
        /// <returns>The result of the operation, i.e. '\alpha * A + this', which is a reference to the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of rows/columns of <paramref name="a"/> does not coincide with the number of rows/columns of the matrix given by the current instance.</exception>
        /// <remarks>The current object will be changed and a reference to the current object will be returned.
        /// <para>
        ///    Use this method for some '+=' or '-=' operator which may performs better than some separate addition.
        /// </para></remarks>
        public DenseMatrix AddAssignment(DenseMatrix a, double alpha)
        {
            Matrix.TestAdditionInput(this, a);
            BLAS.Level1.daxpy(m_RowCount * m_ColumnCount, alpha, a.m_Data, m_Data);
            return this;
        }

        /// <summary>Gets 'C = C + \alpha * A, where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="a">The general band matrix 'A' to add.</param>
        /// <param name="alpha">The scalar factor \alpha.</param>
        /// <returns>The result of the operation, i.e. '\alpha * A + this', which is a reference to the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of rows/columns of <paramref name="a"/> does not coincide with the number of rows/columns of the matrix given by the current instance.</exception>
        /// <remarks>The current object will be changed and a reference to the current object will be returned.
        /// <para>
        ///    Use this method for some '+=' or '-=' operator which may performs better than some separate addition.
        /// </para></remarks>
        public DenseMatrix AddAssignment(GeneralBandMatrix a, double alpha)
        {
            Matrix.TestAdditionInput(this, a);

            int superPlusSubPlusMainDiagonalCount = a.SuperDiagonalCount + a.SubDiagonalCount + 1;
            double[] aData = a.Data;

            for (int i = 0; i < m_RowCount; i++)
            {
                int jMin = Math.Max(i - a.SubDiagonalCount, 0);
                int jMax = Math.Min(i + a.SuperDiagonalCount, m_ColumnCount);

                /* we replace the following code by a corresponding BLAS level 1 call:
                 *
                 *  for (int j = Math.Max(i - a.SubDiagonalCount, 0); j < Math.Min(i + a.SuperDiagonalCount, m_ColumnCount); j++)
                 *     m_Data[i + j * m_RowCount] += alpha * aData[i - j + a.SuperDiagonalCount + j * superPlusSubPluMainDiagonalCount];
                 */
                BLAS.Level1.daxpy(jMax - jMin + 1, alpha, a.Data, m_Data, m_RowCount, superPlusSubPlusMainDiagonalCount, i + jMin * m_RowCount, i - jMin + a.SuperDiagonalCount + jMin * superPlusSubPlusMainDiagonalCount);
            }
            return this;
        }

        /// <summary>Gets 'C = C + \alpha * op(A), where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="a">The dense matrix 'A' to add.</param>
        /// <param name="alpha">The scalar factor \alpha.</param>
        /// <param name="transA">A value indicating whether op(A) = A or op(A) = A'.</param> 
        /// <returns>The result of the operation, i.e. '\alpha * op(A) + this', which is a reference to the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of rows/columns of op(A) does not coincide with the number of rows/columns of the matrix given by the current instance.</exception>
        /// <remarks>The current object will be changed and a reference to the current object will be returned.
        /// <para>
        ///    Use this method for some '+=' or '-=' operator which may performs better than some separate addition.
        /// </para></remarks>
        [CLSCompliant(false)]
        public DenseMatrix AddAssignment(DenseMatrix a, double alpha, BLAS.MatrixTransposeState transA)
        {
            if (transA == BLAS.MatrixTransposeState.NoTranspose)
            {
                return AddAssignment(a, alpha);
            }
            else if (transA == BLAS.MatrixTransposeState.Transpose)
            {
                if (a == null)
                {
                    throw new ArgumentNullException("a", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'a'"));
                }
                if (a.m_ColumnCount != m_RowCount)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "this.RowCount != a.ColumnCount"));
                }
                if (a.m_RowCount != m_ColumnCount)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "this.ColumnCount != a.RowCount"));
                }

                /* c[i+j*m_RowCount] += \alpha * a[j+i*a.RowCount] for i=0,..,m_RowCount-1, j=0,..,m_ColumnCount-1 */
                double[] aData = a.Data;
                for (int i = 0; i < m_RowCount; i++)
                {
                    BLAS.Level1.daxpy(m_ColumnCount, alpha, aData, m_Data, 1, m_RowCount, i * m_ColumnCount, i);
                }
                return this;
            }
            else
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, transA), "transA");
            }
        }

        /// <summary>Gets 'C = C + \alpha * op(A), where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="a">The general band matrix 'A' to add.</param>
        /// <param name="alpha">The scalar factor \alpha.</param>
        /// <param name="transA">A value indicating whether op(A) = A or op(A) = A'.</param> 
        /// <returns>The result of the operation, i.e. '\alpha * op(A) + this', which is just a reference of the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of rows/columns of op(A) does not coincide with the number of rows/columns of the matrix given by the current instance.</exception>
        /// <remarks>The current object will be changed and a reference to the current object will be returned.
        /// <para>
        ///    Use this method for some '+=' or '-=' operator which may performs better than some separate addition.
        /// </para></remarks>
        [CLSCompliant(false)]
        public DenseMatrix AddAssignment(GeneralBandMatrix a, double alpha, BLAS.MatrixTransposeState transA)
        {
            if (transA == BLAS.MatrixTransposeState.NoTranspose)
            {
                return AddAssignment(a, alpha);
            }
            else if (transA == BLAS.MatrixTransposeState.Transpose)
            {
                if (a == null)
                {
                    throw new ArgumentNullException("a", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'a'"));
                }
                if (a.ColumnCount != m_RowCount)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "this.RowCount != a.ColumnCount"));
                }
                if (a.RowCount != m_ColumnCount)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "this.ColumnCount != a.RowCount"));
                }

                double[] aData = a.Data;
                int superPlusSubPlusMainDiagonalCount = a.SuperDiagonalCount + a.SubDiagonalCount + 1;

                for (int i = 0; i < m_RowCount; i++)
                {
                    int jMin = Math.Max(i - a.SuperDiagonalCount, 0);
                    int jMax = Math.Min(i + a.SubDiagonalCount, m_ColumnCount);

                    /* we replace the following code by a corresponding BLAS level 1 call:
                     *
                     *  for (int j = Math.Max(i - a.SuperDiagonalCount, 0); j < Math.Min(i + a.SubDiagonalCount, m_ColumnCount); j++)
                     *     m_Data[i + j * m_RowCount] += alpha * aData[j - i + a.SubDiagonalCount + i * superPlusSubPluMainDiagonalCount];
                     */
                    BLAS.Level1.daxpy(jMax - jMin + 1, alpha, aData, m_Data, 1, m_RowCount, jMin - i + a.SubDiagonalCount + i * superPlusSubPlusMainDiagonalCount, jMin);
                }
                return this;
            }
            else
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, transA), "transA");
            }
        }
        #endregion

        #region "this = \alpha * op(A) * op(B) + \beta * this" methods

        /// <summary>Gets C = \alpha * (A * B) + \beta * C', where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="a">The dense matrix 'A'.</param>
        /// <param name="b">The dense matrix 'B'.</param>
        /// <param name="alpha">The (real) scalar factor \alpha.</param>
        /// <param name="beta">The (real) scalar factor \beta.</param>
        /// <returns>The result of the operation, i.e. '\alpha * (A * B) + \beta * this', which is a reference to the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, 
        /// <para>if the number of columns of <paramref name="a"/> does not coincide with the number of rows of <paramref name="b"/>,</para>
        /// <para>if the number of rows of <paramref name="a"/> is not equal to the number of rows of the current object</para>
        /// <para>if the number of columns of <paramref name="b"/> is not equal to the number of columns of the current object.</para></exception>
        /// <remarks>The current object will be changed.</remarks>
        public DenseMatrix AddAssignment(DenseMatrix a, DenseMatrix b, double alpha, double beta)
        {
            Matrix.TestMultiplicationInput(a, b);
            if (a.m_RowCount != m_RowCount)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentHasWrongDimension, "'a' (RowCount)"));
            }
            if (b.m_ColumnCount != m_ColumnCount)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentHasWrongDimension, "'b' (ColumnCount)"));
            }
            BLAS.Level3.dgemm(a.m_RowCount, b.m_ColumnCount, a.m_ColumnCount, alpha, a.m_Data, b.m_Data, beta, m_Data);
            return this;
        }

        /// <summary>Gets C = \alpha * (op(A) * op(B)) + \beta * C', where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="a">The dense matrix 'A'.</param>
        /// <param name="transA">A value indicating whether op(A) = A or op(A) = A'.</param>
        /// <param name="b">The dense matrix 'B'.</param>
        /// <param name="transB">A value indicating whether op(B) = B or op(B) = B'.</param>
        /// <param name="alpha">The scalar factor \alpha.</param>
        /// <param name="beta">The scalar factor \beta.</param>
        /// <returns>The result of the operation, i.e. '\alpha * (op(A) * op(B)) + \beta * this', which is a reference to the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of row/columns of <paramref name="a"/> and <paramref name="b"/> is not allowed for the operation.</exception>
        /// <remarks>The current object will be changed.</remarks>
        [CLSCompliant(false)]
        public DenseMatrix AddAssignment(DenseMatrix a, BLAS.MatrixTransposeState transA, DenseMatrix b, BLAS.MatrixTransposeState transB, double alpha, double beta)
        {
            if (a == null)
            {
                throw new ArgumentNullException("a", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'a'"));
            }
            if (b == null)
            {
                throw new ArgumentNullException("b", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'b'"));
            }

            // prepare the dgemm (BLAS level 1) parameters:
            int m, n, k;
            if (transA == BLAS.MatrixTransposeState.NoTranspose)
            {
                m = a.m_RowCount;
                k = a.m_ColumnCount;
            }
            else if (transA == BLAS.MatrixTransposeState.Transpose)
            {
                m = a.m_ColumnCount;
                k = a.m_RowCount;
            }
            else
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, transA), "transA");
            }
            if (m != m_RowCount)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "RowCount, op(A).RowCount"));
            }

            if (transB == BLAS.MatrixTransposeState.NoTranspose)
            {
                n = b.m_ColumnCount;
                if (b.m_RowCount != k)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "op(A).ColumnCount, op(B).RowCount"));
                }
            }
            else if (transB == BLAS.MatrixTransposeState.Transpose)
            {
                n = b.m_RowCount;
                if (b.m_ColumnCount != k)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "op(A).ColumnCount, op(B).RowCount"));
                }
            }
            else
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, transB), "transB");
            }
            if (n != m_ColumnCount)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "ColumnCount, op(B).ColumnCount"));
            }

            BLAS.Level3.dgemm(m, n, k, alpha, a.m_Data, b.m_Data, beta, m_Data, transA, transB);
            return this;
        }
        #endregion

        /// <summary>Gets the vector product y := \alpha * C * x, where C represents the matrix specified by the current instance and x is some vector.
        /// </summary>
        /// <param name="x">The vector x with at least <see cref="IMatrix.ColumnCount"/> elements.</param>
        /// <param name="alpha">The (real) scalar \alpha.</param>
        /// <param name="y">The vector y with at least <see cref="IMatrix.RowCount"/> elements (output).</param>
        public void GetVectorProduct(double[] x, double alpha, double[] y)
        {
            if (x == null)
            {
                throw new ArgumentNullException("x", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'x'"));
            }
            if (x.Length < m_ColumnCount)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentOutOfRangeGreater, "x.Length", m_ColumnCount));
            }
            if (y == null)
            {
                throw new ArgumentNullException("y", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'y'"));
            }
            if (y.Length < m_RowCount)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentOutOfRangeGreater, "y.Length", m_RowCount));
            }
            BLAS.Level2.dgemv(m_RowCount, m_ColumnCount, alpha, m_Data, x, 0, y);
        }

        /// <summary>Gets C = D * C, where D is a diagonal matrix and the matrix C is specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="diagonalMatrix">The diagonal matrix, i.e. an array with at least <see cref="IMatrix.RowCount"/> elements.</param>
        /// <returns>The result of the operator, i.e. 'C = D * C', where 'C' represents the matrix specified by the current instance.</returns>
        public DenseMatrix LeftMultiplyDiagonalMatrixAssignment(double[] diagonalMatrix)
        {
            // c_{i,j} = d_{i,i} * c_{i,j}:
            for (int i = 0; i < m_RowCount; i++)
            {
                BLAS.Level1.dscal(m_ColumnCount, diagonalMatrix[i], m_Data, m_RowCount, i);
            }
            return this;
        }
    }
}