/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Globalization;

using Dodoni.BasicComponents;
using Dodoni.MathLibrary.Basics;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.MathLibrary
{
    /// <summary>Represents a general band matrix.
    /// </summary>
    /// <remarks>The matrix entries are provided column-by-column, where <b>each</b> column contains exactly m elements, where m is the 
    /// number of sub-diagonals plus the number of super diagonals plus  one (for the diagonal element) - the general band matrix storage.</remarks>
    public partial class GeneralBandMatrix : IMatrix, IInfoOutputQueriable
    {
        #region private members

        /// <summary>The number of rows, i.e. the first dimension.
        /// </summary>
        private int m_RowCount;

        /// <summary>The number of columns, i.e. the second dimension.
        /// </summary>
        private int m_ColumnCount;

        /// <summary>The matrix entries, provided column-by-column.
        /// </summary>
        private double[] m_Data;

        /// <summary>The number of sub-diagonals of the matrix.
        /// </summary>
        private int m_SubDiagonalCount;

        /// <summary>The number of super-diagonals of the matrix.
        /// </summary>
        private int m_SuperDiagonalCount;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="GeneralBandMatrix"/> class.
        /// </summary>
        /// <param name="rowCount">The number of rows.</param>
        /// <param name="columnCount">The number of columns.</param>
        /// <param name="subDiagonalCount">The number of sub-diagonals of the matrix.</param>
        /// <param name="superDiagonalCount">The number of super-diagonals of the matrix.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if <paramref name="rowCount"/> or <paramref name="columnCount"/> is less or 
        /// equal <c>0</c> or <paramref name="subDiagonalCount"/> or <paramref name="superDiagonalCount"/> is less than <c>0</c>.</exception>
        public GeneralBandMatrix(int rowCount, int columnCount, int subDiagonalCount, int superDiagonalCount)
        {
            Initialize(rowCount, columnCount, subDiagonalCount, superDiagonalCount);
            m_Data = new double[DataLength];
        }

        /// <summary>Initializes a new instance of the <see cref="GeneralBandMatrix"/> class.
        /// </summary>
        /// <param name="rowCount">The number of rows.</param>
        /// <param name="columnCount">The number of columns.</param>
        /// <param name="subDiagonalCount">The number of sub-diagonals of the matrix.</param>
        /// <param name="superDiagonalCount">The number of super-diagonals of the matrix.</param>
        /// <param name="data">The general band matrix provided column-by-column, where <b>each</b> column contains exactly <paramref name="subDiagonalCount"/> + <paramref name="superDiagonalCount"/> + 1 element.</param>
        /// <param name="createDeepCopyOfArgument">A value indicating whether a deep copy of <paramref name="data"/> will be created and stored in the current object.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="data"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if <paramref name="rowCount"/> or <paramref name="columnCount"/> is less or equal <c>0</c> or <paramref name="subDiagonalCount"/> or <paramref name="superDiagonalCount"/> is less than <c>0</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="data"/> contains less elements than <paramref name="columnCount"/> * ( <paramref name="subDiagonalCount"/> + <paramref name="superDiagonalCount"/> + 1) elements.</exception>
        public GeneralBandMatrix(int rowCount, int columnCount, int subDiagonalCount, int superDiagonalCount, double[] data, bool createDeepCopyOfArgument = false)
        {
            Initialize(rowCount, columnCount, subDiagonalCount, superDiagonalCount);
            if (data == null)
            {
                throw new ArgumentNullException("generalBandMatrix");
            }
            if (data.Length < DataLength)
            {
                throw new ArgumentException("generalBandMatrix");
            }
            if (createDeepCopyOfArgument == false)
            {
                m_Data = data;
            }
            else
            {
                m_Data = new double[DataLength];
                BLAS.Level1.dcopy(DataLength, data, m_Data);
            }
        }

        /// <summary>Initializes a new instance of the <see cref="GeneralBandMatrix"/> class.
        /// </summary>
        /// <param name="subDiagonalCount">The number of sub-diagonals of the matrix.</param>
        /// <param name="superDiagonalCount">The number of super-diagonals of the matrix.</param>
        /// <param name="denseBandMatrixData">The dense band matrix, where the first null-based index corresponds to the row, the second to the column index.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="denseBandMatrixData"/> is <c>null</c>.</exception>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="denseBandMatrixData"/> does not represents a dense band matrix.</exception>
        /// <remarks>This constructor creates a deep copy of its argument.</remarks>
        public GeneralBandMatrix(int subDiagonalCount, int superDiagonalCount, double[][] denseBandMatrixData)
        {
            if (denseBandMatrixData == null)
            {
                throw new ArgumentNullException("denseBandMatrixData");
            }
            Initialize(denseBandMatrixData.GetLength(0), denseBandMatrixData[0].Length, subDiagonalCount, superDiagonalCount);
            m_Data = new double[DataLength];

            BLAS.MatrixStorageConversion.CreateDenseBandMatrix(denseBandMatrixData, m_RowCount, m_ColumnCount, m_SubDiagonalCount, m_SuperDiagonalCount, m_Data);
        }

        /// <summary>Initializes a new instance of the <see cref="GeneralBandMatrix"/> class.
        /// </summary>
        /// <param name="subDiagonalCount">The number of sub-diagonals of the matrix.</param>
        /// <param name="superDiagonalCount">The number of super-diagonals of the matrix.</param>
        /// <param name="denseBandMatrixData">The dense band matrix, where the first null-based index corresponds to the row, the second to the column index.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="denseBandMatrixData"/> is <c>null</c>.</exception>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="denseBandMatrixData"/> does not represents a dense band matrix.</exception>
        /// <remarks>This constructor creates a deep copy of its argument.</remarks>
        [CLSCompliant(false)]
        public GeneralBandMatrix(int subDiagonalCount, int superDiagonalCount, double[,] denseBandMatrixData)
        {
            if (denseBandMatrixData == null)
            {
                throw new ArgumentNullException("denseBandMatrixData");
            }
            Initialize(denseBandMatrixData.GetLength(0), denseBandMatrixData.GetLength(1), subDiagonalCount, superDiagonalCount);

            m_Data = new double[DataLength];
            BLAS.MatrixStorageConversion.CreateDenseBandMatrix(denseBandMatrixData, m_RowCount, m_ColumnCount, m_SubDiagonalCount, m_SuperDiagonalCount, m_Data);
        }
        #endregion

        #region public properties

        #region IMaxtrix Members

        /// <summary>Gets the number of rows of the current matrix object.
        /// </summary>
        /// <value>The number of rows.</value>
        public int RowCount
        {
            get { return m_RowCount; }
        }

        /// <summary>Gets the number of columns of the current matrix object.
        /// </summary>
        /// <value>The number of columns.</value>
        public int ColumnCount
        {
            get { return m_ColumnCount; }
        }

        /// <summary>Gets a value indicating whether this instance represents a quadratic matrix.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is quadratic; otherwise, <c>false</c>.
        /// </value>
        public bool IsQuadratic
        {
            get { return (m_ColumnCount == m_RowCount); }
        }

        /// <summary>Gets or sets a specified value.
        /// </summary>
        /// <param name="rowIndex">The null-based row index.</param>
        /// <param name="columnIndex">The null-based column index.</param>
        /// <value>The value of the matrix at the specified coordinate.</value>
        /// <exception cref="IndexOutOfRangeException">Thrown, if the row-/column position is invalid.</exception>
        public double this[int rowIndex, int columnIndex]
        {
            get
            {
                if ((rowIndex > columnIndex + m_SubDiagonalCount) || (columnIndex > rowIndex + m_SuperDiagonalCount))  // i-j > kl or j-i > ku
                {
                    return 0.0;
                }
                return m_Data[rowIndex - columnIndex + m_SuperDiagonalCount + columnIndex * (m_SubDiagonalCount + m_SuperDiagonalCount + 1)];
            }
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Gets the info-output level of detail.
        /// </summary>
        /// <value>The info-output level of detail.</value>
        InfoOutputDetailLevel IInfoOutputQueriable.InfoOutputDetailLevel
        {
            get { return InfoOutputDetailLevel.Full; }
        }
        #endregion

        /// <summary>Gets the number of sub-diagonals of the current <see cref="GeneralBandMatrix"/> object.
        /// </summary>
        public int SubDiagonalCount
        {
            get { return m_SubDiagonalCount; }
        }

        /// <summary>The number of super-diagonals of the current <see cref="GeneralBandMatrix"/> object.
        /// </summary>
        public int SuperDiagonalCount
        {
            get { return m_SuperDiagonalCount; }
        }

        /// <summary>Gets the reference to the internal data, i.e. the matrix entries provided column-by-column, where <b>each</b> column contains
        /// exactly <see cref="SuperDiagonalCount"/> + <see cref="SubDiagonalCount"/> + 1 elements - general band matrix storage.
        /// </summary>
        /// <value>The reference to the internal data.</value>
        public double[] Data
        {
            get { return m_Data; }
        }

        /// <summary>Gets the relevant number of elements of <see cref="Data"/>
        /// </summary>
        /// <value>The number of relevant elements in <see cref="Data"/>.</value>
        public int DataLength
        {
            get { return m_ColumnCount * (1 + m_SubDiagonalCount + m_SuperDiagonalCount); }
        }
        #endregion

        #region public methods

        #region IMatrix Members

        /// <summary>Creates a <see cref="DenseMatrix"/> object from the current object.
        /// </summary>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at
        /// least <see cref="IMatrix.RowCount"/> * <see cref="IMatrix.ColumnCount"/> elements.</param>
        /// <returns>A <see cref="DenseMatrix"/> object that represents the current object.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        public DenseMatrix ToDenseMatrix(double[] matrixData = null)
        {
            double[] dataOfNewDenseMatrix = matrixData;

            if ((dataOfNewDenseMatrix == null) || (dataOfNewDenseMatrix.Length < m_RowCount * m_ColumnCount))
            {
                dataOfNewDenseMatrix = new double[m_RowCount * m_ColumnCount];
            }
            int subPlusSuperPlusMainDiagonalCount = m_SubDiagonalCount + m_SuperDiagonalCount + 1;

            int denseMatrixIndex = 0;
            for (int j = 0; j < m_RowCount; j++)
            {
                int jPlusSubDiagonalCount = j + m_SubDiagonalCount;
                int jMinusSuperDiagonalCount = j - m_SuperDiagonalCount;

                for (int i = 0; i < m_ColumnCount; i++)
                {
                    if ((i > jPlusSubDiagonalCount) || (i < jMinusSuperDiagonalCount))  // i-j > kl or j-i > ku
                    {
                        dataOfNewDenseMatrix[denseMatrixIndex++] = 0.0;
                    }
                    else
                    {
                        dataOfNewDenseMatrix[denseMatrixIndex++] = m_Data[i - j + m_SuperDiagonalCount + j * subPlusSuperPlusMainDiagonalCount];
                    }
                }
            }
            return new DenseMatrix(m_RowCount, m_ColumnCount, dataOfNewDenseMatrix);
        }

        /// <summary>Gets a specific column.
        /// </summary>
        /// <param name="columnIndex">The null-based index of the column.</param>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at
        /// least <see cref="IMatrix.RowCount"/> elements.</param>
        /// <returns>The specified column vector in its <see cref="DenseMatrix"/> representation.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        public DenseMatrix GetColumn(int columnIndex, double[] matrixData = null)
        {
            double[] column = matrixData;
            if ((column == null) || (column.Length < m_RowCount))
            {
                column = new double[m_RowCount];
            }

            // a[i,j] = 0 for i-j>kl or j-i>ku, i.e. i > j+kl or i < j-ku:

            int columnIndexPlusSubDiagonalCount = Math.Min(columnIndex + m_SubDiagonalCount, m_RowCount);
            int columnIndexMinusSuperDiagonalCount = Math.Min(columnIndex - m_SuperDiagonalCount, m_RowCount);

            /* replace the following code by the corresponding BLAS level 1 function call:
             * int indexOffset = columnIndex * (m_SubDiagonalCount + m_SuperDiagonalCount + 1);
             * for (int i = columnIndexPlusSubDiagonalCount; i < columnIndexMinusSuperDiagonalCount; i++)
             * {
             *    column[i] = m_Data[i - columnIndex + m_SuperDiagonalCount + indexOffset];
             * }
             */
            BLAS.Level1.dcopy(
                columnIndexMinusSuperDiagonalCount - columnIndexPlusSubDiagonalCount,
                m_Data,
                column,
                1,
                1,
                m_SuperDiagonalCount + columnIndex * (m_SubDiagonalCount + m_SuperDiagonalCount),
                columnIndexPlusSubDiagonalCount);

            return new DenseMatrix(m_RowCount, 1, column);
        }

        /// <summary>Gets a specific transposed row.
        /// </summary>
        /// <param name="rowIndex">The null-based index of the row.</param>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at
        /// least <see cref="IMatrix.ColumnCount"/> elements.</param>
        /// <returns>The specified transposed row vector in its <see cref="DenseMatrix"/> representation.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        public DenseMatrix GetTransposedRow(int rowIndex, double[] matrixData = null)
        {
            double[] row = matrixData;
            if ((row == null) || (row.Length < m_ColumnCount))
            {
                row = new double[m_ColumnCount];
            }

            // a[i,j] = 0 for i-j>kl or j-i>ku, i.e. j<i-kl or j>i+ku:

            int rowIndexMinusSubDiagonalCount = Math.Min(rowIndex - m_SubDiagonalCount, m_ColumnCount);
            int rowIndexPlusSuperDiagonalCount = Math.Min(rowIndex + m_SuperDiagonalCount, m_ColumnCount);

            int indexOffsetFactor = m_SubDiagonalCount + m_SuperDiagonalCount + 1;

            /* replace the following code by the corresponding BLAS level 1 function call:
             * for (int j = rowIndexMinusSubDiagonalCount; j < rowIndexPlusSuperDiagonalCount; j++)
             * {
             *   row[j] = m_Data[rowIndex - j + m_SuperDiagonalCount + j * indexOffsetFactor];
             * }
             */
            BLAS.Level1.dcopy(
                rowIndexPlusSuperDiagonalCount - rowIndexMinusSubDiagonalCount,
                m_Data,
                row,
                indexOffsetFactor - 1,
                1,
                rowIndex + m_SuperDiagonalCount + rowIndexMinusSubDiagonalCount * (indexOffsetFactor + 1),
                rowIndexMinusSubDiagonalCount);

            return new DenseMatrix(m_ColumnCount, 1, row);
        }

        /// <summary>Gets a specific sub-matrix.
        /// </summary>
        /// <param name="startRowIndex">The null-based index of the upper row.</param>
        /// <param name="endRowIndex">The null-based index of the lower row.</param>
        /// <param name="startColumnIndex">The null-based index of the left column.</param>
        /// <param name="endColumnIndex">The null-based index of the right column.</param>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at
        /// least (<paramref name="endRowIndex"/> - <paramref name="startRowIndex"/> + 1) * (<paramref name="endColumnIndex"/> - <paramref name="startColumnIndex"/> + 1) elements.</param>
        /// <returns>The specified sub-matrix in its <see cref="DenseMatrix"/> representation.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        public DenseMatrix GetSubMatrix(int startRowIndex, int endRowIndex, int startColumnIndex, int endColumnIndex, double[] matrixData = null)
        {
            int subColumnCount = endColumnIndex - startColumnIndex + 1;
            int subRowCount = endRowIndex - startRowIndex + 1;

            if ((subRowCount > m_RowCount) || (subColumnCount > m_ColumnCount))
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "sub-matrix range and given matrix"));
            }
            double[] subMatrix = matrixData;
            if ((subMatrix == null) || (subMatrix.Length < subRowCount * subColumnCount))
            {
                subMatrix = new double[subRowCount * subColumnCount];
            }

            int superPlusSubPlusMainDiagonalCount = m_SuperDiagonalCount + m_SubDiagonalCount + 1;

            int j = startColumnIndex;
            for (int jSubMatrix = 0; jSubMatrix < subColumnCount; jSubMatrix++)
            {
                int i = startRowIndex;

                int subMatrixOffset = jSubMatrix * subRowCount;
                int matrixOffset = j * superPlusSubPlusMainDiagonalCount;
                for (int iSubMatrix = 0; iSubMatrix < subRowCount; iSubMatrix++)
                {
                    if ((i - j > m_SubDiagonalCount) || (j - i > m_SuperDiagonalCount))
                    {
                        subMatrix[i + subMatrixOffset] = 0.0;
                    }
                    else
                    {
                        subMatrix[i + subMatrixOffset] = m_Data[i - j + m_SuperDiagonalCount + matrixOffset];
                    }
                    i++;  // i = startRowIndex + iSubMatrix
                }
                j++;  // j = startColumnInde + jSubMatrix
            }
            return new DenseMatrix(subRowCount, subColumnCount, subMatrix);
        }

        /// <summary>Determines whether the matrix is symmetric.
        /// </summary>
        /// <param name="tolerance">A tolerance taken into account, i.e. if abs(a_{i,j} - a{j,i}) is less than <paramref name="tolerance"/> the values are assumed to be equal.</param>
        /// <returns>
        /// 	<c>true</c> if this instance is symmetric; otherwise, <c>false</c>.
        /// </returns>
        public bool IsSymmetric(double tolerance)
        {
            if (m_RowCount != m_ColumnCount)
            {
                return false;
            }

            int subPlusSuperPlusMainDiagonalCount = m_SubDiagonalCount + m_SuperDiagonalCount + 1;
            for (int j = 0; j < m_ColumnCount; j++)
            {
                for (int i = j - m_SuperDiagonalCount; i < j + m_SubDiagonalCount; j++)
                {
                    int index1 = i - j + m_SuperDiagonalCount + j * subPlusSuperPlusMainDiagonalCount;
                    int index2 = j - i + m_SuperDiagonalCount + i * subPlusSuperPlusMainDiagonalCount;

                    // ignore the entries which are not needed for the matrix representation:
                    if ((index1 >= m_SuperDiagonalCount) && Math.Abs(m_Data[index1] - m_Data[index2]) > tolerance)
                    {
                        return false;
                    }
                }
            }
            return true;
        }

        /// <summary>Gets the trace of a quadratic matrix.
        /// </summary>
        /// <returns>The trace of the matrix, i.e. the sum of the diagonal elements.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if the current object does not represent a quadratic matrix.</exception>
        public double GetTrace()
        {
            if (IsQuadratic == false)
            {
                throw new InvalidOperationException();
            }
            double trace = 0.0;
            for (int j = 0; j < m_RowCount; j++)
            {
                trace += m_Data[m_SuperDiagonalCount + j * (m_SubDiagonalCount + m_SuperDiagonalCount + 1)];
            }
            return trace;
        }

        /// <summary>Gets a specific norm of the current matrix object.
        /// </summary>
        /// <param name="normType">The type of the (matrix) norm.</param>
        /// <returns>The norm of the current instance with respect to the <paramref name="normType"/>.</returns>
        public double GetNorm(Matrix.NormType normType)
        {
            /* here, we assume that the values which are given in the general band matrix storage
             * but not needed for the matrix representation are set to 0.0! */

            switch (normType)
            {
                case Matrix.NormType.FrobeniusNorm:
                    return BLAS.Level1.dnrm2(m_RowCount * m_ColumnCount, m_Data);

                case Matrix.NormType.MaximalNorm:
                    return m_Data[BLAS.Level1.idamax(m_RowCount * m_ColumnCount, m_Data)];

                default: throw new NotImplementedException();
            }
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Gets informations of the current object as a specific <see cref="InfoOutput"/> instance.
        /// </summary>
        /// <param name="infoOutput">The <see cref="InfoOutput"/> object which is to be filled with informations concering the current instance.</param>
        /// <param name="categoryName">The name of the category, i.e. all informations will be added to these category.</param>
        void IInfoOutputQueriable.FillInfoOutput(InfoOutput infoOutput, string categoryName)
        {
            InfoOutputPackage infoOutputCollection = infoOutput.AcquirePackage(categoryName);
            infoOutputCollection.Add(new InfoOutputProperty("RowCount", m_RowCount),
                                     new InfoOutputProperty("ColumnCount", m_ColumnCount));

            var dataTable = new System.Data.DataTable("Data");
            for (int j = 0; j < m_ColumnCount; j++)
            {
                dataTable.Columns.Add(j.ToString(), typeof(double));
            }

            for (int k = 0; k < m_RowCount; k++)
            {
                var row = dataTable.NewRow();
                for (int j = 0; j < m_ColumnCount; j++)
                {
                    row[j] = this[k, j];
                }
                dataTable.Rows.Add(row);
            }
            infoOutputCollection.Add(dataTable);
        }

        /// <summary>Sets the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel"/> property.
        /// </summary>
        /// <param name="infoOutputDetailLevel">The info-output level of detail.</param>
        /// <returns>A value indicating whether the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel"/> has been set to <paramref name="infoOutputDetailLevel"/>.</returns>
        bool IInfoOutputQueriable.TrySetInfoOutputDetailLevel(InfoOutputDetailLevel infoOutputDetailLevel)
        {
            return (infoOutputDetailLevel == InfoOutputDetailLevel.Full);
        }
        #endregion

        /// <summary>Gets a singular value decomposition, i.e. 'A = U * \Sigma *V^t', where the diagonal elements of '\Sigma' are the singular values in descending order.
        /// </summary>
        /// <param name="u">The left singular vectors 'U', i.e. a 'm x m' unitary matrix.</param>
        /// <param name="vt">The right singular vectors 'V^t', i.e. a 'n x n' unitary matrix.</param>
        /// <returns>The singular values in descending order.</returns>
        /// <remarks>The singular values are the roots of the non-negative eigenvalues of A^t*A.
        /// <para>This method changes the current instance!</para></remarks>
        public double[] GetSingularValueDecomposition(out DenseMatrix u, out DenseMatrix vt)
        {
            var uArray = new double[m_RowCount * m_RowCount];
            var vtArray = new double[m_ColumnCount * m_ColumnCount];
            var s = new double[Math.Min(m_RowCount, m_ColumnCount)];

            LAPACK.EigenValues.SingularValueDecomposition.dgbsvd(true, m_RowCount, m_ColumnCount, m_SubDiagonalCount, m_SuperDiagonalCount, m_Data, s, uArray, vtArray);

            u = new DenseMatrix(m_RowCount, m_RowCount, uArray);
            vt = new DenseMatrix(m_ColumnCount, m_ColumnCount, vtArray);
            return s;
        }

        /// <summary>Solves the system of linear equations Ax = b. The current instance will be changed (in-place).
        /// </summary>
        /// <param name="b">The vector b, whose single unique column is the right-hand side for the systems of equations (output: Overwritten by the solution vector x).</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="InvalidOperationException">Thrown, if the current instance does not represent a square matrix.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="b"/> contains less elements than the number of rows/columns of the current instance.</exception>
        /// <remarks>After calling this method the current instance contains the LU decomposition of the specified matrix.</remarks>
        public void SolveSystemOfLinearEquationsInPlace(double[] b)
        {
            if (m_RowCount != m_ColumnCount)
            {
                throw new InvalidOperationException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ObjectIsInvalid, "General band matrix", "not quadratic"));
            }
            if (b == null)
            {
                throw new ArgumentNullException("b");
            }
            if (b.Length < m_RowCount)
            {
                throw new ArgumentException("b");
            }
            int[] ipivot = new int[m_RowCount];
            LAPACK.LinearEquations.DriverRoutines.dgbsv(m_RowCount, m_SubDiagonalCount, m_SuperDiagonalCount, m_Data, b, ipivot);
        }

        /// <summary>Solves the system of linear equations Ax = b.
        /// </summary>
        /// <param name="b">The vector b, whose single unique column is the right-hand side for the systems of equations (output: Overwritten by the solution vector x).</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="InvalidOperationException">Thrown, if the current instance does not represent a square matrix.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="b"/> contains less elements than the number of rows/columns of the current instance.</exception>
        /// <remarks>The current instance will not be changed.</remarks>
        public void SolveSystemOfLinearEquationsOutOfPlace(double[] b)
        {
            if (m_RowCount != m_ColumnCount)
            {
                throw new InvalidOperationException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ObjectIsInvalid, "General band matrix", "not quadratic"));
            }
            if (b == null)
            {
                throw new ArgumentNullException("b");
            }
            if (b.Length < m_RowCount)
            {
                throw new ArgumentException("b");
            }
            int[] ipivot = new int[m_RowCount];
            int length = m_ColumnCount * (1 + m_SubDiagonalCount + m_SuperDiagonalCount);

            double[] data = new double[length];  // create a deep copy of the specific matrix
            BLAS.Level1.dcopy(length, m_Data, data);

            LAPACK.LinearEquations.DriverRoutines.dgbsv(m_RowCount, m_SubDiagonalCount, m_SuperDiagonalCount, data, b, ipivot);
        }

        /// <summary>Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>A <see cref="System.String"/> that represents this instance.</returns>
        public override string ToString()
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.AppendLine(string.Format("Row count: {0:g}, Column count: {0:g}, sub-diagonal count: {0:g}, super-diagonal count: {0:g}", m_RowCount, m_ColumnCount, m_SubDiagonalCount, m_SuperDiagonalCount));

            int subPlusSuperPlusMainDiagonalCount = m_SubDiagonalCount + m_SuperDiagonalCount + 1;

            for (int j = 0; j < m_RowCount; j++)
            {
                int jPlusSubDiagonalCount = j + m_SubDiagonalCount;
                int jMinusSuperDiagonalCount = j - m_SuperDiagonalCount;

                for (int i = 0; i < m_ColumnCount; i++)
                {
                    if ((i > jPlusSubDiagonalCount) || (i < jMinusSuperDiagonalCount))  // i-j > kl or j-i > ku
                    {
                        strBuilder.Append(String.Format("0 "));
                    }
                    else
                    {
                        strBuilder.Append(String.Format("{0:g} ", m_Data[i - j + m_SuperDiagonalCount + j * subPlusSuperPlusMainDiagonalCount]));
                    }
                }
                strBuilder.AppendLine();
            }
            return strBuilder.ToString();
        }
        #endregion

        #region private methods

        /// <summary>Initializes the current instance.
        /// </summary>
        /// <param name="rowCount">The number of rows.</param>
        /// <param name="columnCount">The number of columns.</param>
        /// <param name="subDiagonalCount">The number of sub-diagonal elements.</param>
        /// <param name="superDiagonalCount">The number of super-diagonal elements.</param>
        private void Initialize(int rowCount, int columnCount, int subDiagonalCount, int superDiagonalCount)
        {
            if (rowCount < 1)
            {
                throw new ArgumentOutOfRangeException("rowCount");
            }
            m_RowCount = rowCount;

            if (columnCount < 1)
            {
                throw new ArgumentOutOfRangeException("columnCount");
            }
            m_ColumnCount = columnCount;

            if (subDiagonalCount < 0)
            {
                throw new ArgumentOutOfRangeException("subDiagonalCount");
            }
            m_SubDiagonalCount = subDiagonalCount;

            if (superDiagonalCount < 0)
            {
                throw new ArgumentOutOfRangeException("superDiagonalCount");
            }
            m_SuperDiagonalCount = superDiagonalCount;
        }
        #endregion

        #region public static methods

        #region multiplication operator methods

        /// <summary>The multiplication of two <see cref="GeneralBandMatrix"/> objects, i.e. A * B.
        /// </summary>
        /// <param name="a">The first general band matrix A.</param>
        /// <param name="b">The second general band matrix B.</param>
        /// <returns>The result of the operator, i.e. 'A * B'.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of columns of <paramref name="a"/> does not coincide with the number of rows of <paramref name="b"/>.</exception>
        public static GeneralBandMatrix operator *(GeneralBandMatrix a, GeneralBandMatrix b)
        {
            Matrix.TestMultiplicationInput(a, b);

            int subDiagonalCount = Math.Max(0, a.m_SubDiagonalCount - b.m_SuperDiagonalCount);
            int superDiagonalCount = Math.Max(0, a.m_SuperDiagonalCount - b.m_SubDiagonalCount);

            int length = a.m_RowCount * (subDiagonalCount + superDiagonalCount + 1);
            double[] c = new double[length];

            double[] aData = a.m_Data;
            double[] bData = b.m_Data;
            for (int j = 0; j < b.m_ColumnCount; j++)
            {
                int offsetMatrixC = j * (subDiagonalCount + superDiagonalCount + 1);
                int offsetMatrixB = j * (b.m_SubDiagonalCount + b.m_SuperDiagonalCount + 1);

                for (int i = j - superDiagonalCount; i <= j + subDiagonalCount; i++)
                {
                    double value = 0.0;
                    for (int k = Math.Max(i - a.m_SubDiagonalCount, j - b.m_SuperDiagonalCount); k <= Math.Min(a.m_ColumnCount, Math.Min(i + a.m_SuperDiagonalCount, j + b.m_SubDiagonalCount)); k++)
                    {
                        value += aData[i - k + a.m_SuperDiagonalCount + k * (a.m_SubDiagonalCount + a.m_SuperDiagonalCount + 1)] * bData[k - j + b.m_SuperDiagonalCount + offsetMatrixB];
                    }
                    c[i - j + superDiagonalCount + offsetMatrixC] = value;
                }
            }
            return new GeneralBandMatrix(a.m_RowCount, b.m_ColumnCount, subDiagonalCount, superDiagonalCount, c);
        }

        /// <summary>The multiplication of two <see cref="GeneralBandMatrix"/> objects, i.e. op(A) * op(B).
        /// </summary>
        /// <param name="a">The first general band matrix A.</param>
        /// <param name="transA">A value indicating  whether <paramref name="a"/> will be transposed, i.e.  op(A) = A or op(A) = A'.</param>
        /// <param name="b">The second general band matrix B.</param>
        /// <param name="transB">A value indicating  whether <paramref name="b"/> will be transposed, i.e.  op(B) = B or op(B) = B'.</param>
        /// <returns>The result of the operator, i.e. op(A) * op(B).</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the matrix operation op(A) * op(B) can not be place because of unsuited arguments.</exception>
        [CLSCompliant(false)]
        public static GeneralBandMatrix Multiply(GeneralBandMatrix a, BLAS.MatrixTransposeState transA, GeneralBandMatrix b, BLAS.MatrixTransposeState transB)
        {
            if (a == null)
            {
                throw new ArgumentNullException("a", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'a'"));
            }
            if (b == null)
            {
                throw new ArgumentNullException("b", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'b'"));
            }

            // get the number of rows/columns and of sub-/super-diagonal elements of op(A), op(B) first:
            int subDiagonalCountOpA, subDiagonalCountOpB;
            int superDiagonalCountOpA, superDiagonalCountOpB;

            int m;  // number of rows of op(A) and C
            int n;  // number of columns of op(B) and C
            int k;  // number of columns of op(A) = number of rows of op(B)

            if (transA == BLAS.MatrixTransposeState.NoTranspose)
            {
                subDiagonalCountOpA = a.m_SubDiagonalCount;
                superDiagonalCountOpA = a.m_SuperDiagonalCount;
                m = a.m_RowCount;
                k = a.m_ColumnCount;
            }
            else if (transA == BLAS.MatrixTransposeState.Transpose)
            {
                subDiagonalCountOpA = a.m_SuperDiagonalCount;
                superDiagonalCountOpA = a.m_SubDiagonalCount;

                m = a.m_ColumnCount;
                k = a.m_RowCount;
            }
            else
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, transA), "transA");
            }

            if (transB == BLAS.MatrixTransposeState.NoTranspose)
            {
                n = b.m_ColumnCount;
                if (b.m_RowCount != k)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "op(A).ColumnCount, op(B).RowCount"));
                }
                subDiagonalCountOpB = b.m_SubDiagonalCount;
                superDiagonalCountOpB = b.m_SuperDiagonalCount;
            }
            else if (transB == BLAS.MatrixTransposeState.Transpose)
            {
                n = b.m_RowCount;
                if (b.m_ColumnCount != k)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "op(A).ColumnCount, op(B).RowCount"));
                }
                subDiagonalCountOpB = b.m_SuperDiagonalCount;
                superDiagonalCountOpB = b.m_SubDiagonalCount;
            }
            else
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, transB), "transB");
            }

            // determind the parameters for the result object and allocate memory
            int subDiagonalCount = Math.Max(0, subDiagonalCountOpA - superDiagonalCountOpB);
            int superDiagonalCount = Math.Max(0, superDiagonalCountOpA - subDiagonalCountOpB);

            int length = m * (subDiagonalCount + superDiagonalCount + 1);
            double[] c = new double[length];

            double[] aData = a.m_Data;
            double[] bData = b.m_Data;

            // do a second switch and calculate the result for each case:
            if ((transA == BLAS.MatrixTransposeState.Transpose) && (transB == BLAS.MatrixTransposeState.Transpose))
            {
                for (int j = 0; j < n; j++)
                {
                    int offsetMatrixA = j * (subDiagonalCountOpA + superDiagonalCountOpA + 1);
                    int offsetMatrixC = j * (subDiagonalCount + superDiagonalCount + 1);

                    for (int i = j - superDiagonalCount; i <= j + subDiagonalCount; i++)
                    {
                        double value = 0.0;
                        for (int s = Math.Max(i - subDiagonalCountOpA, j - superDiagonalCountOpB); s <= Math.Min(k, Math.Min(i + superDiagonalCountOpA, j + subDiagonalCountOpB)); s++)
                        {
                            value += aData[s - i + superDiagonalCountOpA + offsetMatrixA] * bData[j - s + superDiagonalCountOpB + s * (subDiagonalCountOpB + superDiagonalCountOpB + 1)];
                        }
                        c[i - j + superDiagonalCount + offsetMatrixC] = value;
                    }
                }
            }
            else if (transA == BLAS.MatrixTransposeState.Transpose)
            {
                for (int j = 0; j < n; j++)
                {
                    int offsetMatrixA = j * (subDiagonalCountOpA + superDiagonalCountOpA + 1);
                    int offsetMatrixB = j * (subDiagonalCountOpB + superDiagonalCountOpB + 1);
                    int offsetMatrixC = j * (subDiagonalCount + superDiagonalCount + 1);

                    for (int i = j - superDiagonalCount; i <= j + subDiagonalCount; i++)
                    {
                        double value = 0.0;
                        for (int s = Math.Max(i - subDiagonalCountOpA, j - superDiagonalCountOpB); s <= Math.Min(k, Math.Min(i + superDiagonalCountOpA, j + subDiagonalCountOpB)); s++)
                        {
                            value += aData[s - i + superDiagonalCountOpA + offsetMatrixA] * bData[s - j + superDiagonalCountOpB + offsetMatrixB];
                        }
                        c[i - j + superDiagonalCount + offsetMatrixC] = value;
                    }
                }
            }
            else if (transB == BLAS.MatrixTransposeState.Transpose)
            {
                for (int j = 0; j < n; j++)
                {
                    int offsetMatrixC = j * (subDiagonalCount + superDiagonalCount + 1);

                    for (int i = j - superDiagonalCount; i <= j + subDiagonalCount; i++)
                    {
                        double value = 0.0;
                        for (int s = Math.Max(i - subDiagonalCountOpA, j - superDiagonalCountOpB); s <= Math.Min(k, Math.Min(i + superDiagonalCountOpA, j + subDiagonalCountOpB)); s++)
                        {
                            value += aData[i - s + superDiagonalCountOpA + s * (subDiagonalCountOpA + superDiagonalCountOpA + 1)] * bData[j - s + superDiagonalCountOpB + s * (subDiagonalCountOpB + superDiagonalCountOpB + 1)];
                        }
                        c[i - j + superDiagonalCount + offsetMatrixC] = value;
                    }
                }
            }
            else  // op(A) = A, op(B) = B
            {
                for (int j = 0; j < n; j++)
                {
                    int offsetMatrixC = j * (subDiagonalCount + superDiagonalCount + 1);
                    int offsetMatrixB = j * (subDiagonalCountOpB + superDiagonalCountOpB + 1);

                    for (int i = j - superDiagonalCount; i <= j + subDiagonalCount; i++)
                    {
                        double value = 0.0;
                        for (int s = Math.Max(i - subDiagonalCountOpA, j - superDiagonalCountOpB); s <= Math.Min(k, Math.Min(i + superDiagonalCountOpA, j + subDiagonalCountOpB)); s++)
                        {
                            value += aData[i - s + superDiagonalCountOpA + s * (subDiagonalCountOpA + superDiagonalCountOpA + 1)] * bData[s - j + superDiagonalCountOpB + offsetMatrixB];
                        }
                        c[i - j + superDiagonalCount + offsetMatrixC] = value;
                    }
                }
            }
            return new GeneralBandMatrix(m, n, subDiagonalCount, superDiagonalCount, c);
        }

        /// <summary>The multiplication of a <see cref="GeneralBandMatrix"/> and a specific scalar.
        /// </summary>
        /// <param name="alpha">The scalar \alpha.</param>
        /// <param name="a">The general band matrix A.</param>
        /// <returns>The result of the operator, i.e. \alpha * A.</returns>
        public static GeneralBandMatrix operator *(double alpha, GeneralBandMatrix a)
        {
            int length = a.DataLength;
            double[] c = new double[length];
            BLAS.Level1.dcopy(length, a.m_Data, c);
            BLAS.Level1.dscal(length, alpha, c);

            return new GeneralBandMatrix(a.m_RowCount, a.m_ColumnCount, a.m_SubDiagonalCount, a.m_SuperDiagonalCount, c);
        }

        /// <summary>The multiplication of a <see cref="GeneralBandMatrix"/> and a specific scalar.
        /// </summary>
        /// <param name="a">The general band matrix A.</param>
        /// <param name="alpha">The scalar \alpha.</param>
        /// <returns>The result of the operator, i.e. \alpha * A.</returns>
        public static GeneralBandMatrix operator *(GeneralBandMatrix a, double alpha)
        {
            int length = a.DataLength;
            double[] c = new double[length];
            BLAS.Level1.dcopy(length, a.m_Data, c);
            BLAS.Level1.dscal(length, alpha, c);

            return new GeneralBandMatrix(a.m_RowCount, a.m_ColumnCount, a.m_SubDiagonalCount, a.m_SuperDiagonalCount, c);
        }
        #endregion

        #region additive operator methods

        /// <summary>The addition of two <see cref="GeneralBandMatrix"/> objects.
        /// </summary>
        /// <param name="a">The first general band matrix A.</param>
        /// <param name="b">The second general band matrix B.</param>
        /// <returns>The result of the operator, i.e. 'A + B'.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of columns of <paramref name="a"/> does not coincide with the number of rows of <paramref name="b"/>.</exception>
        public static GeneralBandMatrix operator +(GeneralBandMatrix a, GeneralBandMatrix b)
        {
            Matrix.TestAdditionInput(a, b);
            int subDiagonalCount = Math.Max(a.m_SubDiagonalCount, b.m_SubDiagonalCount);
            int superDiagonalCount = Math.Max(a.m_SuperDiagonalCount, b.m_SuperDiagonalCount);

            int length = a.m_RowCount * (subDiagonalCount + superDiagonalCount + 1);
            double[] c = new double[length];

            double[] aData = a.m_Data;
            double[] bData = b.m_Data;
            for (int j = 0; j < a.m_ColumnCount; j++)
            {
                int offSetMatrixA = j * (a.m_SubDiagonalCount + a.m_SuperDiagonalCount + 1);
                int offSetMatrixB = j * (b.m_SubDiagonalCount + b.m_SuperDiagonalCount + 1);
                int offsetMatrixC = j * (subDiagonalCount + superDiagonalCount + 1);

                for (int i = j - superDiagonalCount; i <= j + subDiagonalCount; i++)
                {
                    double value = 0.0;
                    if ((i - j <= a.m_SubDiagonalCount) && (j - i <= a.m_SuperDiagonalCount))
                    {
                        value += aData[i - j + a.m_SuperDiagonalCount + offSetMatrixA];
                    }
                    if ((i - j <= b.m_SubDiagonalCount) && (j - i <= b.m_SuperDiagonalCount))
                    {
                        value += bData[i - j + b.m_SuperDiagonalCount + offSetMatrixB];
                    }
                    c[i - j + superDiagonalCount + offsetMatrixC] = value;
                }
            }
            return new GeneralBandMatrix(a.m_RowCount, b.m_ColumnCount, subDiagonalCount, superDiagonalCount, c);
        }

        /// <summary>The subtraction of two <see cref="GeneralBandMatrix"/> objects.
        /// </summary>
        /// <param name="a">The first general band matrix A.</param>
        /// <param name="b">The second general band matrix B.</param>
        /// <returns>The result of the operator, i.e. 'a - b'.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of columns of <paramref name="a"/> does not coincide with the number of rows of <paramref name="b"/>.</exception>
        public static GeneralBandMatrix operator -(GeneralBandMatrix a, GeneralBandMatrix b)
        {
            Matrix.TestAdditionInput(a, b);
            int subDiagonalCount = Math.Max(a.m_SubDiagonalCount, b.m_SubDiagonalCount);
            int superDiagonalCount = Math.Max(a.m_SuperDiagonalCount, b.m_SuperDiagonalCount);

            int length = a.m_RowCount * (subDiagonalCount + superDiagonalCount + 1);
            double[] c = new double[length];

            double[] aData = a.m_Data;
            double[] bData = b.m_Data;
            for (int j = 0; j < a.m_ColumnCount; j++)
            {
                int offSetMatrixA = j * (a.m_SubDiagonalCount + a.m_SuperDiagonalCount + 1);
                int offSetMatrixB = j * (b.m_SubDiagonalCount + b.m_SuperDiagonalCount + 1);
                int offsetMatrixC = j * (subDiagonalCount + superDiagonalCount + 1);

                for (int i = j - superDiagonalCount; i <= j + subDiagonalCount; i++)
                {
                    double value = 0.0;
                    if ((i - j <= a.m_SubDiagonalCount) && (j - i <= a.m_SuperDiagonalCount))
                    {
                        value += aData[i - j + a.m_SuperDiagonalCount + offSetMatrixA];
                    }
                    if ((i - j <= b.m_SubDiagonalCount) && (j - i <= b.m_SuperDiagonalCount))
                    {
                        value -= bData[i - j + b.m_SuperDiagonalCount + offSetMatrixB];
                    }
                    c[i - j + superDiagonalCount + offsetMatrixC] = value;
                }
            }
            return new GeneralBandMatrix(a.m_RowCount, b.m_ColumnCount, subDiagonalCount, superDiagonalCount, c);
        }
        #endregion

        #endregion
    }
}