﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Globalization;

using Dodoni.BasicComponents;
using Dodoni.MathLibrary.Basics;

namespace Dodoni.MathLibrary
{
    public partial class GeneralBandMatrix
    {
        #region scaling operation and "this = \alpha * op(A) + this" methods

        /// <summary>Scales the current matrix, i.e. C = \alpha * C, where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="alpha">The (real) scalar.</param>
        /// <returns>The result of the operation, i.e. '\alpha * this', which is a reference to the current instance.</returns>
        /// <remarks>The current instance will be changed and a reference to the current object will be returned.</remarks>
        public GeneralBandMatrix Scale(double alpha)
        {
            BLAS.Level1.dscal(m_RowCount * m_ColumnCount, alpha, m_Data);
            return this;
        }

        /// <summary>Gets 'C = C + \alpha * A, where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="a">The general band matrix 'A' to add.</param>
        /// <param name="alpha">The scalar factor \alpha.</param>
        /// <returns>The result of the operation, i.e. '\alpha * A + this', which is a reference to the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of rows/columns of <paramref name="a"/> does not coincide with the number of rows/columns 
        /// of the matrix specified by the current instance or if the number of sub-/super diagonal elements of <paramref name="a"/> is greater than the corresponding number of the current instance.</exception>
        /// <remarks>The current object will be changed and a reference to the current object will be returned.
        /// <para>
        ///    Use this method for some '+=' or '-=' operator which may performs better than some separate addition.
        /// </para></remarks>
        public GeneralBandMatrix AddAssignment(GeneralBandMatrix a, double alpha)
        {
            Matrix.TestAdditionInput(this, a);

            if ((a.m_SubDiagonalCount == m_SubDiagonalCount) && (a.m_SuperDiagonalCount == m_SuperDiagonalCount))
            {
                BLAS.Level1.daxpy(m_RowCount * m_ColumnCount, alpha, a.m_Data, m_Data);  // the same matrix-storage format
            }
            else if ((a.m_SubDiagonalCount > m_SubDiagonalCount) || (a.m_SuperDiagonalCount > m_SuperDiagonalCount))
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, "a {sub-/super diagonal count}"));
            }
            else
            {
                /* set "m_Data[i-j+ superDiagonalCount + j * (subDiagonalCount + superDiagonalCount+1)] += alpha * a.m_Data[i-j+ a.SuperDiagonalCount + j * (a.SubDiagonalCount + a.SuperDiagonalCount+1)]" 
                 * for all indices i,j where the right (and left) side is well defined
                 */
                for (int j = 0; j < m_ColumnCount; j++)
                {
                    int offset = j * (m_SubDiagonalCount + m_SuperDiagonalCount + 1);
                    int offsetMatrixA = j * (a.m_SubDiagonalCount + a.m_SuperDiagonalCount + 1);
                    for (int i = Math.Max(j - m_SuperDiagonalCount, j - a.m_SuperDiagonalCount); i <= Math.Min(j + m_SubDiagonalCount, j + a.m_SubDiagonalCount); i++)
                    {
                        m_Data[i - j + m_SuperDiagonalCount + offset] += alpha * a.m_Data[i - j + a.m_SuperDiagonalCount + offsetMatrixA];
                    }
                }
            }
            return this;
        }

        /// <summary>Gets 'C = C + \alpha * op(A), where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="a">The general band matrix 'A' to add.</param>
        /// <param name="alpha">The scalar factor \alpha.</param>
        /// <param name="transA">A value indicating whether op(A) = A or op(A) = A'.</param> 
        /// <returns>The result of the operation, i.e. '\alpha * op(A) + this', which is a reference of the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of rows/columns of op(A) does not coincide with the number of rows/columns of the matrix specified 
        /// by the current instance or if the number of sub-/super diagonal elements of <paramref name="a"/> is greater than the corresponding number of the current instance.</exception>
        /// <remarks>The current object will be changed and a reference to the current object will be returned.
        /// <para>
        ///    Use this method for some '+=' or '-=' operator which may performs better than some separate addition.
        /// </para></remarks>
        [CLSCompliant(false)]
        public GeneralBandMatrix AddAssignment(GeneralBandMatrix a, double alpha, BLAS.MatrixTransposeState transA)
        {
            if (transA == BLAS.MatrixTransposeState.NoTranspose)
            {
                return AddAssignment(a, alpha);
            }
            else if (transA == BLAS.MatrixTransposeState.Transpose)
            {
                if (a == null)
                {
                    throw new ArgumentNullException("a", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'a'"));
                }
                if (a.ColumnCount != m_RowCount)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "this.RowCount != a.ColumnCount"));
                }
                if (a.RowCount != m_ColumnCount)
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "this.ColumnCount != a.RowCount"));
                }
                if ((a.m_SubDiagonalCount > m_SuperDiagonalCount) || (a.m_SuperDiagonalCount > m_SubDiagonalCount))
                {
                    throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, "a {sub-/super diagonal count}"));
                }

                /* set "m_Data[i-j+ superDiagonalCount + j * (subDiagonalCount + superDiagonalCount+1)] += alpha * a.m_Data[j-i+ a.SubDiagonalCount + i * (a.SubDiagonalCount + a.SuperDiagonalCount+1)]" 
                 * for all indices i,j where the right (and left) side is well defined
                 */

                for (int j = 0; j < m_ColumnCount; j++)  // parallel - more improvements possible?
                {
                    int offset = j * (m_SubDiagonalCount + m_SuperDiagonalCount + 1);
                    for (int i = Math.Max(j - m_SuperDiagonalCount, j - a.m_SubDiagonalCount); i <= Math.Min(j + m_SubDiagonalCount, j + a.m_SuperDiagonalCount); i++)
                    {
                        m_Data[i - j + m_SuperDiagonalCount + offset] += alpha * a.m_Data[j - i + a.m_SubDiagonalCount + i * (a.m_SubDiagonalCount + a.m_SuperDiagonalCount + 1)];
                    }
                }
                return this;
            }
            else
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, transA), "transA");
            }
        }
        #endregion

        #region "this = \alpha * op(A) * op(B) + \beta * this" methods

        /// <summary>Gets C = \alpha * (A * B) + \beta * C', where 'C' represents the matrix specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="a">The first matrix A.</param>
        /// <param name="b">The second matrix B.</param>
        /// <param name="alpha">The scalar factor \alpha.</param>
        /// <param name="beta">The scalar factor \beta.</param>
        /// <returns>The result of the operation, i.e. '\alpha * (A * B) + \beta * this', which is a reference to the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="a"/> or <paramref name="b"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, 
        /// <para>if the number of columns of <paramref name="a"/> does not coincide with the number of rows of <paramref name="b"/>.</para>
        /// or
        /// <para>if the number of rows of <paramref name="a"/> is not equal to the number of rows of the current object</para>
        /// or
        /// <para>if the number of columns of <paramref name="b"/> is not equal to the number of columns of the current object.</para></exception>
        /// <exception cref="ArithmeticException">Thrown, if the product A*B represents a general band matrix where the number of
        /// sub-/super-diagonal elements is greater than the corresponding number of the current instance.</exception>
        /// <remarks>The current object will be changed.</remarks>
        public GeneralBandMatrix AddAssignment(GeneralBandMatrix a, GeneralBandMatrix b, double alpha, double beta)
        {
            /* do some checks first: */
            Matrix.TestMultiplicationInput(a, b);
            if (a.m_RowCount != m_RowCount)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentHasWrongDimension, "'A' (RowCount)"));
            }
            if (b.m_ColumnCount != m_ColumnCount)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentHasWrongDimension, "'B' (ColumnCount)"));
            }

            int subDiagonalCount = Math.Max(0, a.m_SubDiagonalCount - b.m_SuperDiagonalCount);
            if (subDiagonalCount > m_SubDiagonalCount)
            {
                throw new ArithmeticException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "'A,B' (sub-diagonal count of A*B exceeds allowed size)"));
            }
            int superDiagonalCount = Math.Max(0, a.m_SuperDiagonalCount - b.m_SubDiagonalCount);
            if (superDiagonalCount > m_SuperDiagonalCount)
            {
                throw new ArithmeticException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentCombinationInvalid, "'A,B' (super-diagonal count of A*B exceeds allowed size)"));
            }

            double[] aData = a.m_Data;
            double[] bData = b.m_Data;
            for (int j = 0; j < b.m_ColumnCount; j++)
            {
                int offsetMatrixC = j * (m_SubDiagonalCount + m_SuperDiagonalCount + 1);
                int offsetMatrixB = j * (b.m_SubDiagonalCount + b.m_SuperDiagonalCount + 1);

                for (int i = j - superDiagonalCount; i <= j + subDiagonalCount; i++)
                {
                    double value = 0.0;
                    for (int k = Math.Max(i - a.m_SubDiagonalCount, j - b.m_SuperDiagonalCount); k <= Math.Min(a.m_ColumnCount, Math.Min(i + a.m_SuperDiagonalCount, j + b.m_SubDiagonalCount)); k++)
                    {
                        value += aData[i - k + a.m_SuperDiagonalCount + k * (a.m_SubDiagonalCount + a.m_SuperDiagonalCount + 1)] * bData[k - j + b.m_SuperDiagonalCount + offsetMatrixB];
                    }
                    m_Data[i - j + m_SuperDiagonalCount + offsetMatrixC] = value + beta * m_Data[i - j + m_SuperDiagonalCount + offsetMatrixC];
                }
            }
            return this;
        }

        /// <summary>Gets A = D * A, where D is a diagonal matrix and the matrix A is specified by the current instance. The current instance will be changed.
        /// </summary>
        /// <param name="diagonalMatrix">The diagonal matrix, i.e. an array with at least <see cref="IMatrix.RowCount"/> elements.</param>
        /// <returns>The result of the operator, i.e. 'A = D * A', where 'A' represents the matrix specified by the current instance.</returns>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="diagonalMatrix"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if the number of elements of <paramref name="diagonalMatrix"/> is less than the number of rows of the current object.</exception>
        public GeneralBandMatrix LeftMultiplyDiagonalMatrixAssignment(double[] diagonalMatrix)
        {
            /* set 'a_{i,j} = d_{i,i} * a_{i,j}' and it holds
             *    a_{ij} = m_Data[i - j + m_SuperDiagonalCount + j*(m_SuperDiagonalCount + m_SubDiagonalCount +1)]
             * for i=0,...,m_RowCount-1 and j=i-m_SubDiagonalCount,..., i+m_SuperDiagonalCount. 
             *
             * Substitute j = (i-m_SubDiagonalCount) + k with k =0,..,k=m_SubDiagonalCount + m_SuperDiagonalCount,
             * i.e. 
             *    a_{ij} = m_Data[i+m_SuperDiagonalCount + (i-m_SubDiagonalCount)*(m_SuperDiagonalCount+m_SubDiagonalCount) + k * (m_SuperDiagonalCount+m_SubDiagonalCount)]
             */

            if (diagonalMatrix == null)
            {
                throw new ArgumentNullException("diagonalMatrix", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'diagonalMatrix'"));
            }
            if (diagonalMatrix.Length < m_RowCount)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentHasWrongDimension, "diagonalMatrix (length < number of rows)"), "diagonalMatrix");
            }

            int subPlusSuperDiagonalCount = m_SubDiagonalCount + m_SuperDiagonalCount;
            int subPlusSuperPlusMainDiagonalCount = subPlusSuperDiagonalCount + 1;

            for (int i = 0; i < m_RowCount; i++)
            {
                BLAS.Level1.dscal(subPlusSuperPlusMainDiagonalCount, diagonalMatrix[i], m_Data, subPlusSuperDiagonalCount, i + m_SuperDiagonalCount + (i - m_SubDiagonalCount) * subPlusSuperDiagonalCount);
            }
            return this;
        }
        #endregion

        /// <summary>Gets the vector product y = \alpha * op(C) * x + \beta * y, where C represents the matrix specified by the current instance and x is some vector.
        /// </summary>
        /// <param name="x">The vector x with at least <see cref="IMatrix.ColumnCount"/> elements if op(C) = C and at least <see cref="IMatrix.RowCount"/> elements if op(C) = C'.</param>
        /// <param name="trans">A value indicating whether op(C) = C or op(C) = C'.</param>
        /// <param name="alpha">The (real) scalar factor \alpha.</param>
        /// <param name="beta">The (real) scalar factor \beta.</param>
        /// <param name="y">The vector y with at least <see cref="IMatrix.RowCount"/> elements if op(C) = C or at least <see cref="IMatrix.ColumnCount"/> elements if op(C) = C' (output).</param>
        [CLSCompliant(false)]
        public void GetVectorProduct(double[] x, BLAS.MatrixTransposeState trans, double alpha, double beta, double[] y)
        {
            if (x == null)
            {
                throw new ArgumentNullException("x", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'x'"));
            }
            int minLengthX, minLengthY;
            if (trans == BLAS.MatrixTransposeState.NoTranspose)
            {
                minLengthX = m_ColumnCount;
                minLengthY = m_RowCount;
            }
            else if (trans == BLAS.MatrixTransposeState.Transpose)
            {
                minLengthX = m_RowCount;
                minLengthY = m_ColumnCount;
            }
            else
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, trans), "trans");
            }

            if (x.Length < minLengthX)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentOutOfRangeGreater, "x.Length", minLengthX));
            }
            if (y == null)
            {
                throw new ArgumentNullException("y", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "'y'"));
            }
            if (y.Length < minLengthY)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentOutOfRangeGreater, "y.Length", minLengthY));
            }
            BLAS.Level2.dgbmv(m_RowCount, m_ColumnCount, m_SubDiagonalCount, m_SuperDiagonalCount, alpha, m_Data, x, beta, y, trans);
        }
    }
}