﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.MathLibrary.GridPointCurves
{
    public partial class CurveExtrapolationLinear
    {
        /// <summary>A linear extrapolation where the slope is specified by the derivative at the first/last grid point.
        /// </summary>
        private class Differentiable : GridPointCurve.Extrapolator
        {
            #region nested classes

            /// <summary>Serves as implementation of the extrapolation approach.
            /// </summary>
            private class Extrapolator : ICurveExtrapolator, IDifferentiableRealValuedCurve
            {
                #region private members

                /// <summary>The curve interpolator in its <see cref="IDifferentiableRealValuedCurve"/> representation.
                /// </summary>
                private IDifferentiableRealValuedCurve m_CurveInterpolator;

                /// <summary>The slope of the linear extrapolation.
                /// </summary>
                private double m_Slope;

                /// <summary>The reference point, which is equal to the lower or upper bound of the curve.
                /// </summary>
                private double m_ReferencePoint;

                /// <summary>The reference value, i.e. the value at the <see cref="m_ReferencePoint"/>.
                /// </summary>
                private double m_ReferenceValue;

                /// <summary>A value indicating whether to extrapolate to +\infinity or -\infinity.
                /// </summary>
                private BuildingDirection m_BuildingDirection;
                #endregion

                #region public constructors

                /// <summary>Initializes a new instance of the <see cref="Extrapolator"/> class.
                /// </summary>
                /// <param name="curveInterpolator">The interpolation approach of the curve.</param>
                /// <param name="buildingDirection">The building direction of the curve extrapolation.</param>
                /// <param name="extrapolatorFactory">The <see cref="GridPointCurve.Extrapolator"/> object that serves as factory for the current object.</param>
                public Extrapolator(ICurveBuilder curveInterpolator, BuildingDirection buildingDirection, GridPointCurve.Extrapolator extrapolatorFactory)
                {
                    if (curveInterpolator == null)
                    {
                        throw new ArgumentNullException("curveInterpolator");
                    }
                    if ((curveInterpolator is IDifferentiableRealValuedCurve) == false)
                    {
                        throw new ArgumentException("curveInterpolator");
                    }
                    m_CurveInterpolator = (IDifferentiableRealValuedCurve)curveInterpolator;

                    m_BuildingDirection = buildingDirection;
                    m_Slope = Double.NaN;
                    m_ReferencePoint = Double.NaN;
                    m_ReferenceValue = Double.NaN;
                    Factory = extrapolatorFactory;
                }
                #endregion

                #region public properties

                #region ICurveExtrapolator Members

                /// <summary>Gets the factory of <see cref="ICurveExtrapolator" /> objects of the same type and configuration.
                /// </summary>
                /// <value>The factory of <see cref="ICurveExtrapolator" /> objects of the same type and configuration.</value>
                public GridPointCurve.Extrapolator Factory
                {
                    get;
                    private set;
                }
                #endregion

                #region IOperable Members

                /// <summary>Gets a value indicating whether this instance is operable.
                /// </summary>
                /// <value><c>true</c> if this instance is operable; otherwise, <c>false</c>.</value>
                public bool IsOperable
                {
                    get { return (Double.IsNaN(m_Slope) == false) && (Double.IsNaN(m_ReferencePoint) == false) && (Double.IsNaN(m_ReferenceValue) == false); }
                }
                #endregion

                #region IRealValuedCurve Members

                /// <summary>Gets the lower bound of the domain of definition.
                /// </summary>
                /// <value>The lower bound of the domain of definition, perhaps <see cref="System.Double.NegativeInfinity"/> or <see cref="System.Double.NaN"/>.</value>
                public double LowerBound
                {
                    get { return (m_BuildingDirection == BuildingDirection.FromFirstGridPoint) ? Double.NegativeInfinity : m_ReferencePoint; }
                }

                /// <summary>Gets the upper bound of the domain of definition.
                /// </summary>
                /// <value>The upper bound of the domain of definition, perhaps <see cref="System.Double.PositiveInfinity"/> or <see cref="System.Double.NaN"/>.</value>
                public double UpperBound
                {
                    get { return (m_BuildingDirection == BuildingDirection.FromLastGridPoint) ? Double.PositiveInfinity : m_ReferencePoint; }
                }
                #endregion

                #region IInfoOutputQueriable Members

                /// <summary>Gets the info-output level of detail.
                /// </summary>
                /// <value>The info-output level of detail.</value>
                public InfoOutputDetailLevel InfoOutputDetailLevel
                {
                    get { return InfoOutputDetailLevel.Full; }
                }
                #endregion

                #endregion

                #region public methods

                #region ICurveExtrapolator Members

                /// <summary>Updates the current curve extrapolator.
                /// </summary>
                /// <remarks>This method should be called if grid points have been changed, added, removed etc. and before evaluating the grid point curve at a specified point.
                /// <para>If no problem occurred, the flag <see cref="IOperable.IsOperable"/> will be set to <c>true</c>.</para>
                /// </remarks>
                public void Update()
                {
                    m_ReferencePoint = (m_BuildingDirection == BuildingDirection.FromFirstGridPoint) ? m_CurveInterpolator.LowerBound : m_CurveInterpolator.UpperBound;
                    m_ReferenceValue = m_CurveInterpolator.GetValue(m_ReferencePoint);
                    m_Slope = m_CurveInterpolator.GetDerivative(m_ReferencePoint);
                }
                #endregion

                #region IRealValuedCurve Members

                /// <summary>Gets the value at a specific argument.
                /// </summary>
                /// <param name="pointToEvaluate">The point to evaluate.</param>
                /// <returns>The value of the curve at <paramref name="pointToEvaluate"/>.</returns>
                /// <remarks>The argument must be an element of the domain of definition, represented by <see cref="IRealValuedCurve.LowerBound"/> and <see cref="IRealValuedCurve.UpperBound"/>.</remarks>
                public double GetValue(double pointToEvaluate)
                {
                    return m_Slope * (pointToEvaluate - m_ReferencePoint) + m_ReferenceValue;
                }

                /// <summary>Gets the value of the integral \int_a^b f(x) dx.
                /// </summary>
                /// <param name="lowerBound">The lower bound.</param>
                /// <param name="upperBound">The upper bound.</param>
                /// <returns>The value of \int_a^b f(x) dx.</returns>
                /// <remarks>The arguments must be elements of the domain of definition, represented by <see cref="IRealValuedCurve.LowerBound"/> and <see cref="IRealValuedCurve.UpperBound"/>.</remarks>
                public double GetIntegral(double lowerBound, double upperBound)
                {
                    /* integration of 't \mapsto a * (t - t1) + x1' : */
                    return m_Slope * (0.5 * (upperBound * upperBound - lowerBound * lowerBound) - m_ReferencePoint * (upperBound - lowerBound)) + m_ReferenceValue * (upperBound - lowerBound);
                }
                #endregion

                #region IDifferentiableRealValuedCurve Members

                /// <summary>Gets the derivative at a specific point with respect to the extrapolation rule.
                /// </summary>
                /// <param name="pointToEvaluate">The point to evaluate.</param>
                /// <returns>The value of the derivative at the <paramref name="pointToEvaluate"/>.</returns>
                double IDifferentiableRealValuedCurve.GetDerivative(double pointToEvaluate)
                {
                    return m_Slope;
                }
                #endregion

                #region IInfoOutputQueriable Members

                /// <summary>Gets informations of the current object as a specific <see cref="InfoOutput" /> instance.
                /// </summary>
                /// <param name="infoOutput">The <see cref="InfoOutput" /> object which is to be filled with informations concering the current instance.</param>
                /// <param name="categoryName">The name of the category, i.e. all informations will be added to these category.</param>
                public void FillInfoOutput(InfoOutput infoOutput, string categoryName = "General")
                {
                    var infoOutputPackage = infoOutput.AcquirePackage(categoryName);

                    infoOutputPackage.Add("Name", Factory.Name.String);
                    infoOutputPackage.Add("Long Name", Factory.LongName.String);
                    infoOutputPackage.Add("Building direction", m_BuildingDirection);

                    infoOutputPackage.Add("Reference Point", m_ReferencePoint);
                    infoOutputPackage.Add("Reference Value", m_ReferenceValue);
                }

                /// <summary>Sets the <see cref="P:Dodoni.BasicComponents.Containers.IInfoOutputQueriable.InfoOutputDetailLevel"/> property.
                /// </summary>
                /// <param name="infoOutputDetailLevel">The info-output level of detail.</param>
                /// <returns>A value indicating whether the <see cref="P:Dodoni.BasicComponents.Containers.IInfoOutputQueriable.InfoOutputDetailLevel"/> has been set to <paramref name="infoOutputDetailLevel"/>.</returns>
                public bool TrySetInfoOutputDetailLevel(InfoOutputDetailLevel infoOutputDetailLevel)
                {
                    return (infoOutputDetailLevel == InfoOutputDetailLevel.Full);
                }
                #endregion

                #endregion
            }
            #endregion

            #region private members

            /// <summary>The name of the curve extrapolation.
            /// </summary>
            private IdentifierString m_Name;

            /// <summary>The long name of the curve extrapolation.
            /// </summary>
            private IdentifierString m_LongName;
            #endregion

            #region public constructors

            /// <summary>Initializes a new instance of the <see cref="Differentiable"/> class.
            /// </summary>
            /// <param name="buildingDirection">The building direction.</param>
            public Differentiable(BuildingDirection buildingDirection)
                : base(buildingDirection)
            {
                switch (buildingDirection)
                {
                    case BuildingDirection.FromFirstGridPoint:
                        m_Name = new IdentifierString("Linear:First");
                        m_LongName = new IdentifierString(CurveResource.LongNameExtrapolationLinearFirstDifferentiable);
                        TrySetAnnotation(CurveResource.AnnotationExtrapolationLinearFirstDifferentiable);
                        break;

                    case BuildingDirection.FromLastGridPoint:
                        m_Name = new IdentifierString("Linear:Last");
                        m_LongName = new IdentifierString(CurveResource.LongNameExtrapolationLinearLastDifferentiable);
                        TrySetAnnotation(CurveResource.AnnotationExtrapolationLinearLastDifferentiable);
                        break;
                    default:
                        throw new NotImplementedException();
                }
            }
            #endregion

            #region public methods

            /// <summary>Creates a <see cref="ICurveExtrapolator"/> object that represents the implementation of the extrapolation approach.
            /// </summary>
            /// <param name="curveInterpolator">The interpolation approach of the curve.</param>
            /// <returns>A <see cref="ICurveExtrapolator"/> object that represents the implementation of the extrapolation approach.</returns>
            public override ICurveExtrapolator Create(ICurveBuilder curveInterpolator)
            {
                return new Extrapolator(curveInterpolator, ExtrapolationBuildingDirection, this);
            }

            /// <summary>Gets the level of grid point dependency, i.e. the number of grid points required for the curve extrapolation.
            /// </summary>
            /// <param name="gridPointCount">The number of grid points.</param>
            /// <returns>The level of grid point dependency, i.e. the number of grid points required for the curve extrapolation.</returns>
            public override int GetLevelOfGridPointDependency(int gridPointCount)
            {
                return 1;  // the first/last grid point is required for extrapolation only
            }
            #endregion

            #region protected methods

            /// <summary>Gets the name of the curve extrapolator.
            /// </summary>
            /// <returns>The name of the curve extrapolator.</returns>
            protected override IdentifierString GetName()
            {
                return m_Name;
            }

            /// <summary>Gets the long name of the curve extrapolator.
            /// </summary>
            /// <returns>The (perhaps) language dependent long name of the curve extrapolator.</returns>
            protected override IdentifierString GetLongName()
            {
                return m_LongName;
            }
            #endregion
        }
    }
}