﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.GridPointCurves
{
    public static partial class GridPointCurve
    {
        /// <summary>Serves as abstract basis class for curve interpolators, i.e. as factory for <see cref="ICurveBuilder"/> objects. Moverover it provides some basic curve interpolation approaches.
        /// </summary>
        public abstract class Interpolator : ICurveBuilderFactory
        {
            #region public static (readonly) members

            /// <summary>The curve interpolation 'none'.
            /// </summary>
            public static readonly Interpolator None = new CurveInterpolationNone();

            /// <summary>The piecewise constant curve interpolation, i.e. the value of the nearest grid point to the left will be returned.
            /// </summary>
            public static readonly Interpolator PiecewiseConstant = new CurveInterpolationPiecewiseConstant();

            /// <summary>The linear curve interpolation.
            /// </summary>
            public static readonly Interpolator Linear = new CurveInterpolationLinear();

            /// <summary>The log-linear curve interpolation.
            /// </summary>
            public static readonly Interpolator LogLinear = new CurveInterpolationLogLinear();

            /// <summary>Provides curve interpolation approaches based on (Cubic) Splines.
            /// </summary>
            public static readonly CurveInterpolationSpline Splines = new CurveInterpolationSpline();
            #endregion

            #region private members

            /// <summary>A short description of the curve interpolator.
            /// </summary>
            private string m_Annotation;
            #endregion

            #region static constructor

            /// <summary>Initializes the <see cref="Interpolator"/> class.
            /// </summary>
            static Interpolator()
            {
            }
            #endregion

            #region protected constructors

            /// <summary>Initializes a new instance of the <see cref="Interpolator"/> class.
            /// </summary>
            /// <param name="minimalRequiredNumberOfGridPoints">The minimal number of grid points required for a successfully application of the interpolation approach.</param>
            /// <param name="exactFitToGridPoints">A value indicating whether grid points are meet exactly.</param>
            protected Interpolator(int minimalRequiredNumberOfGridPoints = 2, bool exactFitToGridPoints = true)
            {
                m_Annotation = String.Empty;
                ExactFitToGridPoints = exactFitToGridPoints;
                MinimalRequiredNumberOfGridPoints = minimalRequiredNumberOfGridPoints;
            }

            /// <summary>Initializes a new instance of the <see cref="Interpolator"/> class.
            /// </summary>
            /// <param name="annotation">The annotation, i.e. short description, of the curve interpolator.</param>
            /// <param name="minimalRequiredNumberOfGridPoints">The minimal number of grid points required for a successfully application of the interpolation approach.</param>
            /// <param name="exactFitToGridPoints">A value indicating whether grid points are meet exactly.</param>
            protected Interpolator(string annotation, int minimalRequiredNumberOfGridPoints = 2, bool exactFitToGridPoints = true)
            {
                m_Annotation = (annotation != null) ? annotation : String.Empty;
                ExactFitToGridPoints = exactFitToGridPoints;
                MinimalRequiredNumberOfGridPoints = minimalRequiredNumberOfGridPoints;
            }
            #endregion

            #region public properties

            #region IIdentifierNameable Members

            /// <summary>Gets the name of the curve interpolator.
            /// </summary>
            /// <value>The language independent name of the curve interpolator.</value>
            public IdentifierString Name
            {
                get { return GetName(); }
            }

            /// <summary>Gets the long name of the curve interpolator.
            /// </summary>
            /// <value>The (perhaps) language dependent long name of the curve interpolator.</value>
            public IdentifierString LongName
            {
                get { return GetLongName(); }
            }
            #endregion

            #region IAnnotatable Members

            /// <summary>Gets a value indicating whether the annotation is read-only.
            /// </summary>
            /// <value><c>true</c> if the annotation of this instance is readonly; otherwise, <c>false</c>.</value>
            public bool HasReadOnlyAnnotation
            {
                get { return false; }
            }

            /// <summary>Gets the annotation of the current instance.
            /// </summary>
            /// <value>The annotation of the current instance.</value>
            public string Annotation
            {
                get { return m_Annotation; }
            }
            #endregion

            #region ICurveBuilderFactory Members

            /// <summary>Gets a value indicating whether grid points are meet exactly.
            /// </summary>
            /// <value>A value indicating whether grid points are meet exactly.</value>
            public bool ExactFitToGridPoints
            {
                get;
                private set;
            }

            /// <summary>Gets the minimal number of grid points required for a successfully application of the interpolation approach.
            /// </summary>
            /// <value>The minimal number of grid points required for a successfully application of the interpolation approach.</value>
            /// <remarks>In general two grid points are required.</remarks>
            public int MinimalRequiredNumberOfGridPoints
            {
                get;
                private set;
            }

            /// <summary>Gets a value indicating whether this instance represents a local approach.
            /// </summary>
            /// <value>
            /// 	<c>true</c> if this instance is local approach; otherwise, <c>false</c>.
            /// </value>
            /// <remarks>In the case of a local approach call <see cref="GetLeftLocalnessLevel(int, int)"/> and <see cref="GetRightLocalnessLevel(int, int)"/> 
            /// for the left and right localness level.
            /// <para>In the case of a global approach all grid points are required for the curve interpolation.</para></remarks>
            public abstract bool IsLocalApproach
            {
                get;
            }
            #endregion

            #endregion

            #region public methods

            #region IAnnotatable Members

            /// <summary>Sets the annotation of the current instance.
            /// </summary>
            /// <param name="annotation">The annotation.</param>
            /// <returns>A value indicating whether the <see cref="Annotation"/> has been changed.</returns>
            public bool TrySetAnnotation(string annotation)
            {
                m_Annotation = (annotation != null) ? annotation : String.Empty;
                return true;
            }
            #endregion

            #region ICurveBuilderFactory Members

            /// <summary>Creates a <see cref="ICurveBuilder"/> object that represents the implementation of the interpolation approach.
            /// </summary>
            /// <returns>A <see cref="ICurveBuilder"/> object that represents the implementation of the interpolation approach.</returns>
            public abstract ICurveBuilder Create();

            /// <summary>Gets the left localness level for a specific grid point, i.e.
            /// changing grid point (t_j,x_j) implies changes on the interval [t_{j-leftLocalnessLevel}, t_{j+rightLocalnessLevel}].
            /// </summary>
            /// <param name="gridPointIndex">The null-based index of the grid point (t_j,x_j).</param>
            /// <param name="gridPointCount">The number of grid points.</param>
            /// <returns>The left localness level with respect to the grid point (t_j,x_j), where j is represented by <paramref name="gridPointIndex"/>
            /// i.e. changing grid point (t_j,x_j) implies changes on the interval [t_{j-leftLocalnessLevel}, t_{j+rightLocalnessLevel}].</returns>
            /// <remarks>The localness level does not depend on the value of the grid point itself.</remarks>
            public abstract int GetLeftLocalnessLevel(int gridPointIndex, int gridPointCount);

            /// <summary>Gets the right localness level for a specific grid point, i.e.
            /// changing grid point (t_j,x_j) implies changes on the interval [t_{j-leftLocalnessLevel}, t_{j+rightLocalnessLevel}].
            /// </summary>
            /// <param name="gridPointIndex">The null-based index of the grid point (t_j,x_j).</param>
            /// <param name="gridPointCount">The number of grid points.</param>
            /// <returns>The right localness level with respect to the grid point (t_j,x_j), where j is represented by <paramref name="gridPointIndex"/>
            /// i.e. changing grid point (t_j,x_j) implies changes on the interval [t_{j-leftLocalnessLevel}, t_{j+rightLocalnessLevel}].</returns>
            /// <remarks>The localness level does not depend on the value of the grid point itself.</remarks>
            public abstract int GetRightLocalnessLevel(int gridPointIndex, int gridPointCount);
            #endregion

            /// <summary>Returns a <see cref="System.String"/> that represents this instance.</summary>
            /// <returns>A <see cref="System.String"/> that represents this instance.</returns>
            public override string ToString()
            {
                return Name.String;
            }
            #endregion

            #region protected methods

            /// <summary>Gets the name of the curve interpolator.
            /// </summary>
            /// <returns>The name of the curve interpolator.</returns>
            protected abstract IdentifierString GetName();

            /// <summary>Gets the long name of the curve interpolator.
            /// </summary>
            /// <returns>The (perhaps) language dependent long name of the curve interpolator.</returns>
            protected abstract IdentifierString GetLongName();
            #endregion
        }
    }
}