﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
namespace Dodoni.MathLibrary.GridPointCurves
{
    /// <summary>Represents curve interpolation approaches based on (Cubic) splines.
    /// </summary>
    [System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)]
    public class CurveInterpolationSpline
    {
        #region public (readonly) members

        /// <summary>Provides curve interpolation approaches based on natural cubic splines.
        /// </summary>
        public readonly GridPointCurve.Interpolator NaturalCubicSpline;

        /// <summary>Provides the Bessel (Hermite) cubic spline curve interpolation.
        /// </summary>
        /// <remarks>The implementation is based on 
        /// <para>P.S. Hagan, G. West, Interpolation methods for curve construction, Applied Mathematical Finance, Vol. 13, No. 2, p.89-129, June 2006.</para>
        /// This interpolation is also named 'Hermite interpolation'. This approach contains already spline boundary conditions.</remarks>
        public readonly GridPointCurve.Interpolator BesselCubicSpline;
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="CurveInterpolationSpline"/> class.
        /// </summary>
        internal CurveInterpolationSpline()
        {
            var naturalCondition = new NaturalCubicSplineBoundaryCondition();
            NaturalCubicSpline = new CurveInterpolationCommonCubicSpline(naturalCondition, "Natural Cubic Spline", CurveResource.LongNameInterpolationNaturalCubicSpline, CurveResource.AnnotationInterpolationNaturalCubicSpline);

            BesselCubicSpline = new CurveInterpolationBesselCubicSpline();
        }
        #endregion

        #region public methods

        /// <summary>Creates a specific cubic spline curve interpolation.
        /// </summary>
        /// <param name="boundaryCondition">The boundary condition.</param>
        /// <returns>The cube spline curve interpolation with respect to the specified boundary conditions.</returns>
        public GridPointCurve.Interpolator Create(CurveInterpolationCubicSpline.BoundaryCondition boundaryCondition)
        {
            return new CurveInterpolationCommonCubicSpline(boundaryCondition);
        }

        /// <summary>Creates a specific cubic spline curve interpolation with respect to a clamped cubic spline, i.e. with specified f'(x_0) and f'(x_n).
        /// </summary>
        /// <param name="derivativeAtFirstPoint">The derivative at first point.</param>
        /// <param name="derivativeAtLastPoint">The derivative at last point.</param>
        /// <returns>The cube spline curve interpolation with respect to the specified clamped cubic spline boundary condition.</returns>
        public GridPointCurve.Interpolator Create(double derivativeAtFirstPoint, double derivativeAtLastPoint)
        {
            var boundaryCondition = ClampedCubicSplineBoundaryCondition.Create(derivativeAtFirstPoint, derivativeAtLastPoint);
            return new CurveInterpolationCommonCubicSpline(boundaryCondition);
        }
        #endregion
    }
}