﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;
using System.Collections.ObjectModel;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.MathLibrary.GridPointCurves
{
    /// <summary>Serves as standard implementation of a grid point curve.
    /// </summary>
    /// <typeparam name="TLabel">The type of the labels.</typeparam>
    internal class StandardParametricGridPointCurve<TLabel> : IGridPointCurve<TLabel>
         where TLabel : IEquatable<TLabel>
    {
        #region nested classes

        /// <summary>Serves as standard implementation of a differentiable grid point curve.
        /// </summary>
        internal class Differentiable : StandardParametricGridPointCurve<TLabel>, IDifferentiableRealValuedCurve
        {
            #region internal/protected constructors

            /// <summary>Initializes a new instance of the <see cref="Differentiable"/> class.
            /// </summary>
            /// <param name="curveParametrizationFactory">The curve parametrization factory.</param>
            /// <param name="curveParametrization">The curve parametrization.</param>
            /// <param name="capacity">The number of elements that the new grid point curve can initially store.</param>
            internal Differentiable(GridPointCurve.Parametrization curveParametrizationFactory, ICurveBuilder curveParametrization, int capacity)
                : base(curveParametrizationFactory, curveParametrization, capacity)
            {
            }

            /// <summary>Initializes a new instance of the <see cref="Differentiable"/> class.
            /// </summary>
            /// <param name="curveParametrizationFactory">The curve parametrization factory.</param>
            /// <param name="gridPointLabels">The grid point labels.</param>
            /// <param name="gridPointArguments">The grid point arguments.</param>
            /// <param name="gridPointValues">The grid point values.</param>
            /// <param name="isReadOnly">A value indicating whether the instance is be read-only.</param>
            protected Differentiable(GridPointCurve.Parametrization curveParametrizationFactory, List<TLabel> gridPointLabels, List<double> gridPointArguments, List<double> gridPointValues, bool isReadOnly = true)
                : base(curveParametrizationFactory, gridPointLabels, gridPointArguments, gridPointValues, isReadOnly)
            {
            }
            #endregion

            #region public methods

            /// <summary>Returns a read-only <see cref="IGridPointCurve{TLabel}"/> wrapper for the current instance.
            /// </summary>
            /// <returns>A <see cref="IGridPointCurve{TLabel}"/> object that acts as a read-only wrapper around the current instance.</returns>
            public override IGridPointCurve<TLabel> AsReadOnly()
            {
                return new Differentiable(m_CurveParametrizationFactory, m_GridPointLabels, m_GridPointArguments, m_GridPointValues);
            }
            #endregion

            #region IDifferentiableRealValuedCurve Members

            /// <summary>Gets the derivative at a specific point.
            /// </summary>
            /// <param name="pointToEvaluate">The point to evaluate.</param>
            /// <returns>The value of the derivative at the <paramref name="pointToEvaluate"/>.</returns>
            /// <remarks>The argument must be an element of the domain of definition, represented by <see cref="IRealValuedCurve.LowerBound"/> and <see cref="IRealValuedCurve.UpperBound"/>.</remarks>
            public double GetDerivative(double pointToEvaluate)
            {
                return ((IDifferentiableRealValuedCurve)m_CurveParametrization).GetDerivative(pointToEvaluate);
            }
            #endregion
        }
        #endregion

        #region private/protected members

        /// <summary>The state of the grid point curve.
        /// </summary>
        private GridPointCurve.State m_State;

        /// <summary>The labels of the grid points.
        /// </summary>
        protected readonly List<TLabel> m_GridPointLabels;

        /// <summary>The arguments of the grid points, i.e. the x-components of the grid points. 
        /// </summary>
        private List<double> m_GridPointArguments;

        /// <summary>The values of the grid points, i.e. the y-components of the grid points.
        /// </summary>
        protected List<double> m_GridPointValues;

        /// <summary>A value indicating whether the instance is read-only.
        /// </summary>
        private bool m_IsReadOnly;

        /// <summary>The curve parametrization factory.
        /// </summary>
        protected readonly GridPointCurve.Parametrization m_CurveParametrizationFactory;

        /// <summary>The curve parametrization.
        /// </summary>
        protected readonly ICurveBuilder m_CurveParametrization;

        /// <summary>The read-only wrapper of the grid point labels.
        /// </summary>
        /// <remarks>This member is used for performance reasons only.</remarks>
        private ReadOnlyCollection<TLabel> m_ReadOnlyGridPointLabels;

        /// <summary>The read-only wrapper of the grid point arguments.
        /// </summary>
        /// <remarks>This member is used for performance reasons only.</remarks>
        private ReadOnlyCollection<double> m_ReadOnlyGridPointArguments;

        /// <summary>The read-only wrapper of the grid point values.
        /// </summary>
        /// <remarks>This member is used for performance reasons only.</remarks>
        private ReadOnlyCollection<double> m_ReadOnlyGridPointValues;
        #endregion

        #region internal/protected constructors

        /// <summary>Initializes a new instance of the <see cref="StandardParametricGridPointCurve&lt;TLabel&gt;"/> class.
        /// </summary>
        /// <param name="curveParametrizationFactory">The curve parametrization factory.</param>
        /// <param name="curveParametrization">The curve parametrization.</param>
        /// <param name="capacity">The number of elements that the new grid point curve can initially store.</param>
        internal StandardParametricGridPointCurve(GridPointCurve.Parametrization curveParametrizationFactory, ICurveBuilder curveParametrization, int capacity = 20)
        {
            m_GridPointLabels = new List<TLabel>(capacity);
            m_GridPointArguments = new List<double>(capacity);
            m_GridPointValues = new List<double>(capacity);

            m_CurveParametrizationFactory = curveParametrizationFactory;
            m_CurveParametrization = curveParametrization;

            m_ReadOnlyGridPointLabels = new ReadOnlyCollection<TLabel>(m_GridPointLabels);
            m_ReadOnlyGridPointValues = new ReadOnlyCollection<double>(m_GridPointValues);
            m_ReadOnlyGridPointArguments = new ReadOnlyCollection<double>(m_GridPointArguments);

            m_State = GridPointCurve.State.GridPointChanged;
            m_IsReadOnly = false;
        }

        /// <summary>Initializes a new instance of the <see cref="StandardParametricGridPointCurve&lt;TLabel&gt;"/> class.
        /// </summary>
        /// <param name="curveParametrizationFactory">The curve parametrization factory.</param>
        /// <param name="gridPointLabels">The grid point labels.</param>
        /// <param name="gridPointArguments">The grid point arguments.</param>
        /// <param name="gridPointValues">The grid point values.</param>
        /// <param name="isReadOnly">A value indicating whether the instance is be read-only.</param>
        protected StandardParametricGridPointCurve(GridPointCurve.Parametrization curveParametrizationFactory, List<TLabel> gridPointLabels, List<double> gridPointArguments, List<double> gridPointValues, bool isReadOnly = true)
        {
            m_CurveParametrizationFactory = curveParametrizationFactory;
            m_CurveParametrization = curveParametrizationFactory.Create();

            m_GridPointLabels = new List<TLabel>(gridPointLabels.Count);
            m_GridPointLabels.AddRange(gridPointLabels);

            m_GridPointArguments = new List<double>(gridPointArguments.Count);
            m_GridPointArguments.AddRange(gridPointArguments);
            m_GridPointValues = new List<double>(gridPointValues.Count);
            m_GridPointValues.AddRange(gridPointValues);

            m_ReadOnlyGridPointLabels = new ReadOnlyCollection<TLabel>(m_ReadOnlyGridPointLabels);
            m_ReadOnlyGridPointValues = new ReadOnlyCollection<double>(m_GridPointValues);
            m_ReadOnlyGridPointArguments = new ReadOnlyCollection<double>(m_GridPointArguments);

            Update();
            m_IsReadOnly = isReadOnly;
        }
        #endregion

        #region public properties

        #region IOperable Members

        /// <summary>Gets a value indicating whether this instance is operable.
        /// </summary>
        /// <value><c>true</c> if this instance is operable; otherwise, <c>false</c>.</value>
        public bool IsOperable
        {
            get { return ((m_State == GridPointCurve.State.NoChangeSinceLastUpdate) && (m_GridPointLabels.Count >= 2) && m_CurveParametrization.IsOperable); }
        }
        #endregion

        #region IGridPointCurve Members

        /// <summary>Gets the number of grid points.
        /// </summary>
        /// <value>The number of grid points.</value>
        public int GridPointCount
        {
            get { return m_GridPointLabels.Count; }
        }

        /// <summary>Gets the grid point arguments, i.e. the labels (on the x-axis) of the curve in its <see cref="System.Double"/> representation.
        /// </summary>
        /// <value>The grid point arguments.</value>
        public IList<double> GridPointArguments
        {
            get { return m_ReadOnlyGridPointArguments; }
        }

        /// <summary>Gets the grid point values with respect to <see cref="IGridPointCurve.GridPointArguments"/>.
        /// </summary>
        /// <value>The grid point values.</value>
        public IList<double> GridPointValues
        {
            get { return m_ReadOnlyGridPointValues; }
        }

        /// <summary>Gets a value indicating whether this instance is read-only.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is read-only; otherwise, <c>false</c>.
        /// </value>
        public bool IsReadOnly
        {
            get { return m_IsReadOnly; }
        }

        /// <summary>Gets a value indicating whether grid points are meet exactly.
        /// </summary>
        /// <value>A value indicating whether grid points are meet exactly.</value>
        public bool ExactFitToGridPoints
        {
            get { return m_CurveParametrization.Factory.ExactFitToGridPoints; }
        }
        #endregion

        #region IGridPointCurve<TLabel> Members

        /// <summary>Gets the labels of the grid points
        /// </summary>
        /// <value>The grid point labels.</value>
        public IList<TLabel> GridPointLabels
        {
            get { return m_ReadOnlyGridPointLabels; }
        }
        #endregion

        #region IRealValuedCurve Members

        /// <summary>Gets the lower bound of the domain of definition.
        /// </summary>
        /// <value>The lower bound of the domain of definition, perhaps <see cref="System.Double.NegativeInfinity"/> or <see cref="System.Double.NaN"/>.
        /// </value>
        public double LowerBound
        {
            get { return m_CurveParametrization.LowerBound; }
        }

        /// <summary>Gets the upper bound of the domain of definition.
        /// </summary>
        /// <value>The upper bound of the domain of definition, perhaps <see cref="System.Double.PositiveInfinity"/> or <see cref="System.Double.NaN"/>.</value>
        public double UpperBound
        {
            get { return m_CurveParametrization.UpperBound; }
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Gets the info-output level of detail.
        /// </summary>
        /// <value>The info-output level of detail.</value>
        public InfoOutputDetailLevel InfoOutputDetailLevel
        {
            get { return InfoOutputDetailLevel.Full; }
        }
        #endregion

        #endregion

        #region public methods

        #region IGridPointCurve<TLabel> Members

        /// <summary>Adds a specific grid point.
        /// </summary>
        /// <param name="label">The label.</param>
        /// <param name="argument">The argument, i.e. the <see cref="System.Double"/> representation of <paramref name="label"/>.</param>
        /// <param name="value">The value.</param>
        /// <returns>The null-based index of the grid point in the curve, the grid points should be ordered with respect to the argument.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IGridPointCurve.IsReadOnly"/> is <c>true</c>.</exception>
        public int Add(TLabel label, double argument, double value)
        {
            if (m_IsReadOnly == true)
            {
                throw new InvalidOperationException();
            }

            int index = m_GridPointArguments.BinarySearch(argument);
            if (index >= 0)
            {
                throw new ArgumentException();
            }
            index = (~index);

            m_GridPointLabels.Insert(index, label);
            m_GridPointArguments.Insert(index, argument);
            m_GridPointValues.Insert(index, value);
            m_State = GridPointCurve.State.GridPointChanged;
            return index;
        }

        /// <summary>Adds a collection of grid points.
        /// </summary>
        /// <param name="values">A collection of grid points, where the first component is the label, the second component is the argument, i.e. the <see cref="System.Double"/> representation of the label,
        /// and the third component of the triple is the value.</param>
        /// <param name="isSorted">A value indicating whether the <see cref="System.Double"/> representation of the labels in <paramref name="values"/> are sorted in ascending order.</param>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IGridPointCurve.IsReadOnly"/> is <c>true</c>.</exception>
        public void AddRange(IEnumerable<Tuple<TLabel, double, double>> values, bool isSorted = false)
        {
            if (values == null)
            {
                throw new ArgumentNullException();
            }
            if ((isSorted == true) && (m_GridPointLabels.Count == 0))
            {
                if (m_IsReadOnly == true)
                {
                    throw new InvalidOperationException();
                }
                foreach (var gridPoint in values)
                {
                    m_GridPointLabels.Add(gridPoint.Item1);
                    m_GridPointArguments.Add(gridPoint.Item2);
                    m_GridPointValues.Add(gridPoint.Item3);
                }
            }
            else
            {
                foreach (var gridPoint in values)
                {
                    Add(gridPoint.Item1, gridPoint.Item2, gridPoint.Item3);
                }
            }
        }

        /// <summary>Removes a specific grid point.
        /// </summary>
        /// <param name="label">The label of the grid point to remove.</param>
        /// <returns>A value indicating whether the grid point with respect to <paramref name="label"/> has been removed.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IGridPointCurve.IsReadOnly"/> is <c>true</c>.</exception>
        public bool TryRemove(TLabel label)
        {
            if (m_IsReadOnly == true)
            {
                throw new InvalidOperationException();
            }
            int labelIndex = m_GridPointLabels.FindIndex(match => match.Equals(label));
            if (labelIndex >= 0)
            {
                m_GridPointLabels.RemoveAt(labelIndex);
                m_GridPointArguments.RemoveAt(labelIndex);
                m_GridPointValues.RemoveAt(labelIndex);
                m_State = GridPointCurve.State.GridPointChanged;
            }
            return false;
        }

        /// <summary>Changes the argument of a specific grid point.
        /// </summary>
        /// <param name="label">The label.</param>
        /// <param name="argument">The argument, i.e. the <see cref="System.Double"/> representation of <paramref name="label"/>.</param>
        /// <returns>A value indicating whether the argument of the specific grid point has been changed to <paramref name="argument"/>.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IGridPointCurve.IsReadOnly"/> is <c>true</c>.</exception>
        public bool TrySetGridPointArgument(TLabel label, double argument)
        {
            if (m_IsReadOnly == true)
            {
                throw new InvalidOperationException();
            }
            int labelIndex = m_GridPointLabels.FindIndex(match => match.Equals(label));
            if (labelIndex >= 0)
            {
                m_GridPointArguments[labelIndex] = argument;
                m_State |= GridPointCurve.State.GridPointArgumentChanged;
            }
            return false;
        }

        /// <summary>Changes the value component of a specific grid point.
        /// </summary>
        /// <param name="label">The label.</param>
        /// <param name="value">The value.</param>
        /// <returns>A value indicating whether the <paramref name="label"/> exists in the curve and the value component has been changed to <paramref name="value"/>.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IGridPointCurve.IsReadOnly"/> is <c>true</c>.</exception>
        public bool TrySetValue(TLabel label, double value)
        {
            if (m_IsReadOnly == true)
            {
                throw new InvalidOperationException();
            }
            int labelIndex = m_GridPointLabels.FindIndex(match => match.Equals(label));
            if (labelIndex >= 0)
            {
                m_GridPointValues[labelIndex] = value;
                m_State |= GridPointCurve.State.GridPointValueChanged;
            }
            return false;
        }

        /// <summary>Returns a read-only <see cref="IGridPointCurve{TLabel}"/> wrapper for the current instance.
        /// </summary>
        /// <returns>A <see cref="IGridPointCurve{TLabel}"/> object that acts as a read-only wrapper around the current instance.</returns>
        public virtual IGridPointCurve<TLabel> AsReadOnly()
        {
            return new StandardParametricGridPointCurve<TLabel>(m_CurveParametrizationFactory, m_GridPointLabels, m_GridPointArguments, m_GridPointValues);
        }
        #endregion

        #region IGridPointCurve Members

        /// <summary>Removes all elements from the <see cref="IGridPointCurve"/> object.
        /// </summary>
        /// <remarks>This method set the <see cref="IOperable.IsOperable"/> flag to <c>false</c>.</remarks>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IGridPointCurve.IsReadOnly"/> is <c>true</c>.</exception>
        public void Clear()
        {
            if (m_IsReadOnly == true)
            {
                throw new InvalidOperationException();
            }
            m_GridPointLabels.Clear();
            m_GridPointArguments.Clear();
            m_GridPointValues.Clear();
            m_State = GridPointCurve.State.GridPointChanged;
        }

        /// <summary>Removes a specific grid point.
        /// </summary>
        /// <param name="gridPointIndex">The null-based index of the grid point.</param>
        /// <remarks>This method set the <see cref="IOperable.IsOperable"/> flag to <c>false</c>.</remarks>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IGridPointCurve.IsReadOnly"/> is <c>true</c>.</exception>
        public void RemoveAt(int gridPointIndex)
        {
            if (m_IsReadOnly == true)
            {
                throw new InvalidOperationException();
            }
            m_GridPointLabels.RemoveAt(gridPointIndex);
            m_GridPointArguments.RemoveAt(gridPointIndex);
            m_GridPointValues.RemoveAt(gridPointIndex);

            m_State = GridPointCurve.State.GridPointChanged;
        }

        /// <summary>Changes a specific grid point argument.
        /// </summary>
        /// <param name="gridPointIndex">The null-based index of the grid point.</param>
        /// <param name="argument">The argument of the specified grid point, i.e. the <see cref="System.Double"/> representation of the label of the x-axis.</param>
        /// <remarks>This method set the <see cref="IOperable.IsOperable"/> flag to <c>false</c>.</remarks>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IGridPointCurve.IsReadOnly"/> is <c>true</c>.</exception>
        public void SetGridPointArgument(int gridPointIndex, double argument)
        {
            if (m_IsReadOnly == true)
            {
                throw new InvalidOperationException();
            }
            m_GridPointArguments[gridPointIndex] = argument;
            m_State |= GridPointCurve.State.GridPointArgumentChanged;
        }

        /// <summary>Changes the value component of a specific grid point.
        /// </summary>
        /// <param name="gridPointIndex">The null-based index of the grid point.</param>
        /// <param name="value">The value.</param>
        /// <remarks>This method set the <see cref="IOperable.IsOperable"/> flag to <c>false</c>.</remarks>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IGridPointCurve.IsReadOnly"/> is <c>true</c>.</exception>
        public void SetValue(int gridPointIndex, double value)
        {
            if (m_IsReadOnly == true)
            {
                throw new InvalidOperationException();
            }
            m_GridPointValues[gridPointIndex] = value;
            m_State |= GridPointCurve.State.GridPointValueChanged;
        }

        /// <summary>Updates the current instance. This method may change <see cref="IOperable.IsOperable"/>.
        /// </summary>
        /// <remarks>Call this method after grid points have been removed, modified or added and before trying to compute a value at a specified argument.
        /// <para>In general this method sets the <see cref="IOperable.IsOperable"/> flag to <c>true</c>.</para></remarks>
        public void Update()
        {
            m_CurveParametrization.Update(m_GridPointLabels.Count, m_GridPointArguments, m_GridPointValues, m_State);
            m_State = GridPointCurve.State.NoChangeSinceLastUpdate;
        }
        #endregion

        #region IRealValuedCurve Members

        /// <summary>Gets the value at a specific argument.
        /// </summary>
        /// <param name="pointToEvaluate">The point to evaluate.</param>
        /// <returns>The value of the curve at <paramref name="pointToEvaluate"/>.</returns>
        /// <remarks>The argument must be an element of the domain of definition, represented by <see cref="IRealValuedCurve.LowerBound"/> and <see cref="IRealValuedCurve.UpperBound"/>.</remarks>
        public double GetValue(double pointToEvaluate)
        {
            return m_CurveParametrization.GetValue(pointToEvaluate);
        }

        /// <summary>Gets the value of the integral \int_a^b f(x) dx.
        /// </summary>
        /// <param name="lowerBound">The lower bound.</param>
        /// <param name="upperBound">The upper bound.</param>
        /// <returns>The value of \int_a^b f(x) dx.</returns>
        /// <remarks>The arguments must be elements of the domain of definition, represented by <see cref="IRealValuedCurve.LowerBound"/> and <see cref="IRealValuedCurve.UpperBound"/>.</remarks>
        public double GetIntegral(double lowerBound, double upperBound)
        {
            return m_CurveParametrization.GetIntegral(lowerBound, upperBound);
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Gets informations of the current object as a specific <see cref="InfoOutput" /> instance.
        /// </summary>
        /// <param name="infoOutput">The <see cref="InfoOutput" /> object which is to be filled with informations concering the current instance.</param>
        /// <param name="categoryName">The name of the category, i.e. all informations will be added to these category.</param>
        public void FillInfoOutput(InfoOutput infoOutput, string categoryName = "General")
        {
            var infoOutputPackage = infoOutput.AcquirePackage(categoryName);
            infoOutputPackage.Add("Count", GridPointCount);
            infoOutputPackage.Add("Is read-only", IsReadOnly);

            m_CurveParametrization.FillInfoOutput(infoOutput, categoryName + ".Parametrization");
        }

        /// <summary>Sets the <see cref="P:Dodoni.BasicComponents.Containers.IInfoOutputQueriable.InfoOutputDetailLevel"/> property.
        /// </summary>
        /// <param name="infoOutputDetailLevel">The info-output level of detail.</param>
        /// <returns>A value indicating whether the <see cref="P:Dodoni.BasicComponents.Containers.IInfoOutputQueriable.InfoOutputDetailLevel"/> has been set to <paramref name="infoOutputDetailLevel"/>.</returns>
        public bool TrySetInfoOutputDetailLevel(InfoOutputDetailLevel infoOutputDetailLevel)
        {
            return (infoOutputDetailLevel == InfoOutputDetailLevel.Full);
        }
        #endregion

        #endregion
    }
}