﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary
{
    /// <summary>Serves as interface for real-valued matrices.
    /// </summary>
    /// <remarks>For special matrices and problems one may apply BLAS, LAPACK functions etc. directly without using specific wrapper classes. 
    /// Therefore we establish wrapper classes for some matrix categories only.</remarks>
    public interface IMatrix
    {
        #region properties

        /// <summary>Gets the number of rows of the current matrix object.
        /// </summary>
        /// <value>The number of rows.</value>
        int RowCount
        {
            get;
        }

        /// <summary>Gets the number of columns of the current matrix object.
        /// </summary>
        /// <value>The number of columns.</value>
        int ColumnCount
        {
            get;
        }

        /// <summary>Gets a specified value.
        /// </summary>
        /// <param name="rowIndex">The null-based row index.</param>
        /// <param name="columnIndex">The null-based column index.</param>
        /// <value>The value of the matrix at the specified coordinate.</value>
        /// <exception cref="IndexOutOfRangeException">Thrown, if the row-/column position is invalid.</exception>
        double this[int rowIndex, int columnIndex]
        {
            get;
        }

        /// <summary>Gets a value indicating whether this instance represents a quadratic matrix.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is quadratic; otherwise, <c>false</c>.
        /// </value>
        bool IsQuadratic
        {
            get;
        }
        #endregion

        #region methods

        /// <summary>Creates a <see cref="DenseMatrix"/> object from the current object.
        /// </summary>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at 
        /// least <see cref="IMatrix.RowCount"/> * <see cref="IMatrix.ColumnCount"/> elements.</param>
        /// <returns>A <see cref="DenseMatrix"/> object that represents the current object.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        DenseMatrix ToDenseMatrix(double[] matrixData = null);

        /// <summary>Gets a specific column.
        /// </summary>
        /// <param name="columnIndex">The null-based index of the column.</param>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at 
        /// least <see cref="IMatrix.RowCount"/> elements.</param>
        /// <returns>The specified column vector in its <see cref="DenseMatrix"/> representation.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        DenseMatrix GetColumn(int columnIndex, double[] matrixData = null);

        /// <summary>Gets a specific transposed row.
        /// </summary>
        /// <param name="rowIndex">The null-based index of the row.</param>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at 
        /// least <see cref="IMatrix.ColumnCount"/> elements.</param>
        /// <returns>The specified transposed row vector in its <see cref="DenseMatrix"/> representation.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        DenseMatrix GetTransposedRow(int rowIndex, double[] matrixData = null);

        /// <summary>Gets a specific sub-matrix.
        /// </summary>
        /// <param name="startRowIndex">The null-based index of the upper row.</param>
        /// <param name="endRowIndex">The null-based index of the lower row.</param>
        /// <param name="startColumnIndex">The null-based index of the left column.</param>
        /// <param name="endColumnIndex">The null-based index of the right column.</param>
        /// <param name="matrixData">The optional (internal) memory allocation to take into account for the <see cref="DenseMatrix"/> object, i.e. with at 
        /// least (<paramref name="endRowIndex"/> - <paramref name="startRowIndex"/> + 1) * (<paramref name="endColumnIndex"/> - <paramref name="startColumnIndex"/> + 1) elements.</param>
        /// <returns>The specified sub-matrix in its <see cref="DenseMatrix"/> representation.</returns>
        /// <remarks>The argument <paramref name="matrixData"/> can be used to reduce reallocation of memory.</remarks>
        DenseMatrix GetSubMatrix(int startRowIndex, int endRowIndex, int startColumnIndex, int endColumnIndex, double[] matrixData = null);

        /// <summary>Determines whether the matrix is symmetric.
        /// </summary>
        /// <param name="tolerance">A tolerance taken into account, i.e. if abs(a_{i,j} - a{j,i}) is less than <paramref name="tolerance"/> the values are assumed to be equal.</param>
        /// <returns>
        /// 	<c>true</c> if this instance is symmetric; otherwise, <c>false</c>.
        /// </returns>
        bool IsSymmetric(double tolerance = MachineConsts.Epsilon);

        /// <summary>Gets the trace of a quadratic matrix.
        /// </summary>
        /// <returns>The trace of the matrix, i.e. the sum of the diagonal elements.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if the current object does not represent a quadratic matrix.</exception>
        double GetTrace();

        /// <summary>Gets a specific norm of the current matrix object.
        /// </summary>
        /// <param name="normType">The type of the (matrix) norm.</param>
        /// <returns>The norm of the current instance with respect to the <paramref name="normType"/>.</returns>
        double GetNorm(Matrix.NormType normType);
        #endregion
    }
}