﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;
using System.Globalization;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.MathLibrary.Miscellaneous
{
    /// <summary>Represents a (general) polynomial with real coefficients.
    /// </summary>
    internal class RealPolynomial : IRealPolynomial
    {
        #region private members

        /// <summary>The coefficients of the polynmial P(x) = a_0 + a_1 * x + a_2 * x^2 + ... + a_n * x^n.
        /// </summary>
        private double[] m_Coefficients;

        /// <summary>The degree of the polynomial.
        /// </summary>
        private int m_Degree;
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="RealPolynomial"/> class.
        /// </summary>
        /// <param name="degree">The degree.</param>
        /// <param name="coefficients">The coefficients, where the null-based index corresponds to the power of the argument, i.e. starting from the absolute coefficient, first oder coefficient etc.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="coefficients"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="coefficients"/> does not represents a polynom of degree <paramref name="degree"/>.</exception>
        /// <remarks>This constructor stores the reference of the <paramref name="coefficients"/>.</remarks>
        internal RealPolynomial(int degree, params double[] coefficients)
        {
            if (coefficients == null)
            {
                throw new ArgumentNullException("coefficients", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "Coefficients"));
            }
            m_Coefficients = coefficients;
            m_Degree = degree;
            if (coefficients[degree] == 0.0)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, "0.0"), "coefficients");
            }
        }

        /// <summary>Initializes a new instance of the <see cref="RealPolynomial"/> class.
        /// </summary>
        /// <param name="degree">The degree.</param>
        /// <param name="coefficients">The coefficients, where the null-based index corresponds to the power of the argument, i.e. starting from the absolute coefficient, first oder coefficient etc.</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="coefficients"/> is <c>null</c>.</exception>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if one of the coefficients is not a number or +/- infinity.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="coefficients"/> does not represents a polynom of degree <paramref name="degree"/>.</exception>
        internal RealPolynomial(int degree, IList<double> coefficients)
        {
            if (coefficients == null)
            {
                throw new ArgumentNullException("coefficients", String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentNull, "Coefficients"));
            }
            m_Degree = degree;

            m_Coefficients = new double[degree + 1];
            for (int j = degree; j >= 0; j--)
            {
                double coefficient = coefficients[j];
                m_Coefficients[j] = coefficient;
                if ((Double.IsNaN(coefficient) || Double.IsInfinity(coefficient)))
                {
                    throw new ArgumentOutOfRangeException("coefficients");
                }
            }
            if (coefficients[degree] == 0.0)
            {
                throw new ArgumentException(String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, "0.0"), "coefficients");
            }
        }
        #endregion

        #region public properties

        #region IOperable Members

        /// <summary>Gets a value indicating whether this instance is operable.
        /// </summary>
        /// <value><c>true</c> if this instance is operable; otherwise, <c>false</c>.</value>
        /// <remarks>This implementation returns <c>true</c>.</remarks>
        public bool IsOperable
        {
            get { return true; }
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Gets the info-output level of detail.
        /// </summary>
        /// <value>The info-output level of detail.</value>
        public InfoOutputDetailLevel InfoOutputDetailLevel
        {
            get { return InfoOutputDetailLevel.Full; }
        }
        #endregion

        #region IRealValuedCurve Members

        /// <summary>Gets the lower bound of the domain of definition.
        /// </summary>
        /// <value>The lower bound of the domain of definition, i.e. <see cref="System.Double.NegativeInfinity"/>.</value>
        double IRealValuedCurve.LowerBound
        {
            get { return Double.NegativeInfinity; }
        }

        /// <summary>Gets the upper bound of the domain of definition.
        /// </summary>
        /// <value>The upper bound of the domain of definition, i.e. <see cref="System.Double.PositiveInfinity"/>.</value>
        double IRealValuedCurve.UpperBound
        {
            get { return Double.PositiveInfinity; }
        }
        #endregion

        #region IPolynomial Members

        /// <summary>Gets the degree of the polynomial.
        /// </summary>
        /// <value>The degree of the polynomial.</value>
        public int Degree
        {
            get { return m_Degree; }
        }

        /// <summary>Gets a value indicating whether this instance is the zero polynomial.
        /// </summary>
        /// <value><c>true</c> if this instance is zero; otherwise, <c>false</c>.</value>
        public bool IsZero
        {
            get { return ((m_Degree == 0) && (m_Coefficients[0] == 0.0)); }
        }
        #endregion

        #endregion

        #region public methods

        #region IComplexValuedCurve Members

        /// <summary>Gets the value at a specific argument.
        /// </summary>
        /// <param name="pointToEvaluate">The point to evaluate.</param>
        /// <returns>The value of the curve at <paramref name="pointToEvaluate" />.</returns>
        public Complex GetValue(Complex pointToEvaluate)
        {
            // Here we use the Horner scheme for evaluating the polynomial: 
            Complex value = m_Coefficients[m_Degree];
            for (int j = m_Degree - 1; j >= 0; j--)
            {
                value = m_Coefficients[j] + pointToEvaluate * value;
            }
            return value;
        }

        /// <summary>Gets the value at a specific argument.
        /// </summary>
        /// <param name="pointToEvaluate">The point to evaluate.</param>
        /// <param name="value">The value at <paramref name="pointToEvaluate" />.</param>
        /// <returns>A value indicating whether <paramref name="value" /> contains valid data.</returns>
        public bool TryGetValue(double pointToEvaluate, out double value)
        {
            value = GetValue(pointToEvaluate);
            return true;
        }
        #endregion

        #region IDifferentiableComplexValuedCurve Members

        /// <summary>Gets the derivative at a specific point.
        /// </summary>
        /// <param name="pointToEvaluate">The point to evaluate.</param>
        /// <returns>The value of the derivative at the <paramref name="pointToEvaluate" />.</returns>
        public Complex GetDerivative(Complex pointToEvaluate)
        {
            Complex value = new Complex(m_Coefficients[m_Degree] * m_Degree, 0.0);
            for (int j = m_Degree - 1; j >= 1; j--)
            {
                value = m_Coefficients[j] * j + value * pointToEvaluate;
            }
            return value;
        }

        /// <summary>Gets the derivative at a specific point.
        /// </summary>
        /// <param name="pointToEvaluate">The point to evaluate.</param>
        /// <returns>The value of the derivative at the <paramref name="pointToEvaluate" />.</returns>
        Complex IDifferentiableComplexValuedCurve.GetDerivative(double pointToEvaluate)
        {
            double value = m_Coefficients[m_Degree] * m_Degree;
            for (int j = m_Degree - 1; j >= 1; j--)
            {
                value = m_Coefficients[j] * j + value * pointToEvaluate;
            }
            return value;
        }
        #endregion

        #region IRealValuedCurve Members

        /// <summary>Gets the value at a specific argument.
        /// </summary>
        /// <param name="pointToEvaluate">The point to evaluate.</param>
        /// <returns>The value of the curve at <paramref name="pointToEvaluate" />.</returns>
        /// <remarks>
        /// The argument must be an element of the domain of definition, represented by <see cref="IRealValuedCurve.LowerBound" /> and <see cref="IRealValuedCurve.UpperBound" />.
        /// </remarks>
        public double GetValue(double pointToEvaluate)
        {
            // Here we use the Horner scheme for evaluating the polynomial: 
            double value = m_Coefficients[m_Degree];

            for (int j = m_Degree - 1; j >= 0; j--)
            {
                value = m_Coefficients[j] + pointToEvaluate * value;
            }
            return value;
        }

        /// <summary>Gets the value of the integral \int_a^b f(x) dx.
        /// </summary>
        /// <param name="lowerBound">The lower bound.</param>
        /// <param name="upperBound">The upper bound.</param>
        /// <returns>The value of \int_a^b f(x) dx.</returns>
        /// <remarks>
        /// The arguments must be elements of the domain of definition, represented by <see cref="IRealValuedCurve.LowerBound" /> and <see cref="IRealValuedCurve.UpperBound" />.
        /// </remarks>
        public double GetIntegral(double lowerBound, double upperBound)
        {
            double upperValue = m_Coefficients[m_Degree] / (m_Degree + 1.0);
            double lowerValue = upperValue;

            for (int j = m_Degree - 1; j >= 0; j--)
            {
                double tempCoefficient = m_Coefficients[j] / (j + 1.0);

                upperValue = tempCoefficient + upperBound * upperValue;
                lowerValue = tempCoefficient + lowerBound * lowerValue;
            }
            return upperBound * upperValue - lowerBound * lowerValue;
        }
        #endregion

        #region IDifferentiableRealValuedCurve Members

        /// <summary>Gets the derivative at a specific point.
        /// </summary>
        /// <param name="pointToEvaluate">The point to evaluate.</param>
        /// <returns>The value of the derivative at the <paramref name="pointToEvaluate" />.</returns>
        /// <remarks>
        /// The argument must be an element of the domain of definition, represented by <see cref="IRealValuedCurve.LowerBound" /> and <see cref="IRealValuedCurve.UpperBound" />.
        /// </remarks>
        public double GetDerivative(double pointToEvaluate)
        {
            double value = m_Coefficients[m_Degree] * m_Degree;
            for (int j = m_Degree - 1; j >= 1; j--)
            {
                value = m_Coefficients[j] * j + value * pointToEvaluate;
            }
            return value;
        }
        #endregion

        #region IRealPolynomial Members

        /// <summary>Gets the real coefficient for a specific <paramref name="order"/>.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <returns>The real coefficient with respect to the specified <paramref name="order"/>.</returns>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="order"/> is negative or <paramref name="order"/> is greater than the <see cref="IPolynomial.Degree"/>.</exception>
        public double GetCoefficient(int order)
        {
            if (order <= m_Degree)
            {
                return m_Coefficients[order];
            }
            throw new IndexOutOfRangeException("order");
        }
        #endregion

        #region IPolynomial Members

        /// <summary>Gets the complex coefficient for a specific <paramref name="order"/>.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <returns>The complex coefficient with respect to the specified <paramref name="order"/>.</returns>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="order"/> is negative or <paramref name="order"/> is greater than the <see cref="IPolynomial.Degree"/>.</exception>
        Complex IPolynomial.GetCoefficient(int order)
        {
            if (order <= m_Degree)
            {
                return new Complex(m_Coefficients[order], 0.0);
            }
            throw new IndexOutOfRangeException("order");
        }

        /// <summary>Gets the (complex) roots of the polynomial.
        /// </summary>
        /// <param name="roots">The roots (output).</param>
        /// <param name="rootFinderApproach">The root finder approach. Some implementations for polynomials with degree up to 3 ignore this parameter because of known analytic solutions.</param>
        /// <remarks>Each root of the polynomial given by the current instance will be added to <paramref name="roots" />, i.e. <see cref="Degree" /> elements.
        /// <para><paramref name="roots" /> will <c>not</c> be cleared before adding roots.</para>
        /// </remarks>
        public void GetRoots(IList<Complex> roots, IPolynomialRootFinder rootFinderApproach)
        {
            rootFinderApproach.GetRoots(m_Degree, m_Coefficients, roots);
        }

        /// <summary>Gets the real roots of the polynomial.
        /// </summary>
        /// <param name="realRoots">The real-valued roots (output).</param>
        /// <param name="rootFinderApproach">The root finder approach. Some implementations for polynomials with degree up to 3 ignore this parameter because of known analytic solutions.</param>
        /// <param name="imaginaryZeroTolerance">The (positive) tolerance taken into account to indicate whether a complex number is interpreted as real number.</param>
        /// <returns>The number or real roots added to <paramref name="realRoots" />.</returns>
        /// <remarks>
        /// The real roots of the polynomial given by the current instance will be added to <paramref name="realRoots" />. Roots are added with respect to their
        /// multiplicity, i.e. a root may add more than once in <paramref name="realRoots" />.
        /// <para><paramref name="realRoots" /> will <c>not</c> be cleared before adding real roots.</para>
        /// </remarks>
        public int GetRealRoots(IList<double> realRoots, IPolynomialRootFinder rootFinderApproach, double imaginaryZeroTolerance = MachineConsts.Epsilon)
        {
            return rootFinderApproach.GetRealRoots(m_Degree, m_Coefficients, realRoots, imaginaryZeroTolerance);
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Sets the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel" /> property.
        /// </summary>
        /// <param name="infoOutputDetailLevel">The info-output level of detail.</param>
        /// <returns>A value indicating whether the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel" /> has been set to <paramref name="infoOutputDetailLevel" />.</returns>
        public bool TrySetInfoOutputDetailLevel(InfoOutputDetailLevel infoOutputDetailLevel)
        {
            return (infoOutputDetailLevel == InfoOutputDetailLevel.Full);
        }

        /// <summary>Gets informations of the current object as a specific <see cref="InfoOutput" /> instance.
        /// </summary>
        /// <param name="infoOutput">The <see cref="InfoOutput" /> object which is to be filled with informations concering the current instance.</param>
        /// <param name="categoryName">The name of the category, i.e. all informations will be added to these category.</param>
        public void FillInfoOutput(InfoOutput infoOutput, string categoryName = InfoOutput.GeneralCategoryName)
        {
            Polynomial.FillInfoOutput(this, infoOutput.AcquirePackage(categoryName));
        }
        #endregion

        /// <summary>Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            StringBuilder strBuilder = new StringBuilder();
            for (int j = m_Degree; j >= 0; j--)
            {
                strBuilder.Append(m_Coefficients[j]);
                if (j > 0)
                {
                    strBuilder.Append("x^");
                    strBuilder.Append(j);
                    strBuilder.Append(" + ");
                }
            }
            return strBuilder.ToString();
        }
        #endregion

        #region internal static methods

        /// <summary>Checks some coefficient and throws some exception if the input is not a real number.
        /// </summary>
        /// <param name="coefficient">The coefficient to check.</param>
        /// <param name="argumentName">The name of the argument.</param>
        /// <param name="errorMessage">The error message.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if <paramref name="coefficient"/> is not a number or +/- infinity.</exception>
        internal static void CheckCoefficient(double coefficient, string argumentName, string errorMessage)
        {
            if (Double.IsNaN(coefficient) || Double.IsInfinity(coefficient))
            {
                throw new ArgumentOutOfRangeException(argumentName, String.Format(CultureInfo.CurrentCulture, ExceptionMessages.ArgumentIsInvalid, errorMessage));
            }
        }
        #endregion
    }
}