﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System.Numerics;
using System.Collections.Generic;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.Miscellaneous
{
    /// <summary>Provides methods to compute roots of a specified polynomial.
    /// </summary>
    public interface IPolynomialRootFinder : IIdentifierNameable
    {
        /// <summary>Gets the roots of a specified polynomial.
        /// </summary>
        /// <param name="degree">The degree of the polynomial.</param>
        /// <param name="coefficients">The coefficients of the polynmial P(z) = a_0 + a_1 * z + a_2 * z^2 + ... + a_n * z^n where the null-based index corresponds 
        /// to the power of the argument, i.e. starting from the absolute coefficient, first oder coefficient etc.</param>
        /// <param name="roots">The <paramref name="degree"/> roots (output).</param>
        /// <remarks><paramref name="roots"/> will <c>not</c> be cleared before adding roots.</remarks>
        void GetRoots(int degree, IList<Complex> coefficients, IList<Complex> roots);

        /// <summary>Gets the roots of a specified (real-valued) polynomial.
        /// </summary>
        /// <param name="degree">The degree of the polynomial.</param>
        /// <param name="coefficients">The coefficients of the polynmial P(z) = a_0 + a_1 * z + a_2 * z^2 + ... + a_n * z^n where the null-based index corresponds 
        /// to the power of the argument, i.e. starting from the absolute coefficient, first oder coefficient etc.</param>
        /// <param name="roots">The <paramref name="degree"/> roots (output).</param>
        /// <remarks><paramref name="roots"/> will <c>not</c> be cleared before adding roots.</remarks>
        void GetRoots(int degree, IList<double> coefficients, IList<Complex> roots);

        /// <summary>Gets the real roots of a specified polynomial.
        /// </summary>
        /// <param name="degree">The degree of the polynomial.</param>
        /// <param name="coefficients">The coefficients of the polynmial P(z) = a_0 + a_1 * z + a_2 * z^2 + ... + a_n * z^n where the null-based index corresponds 
        /// to the power of the argument, i.e. starting from the absolute coefficient, first oder coefficient etc.</param>
        /// <param name="realRoots">The real-valued roots (output).</param>
        /// <param name="rootImaginaryZeroTolerance">The tolerance taken into account to indicate whether a complex number is interpreted as real number.</param>
        /// <returns>The number or real roots added to <paramref name="realRoots"/>.</returns>
        /// <remarks>Roots are added with respect to their multiplicity, i.e. a root may add more than once in <paramref name="realRoots"/>.
        /// <para><paramref name="realRoots"/> will <c>not</c> be cleared before adding real roots.</para>
        /// </remarks>
        int GetRealRoots(int degree, IList<Complex> coefficients, IList<double> realRoots, double rootImaginaryZeroTolerance = MachineConsts.Epsilon);

        /// <summary>Gets the real roots of a specified polynomial.
        /// </summary>
        /// <param name="degree">The degree of the polynomial.</param>
        /// <param name="coefficients">The coefficients of the polynmial P(z) = a_0 + a_1 * z + a_2 * z^2 + ... + a_n * z^n where the null-based index corresponds 
        /// to the power of the argument, i.e. starting from the absolute coefficient, first oder coefficient etc.</param>
        /// <param name="realRoots">The real-valued roots (output).</param>
        /// <param name="rootImaginaryZeroTolerance">The tolerance taken into account to indicate whether a complex number is interpreted as real number.</param>
        /// <returns>The number or real roots added to <paramref name="realRoots"/>.</returns>
        /// <remarks>Roots are added with respect to their multiplicity, i.e. a root may add more than once in <paramref name="realRoots"/>.
        /// <para><paramref name="realRoots"/> will <c>not</c> be cleared before adding real roots.</para>
        /// </remarks>
        int GetRealRoots(int degree, IList<double> coefficients, IList<double> realRoots, double rootImaginaryZeroTolerance = MachineConsts.Epsilon);
    }
}