﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.NumericalIntegrators
{
    /// <summary>Serves as factory for <see cref="ICacheableOneDimNumericalIntegrator"/> objects that allowes a numerical integration of a real-valued function in one variable 
    /// with some specific caching technique.
    /// </summary>
    public abstract class CacheableOneDimNumericalIntegrator : IIdentifierNameable
    {
        #region delegate declarations/nested classes

        /// <summary>Represents a function returning a double-precision floating-point number that has one double-precision floating-point 
        /// number argument which is a element of a specific array and a second argument indicates the null-based index of the first argument.
        /// </summary>
        /// <param name="x_k">The value x_k where to evaluate the function, i.e. the k-th element of the <see cref="CacheableOneDimNumericalIntegrator.Abscissas"/>.</param>
        /// <param name="k">The null-based index <c>k</c> of the point <paramref name="x_k"/>, i.e. of the <see cref="CacheableOneDimNumericalIntegrator.Abscissas"/>.</param>
        /// <returns>The value of the function at <paramref name="x_k"/>.</returns>
        public delegate double FunctionToIntegrate(double x_k, int k);

        private class OneDimNumericalIntegratorWrapper : OneDimNumericalIntegrator
        {
            #region private members

            /// <summary>The cacheable integrator instance to wrap.
            /// </summary>
            private CacheableOneDimNumericalIntegrator m_CacheableIntegrator;
            #endregion

            #region internal constructors

            /// <summary>Initializes a new instance of the <see cref="OneDimNumericalIntegratorWrapper"/> class.
            /// </summary>
            /// <param name="cacheableIntegrator">The cacheable one-dimensional numerical integrator.</param>
            internal OneDimNumericalIntegratorWrapper(CacheableOneDimNumericalIntegrator cacheableIntegrator)
                : base(cacheableIntegrator.LowerBoundType, cacheableIntegrator.UpperBoundType, cacheableIntegrator.WeightingFunction)
            {
                m_CacheableIntegrator = cacheableIntegrator;
            }
            #endregion

            #region public methods

            /// <summary>Creates a new <see cref="IOneDimNumericalIntegrator"/> object.
            /// </summary>
            /// <returns>A new <see cref="IOneDimNumericalIntegrator"/> object.</returns>
            public override IOneDimNumericalIntegrator Create()
            {
                return m_CacheableIntegrator.Create();
            }

            /// <summary>Gets the name of the numerical integrator.
            /// </summary>
            /// <returns>The name of the numerical integrator.</returns>
            protected override IdentifierString GetName()
            {
                return m_CacheableIntegrator.GetName();
            }

            /// <summary>Gets the long name of the numerical integrator.
            /// </summary>
            /// <returns>The (perhaps) language dependent long name of the numerical integrator.</returns>
            protected override IdentifierString GetLongName()
            {
                return m_CacheableIntegrator.GetLongName();
            }
            #endregion
        }
        #endregion

        #region protected constructors

        /// <summary>Initializes a new instance of the <see cref="CacheableOneDimNumericalIntegrator"/> class.
        /// </summary>
        /// <param name="lowerBoundType">The type of the lower bound of the numerical integration approach, i.e. a value indicating whether
        /// <see cref="IOneDimNumericalIntegrator.FunctionToIntegrate"/> will be evaluate at the <see cref="IOneDimNumericalIntegrator.LowerBound"/>.</param>
        /// <param name="upperBoundType">The type of the upper bound of the numerical integration approach, i.e. a value indicating whether 
        /// <see cref="IOneDimNumericalIntegrator.FunctionToIntegrate"/> will be evaluate at the <see cref="IOneDimNumericalIntegrator.UpperBound"/>.</param>
        /// <param name="abscissas">The abscissas, i.e. evaluation points of the numerical integration approach.</param>
        /// <remarks>A read-only collection of the <paramref name="abscissas"/> argument will be stored internally.</remarks>
        protected CacheableOneDimNumericalIntegrator(OneDimNumericalIntegrator.BoundaryType lowerBoundType, OneDimNumericalIntegrator.BoundaryType upperBoundType, IList<double> abscissas)
        {
            LowerBoundType = lowerBoundType;
            UpperBoundType = upperBoundType;
            WeightingFunction = OneDimNumericalIntegrator.WeightFunction.One;
            Abscissas = new ReadOnlyCollection<double>(abscissas);
        }

        /// <summary>Initializes a new instance of the <see cref="CacheableOneDimNumericalIntegrator"/> class.
        /// </summary>
        /// <param name="lowerBoundType">The type of the lower bound of the numerical integration approach, i.e. a value indicating whether
        /// <see cref="IOneDimNumericalIntegrator.FunctionToIntegrate"/> will be evaluate at the <see cref="IOneDimNumericalIntegrator.LowerBound"/>.</param>
        /// <param name="upperBoundType">The type of the upper bound of the numerical integration approach, i.e. a value indicating whether 
        /// <see cref="IOneDimNumericalIntegrator.FunctionToIntegrate"/> will be evaluate at the <see cref="IOneDimNumericalIntegrator.UpperBound"/>.</param>
        /// <param name="abscissas">The abscissas, i.e. evaluation points of the numerical integration approach.</param>
        /// <param name="weightFunction">The weight function of the numerical integration approach.</param>
        /// <remarks>A read-only collection of the <paramref name="abscissas"/> argument will be stored internally.</remarks>
        protected CacheableOneDimNumericalIntegrator(OneDimNumericalIntegrator.BoundaryType lowerBoundType, OneDimNumericalIntegrator.BoundaryType upperBoundType, IList<double> abscissas, OneDimNumericalIntegrator.WeightFunction weightFunction)
        {
            if (weightFunction == null)
            {
                throw new ArgumentNullException("weightFunction");
            }
            WeightingFunction = weightFunction;

            LowerBoundType = lowerBoundType;
            UpperBoundType = upperBoundType;
            Abscissas = new ReadOnlyCollection<double>(abscissas);
        }
        #endregion

        #region public properties

        #region IIdentifierNameable Members

        /// <summary>Gets the name of the numerical integrator.
        /// </summary>
        /// <value>The language independent name of the numerical integrator.</value>
        public IdentifierString Name
        {
            get { return GetName(); }
        }

        /// <summary>Gets the long name of the numerical integrator.
        /// </summary>
        /// <value>The (perhaps) language dependent long name of the numerical integrator.</value>
        public IdentifierString LongName
        {
            get { return GetLongName(); }
        }
        #endregion

        /// <summary>Gets a value indicating whether the <see cref="IOneDimNumericalIntegrator.FunctionToIntegrate"/> will be evaluate at the <see cref="IOneDimNumericalIntegrator.LowerBound"/>.
        /// </summary>
        /// <value>The type of the lower bound.</value>
        public OneDimNumericalIntegrator.BoundaryType LowerBoundType
        {
            get;
            private set;
        }

        /// <summary>Gets a value indicating whether the <see cref="IOneDimNumericalIntegrator.FunctionToIntegrate"/> will be evaluate at the <see cref="IOneDimNumericalIntegrator.UpperBound"/>.
        /// </summary>
        /// <value>The type of the upper bound.</value>
        public OneDimNumericalIntegrator.BoundaryType UpperBoundType
        {
            get;
            private set;
        }

        /// <summary>Gets the weight function w(x) of the numerical integration 
        ///  <para>
        /// \int_a^b w(x) * f(x) dx,
        /// </para>
        /// where f:I \to \R, I: Interval with a = inf I, b = sup I, is the function to integrate and w(x) is a specific strictly positive weight function.
        /// </summary>
        /// <value>The weight function.</value>
        public OneDimNumericalIntegrator.WeightFunction WeightingFunction
        {
            get;
            private set;
        }

        /// <summary>The abscissas, i.e. evaluation points of the numerical integration approach.
        /// </summary>
        /// <value>The abscissas , i.e. evaluation points of the numerical integration approach.</value>
        public IList<double> Abscissas
        {
            get;
            private set;
        }
        #endregion

        #region public methods

        /// <summary>Creates a new <see cref="ICacheableOneDimNumericalIntegrator"/> object.
        /// </summary>
        /// <returns>A new <see cref="ICacheableOneDimNumericalIntegrator"/> object.</returns>
        public abstract ICacheableOneDimNumericalIntegrator Create();

        /// <summary>Returns a <see cref="System.String"/> that represents this instance.</summary>
        /// <returns>A <see cref="System.String"/> that represents this instance.</returns>
        public override string ToString()
        {
            return Name.String;
        }
        #endregion

        #region public static methods

        /// <summary>Performs an implicit conversion from <see cref="CacheableOneDimNumericalIntegrator"/> to <see cref="OneDimNumericalIntegrator"/>.
        /// </summary>
        /// <param name="cacheableOneDimNumericalIntegrator">The cacheable one-dimensional numerical integrator.</param>
        /// <returns>The result of the conversion.</returns>
        public static implicit operator OneDimNumericalIntegrator(CacheableOneDimNumericalIntegrator cacheableOneDimNumericalIntegrator)
        {
            return new OneDimNumericalIntegratorWrapper(cacheableOneDimNumericalIntegrator);
        }
        #endregion

        #region protected methods

        /// <summary>Gets the name of the numerical integrator.
        /// </summary>
        /// <returns>The name of the numerical integrator.</returns>
        protected abstract IdentifierString GetName();

        /// <summary>Gets the long name of the numerical integrator.
        /// </summary>
        /// <returns>The (perhaps) language dependent long name of the numerical integrator.</returns>
        protected abstract IdentifierString GetLongName();
        #endregion
    }
}