﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.MathLibrary.NumericalIntegrators
{
    /// <summary>Serves as interface for integration algorithms of a real-valued function in one real argument, i.e. represents an approximation of
    /// <para>
    /// \int_a^b w(x) * f(x) dx,
    /// </para>
    /// where f:I \to \R, I: Interval with a = inf I, b = sup I, is the function to integrate and w(x) is a specific strictly positive weight function.
    /// </summary>
    public interface IOneDimNumericalIntegrator : IOperable, IInfoOutputQueriable
    {
        #region properties

        /// <summary>Gets or sets the function to integrate.
        /// </summary>
        /// <value>The function to integrate.</value>
        Func<double, double> FunctionToIntegrate
        {
            get;
            set;
        }

        /// <summary>Gets the lower integration bound.
        /// </summary>
        /// <value>The lower integration bound.</value>
        double LowerBound
        {
            get;
        }

        /// <summary>Gets the upper integration bound.
        /// </summary>
        /// <value>The upper integration bound.</value>
        double UpperBound
        {
            get;
        }

        /// <summary>Gets the factory for further <see cref="IOneDimNumericalIntegrator"/> objects of the same type and with the same configuration.
        /// </summary>
        /// <value>The factory for further <see cref="IOneDimNumericalIntegrator"/> objects of the same type and with the same configuration.</value>
        OneDimNumericalIntegrator Factory
        {
            get;
        }
        #endregion

        #region methods

        /// <summary>Gets the value of the integral \int_a^b w(x) * f(x) dx.
        /// </summary>
        /// <returns>An approximation of the specific integral.</returns>
        double GetValue();

        /// <summary>Gets the state of the numerical integrator following an execution of the algorithm.
        /// </summary>
        /// <value>The state of the numerical integrator following an execution of the algorithm.</value>
        OneDimNumericalIntegrator.State State
        {
            get;
        }

        /// <summary>Sets the lower and upper integration bounds.
        /// </summary>
        /// <param name="lowerBound">The lower integration bound.</param>
        /// <param name="upperBound">The upper integration bound.</param>
        /// <exception cref="ArgumentException">Thrown, if one of the argument is invalid for the numerical integrator.</exception>
        void SetBounds(double lowerBound, double upperBound);

        /// <summary>Sets the lower integration bound.
        /// </summary>
        /// <param name="lowerBound">The lower integration bound.</param>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="lowerBound"/> is invalid for the numerical integrator.</exception>
        void SetLowerBound(double lowerBound);

        /// <summary>Sets the upper integration bound.
        /// </summary>
        /// <param name="upperBound">The upper integration bound.</param>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="upperBound"/> is invalid for the numerical integrator.</exception>
        void SetUpperBound(double upperBound);
        #endregion
    }
}
