﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

namespace Dodoni.MathLibrary.NumericalIntegrators
{
    public partial class OneDimNumericalIntegrator
    {
        /// <summary>Represents the weight function w(x) in the numerical integration 
        ///  <para>
        ///   \int_a^b w(x) * f(x) dx.
        /// </para>
        /// </summary>
        public class WeightFunction
        {
            #region public static (readonly) members

            /// <summary>The weight function w(x) with w(x) = 1.0 for each x.
            /// </summary>
            public static readonly WeightFunction One = new WeightFunction();
            #endregion

            #region private members

            /// <summary>The delegate that represents the weight function.
            /// </summary>
            private Func<double, double> m_WeightFunctionDelegate;
            #endregion

            #region public constructors

            /// <summary>Initializes a new instance of the <see cref="WeightFunction"/> class.
            /// </summary>
            /// <remarks>The weight function w(x) is assumed to be 1.0.</remarks>
            public WeightFunction()
            {
                IsOne = true;
                m_WeightFunctionDelegate = x => 1.0;
            }

            /// <summary>Initializes a new instance of the <see cref="WeightFunction"/> class.
            /// </summary>
            /// <param name="weightFunction">The weight function.</param>
            public WeightFunction(Func<double, double> weightFunction)
            {
                IsOne = false;
                m_WeightFunctionDelegate = weightFunction;
            }
            #endregion

            #region static constructors

            /// <summary>Initializes the <see cref="WeightFunction"/> class.
            /// </summary>
            static WeightFunction()
            {
            }
            #endregion

            #region public properties

            /// <summary>Gets a value indicating whether the weight function w(x) satisfied w(x) = 1 for each x.
            /// </summary>
            /// <value><c>true</c> if the weight function is equal to one; otherwise, <c>false</c>.</value>
            public bool IsOne
            {
                get;
                private set;
            }
            #endregion

            #region public methods

            /// <summary>Gets the value of the weight function at a specified point.
            /// </summary>
            /// <param name="x">The argument of the weight function.</param>
            /// <returns>The value of the weight function w(<paramref name="x"/>).</returns>
            public double GetValue(double x)
            {
                return (IsOne == true) ? 1.0 : m_WeightFunctionDelegate(x);
            }
            #endregion

            #region public static methods

            /// <summary>Creates a new <see cref="WeightFunction"/> object.
            /// </summary>
            /// <param name="weightFunction">The weight function.</param>
            public static WeightFunction Create(Func<double, double> weightFunction)
            {
                return new WeightFunction(weightFunction);
            }
            #endregion
        }
    }
}
