﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.ProbabilityTheory.Distributions
{
    /// <summary>Provides methods for cummulative distribution function etc. with respect to the Log-ormal distribution.
    /// </summary>
    /// <remarks>The density is specified by 
    /// <para>f(x) = 1/(\sigma * sqrt(2 \pi} * x) * exp( - [ln x-\mu]^2/(2 \sigma^2) for x > 0</para>
    /// and f(x) = 0 for x \leq 0.
    /// </remarks>
    public class LogNormalDistribution : IContinousDistribution1d
    {
        #region private members

        /// <summary>The mean of the log-normal distribution.
        /// </summary>
        private double m_Mean;

        /// <summary>The standard deviation of the log-normal distribution.
        /// </summary>
        private double m_Sigma;

        /// <summary>The name of the distribution.
        /// </summary>
        private IdentifierString m_Name;
        #endregion

        #region public static (readonly) members

        /// <summary>The Standard Log-Normal distribution.
        /// </summary>
        public static readonly LogNormalDistribution Standard;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="LogNormalDistribution"/> class.
        /// </summary>
        /// <param name="mean">The mean.</param>
        /// <param name="sigma">The standard deviation.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if one of the arguments is not valid.</exception>
        public LogNormalDistribution(double mean, double sigma)
        {
            if (sigma <= 0)
            {
                throw new ArgumentOutOfRangeException("sigma", String.Format(ExceptionMessages.ArgumentOutOfRange, sigma));
            }
            m_Sigma = sigma;
            if (Double.IsNaN(mean) == true)
            {
                throw new ArgumentOutOfRangeException("mean", String.Format(ExceptionMessages.ArgumentIsNaN, mean));
            }
            m_Mean = mean;
            m_Name = new IdentifierString(String.Format("Log-N({0}, {1})", m_Mean, m_Sigma * m_Sigma));
        }

        /// <summary>Initializes the <see cref="LogNormalDistribution" /> class.
        /// </summary>
        static LogNormalDistribution()
        {
            Standard = new LogNormalDistribution(0.0, 1.0);
        }
        #endregion

        #region public properties

        #region IIdentifierNameable Members

        /// <summary>Gets the name of the current instance.
        /// </summary>
        /// <value>The language independent name of the current instance.</value>
        public IdentifierString Name
        {
            get { return m_Name; }
        }

        /// <summary>Gets the long name of the current instance.
        /// </summary>
        /// <value>The (perhaps) language dependent long name of the current instance.</value>
        public IdentifierString LongName
        {
            get { return m_Name; }
        }
        #endregion

        #endregion

        #region public methods

        #region IContinousDistribution1d Members

        /// <summary>Evaluate the density function at a specified point.
        /// </summary>
        /// <param name="x">The value where to evaluate.</param>
        /// <returns>A <see cref="System.Double" /> which represents the value of the density function.</returns>
        public double GetDensityValue(double x)
        {
            if (x <= 0)
            {
                return 0.0;
            }
            double logOfX = Math.Log(x);
            return 1.0 / (m_Sigma * x * MathConsts.SqrtTwoPi) * Math.Exp(-(logOfX - m_Mean) * (logOfX - m_Mean) / (2 * m_Sigma * m_Sigma));

        }
        #endregion

        #region IDistribution1d Members

        /// <summary>Evaluate the cummulative distribution function at a specified point.
        /// </summary>
        /// <param name="x">The value where to evaluate.</param>
        /// <returns>A <see cref="System.Double" /> which represents the value of the cummulative distribution function.</returns>
        public double GetCdfValue(double x)
        {
            if (x <= 0)
            {
                return 0.0;
            }
            return NormalDistribution.GetStandardCdfValue((Math.Log(x) - m_Mean) / m_Sigma);
        }

        /// <summary>Gets the inverse of the cummulative distribution function at a specified point.
        /// </summary>
        /// <param name="probability">The probability where to evaluate.</param>
        /// <returns>A <see cref="System.Double" /> which represents the value of the inverse cummulative distribution function.</returns>
        public double GetInverseCdfValue(double probability)
        {
            return Math.Exp(NormalDistribution.GetStandardInverseCdfValue(probability) * m_Sigma + m_Mean);
        }
        #endregion

        #endregion

        #region public static methods

        /// <summary>Creates a new <see cref="LogNormalDistribution"/> instance.
        /// </summary>
        /// <param name="mean">The mean.</param>
        /// <param name="sigma">The standard deviation.</param>
        /// <returns>The <see cref="LogNormalDistribution"/> object with respect to the specified mean and standard deviation.</returns>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if one of the arguments is not valid.</exception>
        public static LogNormalDistribution Create(double mean, double sigma)
        {
            return new LogNormalDistribution(mean, sigma);
        }

        /// <summary>Evaluate the density of a standard Log-Normal random variable at a specified point.
        /// </summary>
        /// <param name="x">The value where to evaluate.</param>
        /// <returns>A <see cref="System.Double"/> which reflects the value of the density function.</returns>
        public static double GetStandardDensityValue(double x)
        {
            if (x <= 0)
            {
                return 0.0;
            }
            double logOfX = Math.Log(x);
            return 1.0 / (x * MathConsts.SqrtTwoPi) * Math.Exp(-logOfX * logOfX / 2.0);
        }

        /// <summary>Evaluate the cummulative distribution function of a standard Log-Normal random variable at a specified point.
        /// </summary>
        /// <param name="x">The value where to evaluate.</param>
        /// <returns>A <see cref="System.Double"/> which reflects the value of the cummulative distribution function.</returns>
        /// <remarks>The implementation is based on <para>'Better approximations to cumulative normal functions', G. West.</para></remarks>
        public static double GetStandardCdfValue(double x)
        {
            if (x <= 0)
            {
                return 0.0;
            }
            return NormalDistribution.GetStandardCdfValue((Math.Log(x)));
        }

        /// <summary>Gets the inverse of the standard cummulative distribution function.
        /// </summary>
        /// <param name="probability">The probability where to evaluate.</param>
        /// <returns>A <see cref="System.Double"/> which reflects the value of the inverse cummulative distribution function.</returns>
        public static double GetStandardInverseCdfValue(double probability)
        {
            return Math.Exp(NormalDistribution.GetStandardInverseCdfValue(probability));
        }
        #endregion
    }
}