﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.ProbabilityTheory.Distributions
{
    /// <summary>Provides methods for cummulative distribution function etc. with respect to the Normal distribution.
    /// </summary>
    public class NormalDistribution : IContinousDistribution1d
    {
        #region private members

        /// <summary>The mean of the normal distribution.
        /// </summary>
        private double m_Mean;

        /// <summary>The standard deviation of the normal distribution.
        /// </summary>
        private double m_Sigma;

        /// <summary>The name of the distribution.
        /// </summary>
        private IdentifierString m_Name;

        #region constants for the inverse cummulative standard normal distribution function

        /// <summary>The border for the lower region in the algorithm for the inverse cummulative standard normal distribution function.
        /// </summary>
        private static readonly double sm_LOW = 0.02425;

        /// <summary>The border for the upper region in the algorithm for the inverse cummulative standard normal distribution function.
        /// </summary>
        private static readonly double sm_HIGH = 0.97575;

        /// <summary>Coefficients in rational approximation of the inverse cummulative standard normal distribution function.
        /// </summary>
        private static readonly double[] sm_InvCDFCoefficientA = {
        -3.969683028665376e+01,
         2.209460984245205e+02,
        -2.759285104469687e+02,
         1.383577518672690e+02,
        -3.066479806614716e+01,
         2.506628277459239e+00};

        /// <summary>Coefficients in rational approximation of the inverse cummulative standard normal distribution function.
        /// </summary>
        private static readonly double[] sm_InvCDFCoefficientB = {
        -5.447609879822406e+01,
         1.615858368580409e+02,
        -1.556989798598866e+02,
         6.680131188771972e+01,
        -1.328068155288572e+01};

        /// <summary>Coefficients in rational approximation of the inverse cummulative standard normal distribution function.
        /// </summary>
        private static readonly double[] sm_InvCDFCoefficientC ={
        -7.784894002430293e-03,
        -3.223964580411365e-01,
        -2.400758277161838e+00,
        -2.549732539343734e+00,
         4.374664141464968e+00,
         2.938163982698783e+00};

        /// <summary>Coefficients in rational approximation of the inverse cummulative standard normal distribution function.
        /// </summary>
        private static readonly double[] sm_InvCDFCoefficientD ={
        7.784695709041462e-03,
        3.224671290700398e-01,
        2.445134137142996e+00,
        3.754408661907416e+00};
        #endregion

        #endregion

        #region public static (readonly) members

        /// <summary>The Standard Normal distribution, i.e. N(0,1).
        /// </summary>
        public static readonly NormalDistribution Standard;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="NormalDistribution"/> class.
        /// </summary>
        /// <param name="mean">The mean.</param>
        /// <param name="sigma">The standard deviation.</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if one of the arguments is not valid.</exception>
        public NormalDistribution(double mean, double sigma)
        {
            if (sigma <= 0)
            {
                throw new ArgumentOutOfRangeException("sigma", String.Format(ExceptionMessages.ArgumentOutOfRange, sigma));
            }
            m_Sigma = sigma;
            if (Double.IsNaN(mean))
            {
                throw new ArgumentOutOfRangeException("mean", String.Format(ExceptionMessages.ArgumentIsNaN, mean));
            }
            m_Mean = mean;
            m_Name = new IdentifierString(String.Format("Normal distribution N({0}, {1})", m_Mean, m_Sigma * m_Sigma));
        }

        /// <summary>Initializes the <see cref="NormalDistribution" /> class.
        /// </summary>
        static NormalDistribution()
        {
            Standard = new NormalDistribution(0.0, 1.0);
        }
        #endregion

        #region public properties

        #region IIdentifierNameable Members

        /// <summary>Gets the name of the current instance.
        /// </summary>
        /// <value>The language independent name of the current instance.</value>
        public IdentifierString Name
        {
            get { return m_Name; }
        }

        /// <summary>Gets the long name of the current instance.
        /// </summary>
        /// <value>The (perhaps) language dependent long name of the current instance.</value>
        public IdentifierString LongName
        {
            get { return m_Name; }
        }
        #endregion

        #endregion

        #region public methods

        #region IContinousDistribution1d Members

        /// <summary>Evaluate the density function at a specified point.
        /// </summary>
        /// <param name="x">The value where to evaluate.</param>
        /// <returns>A <see cref="System.Double" /> which represents the value of the density function.</returns>
        public double GetDensityValue(double x)
        {
            return MathConsts.OneOverSqrtTwoPi / m_Sigma * Math.Exp(-(x - m_Mean) * (x - m_Mean) / (2.0 * m_Sigma));
        }
        #endregion

        #region IDistribution1d Members

        /// <summary>Evaluate the cummulative distribution function at a specified point.
        /// </summary>
        /// <param name="x">The value where to evaluate.</param>
        /// <returns>A <see cref="System.Double" /> which represents the value of the cummulative distribution function.</returns>
        public double GetCdfValue(double x)
        {
            return GetStandardCdfValue((x - m_Mean) / m_Sigma);
        }

        /// <summary>Gets the inverse of the cummulative distribution function at a specified point.
        /// </summary>
        /// <param name="probability">The probability where to evaluate.</param>
        /// <returns>A <see cref="System.Double" /> which represents the value of the inverse cummulative distribution function.</returns>
        public double GetInverseCdfValue(double probability)
        {
            return GetStandardInverseCdfValue(probability) * m_Sigma + m_Mean;
        }
        #endregion

        #endregion

        #region public static methods

        /// <summary>Creates a new <see cref="NormalDistribution"/> instance.
        /// </summary>
        /// <param name="mean">The mean.</param>
        /// <param name="sigma">The standard deviation.</param>
        /// <returns>The <see cref="NormalDistribution"/> object with respect to the specified mean and standard deviation.</returns>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if one of the arguments is not valid.</exception>
        public static NormalDistribution Create(double mean, double sigma)
        {
            return new NormalDistribution(mean, sigma);
        }

        /// <summary>Evaluate the density of a standard normal random variable at a specified point.
        /// </summary>
        /// <param name="x">The value where to evaluate.</param>
        /// <returns>A <see cref="System.Double"/> which reflects the value of the density function.</returns>
        public static double GetStandardDensityValue(double x)
        {
            return MathConsts.OneOverSqrtTwoPi * Math.Exp(-x * x * 0.5);
        }

        /// <summary>Evaluate the cummulative distribution function of a standard normal random variable at a specified point.
        /// </summary>
        /// <param name="x">The value where to evaluate.</param>
        /// <returns>A <see cref="System.Double"/> which reflects the value of the cummulative distribution function.</returns>
        /// <remarks>The implementation is based on <para>'Better approximations to cumulative normal functions', G. West.</para></remarks>
        public static double GetStandardCdfValue(double x)
        {
            double xAbs = Math.Abs(x);
            double cdfValue = 0.0;
            double build;

            if (xAbs <= 37)
            {
                double expontential = Math.Exp(-xAbs * xAbs / 2.0);
                if (xAbs < 7.07106781186547)
                {
                    build = 3.52624965998911E-02 * xAbs + 0.700383064443688;
                    build = build * xAbs + 6.37396220353165;
                    build = build * xAbs + 33.912866078383;
                    build = build * xAbs + 112.079291497871;
                    build = build * xAbs + 221.213596169931;
                    build = build * xAbs + 220.206867912376;
                    cdfValue = expontential * build;
                    build = 8.83883476483184E-02 * xAbs + 1.75566716318264;
                    build = build * xAbs + 16.064177579207;
                    build = build * xAbs + 86.7807322029461;
                    build = build * xAbs + 296.564248779674;
                    build = build * xAbs + 637.333633378831;
                    build = build * xAbs + 793.826512519948;
                    build = build * xAbs + 440.413735824752;
                    cdfValue = cdfValue / build;
                }
                else
                {
                    build = xAbs + 0.65;
                    build = xAbs + 4 / build;
                    build = xAbs + 3 / build;
                    build = xAbs + 2 / build;
                    build = xAbs + 1 / build;
                    cdfValue = expontential / build / 2.506628274631;
                }
            }
            if (x > 0)
            {
                return 1.0 - cdfValue;
            }
            return cdfValue;
        }

        /// <summary>Gets the inverse of the standard cummulative distribution function.
        /// </summary>
        /// <param name="probability">The probability where to evaluate.</param>
        /// <returns>A <see cref="System.Double"/> which reflects the value of the inverse cummulative distribution function.</returns>
        /// <remarks>
        /// The algorithm uses a minimax approximation by rational functions and the result has a relative error whose absolute value is less 
        /// than 1.15e-9. Adapted from Peter's Perl version by Chad Sprouse (http://home.online.no/~pjacklam/notes/invnorm/impl/sprouse/).
        /// </remarks>
        public static double GetStandardInverseCdfValue(double probability)
        {
            double q, r;

            if ((probability < 0.0) || (probability > 1))
            {
                throw new ArgumentOutOfRangeException("probability", String.Format(ExceptionMessages.ArgumentOutOfConstraint, probability, "[0;1]"));
            }
            else if (probability == 0)
            {
                return double.MinValue;
            }
            else if (probability == 1)
            {
                return double.MaxValue;
            }
            else if (probability < sm_LOW) /* Rational approximation for lower region */
            {
                q = Math.Sqrt(-2 * Math.Log(probability));
                return (((((sm_InvCDFCoefficientC[0] * q + sm_InvCDFCoefficientC[1]) * q + sm_InvCDFCoefficientC[2]) * q + sm_InvCDFCoefficientC[3]) * q + sm_InvCDFCoefficientC[4]) * q + sm_InvCDFCoefficientC[5]) /
                    ((((sm_InvCDFCoefficientD[0] * q + sm_InvCDFCoefficientD[1]) * q + sm_InvCDFCoefficientD[2]) * q + sm_InvCDFCoefficientD[3]) * q + 1);
            }
            else if (probability > sm_HIGH) /* Rational approximation for upper region */
            {
                q = Math.Sqrt(-2 * Math.Log(1 - probability));
                return -(((((sm_InvCDFCoefficientC[0] * q + sm_InvCDFCoefficientC[1]) * q + sm_InvCDFCoefficientC[2]) * q + sm_InvCDFCoefficientC[3]) * q + sm_InvCDFCoefficientC[4]) * q + sm_InvCDFCoefficientC[5]) /
                    ((((sm_InvCDFCoefficientD[0] * q + sm_InvCDFCoefficientD[1]) * q + sm_InvCDFCoefficientD[2]) * q + sm_InvCDFCoefficientD[3]) * q + 1);
            }
            else
            {
                /* Rational approximation for central region */
                q = probability - 0.5;
                r = q * q;
                return (((((sm_InvCDFCoefficientA[0] * r + sm_InvCDFCoefficientA[1]) * r + sm_InvCDFCoefficientA[2]) * r + sm_InvCDFCoefficientA[3]) * r + sm_InvCDFCoefficientA[4]) * r + sm_InvCDFCoefficientA[5]) * q /
                    (((((sm_InvCDFCoefficientB[0] * r + sm_InvCDFCoefficientB[1]) * r + sm_InvCDFCoefficientB[2]) * r + sm_InvCDFCoefficientB[3]) * r + sm_InvCDFCoefficientB[4]) * r + 1);
            }
        }
        #endregion
    }
}