﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Linq;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.MathLibrary.GridPointCurves;

namespace Dodoni.MathLibrary.Surfaces
{
    internal abstract partial class VerticalHorizontalWiseSurface2d<THorizontalLabel, TVerticalLabel> : IGridPointSurface2d<THorizontalLabel, TVerticalLabel>
        where THorizontalLabel : IComparable<THorizontalLabel>
        where TVerticalLabel : IComparable<TVerticalLabel>, IEquatable<TVerticalLabel>
    {
        /// <summary>Represents a two-dimensional grid point surface, where a parametrization takes place in horizontal direction.
        /// </summary>
        internal class HParametrization : VerticalHorizontalWiseSurface2d<THorizontalLabel, TVerticalLabel>
        {
            #region private members

            /// <summary>The interpolator factory along horizontal direction.
            /// </summary>
            private GridPointCurve.Parametrization m_HorizontalParametrizationFactory;

            /// <summary>The interpolation approach along vertical direction.
            /// </summary>
            private ICurveBuilder m_HorizontalParametrization;

            /// <summary>A temporary array that contains the values for a interpolation etc. along horizontal direction, i.e. at most <c>m_HorizontalDoubleLabels.Length</c> elements.
            /// </summary>
            /// <remarks>This member is used for performance reason.</remarks>
            private double[] m_TempValuesForHorizontalEvaluation;

            /// <summary>A temporary array that contains the <see cref="System.Double"/> representation of some horizontal labels.
            /// </summary>
            /// <remarks>This member is used for performance reason.</remarks>
            private double[] m_TempHorizontalDoubleLabels;
            #endregion

            #region internal constructors

            /// <summary>Initializes a new instance of the <see cref="HParametrization"/> class.
            /// </summary>
            /// <param name="gridPointMatrix">The grid point matrix.</param>
            /// <param name="verticalCurveFactory">A factory for grid point curves along vertical direction, i.e. taken into account a specified interpolation, parametrization etc.</param>
            /// <param name="horizontalParametrization">The interpolation approach along horizontal direction.</param>
            internal HParametrization(LabelMatrix<THorizontalLabel, TVerticalLabel> gridPointMatrix, Func<TVerticalLabel, IGridPointCurveFactory<TVerticalLabel>> verticalCurveFactory, GridPointCurve.Parametrization horizontalParametrization)
                : base(gridPointMatrix, verticalCurveFactory)
            {
                if (horizontalParametrization == null)
                {
                    throw new NullReferenceException("horizontalParametrization");
                }
                m_HorizontalParametrizationFactory = horizontalParametrization;
                m_HorizontalParametrization = horizontalParametrization.Create();

                m_TempValuesForHorizontalEvaluation = new double[m_HorizontalDoubleLabels.Length];
                m_TempHorizontalDoubleLabels = new double[m_GridPointMatrix.ColumnCount];
            }
            #endregion

            #region public methods

            /// <summary>Gets a value of the surface at a specified (x,y) coordinate.
            /// </summary>
            /// <param name="x">The x coordinate of the point.</param>
            /// <param name="y">The y coordinate of the point.</param>
            /// <returns>The value of the surface at (<paramref name="x"/>, <paramref name="y"/>).</returns>
            public override double GetValue(double x, double y)
            {
                int columnCount = m_HorizontalDoubleLabels.Length;

                /* compute values w.r.t. to the y-coordinate and each relevant column, i.e. store values to take into account for horizontal 
                 * interpolation. In the case of a non-local approach, one may improve performance by indicating that the labels will not be changed: */
                int k = 0;
                for (int j = 0; j < columnCount; j++)
                {
                    m_TempValuesForHorizontalEvaluation[k++] = m_CurvesAlongVerticalDirection[j].GetValue(y);
                }

                GridPointCurve.State horizontalCurveState = GridPointCurve.State.GridPointChanged;
                if ((m_HorizontalParametrization.IsOperable == true) && (m_HorizontalParametrization.GridPointCount == columnCount))
                {
                    horizontalCurveState = GridPointCurve.State.GridPointValueChanged;
                }
                m_HorizontalParametrization.Update(columnCount, m_HorizontalDoubleLabels, m_TempValuesForHorizontalEvaluation, horizontalCurveState);
                if ((x < m_HorizontalParametrization.LowerBound) || (x > m_HorizontalParametrization.UpperBound))
                {
                    throw new ArgumentOutOfRangeException();
                }
                return m_HorizontalParametrization.GetValue(x);
            }

            /// <summary>Gets a value at a specified point (<paramref name="x"/>, <paramref name="rowIndex"/>).
            /// </summary>
            /// <param name="x">The x coordinate of the point.</param>
            /// <param name="rowIndex">The null-based row index of the point.</param>
            /// <returns>The value of the surface at (<paramref name="x"/>, <paramref name="rowIndex"/>).</returns>
            public override double GetValue(double x, int rowIndex)
            {
                if (m_VerticalExactFitToGridPoints == false)
                {
                    if (m_HorizontalLabelMapping == null)
                    {
                        return GetValue(x, m_GridPointMatrix.VerticalDoubleLabels[rowIndex]);
                    }
                    else
                    {
                        return GetValue(x, m_GridPointMatrix.VerticalDoubleLabels[m_HorizontalLabelMapping[rowIndex]]);
                    }
                }

                /* if the vertical curve fitting meets the grid points, an evaluation of the vertical curve fitting is not necessary. Here, we do not distinguish 
                 * between local and non-local interpolation approach and we take into account all values (!= NaN) along horizontal direction: */

                int numberOfRelevantColumns = 0;

                for (int j = 0; j < m_GridPointMatrix.ColumnCount; j++)
                {
                    double gridValue = m_GridPointMatrix[rowIndex, j];
                    if (Double.IsNaN(gridValue) == false)
                    {
                        m_TempValuesForHorizontalEvaluation[numberOfRelevantColumns] = gridValue;
                        m_TempHorizontalDoubleLabels[numberOfRelevantColumns] = m_GridPointMatrix.HorizontalDoubleLabels[j];
                        numberOfRelevantColumns++;
                    }
                }
                GridPointCurve.State horizontalCurveState = GridPointCurve.State.GridPointChanged;
                if ((m_HorizontalParametrization.GridPointCount == m_GridPointMatrix.ColumnCount) && (numberOfRelevantColumns == m_GridPointMatrix.ColumnCount))
                {
                    horizontalCurveState = GridPointCurve.State.GridPointValueChanged;
                }
                m_HorizontalParametrization.Update(numberOfRelevantColumns, m_TempHorizontalDoubleLabels, m_TempValuesForHorizontalEvaluation, horizontalCurveState);

                if ((x < m_HorizontalParametrization.LowerBound) || (x > m_HorizontalParametrization.UpperBound))
                {
                    throw new ArgumentOutOfRangeException();
                }
                return m_HorizontalParametrization.GetValue(x);
            }
            #endregion
        }
    }
}