﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Collections.Generic;

using NUnit.Framework;

namespace Dodoni.MathLibrary.GridPointCurves
{
    /// <summary>Serves as unit test class for the linear curve interpolator.
    /// </summary>
    public class InterpolatorLinearTests
    {
        /// <summary>A test function for the GetIntegral method of a linear interpolator object.
        /// </summary>
        /// <param name="lowerBound">The lower bound.</param>
        /// <param name="upperBound">The upper bound.</param>
        [TestCase(0.0, 1.2)]
        [TestCase(-0.05, 2.75)]
        [TestCase(1.8, 4.75)]
        public void GetIntegral_TestCase_BenchmarkNumericalIntegrator(double lowerBound, double upperBound)
        {
            var linearInterpolator = GetTestCaseInterpolatorObject();

            double actual = linearInterpolator.GetIntegral(lowerBound, upperBound);

            BenchmarkIntegrator benchmarkIntegrator = new BenchmarkIntegrator();
            benchmarkIntegrator.FunctionToIntegrate = linearInterpolator.GetValue;

            double expected = benchmarkIntegrator.GetValue(lowerBound, upperBound);

            Assert.That(actual, Is.EqualTo(expected).Within(1E-5));
        }

        /// <summary>A test function for the GetIntegral method of a linear interpolator object.
        /// </summary>
        [Test]
        public void GetIntegral_TestCaseFromGridPointToGridPoint_AnalyticResult()
        {
            var linearInterpolator = GetTestCaseInterpolatorObject();

            for (int j = 0; j < linearInterpolator.GridPointCount - 1; j++)
            {
                double lowerIntegralBound = linearInterpolator.GridPointArguments[j];
                double upperIntegralBound = linearInterpolator.GridPointArguments[j + 1];

                double actual = linearInterpolator.GetIntegral(lowerIntegralBound, upperIntegralBound);

                double m = (linearInterpolator.GridPointValues[j + 1] - linearInterpolator.GridPointValues[j]) / (linearInterpolator.GridPointArguments[j + 1] - linearInterpolator.GridPointArguments[j]);
                double b = linearInterpolator.GridPointValues[j] - m * linearInterpolator.GridPointArguments[j];

                double expected = 0.5 * m * upperIntegralBound * upperIntegralBound + upperIntegralBound * b - (0.5 * m * lowerIntegralBound * lowerIntegralBound + b * lowerIntegralBound);

                Assert.That(actual, Is.EqualTo(expected).Within(1E-6), String.Format("Integral from gridpoints: {0} to {1}; null-based index of the left grid point {2}.", lowerIntegralBound, upperIntegralBound, j));
            }
        }

        /// <summary>A test function for the GetIntegral method of a linear interpolator object.
        /// </summary>
        [Test]
        public void GetIntegral_TestCaseInsideTwoGridPoints_AnalyticResult()
        {
            var linearInterpolator = GetTestCaseInterpolatorObject();

            for (int j = 0; j < linearInterpolator.GridPointCount - 1; j++)
            {
                double width = linearInterpolator.GridPointArguments[j + 1] - linearInterpolator.GridPointArguments[j];

                double lowerIntegralBound = linearInterpolator.GridPointArguments[j] + width / 4.0;
                double upperIntegralBound = linearInterpolator.GridPointArguments[j + 1] - width / 4.0;

                double actual = linearInterpolator.GetIntegral(lowerIntegralBound, upperIntegralBound);

                double m = (linearInterpolator.GridPointValues[j + 1] - linearInterpolator.GridPointValues[j]) / (linearInterpolator.GridPointArguments[j + 1] - linearInterpolator.GridPointArguments[j]);
                double b = linearInterpolator.GridPointValues[j] - m * linearInterpolator.GridPointArguments[j];

                double expected = 0.5 * m * upperIntegralBound * upperIntegralBound + upperIntegralBound * b - (0.5 * m * lowerIntegralBound * lowerIntegralBound + b * lowerIntegralBound);

                Assert.That(actual, Is.EqualTo(expected).Within(1E-6), String.Format("Integral from gridpoints: {0} to {1}; null-based index of the left grid point {2}.", lowerIntegralBound, upperIntegralBound, j));
            }
        }

        /// <summary>A test function for the GetIntegral method of a linear interpolator object.
        /// </summary>
        [Test]
        public void GetIntegral_TestCase_AnalyticResult()
        {
            var linearInterpolator = GetTestCaseInterpolatorObject();

            for (int j = 0; j < linearInterpolator.GridPointCount - 2; j++)
            {
                var lowerGridPointArgument = linearInterpolator.GridPointArguments[j];
                var lowerGridPointValue = linearInterpolator.GridPointValues[j];

                var midGridPointArgument = linearInterpolator.GridPointArguments[j + 1];
                var midGridPointValue = linearInterpolator.GridPointValues[j + 1];

                var higherGridPointArgument = linearInterpolator.GridPointArguments[j + 2];
                var higherGridPointValue = linearInterpolator.GridPointValues[j + 2];

                var width = Math.Min(midGridPointArgument - lowerGridPointArgument, higherGridPointArgument - midGridPointArgument);

                var lowerIntegralBound = lowerGridPointArgument + width / 4.0;
                var upperIntegralBound = higherGridPointArgument - width / 4.0;

                var actual = linearInterpolator.GetIntegral(lowerIntegralBound, upperIntegralBound);


                var m1 = (midGridPointValue - lowerGridPointValue) / (midGridPointArgument - lowerGridPointArgument);
                var b1 = lowerGridPointValue - m1 * lowerGridPointArgument;

                var integralValueFromlowerBoundToMidGridPoint = 0.5 * m1 * midGridPointArgument * midGridPointArgument + midGridPointArgument * b1 - (0.5 * m1 * lowerIntegralBound * lowerIntegralBound + b1 * lowerIntegralBound);

                var m2 = (higherGridPointValue - midGridPointValue) / (higherGridPointArgument - midGridPointArgument);
                var b2 = midGridPointValue - m2 * midGridPointArgument;

                double integralValueFromMidGridPointToUpperBound = 0.5 * m2 * upperIntegralBound * upperIntegralBound + upperIntegralBound * b2 - (0.5 * m2 * midGridPointArgument * midGridPointArgument + b2 * midGridPointArgument);

                double expected = integralValueFromlowerBoundToMidGridPoint + integralValueFromMidGridPointToUpperBound;
                Assert.That(actual, Is.EqualTo(expected).Within(1E-6), String.Format("Integral from gridpoints: {0} to {1}; null-based index of the left grid point {2}.", lowerIntegralBound, upperIntegralBound, j));
            }
        }

        /// <summary>A test function for the GetValue method of a linear interpolator object.
        /// </summary>
        /// <param name="x">The point where to evaluate.</param>
        /// <param name="leftGridPointIndex">The null-based index of the left grid point with respect to <paramref name="x"/>.</param>
        [TestCase(1.0, 2)]
        [TestCase(1.5, 2)]
        [TestCase(-0.5, 0)]
        [TestCase(-0.2, 0)]
        [TestCase(5.0, 3)]
        public void GetValue_TestCaseData_AnalyticValue(double x, int leftGridPointIndex)
        {
            var linearInterpolator = GetTestCaseInterpolatorObject();

            double actual = linearInterpolator.GetValue(x);

            double m = (linearInterpolator.GridPointValues[leftGridPointIndex] - linearInterpolator.GridPointValues[leftGridPointIndex + 1]) / (linearInterpolator.GridPointArguments[leftGridPointIndex] - linearInterpolator.GridPointArguments[leftGridPointIndex + 1]);
            double b = linearInterpolator.GridPointValues[leftGridPointIndex] - m * linearInterpolator.GridPointArguments[leftGridPointIndex];
            double expected = m * x + b;

            Assert.That(actual, Is.EqualTo(expected).Within(1E-6));
        }

        #region protected methods

        /// <summary>Gets the test case interpolator object.
        /// </summary>
        /// <returns></returns>
        protected ICurveBuilder GetTestCaseInterpolatorObject()
        {
            var linearInterpolator = GridPointCurve.Interpolator.Linear.Create();

            linearInterpolator.Update(5, new[] { -0.5, 0.0, 0.1, 2.75, 5.0 },
                                         new[] { 1.0, -2.5, 3.0, -1.25, 1.75 });
            return linearInterpolator;
        }
        #endregion
    }
}