﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Linq;
using System.Numerics;
using System.Collections.Generic;

using NUnit.Framework;
using Dodoni.MathLibrary.Basics;

namespace Dodoni.MathLibrary.Miscellaneous
{
    /// <summary>Serves as abstract unit test class for <see cref="IPolynomial"/> objects.
    /// </summary>
    [TestFixture]
    public abstract class PolynomialTests
    {
        /// <summary>A test function for <see cref="ComplexPolynomialFactory.GetCoefficientsByRoots(int, IList{Complex}, IList{Complex}, Complex)"/>.
        /// </summary>
        /// <param name="degree">The degree.</param>
        /// <param name="coefficients">The coefficients of the polynomial.</param>
        [TestCaseSource("GetComplexCoefficientsByRootsTestCaseData")]
        public void GetCoefficientsByRoots_TestCasePolynomialCoefficients_Coefficients(int degree, Complex[] coefficients)
        {
            var expectedCoefficients = coefficients.ToArray();

            var polynomial = Polynomial.Complex.Create(degree, coefficients);
            var polynomialRootFinder = GetPolynomialRootFinder();

            var roots = new List<Complex>();
            polynomial.GetRoots(roots, polynomialRootFinder);

            var actualCoefficients = new Complex[degree + 1];
            Polynomial.Complex.GetCoefficientsByRoots(degree, roots, actualCoefficients, coefficients[degree]);

            Assert.That(actualCoefficients, new ComplexArrayNUnitConstraint(expectedCoefficients, tolerance: 1E-8));
        }

        /// <summary>Gets the test case data for polynomials with complex coefficients.
        /// </summary>
        /// <value>The test data for polynomials with complex coefficients.</value>
        public IEnumerable<TestCaseData> GetComplexCoefficientsByRootsTestCaseData
        {
            get
            {
                yield return new TestCaseData(5, new Complex[] { 1.25, 2, 3, 4, 5, 1.0 });
                yield return new TestCaseData(6, new Complex[] { 1.25 + 0.5 * Complex.ImaginaryOne, 2, 3, 4 * Complex.ImaginaryOne, 5.25, 1.0, -0.75 });
                yield return new TestCaseData(7, new Complex[] { 4.25 + 1.25 * Complex.ImaginaryOne, 2, 3 * Complex.ImaginaryOne, 1.65 * Complex.ImaginaryOne, 5.25, 1.0, -0.75, 3.62 * Complex.ImaginaryOne });
            }
        }

        /// <summary>A test function for <see cref="RealPolynomialFactory.GetCoefficientsByRoots(int, IList{Complex}, IList{double}, double,double)"/>.
        /// </summary>
        /// <param name="degree">The degree.</param>
        /// <param name="coefficients">The coefficients of the polynomial.</param>
        [TestCaseSource("GetRealCoefficientsByRootsTestCaseData")]
        public void GetCoefficientsByRoots_TestCasePolynomialCoefficients_Coefficients(int degree, double[] coefficients)
        {
            var expectedCoefficients = coefficients.ToArray();

            var polynomial = Polynomial.Real.Create(degree, coefficients);
            var polynomialRootFinder = GetPolynomialRootFinder();

            var roots = new List<Complex>();
            polynomial.GetRoots(roots, polynomialRootFinder);

            var actualCoefficients = new double[degree + 1];
            Polynomial.Real.GetCoefficientsByRoots(degree, roots, actualCoefficients, coefficients[degree]);

            Assert.That(actualCoefficients, Is.EqualTo(expectedCoefficients).AsCollection.Within(1E-8));
        }


        /// <summary>Gets the test case data for polynomials with real coefficients.
        /// </summary>
        /// <value>The test data for polynomials with real coefficients.</value>
        public IEnumerable<TestCaseData> GetRealCoefficientsByRootsTestCaseData
        {
            get
            {
                yield return new TestCaseData(5, new double[] { 1.25, 2, 3, 4, 5, 1.0 });
                yield return new TestCaseData(6, new double[] { 1.25, 2, 3, 4, 5.25, 1.0, -0.75 });
                yield return new TestCaseData(7, new double[] { 4.25, 2, 3, 1.65, 5.25, 1.0, -0.75, 3.62 });
            }
        }

        /// <summary>Gets the <see cref="IPolynomialRootFinder"/> object under test.
        /// </summary>
        /// <returns>The <see cref="IPolynomialRootFinder"/> under test.</returns>
        protected abstract IPolynomialRootFinder GetPolynomialRootFinder();
    }
}