﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Linq;
using System.Numerics;
using System.Collections.Generic;

using NUnit.Framework;
using Dodoni.MathLibrary.Basics;

namespace Dodoni.MathLibrary.Miscellaneous
{
    /// <summary>Serves as unit test class for analytical polynomial root finder approaches.
    /// </summary>
    [TestFixture]
    public class AnalyticalPolynomialRootFinderTests
    {
        /// <summary>A test function for <see cref="Polynomial.RootFinder.Analytical.GetRoots(double, double, double, double, double, out System.Numerics.Complex, out System.Numerics.Complex, out System.Numerics.Complex, out System.Numerics.Complex)"/>
        /// </summary>
        /// <param name="absoluteCoefficient">The absolute coefficient.</param>
        /// <param name="firstOrderCoefficient">The first order coefficient.</param>
        /// <param name="secondOrderCoefficient">The second order coefficient.</param>
        /// <param name="thirdOrderCoefficient">The third order coefficient.</param>
        /// <param name="fourthOrderCoefficient">The fourth order coefficient.</param>
        [TestCase(1, 2, 3, 4, 5)]
        [TestCase(1, 2, 3, 4, 0.0)]
        [TestCase(1, 2, 3, 0.0, 0.0)]
        public void GetRoots_TestCaseData_ZeroFunctionValue(double absoluteCoefficient, double firstOrderCoefficient, double secondOrderCoefficient, double thirdOrderCoefficient, double fourthOrderCoefficient)
        {
            Complex[] roots = new Complex[4];
            int rootCount = Polynomial.RootFinder.Analytical.GetRoots(absoluteCoefficient, firstOrderCoefficient, secondOrderCoefficient, thirdOrderCoefficient, fourthOrderCoefficient, out roots[0], out roots[1], out roots[2], out roots[3]);

            Assert.That(rootCount, Is.GreaterThan(0), "No root found for the specified polynomial!");

            var polynomial = Polynomial.Real.Create(absoluteCoefficient, firstOrderCoefficient, secondOrderCoefficient, thirdOrderCoefficient, fourthOrderCoefficient);
            var functionValues = roots.Select(z => polynomial.GetValue(z)).ToArray();

            for (int j = 0; j < rootCount; j++)
            {
                Assert.That(Complex.Abs(functionValues[j]), Is.EqualTo(0.0).Within(1E-8));
            }
        }

        /// <summary>A test function for <see cref="Polynomial.RootFinder.Analytical.GetRoots(Complex, Complex, Complex, Complex, out System.Numerics.Complex, out System.Numerics.Complex, out System.Numerics.Complex)"/>
        /// </summary>
        /// <param name="absoluteCoefficient">The absolute coefficient.</param>
        /// <param name="firstOrderCoefficient">The first order coefficient.</param>
        /// <param name="secondOrderCoefficient">The second order coefficient.</param>
        /// <param name="thirdOrderCoefficient">The third order coefficient.</param>
        [TestCaseSource("ComplexTestCaseData")]
        public void GetRoots_ComplexTestCaseData_ZeroFunctionValue(Complex absoluteCoefficient, Complex firstOrderCoefficient, Complex secondOrderCoefficient, Complex thirdOrderCoefficient)
        {
            Complex[] roots = new Complex[3];
            int rootCount = Polynomial.RootFinder.Analytical.GetRoots(absoluteCoefficient, firstOrderCoefficient, secondOrderCoefficient, thirdOrderCoefficient, out roots[0], out roots[1], out roots[2]);

            Assert.That(rootCount, Is.GreaterThan(0), "No root found for the specified polynomial!");

            var polynomial = Polynomial.Complex.Create(absoluteCoefficient, firstOrderCoefficient, secondOrderCoefficient, thirdOrderCoefficient);
            var functionValues = roots.Select(z => polynomial.GetValue(z)).ToArray();

            for (int j = 0; j < rootCount; j++)
            {
                Assert.That(Complex.Abs(functionValues[j]), Is.EqualTo(0.0).Within(1E-8));
            }
        }

        /// <summary>Gets the test case data for root calculation with respect to complex coefficients.
        /// </summary>
        /// <value>The test case data with complex coefficients.
        /// </value>
        public IEnumerable<TestCaseData> ComplexTestCaseData
        {
            get
            {
                yield return new TestCaseData((Complex)1.0, (Complex)2.0, (Complex)3.0, (Complex)4.0);
                yield return new TestCaseData((Complex)1, (Complex)2, (Complex)3, (Complex)0.0);
                yield return new TestCaseData((Complex)1, (Complex)2, (Complex)0.0, (Complex)0.0);
            }
        }
    }
}