﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Linq;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.MathLibrary.Basics.LowLevel.Native;

namespace Dodoni.MathLibrary.ProbabilityTheory.MonteCarloEngine
{
    /// <summary>Represents the .net wrapper for Random Number Generators of AMD's ACML library.
    /// </summary>    
    [RandomNumberLibrary]
    public class AcmlRandomNumberLibrary : IRandomNumberLibrary
    {
        #region private members

        /// <summary>The name of the library.
        /// </summary>
        private static readonly IdentifierString sm_Name = new IdentifierString("ACML");
        #endregion

        #region public (readonly) members

        /// <summary>The NAG basic generator is a linear congruential generator.
        /// </summary>
        public readonly AcmlPseudoRandomNumberGenerator NAG;

        /// <summary>The Wichmann-Hill base generator uses a combination of four linear congruential generators.
        /// </summary>
        public readonly AcmlPseudoRandomNumberGenerator WichmannHill;

        /// <summary>The Mersenne Twister is a twisted generalized feedback shift register generator
        /// </summary>
        public readonly AcmlPseudoRandomNumberGenerator MT19937;

        /// <summary>The L'Ecuyer's combined recursive generator combines two multiple recursive generators.
        /// </summary>
        public readonly AcmlPseudoRandomNumberGenerator Ecuyer;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="AcmlRandomNumberLibrary"/> class.
        /// </summary>
        public AcmlRandomNumberLibrary()
        {
            NAG = new AcmlPseudoRandomNumberGenerator(1, 16, "NAG_acml", new AcmlRandomNumberInitialConditions.Descriptor(), AcmlResources.NAG, RandomNumberSequence.SplittingApproach.LeapFrog | RandomNumberSequence.SplittingApproach.SkipAhead, 1, this);
            WichmannHill = new AcmlPseudoRandomNumberGenerator(2, 20, "WichmannHill_acml", new AcmlRandomNumberInitialConditions.Descriptor(), AcmlResources.WichmannHill, RandomNumberSequence.SplittingApproach.LeapFrog | RandomNumberSequence.SplittingApproach.SkipAhead, 273, this);
            MT19937 = new AcmlPseudoRandomNumberGenerator(3, 633, "MersenneTwister_acml", new AcmlRandomNumberInitialConditions.Descriptor(), AcmlResources.MersenneTwister, RandomNumberSequence.SplittingApproach.None, 1, this);
            Ecuyer = new AcmlPseudoRandomNumberGenerator(4, 61, "Ecuyer_acml", new AcmlRandomNumberInitialConditions.Descriptor(), AcmlResources.Ecuyer, RandomNumberSequence.SplittingApproach.LeapFrog | RandomNumberSequence.SplittingApproach.SkipAhead, 1, this);
        }
        #endregion

        #region public properties

        #region IIdentifierNameable Members

        /// <summary>Gets the long name of the Random Number Library.
        /// </summary>
        /// <value>The (perhaps) language dependent long name of the Random Number library.
        /// </value>
        public IdentifierString LongName
        {
            get { return sm_Name; }
        }

        /// <summary>Gets the name of the Random Number Library.
        /// </summary>
        /// <value>The language independent name of the Random Number library.</value>
        public IdentifierString Name
        {
            get { return sm_Name; }
        }
        #endregion

        #region IAnnotatable Members

        /// <summary>Gets the annotation of the current instance.
        /// </summary>
        /// <value>The annotation of the current instance.</value>
        public string Description
        {
            get { return String.Format(AcmlResources.LibraryDescription, AcmlNativeWrapper.dllName); }
        }
        #endregion

        #region IRandomNumberLibrary Members

        /// <summary>Gets the collection of Pseudo-Random Number Generators provided by the library.
        /// </summary>
        /// <value>The collection of Pseudo-Random Number Generator.</value>
        public IEnumerable<IPseudoRandomNumberGenerator> PseudoRandomNumberGenerators
        {
            get
            {
                yield return NAG;
                yield return WichmannHill;
                yield return MT19937;
                yield return Ecuyer;
            }
        }

        /// <summary>Gets the collection of Quasi-Random Number Generators provided by the library.
        /// </summary>
        /// <value>The collection of Quasi-Random Number Generator.</value>
        public IEnumerable<IQuasiRandomNumberGenerator> QuasiRandomNumberGenerator
        {
            get { return Enumerable.Empty<IQuasiRandomNumberGenerator>(); }
        }
        #endregion

        #endregion
    }
}