﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Runtime.InteropServices;
using System.ComponentModel.Composition;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.Basics.LowLevel.Native
{
    /// <summary>Represents a wrapper for BLAS operation with respect to the C-interface (CBLAS). See http://www.netlib.org/blas for further information.
    /// </summary>
    [Export(typeof(BLAS.ILibrary))]
    public partial class CBlasNativeWrapper : BLAS.ILibrary
    {
        #region public const members

        /// <summary>The name of the dll needed for the Basic Linear Algebra Subprograms (BLAS), Fortran interface.
        /// </summary>
        public const string dllName = "libCBLAS.dll";

        /// <summary>The calling convention of the external BLAS dll.
        /// </summary>
        public const CallingConvention callingConvention = CallingConvention.Cdecl;
        #endregion

        #region private members

        /// <summary>The name of the Library.
        /// </summary>
        private IdentifierString m_Name;

        /// <summary>Level 1 operations.
        /// </summary>
        private ILevel1BLAS m_Level1;

        /// <summary>Level 2 operations.
        /// </summary>
        private ILevel2BLAS m_Level2;

        /// <summary>Level 3 operations.
        /// </summary>
        private ILevel3BLAS m_Level3;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="CBlasNativeWrapper" /> class.
        /// </summary>
        public CBlasNativeWrapper()
        {
            m_Name = new IdentifierString("CBLAS");
            m_Level1 = new Level1CBLAS();
            m_Level2 = new Level2CBLAS();
            m_Level3 = new Level3CBLAS();
        }
        #endregion

        #region protected constructors

        /// <summary>Initializes a new instance of the <see cref="CBlasNativeWrapper" /> class.
        /// </summary>
        /// <param name="name">The name of the Library.</param>
        /// <param name="level1">The implementation of level 1 BLAS functions.</param>
        /// <param name="level2">The implementation of level 2 BLAS functions.</param>
        /// <param name="level3">The implementation of level 3 BLAS functions.</param>
        protected CBlasNativeWrapper(IdentifierString name, ILevel1BLAS level1, ILevel2BLAS level2, ILevel3BLAS level3)
        {
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            m_Name = name;
            if (level1 == null)
            {
                throw new ArgumentNullException("level1");
            }
            m_Level1 = level1;
            if (level2 == null)
            {
                throw new ArgumentNullException("level2");
            }
            m_Level2 = level2;
            if (level3 == null)
            {
                throw new ArgumentNullException("level3");
            }
            m_Level3 = level3;
        }
        #endregion

        #region public properties

        #region IIdentifierNameable Members

        /// <summary>Gets the name of the BLAS Library.
        /// </summary>
        /// <value>The name of the BLAS Library.</value>
        public IdentifierString Name
        {
            get { return m_Name; }
        }

        /// <summary>Gets the long name of the BLAS Library.
        /// </summary>
        /// <value>The long name of the BLAS Library.</value>
        public IdentifierString LongName
        {
            get { return m_Name; }
        }
        #endregion

        #region IAnnotatable Members

        /// <summary>Gets a value indicating whether the annotation is read-only.
        /// </summary>
        /// <value>
        /// <c>true</c> if the annotation of this instance is readonly; otherwise, <c>false</c>.</value>
        bool IAnnotatable.HasReadOnlyAnnotation
        {
            get { return true; }
        }

        /// <summary>Gets a description of the BLAS Library.
        /// </summary>
        /// <value>The description of the BLAS Library.</value>
        public virtual string Annotation
        {
            get { return String.Format(ResourceFile.Description, dllName); }
        }
        #endregion

        #region ILibrary Members

        /// <summary>Gets Level 1 operations, i.e. vector operations.
        /// </summary>
        /// <value>Level 1 operations.</value>
        public ILevel1BLAS Level1
        {
            get { return m_Level1; }
        }

        /// <summary>Gets Level 2 operations, i.e. matrix-vector operations.
        /// </summary>
        /// <value>Level 2 operations.</value>
        public ILevel2BLAS Level2
        {
            get { return m_Level2; }
        }

        /// <summary>Gets Level 3 operations, i.e. matrix-matrix operations.
        /// </summary>
        /// <value>Level 3 operations.</value>
        public ILevel3BLAS Level3
        {
            get { return m_Level3; }
        }
        #endregion

        #endregion

        #region public methods

        #region IAnnotatable Members

        /// <summary>Sets the annotation of the current instance.
        /// </summary>
        /// <param name="annotation">The annotation.</param>
        /// <returns>A value indicating whether the <see cref="Annotation" /> has been changed.</returns>
        bool IAnnotatable.TrySetAnnotation(string annotation)
        {
            return false;
        }
        #endregion

        /// <summary>Returns a <see cref="System.String" /> that represents this instance.
        /// </summary>
        /// <returns>A <see cref="System.String" /> that represents this instance.</returns>
        public override string ToString()
        {
            return m_Name.String;
        }
        #endregion
    }
}