﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Numerics;
using System.Security;
using System.Runtime.InteropServices;

namespace Dodoni.MathLibrary.Basics.LowLevel.Native
{
    public static class MklExtensions
    {
        /// <summary>The name of the external Lapack dll.
        /// </summary>
        private const string sm_DllName = "libLAPACK_mkl.dll";


        [DllImport(sm_DllName, EntryPoint = "DDTTRFB", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _ddttrfb(ref int n, IntPtr dl, IntPtr d, IntPtr du, out int info);

        [DllImport(sm_DllName, EntryPoint = "DPSTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern int _dpstrf(ref char uplo, ref int n, IntPtr a, ref int lda, IntPtr piv, out int rank, ref double tol, IntPtr work, out int info);


        [DllImport(sm_DllName, EntryPoint = "ZPSTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern int _zpstrf(ref char uplo, ref int n, IntPtr a, ref int lda, IntPtr piv, out int rank, ref double tol, IntPtr work, out int info);

        [DllImport(sm_DllName, EntryPoint = "ZDTTRFB", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zdttrfb(ref int n, IntPtr dl, IntPtr d, IntPtr du, out int info);


        /// <summary>Computes the factorization of a diagonally dominant triangular matrix, i.e. A = L_1 * U * L_, 
        /// <list type="bullet">
        ///  <item>
        ///  <description>where L_1 and L_2 are unit lower bidiagonal with k and n - k - 1 subdiagonal elements, respectively, where k = n/2,</description>
        ///  </item>
        ///  <item>
        ///   <description>U is an upper bidiagonal matrix with nonzeroes in only the main diagonal and first superdiagonal.</description>
        ///  </item>
        /// </list>
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="dl">The (<paramref name="n"/> - 1) subdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) multipliers that defines the matrix L from the LU factorization.</param>
        /// <param name="d">The <paramref name="n"/> diagonal elements of the input matrix; overwritten by the <paramref name="n"/> diagonal element reciprocals of the upper triangular matrix U from the LU factorization.</param>
        /// <param name="du">The (<paramref name="n"/> - 1) superdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) elements of the superdiagonal of U.</param>
        public static void ddttrfb(this LapackLinearEquations.IMatrixFactorization lapack, int n, double[] dl, double[] d, double[] du)
        {
            int info;
            unsafe
            {
                fixed (double* dlPtr = dl, dPtr = d, duPtr = du)
                {
                    _ddttrfb(ref n, (IntPtr)dlPtr, (IntPtr)dPtr, (IntPtr)duPtr, out info);
                }
            }
        }

        /// <summary>Computes the factorization of a diagonally dominant triangular matrix, i.e. A = L_1 * U * L_, 
        /// <list type="bullet">
        ///  <item>
        ///  <description>where L_1 and L_2 are unit lower bidiagonal with k and n - k - 1 subdiagonal elements, respectively, where k = n/2,</description>
        ///  </item>
        ///  <item>
        ///   <description>U is an upper bidiagonal matrix with nonzeroes in only the main diagonal and first superdiagonal.</description>
        ///  </item>
        /// </list>
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="dl">The (<paramref name="n"/> - 1) subdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) multipliers that defines the matrix L from the LU factorization.</param>
        /// <param name="d">The <paramref name="n"/> diagonal elements of the input matrix; overwritten by the <paramref name="n"/> diagonal element reciprocals of the upper triangular matrix U from the LU factorization.</param>
        /// <param name="du">The (<paramref name="n"/> - 1) superdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) elements of the superdiagonal of U.</param>
        public static void zdttrfb(this LapackLinearEquations.IMatrixFactorization lapack, int n, Complex[] dl, Complex[] d, Complex[] du)
        {
            int info;
            unsafe
            {
                fixed (Complex* dlPtr = dl, dPtr = d, duPtr = du)
                {
                    _zdttrfb(ref n, (IntPtr)dlPtr, (IntPtr)dPtr, (IntPtr)duPtr, out info);
                }
            }
        }


        /// <summary>
        /// Computes the Cholesky decomposition with complete pivoting of a real symmetric positive-definite matrix, i.e. P' * A * P = U' * U or P' * A * P = L * L', where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the upper or lower triangular matrix U, L respectively.</param>
        /// <param name="factorizationType">A value indicating whether the upper or lower triangular part of matrix A is stored and how matrix A is factored.</param>
        /// <param name="pRowIndex">The nonzero entries of permutation matrix P, i.e. P( pRowIndex(k) , k ) = 1 for k = 1,.., <paramref name="n"/> - 1 (output).</param>
        /// <param name="work">A workspace array of length at least 2 * n <paramref name="n"/>.</param>
        /// <param name="tolerance">The tolerance for the termination condition of the algorithm, i.e. if pivot elements &lt; <paramref name="tolerance"/>.</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>.</param>
        /// <returns>
        /// The number of steps the algorithm completed.
        /// </returns>
        public static int dpstrf(this LapackLinearEquations.IMatrixFactorization lapack, int n, double[] a, BLAS.TriangularMatrixType factorizationType, int[] pRowIndex, double[] work, double tolerance, int lda)
        {
            int info;
            char uplo = 'l';

            int rank; // output!!!--> Teil des outputs!
            unsafe
            {
                fixed (double* aPtr = a, workPtr = work)
                {
                    fixed (int* piv = pRowIndex)
                    {
                        return _dpstrf(ref uplo, ref n, (IntPtr)aPtr, ref lda, (IntPtr)piv, out rank, ref tolerance, (IntPtr)workPtr, out info);
                    }
                }
            }
        }

        /// <summary>
        /// Computes the Cholesky decomposition with complete pivoting of a real symmetric positive-definite matrix, i.e. conj(P') * A * P = conj(U') * U or conj(P') * A * P = L * conj(L'), where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the upper or lower triangular matrix U, L respectively.</param>
        /// <param name="factorizationType">A value indicating whether the upper or lower triangular part of matrix A is stored and how matrix A is factored.</param>
        /// <param name="pRowIndex">The nonzero entries of permutation matrix P, i.e. P( pRowIndex(k) , k ) = 1 for k = 1,.., <paramref name="n"/> - 1 (output).</param>
        /// <param name="work">A workspace array of length at least 2 * n <paramref name="n"/>.</param>
        /// <param name="tolerance">The tolerance for the termination condition of the algorithm, i.e. if pivot elements &lt; <paramref name="tolerance"/>.</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>.</param>
        /// <returns>
        /// The number of steps the algorithm completed.
        /// </returns>
        public static int zpstrf(this LapackLinearEquations.IMatrixFactorization lapack, int n, Complex[] a, BLAS.TriangularMatrixType factorizationType, int[] pRowIndex, Complex[] work, double tolerance, int lda)
        {
            int info;
            char uplo = 'l';

            int rank; // output!!!--> Teil des outputs!
            unsafe
            {
                fixed (Complex* aPtr = a, workPtr = work)
                {
                    fixed (int* piv = pRowIndex)
                    {
                        return _zpstrf(ref uplo, ref n, (IntPtr)aPtr, ref lda, (IntPtr)piv, out rank, ref tolerance, (IntPtr)workPtr, out info);
                    }
                }
            }
        }
    }
}