﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Linq;
using System.Collections.Generic;

namespace Dodoni.MathLibrary.ProbabilityTheory.MonteCarloEngine
{
    /// <summary>Represents initial conditions of a specified Random Number Generator of the MKL Library, <seealso cref="IRandomNumberGenerator"/>.
    /// </summary>
    /// <remarks>This class is used to indicate whether additional initial conditions are required for the creation of <see cref="IRandomNumberStream"/> objects. 
    /// Moreover it contains a short description and perhaps a set of standard initial conditions etc.</remarks>
    internal partial class MklRandomNumberInitialConditions
    {
        /// <summary>Serves as descriptor and factory for initial conditions of a specified Random Number Generator of the MKL Library.
        /// </summary>
        internal class Descriptor : RandomNumberInitialConditions.Descriptor
        {
            #region internal constructors

            /// <summary>Initializes a new instance of the <see cref="Descriptor" /> class.
            /// </summary>
            /// <param name="usage">The usage of initial condition parameters used to generate <see cref="IRandomNumberStream"/> instances..</param>
            /// <param name="description">A description of [optional/additional] initial conditions.</param>
            internal Descriptor(RandomNumberInitialConditions.ParametersUsage usage = RandomNumberInitialConditions.ParametersUsage.Optional, string description = "")
                : base(usage, description)
            {
            }
            #endregion

            #region public properties

            /// <summary>Gets the number of retrievable initial conditions for the specified Random Number Generator.
            /// </summary>
            /// <value>The number of retrievable initial conditions for a specified Random Number Generator.
            /// </value>
            public override int Count
            {
                get { return 0; }
            }

            /// <summary>Gets a specified retrievable <see cref="RandomNumberInitialConditions" /> object.
            /// </summary>
            /// <value>The retrievable initial conditions in its <see cref="RandomNumberInitialConditions" /> representation.
            /// </value>
            /// <param name="index">The null-based index.</param>
            public override RandomNumberInitialConditions this[int index]
            {
                get { throw new IndexOutOfRangeException(); }
            }

            /// <summary>Gets a specified retrievable <see cref="RandomNumberInitialConditions"/> object.
            /// </summary>
            /// <value>The retrievable initial conditions in its <see cref="RandomNumberInitialConditions"/> representation.
            /// </value>
            /// <param name="name">The name of the initial condition parameters.</param>
            public override RandomNumberInitialConditions this[string name]
            {
                get { throw new ArgumentException(); }
            }

            /// <summary>Gets the collection of retrievable initial conditions in its <see cref="RandomNumberInitialConditions"/> representation.
            /// </summary>
            /// <value>The collection of retrievable initial conditions in its <see cref="RandomNumberInitialConditions"/> representation.
            /// </value>
            public override IEnumerable<RandomNumberInitialConditions> Values
            {
                get { return Enumerable.Empty<RandomNumberInitialConditions>(); }
            }
            #endregion

            #region public methods

            /// <summary>Creates specified initial conditions for the Random Number Generator.
            /// </summary>
            /// <typeparam name="T">The type of initial conditions.</typeparam>
            /// <param name="initialConditions">The initial conditions.</param>
            /// <returns>The initial conditions for the Random Number Generator in its <see cref="RandomNumberInitialConditions"/> representation.</returns>
            /// <exception cref="InvalidOperationException">Thrown, if the arguments for the initial conditions are not suitable for the specified Random Number Generator; perhaps individual initial conditions are not allowed at all.</exception>
            public override RandomNumberInitialConditions Create<T>(T initialConditions)
            {
                if (initialConditions is int)
                {
                    var seed = (int)(object)initialConditions;
                    return new SingleSeed((uint)seed);
                }
                else if (initialConditions is uint)
                {
                    uint seed = (uint)(object)initialConditions;
                    return new SingleSeed(seed);
                }
                else if (initialConditions is uint[])
                {
                    var seedArray = (uint[])(object)initialConditions;
                    int n = seedArray.Length;
                    return new MultiSeed(n, seedArray);
                }
                throw new InvalidOperationException();
            }

            /// <summary>Creates specified initial conditions for the Random Number Generator.
            /// </summary>
            /// <typeparam name="T1">The type of first argument of the initial conditions.</typeparam>
            /// <typeparam name="T2">The type of second argument of the initial conditions.</typeparam>
            /// <param name="initialCondition1">The first initial condition.</param>
            /// <param name="initialCondition2">The second initial condition.</param>
            /// <returns>The initial conditions for the Random Number Generator in its <see cref="RandomNumberInitialConditions"/> representation.</returns>
            /// <exception cref="InvalidOperationException">Thrown, if the arguments for the initial conditions are not suitable for the specified Random Number Generator; perhaps individual initial conditions are not allowed at all.</exception>
            public override RandomNumberInitialConditions Create<T1, T2>(T1 initialCondition1, T2 initialCondition2)
            {
                if ((initialCondition1 is int) && (initialCondition2 is uint[]))
                {
                    int n = (int)(object)initialCondition1;
                    var seedArray = (uint[])(object)initialCondition2;

                    return new MultiSeed(n, seedArray);
                }
                throw new InvalidOperationException();
            }

            /// <summary>Creates specified initial conditions for the Random Number Generator.
            /// </summary>
            /// <typeparam name="T1">The type of first argument of the initial conditions.</typeparam>
            /// <typeparam name="T2">The type of second argument of the initial conditions.</typeparam>
            /// <typeparam name="T3">The type of third argument of the initial conditions.</typeparam>
            /// <param name="initialCondition1">The first initial condition.</param>
            /// <param name="initialCondition2">The second initial condition.</param>
            /// <param name="initialCondition3">The third initial condition.</param>
            /// <returns>The initial conditions for the Random Number Generator in its <see cref="RandomNumberInitialConditions"/> representation.</returns>
            /// <exception cref="InvalidOperationException">Thrown, if the arguments for the initial conditions are not suitable for the specified Random Number Generator; perhaps individual initial conditions are not allowed at all.</exception>
            public override RandomNumberInitialConditions Create<T1, T2, T3>(T1 initialCondition1, T2 initialCondition2, T3 initialCondition3)
            {
                throw new InvalidOperationException();
            }
            #endregion
        }
    }
}
