﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.ProbabilityTheory.MonteCarloEngine
{
    /// <summary>Provides settings for Random Number Sequences with respect to Intel's MKL Library.
    /// </summary>
    public partial class MklRandomNumberSequence
    {
        /// <summary>Represents the generation mode provided for Random Number generation following a Gaussian (Normal) distribution.
        /// </summary>
        public class GaussianGenerationMode : RandomNumberSequence.GaussianGenerationMode
        {
            /// <summary>Generates normally distributed random number x thru the pair of uniformly distributed numbers u1 and u2 according to the formula
            /// <para>x=sqrt(-ln(u1))*sin(2*Pi*u2).</para>
            /// </summary>
            public static readonly GaussianGenerationMode BoxMuller = new GaussianGenerationMode("BoxMuller", 0, MklResources.Gaussian_BoxMuller);

            /// <summary>Generates pair of normally distributed random numbers x1 and x2 thru the pair of uniformly dustributed numbers u1 and u2 according to the formula
            /// <para>x1=sqrt(-ln(u1))*sin(2*Pi*u2), x2=sqrt(-ln(u1))*cos(2*Pi*u2).</para>
            /// </summary>
            /// <remarks>Implementation correctly works with odd vector lengths.</remarks>
            public static readonly GaussianGenerationMode BoxMuller2 = new GaussianGenerationMode("BoxMuller2", 1, MklResources.Gaussian_BoxMuller2);

            /// <summary>The inverse cumulative distribution approach.
            /// </summary>
            public static readonly GaussianGenerationMode InverseCumulativeDistributionFunction = new GaussianGenerationMode("InverseCumulativeDistributionFunction", 2, MklResources.GaussianInverseCumulativeDistributionFunction);

            /// <summary>The magic number, i.e. ID needed by Intel's MKL library (VSL).
            /// </summary>
            internal readonly int MagicNumber;

            /// <summary>Initializes a new instance of the <see cref="GaussianGenerationMode" /> class.
            /// </summary>
            /// <param name="name">The name of the generation mode.</param>
            /// <param name="magicNumber">The magic number, i.e. ID needed by Intel's MKL library.</param>
            /// <param name="description">The description of the generation mode.</param>
            private GaussianGenerationMode(string name, int magicNumber, string description)
                : base(IdentifierString.Create(name), description)
            {
                MagicNumber = magicNumber;
            }
        }
    }
}