﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.Finance.Compoundings;

namespace Dodoni.Finance
{
    /// <summary>Serves as factory for compounding conventions.
    /// </summary>
    public static class Compounding
    {
        #region public static (readonly) members

        /// <summary>Zero rate compounding conventions, i.e. for zero-coupon bond prices P(t,T).
        /// </summary>
        public static ZeroRateCompounding ZeroRate = new ZeroRateCompounding();

        /// <summary>Compounding conventions for interest rates.
        /// </summary>
        public static InterestRateCompounding InterestRate = new InterestRateCompounding();
        #endregion

        #region public (static) methods

        /// <summary>Gets a zero rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="zeroRate">The zero rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="zeroRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="timeToMaturity">The time to maturity of the underlying zero-coupon bond.</param>
        /// <returns>The zero rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        /// <remarks>Internally first the zero-coupon bond price will be computed with respect to <paramref name="sourceCompounding"/> afterwards
        /// the zero rate will be computed with respect to <paramref name="destinationCompounding"/>. Therefore it may cause some overhead. In
        /// time critical situations one may use a type safed overload of this method instead.</remarks>
        public static double GetConvertedZeroRate(double zeroRate, IZeroRateCompounding sourceCompounding, IZeroRateCompounding destinationCompounding, double timeToMaturity = 1.0)
        {
            if (sourceCompounding == null)
            {
                throw new ArgumentNullException("sourceCompounding");
            }
            double zeroCouponPrice = sourceCompounding.GetZeroCouponBondPrice(zeroRate, timeToMaturity);

            if (destinationCompounding == null)
            {
                throw new ArgumentNullException("destinationCompounding");
            }
            return destinationCompounding.GetZeroRate(zeroCouponPrice, timeToMaturity);
        }

        /// <summary>Gets a zero rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="zeroRate">The zero rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="zeroRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="timeToMaturity">The time to maturity of the underlying zero-coupon bond.</param>
        /// <returns>The zero rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        public static double GetConvertedZeroRate(double zeroRate, ContinuouslyZeroRateCompounding sourceCompounding, PeriodicZeroRateCompounding destinationCompounding, double timeToMaturity = 1.0)
        {
            if (sourceCompounding == null)
            {
                throw new ArgumentNullException("sourceCompounding");
            }
            if (destinationCompounding == null)
            {
                throw new ArgumentNullException("destinationCompounding");
            }
            return (Math.Exp(zeroRate / destinationCompounding.PaymentsPerYear) - 1.0) * destinationCompounding.PaymentsPerYear;
        }

        /// <summary>Gets a zero rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="zeroRate">The zero rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="zeroRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="timeToMaturity">The time to maturity of the underlying zero-coupon bond.</param>
        /// <returns>The zero rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        public static double GetConvertedZeroRate(double zeroRate, PeriodicZeroRateCompounding sourceCompounding, ContinuouslyZeroRateCompounding destinationCompounding, double timeToMaturity = 1.0)
        {
            if (sourceCompounding == null)
            {
                throw new ArgumentNullException("sourceCompounding");
            }
            if (destinationCompounding == null)
            {
                throw new ArgumentNullException("destinationCompounding");
            }
            return sourceCompounding.PaymentsPerYear * Math.Log(1.0 + zeroRate / sourceCompounding.PaymentsPerYear);
        }

        /// <summary>Gets a interest rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="interestRate">The interest rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="interestRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="interestPeriodLength">The length of the interest period.</param>
        /// <returns>The interest rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        /// <remarks>Internally first the normalized interest will be computed with respect to <paramref name="sourceCompounding"/>. Afterwards
        /// the interest rate will be computed with respect to <paramref name="destinationCompounding"/>. Therefore it may cause some overhead. In
        /// time critical situations one may use a type safed overload of this method instead.</remarks>
        public static double GetConvertedInterestRate(double interestRate, IInterestRateCompounding sourceCompounding, IInterestRateCompounding destinationCompounding, double interestPeriodLength)
        {
            if (sourceCompounding == null)
            {
                throw new ArgumentNullException("sourceCompounding");
            }
            double normalizedInterest = sourceCompounding.GetNormalizedInterest(interestRate, interestPeriodLength);

            if (destinationCompounding == null)
            {
                throw new ArgumentNullException("destinationCompounding");
            }

            double rate;
            if (destinationCompounding.TryGetImpliedInterestRate(normalizedInterest, interestPeriodLength, out rate) == false)
            {
                throw new ArgumentException(String.Format("Can not compute implied interest rate with respect to the normalized interest amount {0}, interest period length {1} and compounding convention {2}.", normalizedInterest, interestPeriodLength, destinationCompounding.Name.String));
            }
            return rate;
        }

        /// <summary>Gets a interest rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="interestRate">The interest rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="interestRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="interestPeriodLength">The length of the interest period.</param>
        /// <returns>The interest rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        public static double GetConvertedInterestRate(double interestRate, ContinuouslyInterestCompounding sourceCompounding, PeriodicInterestCompounding destinationCompounding, double interestPeriodLength)
        {
            if (destinationCompounding == null)
            {
                throw new ArgumentNullException("destinationCompounding");
            }
            return (Math.Exp(interestRate / destinationCompounding.PaymentsPerYear) - 1.0) * destinationCompounding.PaymentsPerYear;
        }

        /// <summary>Gets a interest rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="interestRate">The interest rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="interestRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="interestPeriodLength">The length of the interest period.</param>
        /// <returns>The interest rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        public static double GetConvertedInterestRate(double interestRate, ContinuouslyInterestCompounding sourceCompounding, SimpleInterestCompounding destinationCompounding, double interestPeriodLength)
        {
            return (Math.Exp(interestRate * interestPeriodLength) - 1.0) / interestPeriodLength;
        }

        /// <summary>Gets a interest rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="interestRate">The interest rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="interestRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="interestPeriodLength">The length of the interest period.</param>
        /// <returns>The interest rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        public static double GetConvertedInterestRate(double interestRate, PeriodicInterestCompounding sourceCompounding, ContinuouslyInterestCompounding destinationCompounding, double interestPeriodLength)
        {
            if (sourceCompounding == null)
            {
                throw new ArgumentNullException("sourceCompounding");
            }
            return sourceCompounding.PaymentsPerYear * Math.Log(1.0 + interestRate / sourceCompounding.PaymentsPerYear);
        }

        /// <summary>Gets a interest rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="interestRate">The interest rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="interestRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="interestPeriodLength">The length of the interest period.</param>
        /// <returns>The interest rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        public static double GetConvertedInterestRate(double interestRate, PeriodicInterestCompounding sourceCompounding, SimpleInterestCompounding destinationCompounding, double interestPeriodLength)
        {
            if (sourceCompounding == null)
            {
                throw new ArgumentNullException("sourceCompounding");
            }
            return (Math.Pow(1.0 + interestRate / sourceCompounding.PaymentsPerYear, interestPeriodLength * sourceCompounding.PaymentsPerYear) - 1.0) / interestPeriodLength;
        }

        /// <summary>Gets a interest rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="interestRate">The interest rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="interestRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="interestPeriodLength">The length of the interest period.</param>
        /// <returns>The interest rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        public static double GetConvertedInterestRate(double interestRate, SimpleInterestCompounding sourceCompounding, ContinuouslyInterestCompounding destinationCompounding, double interestPeriodLength)
        {
            return Math.Log(1.0 + interestRate * interestPeriodLength) / interestPeriodLength;
        }

        /// <summary>Gets a interest rate with respect to a specific compounding convention.
        /// </summary>
        /// <param name="interestRate">The interest rate.</param>
        /// <param name="sourceCompounding">The compounding convention of the input <paramref name="interestRate"/>.</param>
        /// <param name="destinationCompounding">The compounding convention to convert to.</param>
        /// <param name="interestPeriodLength">The length of the interest period.</param>
        /// <returns>The interest rate in its <paramref name="destinationCompounding"/> compounding convention.</returns>
        public static double GetConvertedInterestRate(double interestRate, SimpleInterestCompounding sourceCompounding, PeriodicInterestCompounding destinationCompounding, double interestPeriodLength)
        {
            if (destinationCompounding == null)
            {
                throw new ArgumentNullException("destinationCompounding");
            }
            return (Math.Exp(Math.Log(1.0 + interestRate * interestPeriodLength) / (interestPeriodLength * destinationCompounding.PaymentsPerYear)) - 1.0) * destinationCompounding.PaymentsPerYear;
        }
        #endregion
    }
}