﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.Finance.DateFactory;
using Dodoni.BasicComponents.Containers;
using Dodoni.BasicComponents.LogFileFactory;

namespace Dodoni.Finance.Compoundings
{
    /// <summary>The compounding conventions for interest rates.
    /// </summary>
    public class InterestRateCompounding : IEnumerable<IInterestRateCompounding>
    {
        #region private (static) members

        /// <summary>The object type ID for logging.
        /// </summary>
        private static readonly System.Guid sm_ObjectTypeID = new System.Guid(0xd303c52c, 0x7388, 0x4390, 0x92, 0x23, 0x5b, 0x1a, 0x67, 0xe2, 0x7a, 0x70);

        /// <summary>The internal pool of the compounding conventions.
        /// </summary>
        private IdentifierNameableDictionary<IInterestRateCompounding> m_Pool;

        /// <summary>The log file.
        /// </summary>
        private ILogFile m_LogFile;
        #endregion

        #region public members

        /// <summary>The simple (i.e. linear) compounding, i.e. 'Notional' * (1.0 + r * t).
        /// </summary>
        public readonly SimpleInterestCompounding Simple = new SimpleInterestCompounding();

        /// <summary>The continuously compounding, i.e. 'Notional' * exp( t * r).
        /// </summary>
        public readonly ContinuouslyInterestCompounding Continuously = new ContinuouslyInterestCompounding();

        /// <summary>The annually compoundingconvention, i.e. 'Notional' * (1.0 + r)^t.
        /// </summary>
        public readonly PeriodicInterestCompounding Annually = new PeriodicInterestCompounding(DateScheduleFrequency.Annually);

        /// <summary>The semi-annually compounding, i.e. 'Notional' * (1.0 + r/2)^{t * 2}.
        /// </summary>
        public readonly PeriodicInterestCompounding SemiAnnually = new PeriodicInterestCompounding(DateScheduleFrequency.SemiAnnually);

        /// <summary>The quarterly compounding, i.e. 'Notional' *  (1.0 + r/4)^{t * 4}.
        /// </summary>
        public readonly PeriodicInterestCompounding Quarterly = new PeriodicInterestCompounding(DateScheduleFrequency.Quarterly);

        /// <summary>The bi-monthly (=two month) compounding, i.e. 'Notional' * (1.0 + r/6)^{t * 6}.
        /// </summary>
        public readonly PeriodicInterestCompounding BiMonthly = new PeriodicInterestCompounding(DateScheduleFrequency.BiMonthly);

        /// <summary>The monthly compounding, i.e. 'Notional' * (1.0 + r/12)^{t * 12}.
        /// </summary>
        public readonly PeriodicInterestCompounding Monthly = new PeriodicInterestCompounding(DateScheduleFrequency.Monthly);

        /// <summary>The daily compounding, i.e. 'Notional' * (1.0 + r/n)^{t * n}, where n = 365. 
        /// </summary>
        public readonly PeriodicInterestCompounding Daily = new PeriodicInterestCompounding(DateScheduleFrequency.Daily);
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="InterestRateCompounding"/> class.
        /// </summary>
        internal InterestRateCompounding()
        {
            m_Pool = new IdentifierNameableDictionary<IInterestRateCompounding>(Simple, Continuously, Annually, SemiAnnually, Quarterly, BiMonthly, Monthly, Daily);

            m_LogFile = LoggingFactory.CreateGlobalLogFile("Interest Compounding convention", "Pool", sm_ObjectTypeID);
            m_LogFile.AddMessage(LoggingFactory.Infos.CreateObject);
        }
        #endregion

        #region public properties

        /// <summary>Gets the number of compounding conventions.
        /// </summary>
        /// <value>The number of compounding conventions.</value>
        public int Count
        {
            get { return m_Pool.Count; }
        }

        /// <summary>Gets the compounding conventions.
        /// </summary>
        /// <value>The compounding conventions.</value>
        public IEnumerable<IInterestRateCompounding> Values
        {
            get { return m_Pool.Values; }
        }

        /// <summary>Gets the compunding convention names.
        /// </summary>
        /// <value>The compounding convention names.</value>
        public IEnumerable<string> Names
        {
            get { return m_Pool.Names; }
        }
        #endregion

        #region public methods

        #region IEnumerable<IInterestRateCompounding> Members

        /// <summary>Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>An <see cref="T:System.Collections.IEnumerator"/> object that can be used to iterate through the collection.
        /// </returns>
        public IEnumerator<IInterestRateCompounding> GetEnumerator()
        {
            return m_Pool.GetEnumerator();
        }
        #endregion

        #region IEnumerable Members

        /// <summary>Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>An <see cref="T:System.Collections.IEnumerator"/> object that can be used to iterate through the collection.
        /// </returns>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return m_Pool.GetEnumerator();
        }
        #endregion

        /// <summary>Gets the compounding convention names in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <returns>A collection of compounding convention names in its <see cref="IdentifierString"/> representation.</returns>
        public IEnumerable<IdentifierString> GetIdentifierStringNames()
        {
            return m_Pool.GetNamesAsIdentifierStrings();
        }

        /// <summary>Gets a specified compounding convention.
        /// </summary>
        /// <param name="name">The name of the compounding convention to search.</param>
        /// <param name="value">The compounding convention (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetValue(IdentifierString name, out IInterestRateCompounding value)
        {
            return m_Pool.TryGetValue(name, out value);
        }

        /// <summary>Gets a specified compounding convention.
        /// </summary>
        /// <param name="name">The name of the compounding convention to search.</param>
        /// <param name="value">The compounding convention (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetValue(string name, out IInterestRateCompounding value)
        {
            return m_Pool.TryGetValue(name, out value);
        }

        /// <summary>Adds a specified compounding convention.
        /// </summary>
        /// <param name="value">The compounding convention to add.</param>
        /// <returns>A value indicating whether <paramref name="value"/> has been added.</returns>
        public ItemAddedState Add(IInterestRateCompounding value)
        {
            ItemAddedState state = m_Pool.Add(value);
            m_LogFile.AddPoolAddingLogFileMessage(state, value.Name);
            return state;
        }
        #endregion
    }
}