﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

using Dodoni.BasicComponents;

namespace Dodoni.Finance
{
    /// <summary>Serves as interface for the calculation of a specific zero-coupon zero rate representation and vice a versa.
    /// </summary>
    public interface IZeroRateCompounding : IIdentifierNameable, IAnnotatable
    {
        /// <summary>Gets the minimal time-to-maturity, i.e. a function call of <see cref="GetZeroRate(double, double)"/> with a <c>time-to-maturity</c> argument which is smaller 
        /// than this number will throw an <see cref="ArgumentOutOfRangeException"/>. 
        /// </summary>
        /// <value>The minimal <c>time-to-maturity</c> argument.</value>
        double MinimalTimeToMaturity
        {
            get;
        }

        /// <summary>Gets an equivalent zero rate representation of a specific zero-coupon bond.
        /// </summary>
        /// <param name="zeroCouponPrice">The value P(t,T) of a specific zero-coupon bond with time to maturity T-t.</param>
        /// <param name="timeToMaturity">The time to maturity of the zero-coupon bond.</param>
        /// <returns>An equivalent zero rate representation of <paramref name="zeroCouponPrice"/>.</returns>
        /// <exception cref="ArgumentOutOfRangeException">Thrown, if <paramref name="timeToMaturity"/> is less than <see cref="IZeroRateCompounding.MinimalTimeToMaturity"/>.</exception>
        double GetZeroRate(double zeroCouponPrice, double timeToMaturity);        

        /// <summary>Gets a zero-coupon bond price with respect to a specific zero rate.
        /// </summary>
        /// <param name="zeroRate">The zero rate representation, compounded with respect to the current compounding rule.</param>
        /// <param name="timeToMaturity">The time to maturity of the zero-coupon bond.</param>
        /// <returns>A zero-coupon bond price P(t,T) which equivalent zero rate representation is equal to <paramref name="zeroRate"/>; the maturity <c>T</c> as well as the reference date <c>t</c>
        /// of the zero-coupon bond is undefined, but the time-to-maturity T-t is known.</returns>
        /// <remarks>This method is the inverse operation of <see cref="IZeroRateCompounding.GetZeroRate(double, double)"/>, i.e. applying the
        /// output of <see cref="IZeroRateCompounding.GetZeroRate(double, double)"/> to this method should return the original discount factor input.</remarks>
        /// <exception cref="ArgumentException">Thrown, if there is no zero-coupon bond price available with equivalent zero rate <paramref name="zeroRate"/>.</exception>
        double GetZeroCouponBondPrice(double zeroRate, double timeToMaturity);
    }
}