﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.Finance.DateFactory;
using Dodoni.BasicComponents.Containers;
using Dodoni.BasicComponents.LogFileFactory;

namespace Dodoni.Finance.Compoundings
{
    /// <summary>The zero rate compounding convention of a zero-coupon bond.
    /// </summary>
    public class ZeroRateCompounding : IEnumerable<IZeroRateCompounding>
    {
        #region private (static) members

        /// <summary>The object type ID for logging.
        /// </summary>
        private static readonly System.Guid sm_ObjectTypeID = new System.Guid(0xe092f90b, 0x5952, 0x4901, 0x95, 0x70, 0xe1, 0x8f, 0x19, 0xc3, 0xde, 0x89);

        /// <summary>The internal pool of the compounding conventions.
        /// </summary>
        private IdentifierNameableDictionary<IZeroRateCompounding> m_Pool;

        /// <summary>The log file.
        /// </summary>
        private ILogFile m_LogFile;
        #endregion

        #region public members

        /// <summary>The continuously compounding, i.e. P(t,T) = exp( -(T-t) * r).
        /// </summary>
        public readonly ContinuouslyZeroRateCompounding Continuously = new ContinuouslyZeroRateCompounding();

        /// <summary>The annually compounding, i.e. P(t,T) = (1.0 + r)^{-(T-t)}.
        /// </summary>
        public readonly PeriodicZeroRateCompounding Annually = new PeriodicZeroRateCompounding(DateScheduleFrequency.Annually);

        /// <summary>The semi-annually compounding, i.e. P(t,T) = (1.0 + r/2)^{-(T-t) * 2}.
        /// </summary>
        public readonly PeriodicZeroRateCompounding SemiAnnually = new PeriodicZeroRateCompounding(DateScheduleFrequency.SemiAnnually);

        /// <summary>The quarterly compounding, i.e. P(t,T) = (1.0 + r/4)^{-(T-t) * 4}.
        /// </summary>
        public readonly PeriodicZeroRateCompounding Quarterly = new PeriodicZeroRateCompounding(DateScheduleFrequency.Quarterly);

        /// <summary>The bi-monthly (=two month) compounding, i.e. P(t,T) = (1.0 + r/6)^{-(T-t) * 6}.
        /// </summary>
        public readonly PeriodicZeroRateCompounding BiMonthly = new PeriodicZeroRateCompounding(DateScheduleFrequency.BiMonthly);

        /// <summary>The monthly compounding, i.e. P(t,T) = (1.0 + r/12)^{-(T-t) * 12}.
        /// </summary>
        public readonly PeriodicZeroRateCompounding Monthly = new PeriodicZeroRateCompounding(DateScheduleFrequency.Monthly);

        /// <summary>The daily compounding, i.e. P(t,T) = (1.0 + r/n)^{- (T-t) * n}, where n = 365. 
        /// </summary>
        public readonly PeriodicZeroRateCompounding Daily = new PeriodicZeroRateCompounding(DateScheduleFrequency.Daily);
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="ZeroRateCompounding"/> class.
        /// </summary>
        internal ZeroRateCompounding()
        {
            m_Pool = new IdentifierNameableDictionary<IZeroRateCompounding>(Continuously, Annually, SemiAnnually, Quarterly, BiMonthly, Monthly, Daily);

            m_LogFile = LoggingFactory.CreateGlobalLogFile("Zero rate Compounding convention", "Pool", sm_ObjectTypeID);
            m_LogFile.AddMessage(LoggingFactory.Infos.CreateObject);
        }
        #endregion

        #region public properties

        /// <summary>Gets the number of compounding conventions.
        /// </summary>
        /// <value>The number of compounding conventions.</value>
        public int Count
        {
            get { return m_Pool.Count; }
        }

        /// <summary>Gets the compounding conventions.
        /// </summary>
        /// <value>The compounding conventions.</value>
        public IEnumerable<IZeroRateCompounding> Values
        {
            get { return m_Pool.Values; }
        }

        /// <summary>Gets the compunding convention names.
        /// </summary>
        /// <value>The compounding convention names.</value>
        public IEnumerable<string> Names
        {
            get { return m_Pool.Names; }
        }
        #endregion

        #region public methods

        #region IEnumerable<IZeroRateCompounding> Members

        /// <summary>Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>An <see cref="T:System.Collections.IEnumerator"/> object that can be used to iterate through the collection.
        /// </returns>
        public IEnumerator<IZeroRateCompounding> GetEnumerator()
        {
            return m_Pool.GetEnumerator();
        }

        #endregion

        #region IEnumerable Members

        /// <summary>Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>An <see cref="T:System.Collections.IEnumerator"/> object that can be used to iterate through the collection.
        /// </returns>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return m_Pool.GetEnumerator();
        }
        #endregion

        /// <summary>Gets the compounding convention names in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <returns>A collection of compounding convention names in its <see cref="IdentifierString"/> representation.</returns>
        public IEnumerable<IdentifierString> GetIdentifierStringNames()
        {
            return m_Pool.GetNamesAsIdentifierStrings();
        }

        /// <summary>Gets a specified compounding convention.
        /// </summary>
        /// <param name="name">The name of the compounding convention to search.</param>
        /// <param name="value">The compounding convention (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetValue(IdentifierString name, out IZeroRateCompounding value)
        {
            return m_Pool.TryGetValue(name, out value);
        }

        /// <summary>Gets a specified compounding convention.
        /// </summary>
        /// <param name="name">The name of the compounding convention to search.</param>
        /// <param name="value">The compounding convention (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetValue(string name, out IZeroRateCompounding value)
        {
            return m_Pool.TryGetValue(name, out value);
        }

        /// <summary>Adds a specified compounding convention.
        /// </summary>
        /// <param name="value">The compounding convention to add.</param>
        /// <returns>A value indicating whether <paramref name="value"/> has been added.</returns>
        public ItemAddedState Add(IZeroRateCompounding value)
        {
            ItemAddedState state = m_Pool.Add(value);
            m_LogFile.AddPoolAddingLogFileMessage(state, value.Name);
            return state;
        }
        #endregion
    }
}