﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Linq;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.Finance.Utilities;
using Dodoni.BasicComponents.Containers;
using Dodoni.BasicComponents.LogFileFactory;
using Dodoni.Finance.MarketConventionTemplates;

namespace Dodoni.Finance
{
    /// <summary>Represents a currency, i.e. contains a unique name as well as standard market conventions etc.
    /// </summary>
    public abstract class Currency : IIdentifierNameable, IEquatable<Currency>
    {
        #region nested stuff

        /// <summary>Handles the initialization event of <see cref="Currency"/> which will be raised before starting to query a specific currency.
        /// </summary>
        public class InitializeEventArgs : EventArgs
        {
            #region internal constructors

            /// <summary>Initializes a new instance of the <see cref="InitializeEventArgs"/> class.
            /// </summary>
            internal InitializeEventArgs()
            {
            }
            #endregion

            #region public methods

            /// <summary>Registers a specific <see cref="Currency"/> object.
            /// </summary>
            /// <param name="value">The <see cref="Currency"/> object to register.</param>
            /// <returns>A value indicating whether <paramref name="value"/> has been inserted.</returns>
            public ItemAddedState Add(Currency value)
            {
                return Currency.Add(value);
            }

            /// <summary>Registers a collection of <see cref="Currency"/> objects.
            /// </summary>
            /// <param name="values">The collection of <see cref="Currency"/> objects to register.</param>
            /// <returns>For each element of <paramref name="values"/> a value indicating whether the <see cref="Currency"/>
            /// object has been inserted.</returns>
            public ItemAddedState[] Add(params Currency[] values)
            {
                if (values == null)
                {
                    return null;
                }
                IList<ItemAddedState> stateList = new List<ItemAddedState>();
                for (int j = 0; j < values.Length; j++)
                {
                    stateList.Add(Currency.Add(values[j]));
                }
                return stateList.ToArray();
            }
            #endregion
        }
        #endregion

        #region public static (readonly) members

        /// <summary>The collection of currencies with respect to 'Africa'.
        /// </summary>
        public static readonly CurrencyCollection.Africa Africa = new CurrencyCollection.Africa();

        /// <summary>The collection of currencies with respect to 'America'.
        /// </summary>
        public static readonly CurrencyCollection.America America = new CurrencyCollection.America();

        /// <summary>The collection of currencies with respect to 'Asia'.
        /// </summary>
        public static readonly CurrencyCollection.Asia Asia = new CurrencyCollection.Asia();

        /// <summary>The collection of currencies with respect to 'Europe'.
        /// </summary>
        public static readonly CurrencyCollection.Europe Europe = new CurrencyCollection.Europe();

        /// <summary>The collection of currencies with respect to 'Oceania'.
        /// </summary>
        public static readonly CurrencyCollection.Oceania Oceania = new CurrencyCollection.Oceania();
        #endregion

        #region private static members

        /// <summary>The pool of the currencies.
        /// </summary>
        private static IdentifierNameableDictionary<Currency> sm_Pool;

        /// <summary>The logfile for the currency object pool.
        /// </summary>
        private static ILogFile sm_LogFile;

        /// <summary>The object type ID, i.e. {C515DB42-8DA4-42B8-BC39-CDC7F93EE7B6}.
        /// </summary>
        private static Guid sm_ObjectTypeID = new Guid(0xc515db42, 0x8da4, 0x42b8, 0xbc, 0x39, 0xcd, 0xc7, 0xf9, 0x3e, 0xe7, 0xb6);

        /// <summary>Occurs before quering <see cref="Currency"/> objects.
        /// </summary>
        private static event Action<InitializeEventArgs> sm_Initialize;

        /// <summary>A value indicating whether the user has added or removed at least one event to the <see cref="Currency.Initialize"/> event handler
        /// since the last call of the event-handler.
        /// </summary>
        /// <remarks>This member is used for performance reason only.</remarks>
        private static bool sm_InitializeChanged = true;
        #endregion

        #region private/public (readonly) members

        /// <summary>The name of the currency, i.e. the ISO code.
        /// </summary>
        private IdentifierString m_Name;

        /// <summary>The (perhaps language dependent) long name of the currency.
        /// </summary>
        private IdentifierString m_LongName;

        /// <summary>The standard market conventions.
        /// </summary>
        public readonly ReadOnlyMarketConventions MarketConventions;
        #endregion

        #region static constructors

        /// <summary>Initializes a new instance of the <see cref="Currency"/> class.
        /// </summary>
        static Currency()
        {
            sm_LogFile = LoggingFactory.CreateGlobalLogFile("Currency", "Pool", sm_ObjectTypeID);
            sm_LogFile.AddMessage(LoggingFactory.Infos.CreateObject);
        }
        #endregion

        #region protected constructors

        /// <summary>Initializes a new instance of the <see cref="Currency"/> class.
        /// </summary>
        /// <param name="currencyName">The (ISO) name of the currency.</param>
        /// <param name="standardMarketConventions">The standard Market conventions.</param>
        /// <param name="currencyLongName">The long name of the currency.</param>
        /// <exception cref="ArgumentNullException">Thrown, if one of the arguments (except <paramref name="currencyLongName"/>) is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="standardMarketConventions"/> is not completely defined.</exception>
        protected Currency(IdentifierString currencyName, ReadOnlyMarketConventions standardMarketConventions, string currencyLongName)
        {
            if (currencyName == null)
            {
                throw new ArgumentNullException("currencyName");
            }
            if ((currencyName.IDString == null) || (currencyName.IDString.Length == 0))
            {
                throw new ArgumentException(String.Format(ExceptionMessages.ArgumentIsInvalid, "Currency name"), "currencyName");
            }
            if (standardMarketConventions == null)
            {
                throw new ArgumentNullException("standardMarketConventions");
            }
            m_Name = currencyName;
            m_LongName = (currencyLongName != null) ? currencyLongName.ToIdentifierString() : m_Name;
            MarketConventions = standardMarketConventions;
        }

        /// <summary>Initializes a new instance of the <see cref="Currency"/> class.
        /// </summary>
        /// <param name="currencyName">The (ISO) name of the currency.</param>
        /// <param name="standardMarketConventions">The standard Market conventions.</param>
        /// <param name="currencyLongName">The long name of the currency.</param>
        /// <exception cref="ArgumentNullException">Thrown, if one of the arguments (except <paramref name="currencyLongName"/>) is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="standardMarketConventions"/> is not completely defined.</exception>
        protected Currency(IdentifierString currencyName, ReadOnlyMarketConventions standardMarketConventions, IdentifierString currencyLongName = null)
        {
            if (currencyName == null)
            {
                throw new ArgumentNullException("currencyName");
            }
            if ((currencyName.IDString == null) || (currencyName.IDString.Length == 0))
            {
                throw new ArgumentException(String.Format(ExceptionMessages.ArgumentIsInvalid, "Currency name"), "currencyName");
            }
            if (standardMarketConventions == null)
            {
                throw new ArgumentNullException("standardMarketConventions");
            }
            m_Name = currencyName;
            m_LongName = (currencyLongName != null) ? currencyLongName : m_Name;
            MarketConventions = standardMarketConventions;
        }
        #endregion

        #region public properties

        #region IIdentifierNameable Members

        /// <summary>Gets the name of the currency, i.e. the ISO code.
        /// </summary>
        /// <value>The ISO code of the currency.</value>
        public IdentifierString Name
        {
            get { return m_Name; }
        }

        /// <summary>Gets the long name of the currency.
        /// </summary>
        /// <value>The (perhaps language dependent) long name of the currency.</value>
        public IdentifierString LongName
        {
            get { return m_LongName; }
        }
        #endregion

        #endregion

        #region public methods

        #region IEquatable<Currency> Members

        /// <summary>Indicates whether the current object is equal to another object of the same type.
        /// </summary>
        /// <param name="other">An object to compare with this object.</param>
        /// <returns><c>true</c> if the current object is equal to the <paramref name="other"/> parameter; otherwise, false.</returns>
        public bool Equals(Currency other)
        {
            if (other == null)
            {
                return false;
            }
            return (m_Name.IDString == other.m_Name.IDString);
        }
        #endregion

        /// <summary>Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return (string)m_Name;
        }
        #endregion

        #region public static properties

        /// <summary>Gets the number of currency objects.
        /// </summary>
        /// <value>The number of currency objects.</value>
        public static int Count
        {
            get
            {
                OnInitialize();
                return sm_Pool.Count;
            }
        }

        /// <summary>Gets the collection of currency objects.
        /// </summary>
        /// <value>The collection of currency objects.</value>
        public static IEnumerable<Currency> Values
        {
            get
            {
                OnInitialize();
                return sm_Pool.Values;
            }
        }

        /// <summary>Gets the (ISO) names of the currency objects.
        /// </summary>
        /// <value>The (ISO) names of the currency objects.</value>
        public static IEnumerable<string> Names
        {
            get
            {
                OnInitialize();
                return sm_Pool.Names;
            }
        }

        /// <summary>Occurs before quering a specific <see cref="Currency"/> object.
        /// </summary>
        public static event Action<InitializeEventArgs> Initialize
        {
            add
            {
                sm_InitializeChanged = true;
                sm_Initialize += value;
            }
            remove
            {
                sm_InitializeChanged = true;
                sm_Initialize -= value;
            }
        }
        #endregion

        #region public static methods

        /// <summary>Gets the currency (ISO) names in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <returns>A collection of currency (ISO) names in its <see cref="IdentifierString"/> representation.</returns>
        public static IEnumerable<IdentifierString> GetIdentifierStringNames()
        {
            OnInitialize();
            return sm_Pool.GetNamesAsIdentifierStrings();
        }

        /// <summary>Gets a specific currency.
        /// </summary>
        /// <param name="name">The name of the currency to search.</param>
        /// <param name="value">The value (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public static bool TryGetValue(IdentifierString name, out Currency value)
        {
            OnInitialize();
            return sm_Pool.TryGetValue(name, out value);
        }

        /// <summary>Gets a specific currency.
        /// </summary>
        /// <param name="name">The name of the currency to search.</param>
        /// <param name="value">The value (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public static bool TryGetValue(string name, out Currency value)
        {
            OnInitialize();
            return sm_Pool.TryGetValue(name, out value);
        }

        /// <summary>Adds the specified currency.
        /// </summary>
        /// <param name="value">The currency to add.</param>
        /// <returns>A value indicating whether <paramref name="value"/> has been added.</returns>
        public static ItemAddedState Add(Currency value)
        {
            ItemAddedState state = sm_Pool.Add(value);
            sm_LogFile.AddPoolAddingLogFileMessage(state, (value != null) ? value.Name : null);
            return state;
        }
        #endregion

        #region private (static) methods

        /// <summary>Raises the <see cref="Initialize"/> event.
        /// </summary>
        private static void OnInitialize()
        {
            if (sm_InitializeChanged == true)
            {
                /* first remove all currency objects */
                sm_Pool = new IdentifierNameableDictionary<Currency>(100, isReadOnlyExceptAdding: true);  // clear is not allowed, it is readonly

                /* add all currency objects */
                if (sm_Initialize != null)
                {
                    InitializeEventArgs eventArgs = new InitializeEventArgs();
                    sm_Initialize(eventArgs);
                }
                sm_InitializeChanged = false;
            }
        }
        #endregion
    }
}