﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.Finance;
using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;
using Dodoni.BasicComponents.LogFileFactory;

namespace Dodoni.Finance.DateFactory.TimeframeDescriptions
{
    /// <summary>A time span, where the start date (=settlement date) as well as the end date (=maturity date) is given in its <see cref="TenorTimeSpan"/> representation, 
    /// moreover for the start date (=settlement date) a number of business days to settle will be take into account as well.
    /// <para>The implementation works in the following way:
    ///  <list type="number">
    ///   <item><description>Apply the SpotDateAdjustment to the reference date; call it spot date.</description></item>
    ///   <item><description>For regular tenor with a number of business days to settle [in short: BD-ToSettle] != 0 or tomorrow-next, point the spot date to the 
    ///   next (or previous if BD-ToSettle is lesss than 0) business day and add BD-ToSettle business days. This is the final spot date.</description></item>
    ///   <item><description>Apply the start tenor to the spot date and apply the StartDateAdjustment. This is the start date of the period.</description></item>
    ///   <item><description>Add the tenor to the start date and apply the EndDateAdjustment which gives us the end date.</description></item>
    /// </list></para>
    /// </summary>
    internal class ForwardStartingTimeframeDescription : ITimeframeDescription
    {
        #region public (readonly) members

        /// <summary>The business day convention used to compute the spot date, i.e. reference date + a number of business days to settle (if <see cref="StartDateTenor"/> is regular).
        /// </summary>
        public readonly IBusinessDayConvention SpotDateAdjustment;

        /// <summary>The business day convention used to compute the start date.
        /// </summary>
        public readonly IBusinessDayConvention StartDateAdjustment;

        /// <summary>The business day convention used to compute the end date.
        /// </summary>
        public readonly IBusinessDayConvention EndDateAdjustment;

        /// <summary>The tenor with respect to the start date of the time span.
        /// </summary>
        public readonly TenorTimeSpan StartDateTenor;

        /// <summary>The tenor with respect to the end date of the time span.
        /// </summary>
        public readonly TenorTimeSpan EndDateTenor;

        /// <summary>The number of business days to settle which will be taken into account for the spot date calculation if and only if <see cref="StartDateTenor"/> is regular.
        /// </summary>
        public readonly int BusinessDaysToSettle;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="ForwardStartingTimeframeDescription"/> class.
        /// </summary>
        /// <param name="startDateTenor">The tenor use to calculate the start date.</param>
        /// <param name="endDateTenor">The tenor used to calculate the end date.</param>
        /// <param name="spotDateAdjustment">A business day convention used to compute the spot date, i.e. reference date + a number of business days to settle (if <paramref name="startDateTenor"/> is regular).</param>
        /// <param name="startDateAdjustment">A business day convention used to compute the start date of the period.</param>
        /// <param name="endDateAdjustment">A business day convention used to compute the end date of the period.</param>
        /// <param name="businessDaysToSettle">The number of business days used to calculate the spot date; this parameter will be taken into if and only if <paramref name="startDateTenor"/> is a regular tenor (i.e. neigther overnight nor tomorrow-next).</param>
        /// <exception cref="ArgumentException">Thrown, if <paramref name="startDateTenor"/> or <paramref name="endDateTenor"/> represents a negative time span.</exception>
        /// <exception cref="ArgumentNullException">Thrown, if one of the <see cref="IBusinessDayConvention"/> arguments is <c>null</c>.</exception>
        public ForwardStartingTimeframeDescription(TenorTimeSpan startDateTenor, TenorTimeSpan endDateTenor, IBusinessDayConvention spotDateAdjustment, IBusinessDayConvention startDateAdjustment, IBusinessDayConvention endDateAdjustment, int businessDaysToSettle = 0)
        {
            if (startDateTenor.IsPositive == false)
            {
                throw new ArgumentException(String.Format(ExceptionMessages.ArgumentIsInvalid, startDateTenor.ToString()), "startDateTenor");
            }
            StartDateTenor = startDateTenor;
            if (endDateTenor.IsPositive == false)
            {
                throw new ArgumentException(String.Format(ExceptionMessages.ArgumentIsInvalid, endDateTenor.ToString()), "endDateTenor");
            }
            EndDateTenor = endDateTenor;
            if (spotDateAdjustment == null)
            {
                throw new ArgumentNullException("spotDateAdjustment");
            }
            SpotDateAdjustment = spotDateAdjustment;
            if (startDateAdjustment == null)
            {
                throw new ArgumentNullException("startDateAdjustment");
            }
            StartDateAdjustment = startDateAdjustment;
            if (endDateAdjustment == null)
            {
                throw new ArgumentNullException("endDateAdjustment");
            }
            EndDateAdjustment = endDateAdjustment;
            BusinessDaysToSettle = businessDaysToSettle;
        }
        #endregion

        #region public properties

        #region IInfoOutputQueriable Members

        /// <summary>Gets the info-output level of detail.
        /// </summary>
        /// <value>The info-output level of detail.</value>
        public InfoOutputDetailLevel InfoOutputDetailLevel
        {
            get { return InfoOutputDetailLevel.Full; }
        }
        #endregion

        #endregion

        #region public methods

        #region ITimeframeDescription Members

        /// <summary>Gets the start and end date of the timeframe.
        /// </summary>
        /// <param name="referenceDate">A reference date (can be a business day or a holiday).</param>
        /// <param name="holidayCalendar">The (settlement) holiday calendar.</param>
        /// <param name="startDate">The start date of the time span with respect to <paramref name="referenceDate"/> (output).</param>
        /// <param name="endDate">The end date of the time span with respect to <paramref name="referenceDate"/> (output).</param>
        /// <param name="logFile">An optional log file.</param>
        /// <returns>The settlement date (=spot date), i.e. the (perhaps adjusted) reference date plus a specific number of business days.
        /// </returns>
        public DateTime GetStartAndEndDate(DateTime referenceDate, IHolidayCalendar holidayCalendar, out DateTime startDate, out DateTime endDate, ILogFile logFile = null)
        {
            DateTime adjReferenceDate = SpotDateAdjustment.GetAdjustedDate(referenceDate, holidayCalendar);
            if ((logFile != null) && (adjReferenceDate != referenceDate))
            {
                logFile.AddMessage(LoggingFactory.Infos.Generic, String.Format("Reference date {0} has been adjusted to {1}.", referenceDate.ToShortDateString(), adjReferenceDate.ToShortDateString()));
            }

            DateTime spotDate = adjReferenceDate;

            switch (StartDateTenor.TenorType)
            {
                case TenorType.Overnight:
                    break;  // nothing to do

                case TenorType.TomorrowNext:
                    if ((SpotDateAdjustment.AdjustmentType != BusinessDayAdjustmentType.AdjustmentToBusinessDay) && (holidayCalendar.IsBusinessDay(spotDate) == false))
                    {
                        spotDate = holidayCalendar.GetForwardAdjustedBusinessDay(spotDate);
                    }
                    spotDate = holidayCalendar.AddBusinessDays(spotDate, 1);
                    break;

                case TenorType.RegularTenor:
                    if ((SpotDateAdjustment.AdjustmentType != BusinessDayAdjustmentType.AdjustmentToBusinessDay) && (holidayCalendar.IsBusinessDay(spotDate) == false))
                    {
                        if (BusinessDaysToSettle > 0)
                        {
                            spotDate = holidayCalendar.GetForwardAdjustedBusinessDay(spotDate);
                        }
                        else if (BusinessDaysToSettle < 0)
                        {
                            spotDate = holidayCalendar.GetPreviousAdjustedBusinessDay(spotDate);
                        }
                    }
                    if (BusinessDaysToSettle != 0)
                    {
                        spotDate = holidayCalendar.AddBusinessDays(spotDate, BusinessDaysToSettle);
                    }
                    break;

                default:
                    throw new NotImplementedException();
            }
            startDate = StartDateAdjustment.GetAdjustedDate(spotDate.AddTenorTimeSpan(StartDateTenor), holidayCalendar);
            endDate = EndDateAdjustment.GetAdjustedDate(startDate.AddTenorTimeSpan(EndDateTenor), holidayCalendar);
            return spotDate;
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Gets informations of the current object as a specific <see cref="T:Dodoni.BasicComponents.Containers.InfoOutput"/> instance.
        /// </summary>
        /// <param name="infoOutput">The <see cref="T:Dodoni.BasicComponents.Containers.InfoOutput"/> object which is to be filled with informations concering the current instance.</param>
        /// <param name="categoryName">The name of the category, i.e. all informations will be added to these category.</param>
        public void FillInfoOutput(InfoOutput infoOutput, string categoryName = "General")
        {
            InfoOutputPackage infoOutputCollection = infoOutput.AcquirePackage(categoryName);
            infoOutputCollection.Add("Start Tenor", StartDateTenor);
            infoOutputCollection.Add("End Tenor", StartDateTenor);
            infoOutputCollection.Add("Spot date adjustment", SpotDateAdjustment);
            infoOutputCollection.Add("Start date adjustment", StartDateAdjustment);
            infoOutputCollection.Add("End date adjustment", EndDateAdjustment);
            if (StartDateTenor.TenorType == TenorType.RegularTenor)
            {
                infoOutputCollection.Add("Business days to settle", BusinessDaysToSettle);
            }
        }

        /// <summary>Sets the <see cref="P:Dodoni.BasicComponents.Containers.IInfoOutputQueriable.InfoOutputDetailLevel"/> property.
        /// </summary>
        /// <param name="infoOutputDetailLevel">The info-output level of detail.</param>
        /// <returns>A value indicating whether the <see cref="P:Dodoni.BasicComponents.Containers.IInfoOutputQueriable.InfoOutputDetailLevel"/> has been set to <paramref name="infoOutputDetailLevel"/>.
        /// </returns>
        public bool TrySetInfoOutputDetailLevel(InfoOutputDetailLevel infoOutputDetailLevel)
        {
            return (infoOutputDetailLevel == InfoOutputDetailLevel.Full);
        }
        #endregion

        /// <summary>Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return StartDateTenor.ToString() + "x" + EndDateTenor.ToString();
        }
        #endregion
    }
}