﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.MathLibrary.Basics;
using Dodoni.Finance.DateFactory;

namespace Dodoni.Finance.MarketConventionTemplates
{
    /// <summary>Serves as mutable wrapper class for Bond market conventions.
    /// </summary>
    public class BondMarketConventions
    {
        #region private members

        /// <summary>The number of business days to settle.
        /// </summary>
        private int? m_BusinessDaysToSettle = null;

        /// <summary>The state of the number of business days to settle.
        /// </summary>
        private ConventionState m_BusinessDaysToSettleState = ConventionState.Undefined;

        /// <summary>The day count convention.
        /// </summary>
        private IDayCountConvention m_DayCountConvention;

        /// <summary>The state of the day count convention.
        /// </summary>
        private ConventionState m_DayCountConventionState = ConventionState.Undefined;

        /// <summary>The business day convention.
        /// </summary>
        private IBusinessDayConvention m_BusinessDayConvention;

        /// <summary>The state of the business day convention.
        /// </summary>
        private ConventionState m_BusinessDayConventionState = ConventionState.Undefined;

        /// <summary>The coupon frequency.
        /// </summary>
        private IDateScheduleFrequency m_CouponFrequency;

        /// <summary>The state of the coupon frequency.
        /// </summary>
        private ConventionState m_CouponFrequencyState = ConventionState.Undefined;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="BondMarketConventions"/> class.
        /// </summary>
        public BondMarketConventions()
        {
        }
        #endregion

        #region public properties

        /// <summary>Gets a value indicating whether this instance is completely defined, i.e. all
        /// members and properties contain proper values.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is completely defined; otherwise, <c>false</c>.
        /// </value>
        public bool IsCompletelyDefined
        {
            get { return (m_DayCountConvention != null) && (m_BusinessDayConvention != null) && (m_CouponFrequency != null) && (m_BusinessDaysToSettle != null); }
        }

        /// <summary>Gets or sets the number of business days to settle with respect to the Bond market.
        /// </summary>
        /// <value>
        /// The number of business days to settle or <c>null</c> if the current instance
        /// does not contain the number of business days to settle.
        /// </value>
        /// <remarks>The number of business days to settle (with respect to bond market conventions) will be
        /// used for example in the bootstrap approach of interest rate curves.</remarks>
        public int? BusinessDaysToSettle
        {
            get { return m_BusinessDaysToSettle; }
            set
            {
                m_BusinessDaysToSettle = value;
                m_BusinessDaysToSettleState = (value != null) ? ConventionState.UserInput : ConventionState.Undefined;
            }
        }

        /// <summary>Gets the state of the number of business days to settle.
        /// </summary>
        /// <value>The state of the number of business days to settle.</value>
        public ConventionState BusinessDaysToSettleState
        {
            get { return m_BusinessDaysToSettleState; }
        }

        /// <summary>Gets or sets the Bond market day count convention.
        /// </summary>
        /// <value>The day count convention of the Bond market.</value>
        public IDayCountConvention DayCountConvention
        {
            get { return m_DayCountConvention; }
            set
            {
                m_DayCountConvention = value;
                m_DayCountConventionState = (value != null) ? ConventionState.UserInput : ConventionState.Undefined;
            }
        }

        /// <summary>Gets the state of the day count convention.
        /// </summary>
        /// <value>The state of the day count convention.</value>
        public ConventionState DayCountConventionState
        {
            get { return m_DayCountConventionState; }
        }

        /// <summary>Gets or sets the Bond market business day convention.
        /// </summary>
        /// <value>The business day convention of the Bond market.</value>
        public IBusinessDayConvention BusinessDayConvention
        {
            get { return m_BusinessDayConvention; }
            set
            {
                m_BusinessDayConvention = value;
                m_BusinessDayConventionState = (value != null) ? ConventionState.UserInput : ConventionState.Undefined;
            }
        }

        /// <summary>Gets the state of the business day convention.
        /// </summary>
        /// <value>The state of the business day convention.</value>
        public ConventionState BusinessDayConventionState
        {
            get { return m_BusinessDayConventionState; }
        }

        /// <summary>Gets or sets the Bond coupon frequency.
        /// </summary>
        /// <value>The coupon frequency.</value>
        public IDateScheduleFrequency CouponFrequency
        {
            get { return m_CouponFrequency; }
            set
            {
                m_CouponFrequency = value;
                m_CouponFrequencyState = (value != null) ? ConventionState.UserInput : ConventionState.Undefined;
            }
        }

        /// <summary>Gets the state of the coupon frequency.
        /// </summary>
        /// <value>The state of the coupon frequency.</value>
        public ConventionState CouponFrequencyState
        {
            get { return m_CouponFrequencyState; }
        }
        #endregion

        #region public methods

        /// <summary>Returns a read-only wrapper for the current Bond Market conventions.
        /// </summary>
        /// <returns></returns>
        /// <exception cref="InvalidOperationException">Thrown, if <see cref="IsCompletelyDefined"/> is <c>false</c>.</exception>
        public ReadOnlyBondMarketConventions AsReadOnly()
        {
            if (IsCompletelyDefined == false)
            {
                throw new InvalidOperationException(String.Format(ExceptionMessages.ObjectIsInvalid, "Bond market conventions", "asReadOnly"));
            }
            return new ReadOnlyBondMarketConventions(this);
        }
        #endregion

        #region internal methods

        /// <summary>Sets the business days to settle and sets the <see cref="BusinessDaysToSettleState"/> to <see cref="ConventionState.StandardValue"/>.
        /// </summary>
        /// <param name="businessDaysToSettle">The number of business days to settle.</param>
        internal void SetStandardBusinessDaysToSettle(int businessDaysToSettle)
        {
            m_BusinessDaysToSettle = businessDaysToSettle;
            m_BusinessDaysToSettleState = ConventionState.StandardValue;
        }

        /// <summary>Sets the day count convention and sets the <see cref="DayCountConventionState"/> to <see cref="ConventionState.StandardValue"/>.
        /// </summary>
        /// <param name="dayCountConvention">The day count convention.</param>
        internal void SetStandardDayCountconvention(IDayCountConvention dayCountConvention)
        {
            if (dayCountConvention == null)
            {
                throw new ArgumentNullException("dayCountConvention");
            }
            m_DayCountConvention = dayCountConvention;
            m_DayCountConventionState = ConventionState.StandardValue;
        }

        /// <summary>Sets the business day convention and sets the <see cref="BusinessDayConventionState"/> to <see cref="ConventionState.StandardValue"/>.
        /// </summary>
        /// <param name="businessDayConvention">The business day convention.</param>
        internal void SetStandardBusinessDayConvention(IBusinessDayConvention businessDayConvention)
        {
            if (businessDayConvention == null)
            {
                throw new ArgumentNullException("businessDayConvention");
            }
            m_BusinessDayConvention = businessDayConvention;
            m_BusinessDayConventionState = ConventionState.StandardValue;
        }

        /// <summary>Sets the coupon frequency and sets the <see cref="CouponFrequencyState"/> to <see cref="ConventionState.StandardValue"/>.
        /// </summary>
        /// <param name="couponFrequency">The Bond coupon frequency.</param>
        internal void SetStandardCouponFrequency(IDateScheduleFrequency couponFrequency)
        {
            if (couponFrequency == null)
            {
                throw new ArgumentNullException("couponFrequency");
            }
            m_CouponFrequency = couponFrequency;
            m_CouponFrequencyState = ConventionState.StandardValue;
        }
        #endregion
    }
}