﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;

namespace Dodoni.Finance.MarketConventionTemplates
{
    /// <summary>Serves as read-only wrapper for Market conventions, i.e. for Bond market, Swap market etc. 
    /// </summary>
    /// <remarks>Each currency contains a <see cref="ReadOnlyMarketConventions"/> object which represents the
    /// standard Market conventions with respect to the specific currency.</remarks>
    public abstract class ReadOnlyMarketConventions
    {
        #region public (readonly) members

        /// <summary>The Bond Market conventions.
        /// </summary>
        public readonly ReadOnlyBondMarketConventions BondMarketConventions;

        /// <summary>The Credit Market conventions.
        /// </summary>
        public readonly ReadOnlyCreditMarketConventions CreditMarketConventions;

        /// <summary>The Inflation Market conventions.
        /// </summary>
        public readonly ReadOnlyInflationMarketConventions InflationMarketConventions;

        /// <summary>The Money Market conventions.
        /// </summary>
        public readonly ReadOnlyMoneyMarketConventions MoneyMarketConventions;

        /// <summary>The Swap Market conventions.
        /// </summary>
        public readonly ReadOnlySwapMarketConventions SwapMarketConventions;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="ReadOnlyMarketConventions"/> class.
        /// </summary>
        /// <param name="bondMarketConventions">The bond market conventions.</param>
        /// <param name="creditMarketConventions">The credit market conventions.</param>
        /// <param name="inflationMarketConventions">The inflation market conventions.</param>
        /// <param name="moneyMarketConventions">The money market conventions.</param>
        /// <param name="swapMarketConventions">The swap market conventions.</param>
        /// <exception cref="ArgumentNullException">Thrown, if one of the arguments is <c>null</c>.</exception>
        public ReadOnlyMarketConventions(ReadOnlyBondMarketConventions bondMarketConventions, ReadOnlyCreditMarketConventions creditMarketConventions, ReadOnlyInflationMarketConventions inflationMarketConventions, ReadOnlyMoneyMarketConventions moneyMarketConventions, ReadOnlySwapMarketConventions swapMarketConventions)
        {
            if (bondMarketConventions == null)
            {
                throw new ArgumentNullException("bondMarketConventions");
            }
            BondMarketConventions = bondMarketConventions;

            if (creditMarketConventions == null)
            {
                throw new ArgumentNullException("creditMarketConventions");
            }
            CreditMarketConventions = creditMarketConventions;

            if (inflationMarketConventions == null)
            {
                throw new ArgumentNullException("inflationMarketConventions");
            }
            InflationMarketConventions = inflationMarketConventions;

            if (moneyMarketConventions == null)
            {
                throw new ArgumentNullException("moneyMarketConventions");
            }
            MoneyMarketConventions = moneyMarketConventions;

            if (swapMarketConventions == null)
            {
                throw new ArgumentNullException("swapMarketConventions");
            }
            SwapMarketConventions = swapMarketConventions;
        }
        #endregion

        #region public properties

        /// <summary>Gets the (settlement) holiday calendar.
        /// </summary>
        /// <value>The (settlement) holiday calendar.</value>
        public IHolidayCalendar HolidayCalendar
        {
            get { return GetHolidayCalendar(); }
        }
        #endregion

        #region protected methods

        /// <summary>Gets the (settlement) holiday calendar.
        /// </summary>
        /// <returns>The (settlement) holiday calendar.</returns>
        /// <remarks>Perhaps the holiday calendar is a element of a holiday calendar pool and referenced by its name only. If 
        /// the user has overwritten the standard holiday calendar implementation, then this new implementation will be used.</remarks>
        protected abstract IHolidayCalendar GetHolidayCalendar();
        #endregion
    }
}