﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.MathLibrary;
using Dodoni.MathLibrary.Basics;
using Dodoni.MathLibrary.ProbabilityTheory.Distributions;

namespace Dodoni.Finance.StandardModels.Bachelier
{
    /// <summary>Serves as abstract basis class for Bachelier (=Normal Black) based options, i.e. the underlying
    /// is assumed to be normal distributed with a constant volatility etc.
    /// </summary>
    /// <remarks>See
    /// <list type="bullet">
    /// <item>
    /// <description>Kazuhiro Iwasawa, 'Analytic formula for the European Normal Black Scholes Formula', Dec. 2001,</description></item>
    /// <item>
    /// <description>M. Musiela, M. Rutkowski, 'Martingal Methods in Finance Modelling', Springer-Verlag, second edition, 123-128,</description></item>
    /// <item>
    /// <description>Jaehyuk Choi, Kwangmoon Kim, and Minsuk Kwak, 'Numerical approximation of the implied volatility under arithmetic Brownian motion', 2007.</description></item>
    /// </list>
    /// We implement the case r=0 and the initial value will be interpreted as forward, i.e. we assume
    /// <para>
    ///    d St = F - S_0 + \sigma dW_t, thus S_t = F + \sigma \cdot W_t,
    /// </para>
    /// where F denote the forward, W_t is a Brownian motion and S_t is the model based 
    /// value of the underlying at time t.
    /// </remarks>
    public abstract class BachelierOption
    {
        #region private consts

        /// <summary>The maximal number of iterations for the computation of the implied strike.
        /// </summary>
        private const int MaxNumberIterationImpliedStrike = 1000;

        /// <summary>The maximal number of iterations for the computation of the implied volatility with respect to Newton's approach.
        /// </summary>
        private const int MaxNumberIterationNewtonImpliedVolatility = 100;

        /// <summary>The lower bound of \eta with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double m_EtaLowerBound = 0.049;

        /// <summary>The upper bound of \eta with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double m_EtaUpperBound = 1.0;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double a0 = 3.994961687345134E-1;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double a1 = 2.100960795068497E+1;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double a2 = 4.980340217855084E+1;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double a3 = 5.988761102690991E+2;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double a4 = 1.848489695437094E+3;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double a5 = 6.106322407867059E+3;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double a6 = 2.493415285349361E+4;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double a7 = 1.266458051348246E+4;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b0 = 1.000000000000000E+0;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b1 = 4.990534153589422E+1;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b2 = 3.093573936743112E+1;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b3 = 1.495105008310999E+3;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b4 = 1.323614537899738E+3;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b5 = 1.598919697679745E+4;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b6 = 2.392008891720782E+4;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b7 = 3.608817108375034E+3;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b8 = -2.067719486400926E+2;

        /// <summary>Constant with respect to 'Numerical approximation of the implied volatility under arithmetic Brownian motion', J. Choi, K. Kim, M. Kwak, 2007.
        /// </summary>
        private const double b9 = 1.174240599306013E+1;
        #endregion

        #region protected constructors

        /// <summary>Initializes a new instance of the <see cref="BachelierOption"/> class.
        /// </summary>
        protected BachelierOption()
        {
        }
        #endregion

        #region protected static methods

        /// <summary>Gets the implied Bachelier (=Normal Black) volatility for a specific value (price) of a straddle option.
        /// </summary>
        /// <param name="strikeMinusForward">'Strike minus forward'.</param>
        /// <param name="sqrtOfTime">The square root of the <see cref="System.Double"/> representation of the time span between valuation date and expiry date.</param>
        /// <param name="undiscountedValue">The value of the straddle at the time of expiry, thus the price but <b>not</b> discounted to time 0.</param>
        /// <param name="impliedVolatility">The implied Bachelier (i.e. Normal Black) volatility (output).</param>
        /// <returns>A value indicating whether <paramref name="impliedVolatility"/> contains valid data.
        /// </returns>
        /// <remarks>The implementation is based on
        /// <para>'Numerical approximation of the implied volatility under arithmetic Brownian motion', 
        /// Jaehyuk Choi, Kwangmoon Kim, and Minsuk Kwak, 2007.</para></remarks>
        protected static ImpliedCalculationResultState TryGetStraddleImpliedVolatility(double strikeMinusForward, double sqrtOfTime, double undiscountedValue, out double impliedVolatility)
        {
            double forwardMinusStrikeDivValue = -strikeMinusForward / undiscountedValue;

            /* use formula (14) & (15) and the fact 
             * tanh^{-1}(x) = 0.5 * log( (1+x) / (1-x) ), |x| < 1
             */

            if (Math.Abs(forwardMinusStrikeDivValue) < 1.0)  // a far-out-of-the-money option?
            {
                double eta = 2.0 * forwardMinusStrikeDivValue / Math.Log((1.0 + forwardMinusStrikeDivValue) / (1.0 - forwardMinusStrikeDivValue));
                if ((eta >= m_EtaLowerBound) && (eta <= m_EtaUpperBound))  // is false if NaN
                {
                    impliedVolatility = MathConsts.SqrtPiOverTwo / sqrtOfTime * undiscountedValue * GetValueOfImpliedStraddleVolaHelperFunction(eta);

                    /* to be sure that the result of the approach in 'Numerical approximation of the implied volatility under arithmetic Brownian motion' is accurate
                     * for our needs, we apply the Newton approach. In general we need at most one or two iterations only:
                     */
                    return TryGetStraddleImpledVolatility(strikeMinusForward, sqrtOfTime, undiscountedValue, impliedVolatility, out impliedVolatility);
                }
            }
            impliedVolatility = Double.NaN;
            return ImpliedCalculationResultState.InputError;
        }

        /// <summary>Gets the implied Bachelier (=Normal Black) volatility for a specific value (price) of a straddle option.
        /// </summary>
        /// <param name="strikeMinusForward">'Strike minus forward'.</param>
        /// <param name="sqrtOfTime">The square root of the <see cref="System.Double"/> representation of the time span between valuation date and expiry date.</param>
        /// <param name="undiscountedValue">The value of the straddle at the time of expiry, thus the price but <b>not</b> discounted to time 0.</param>
        /// <param name="initialVolatility">The initial volatility of the Newton method.</param>
        /// <param name="impliedVolatility">The implied Bachelier (i.e. Normal Black) volatility (output).</param>
        /// <returns>A value indicating whether <paramref name="impliedVolatility"/> contains valid data.
        /// </returns>
        protected static ImpliedCalculationResultState TryGetStraddleImpledVolatility(double strikeMinusForward, double sqrtOfTime, double undiscountedValue, double initialVolatility, out double impliedVolatility)
        {
            /* we apply Newtons method to the function 
             * 
             *   f(\sigma) = [given] undiscountedValue - (F-K) *(2N(d) -1) -2 * \sigma * sqrt(T) * n(d),
             *   
             *  where d = (F-K) / (\sigma * sqrt(T)), n is the density and N the cummulative distribution function of the normal distribution
             */

            impliedVolatility = initialVolatility;
            for (int j = 0; j < MaxNumberIterationNewtonImpliedVolatility; j++)
            {
                double d = -strikeMinusForward / (impliedVolatility * sqrtOfTime);
                double f = undiscountedValue + strikeMinusForward * (2.0 * NormalDistribution.GetStandardCdfValue(d) - 1.0) - 2.0 * impliedVolatility * sqrtOfTime * NormalDistribution.GetStandardDensityValue(d);

                if (Math.Abs(f) < MachineConsts.TinyEpsilon)
                {
                    return ImpliedCalculationResultState.ProperResult;
                }
                double dDash = -d / impliedVolatility;
                double fDash = NormalDistribution.GetStandardDensityValue(d) * (impliedVolatility * d * dDash + 2.0 * strikeMinusForward * NormalDistribution.GetStandardCdfValue(dDash) - 2 * sqrtOfTime);

                impliedVolatility = impliedVolatility - f / fDash;
            }
            return ImpliedCalculationResultState.NoProperResult;
        }

        /// <summary>Gets the implied strike for a given value (price) and Bachelier (=Normal Black) volatility of a specific european call or put option.
        /// </summary>
        /// <param name="theta">A value indicating whether a call (=1) or put (=-1) is given.</param>
        /// <param name="forward">The forward.</param>
        /// <param name="undiscountedValue">The value of the option at the time of expiry, thus the price but <b>not</b> discounted to time 0.</param>
        /// <param name="bachelierVolatility">The Bachelier (=Normal Black) volatility.</param>
        /// <param name="sqrtOfTime">The square root of the time span between valuation date and expiry date.</param>
        /// <param name="impliedStrike">The implied strike (output), input represents the initial value of the Newton-Method.</param>
        /// <returns>Returns a value indicating whether <paramref name="impliedStrike"/> contains valid data.
        /// </returns>
        /// <remarks>The implementation is based on the Newton approach and the forward is used as initial guess.</remarks>
        protected static ImpliedCalculationResultState TryGetPlainVanillaImpliedStrike(int theta, double forward, double undiscountedValue, double bachelierVolatility, double sqrtOfTime, ref double impliedStrike)
        {
            /* we apply Newtons method to the function
             *     H(K) = ln( f(K) / c),
             * where f(K) is the (undiscounted) value of the option and c is the given undiscounted value 
             * of the option. It holds H'(K) = f'(K)/f(K) and Newtons method gives
             * 
             *      K_{n+1} = K_n - H(K_n)/H'(K_n) = K_n - ln(f(K)/c) * f(K) / f'(K).
             */
            double absOfValueAtExpiry = Math.Abs(undiscountedValue);
            double optionValueForGivenStrike = undiscountedValue;

            double d, CDF_Theta_d;
            for (int i = 1; i <= MaxNumberIterationImpliedStrike; i++)
            {
                if (impliedStrike < MachineConsts.SuperTinyEpsilon)
                {
                    optionValueForGivenStrike = forward + impliedStrike;
                    impliedStrike = forward - theta * undiscountedValue;
                }
                else
                {
                    d = (forward - impliedStrike) / (bachelierVolatility * sqrtOfTime);
                    CDF_Theta_d = NormalDistribution.GetStandardCdfValue(theta * d);
                    optionValueForGivenStrike = theta * (forward - impliedStrike) * CDF_Theta_d + bachelierVolatility * sqrtOfTime * NormalDistribution.GetStandardDensityValue(theta * d);

                    impliedStrike += theta * optionValueForGivenStrike * Math.Log(optionValueForGivenStrike / undiscountedValue) / CDF_Theta_d;
                }
                if (Math.Abs(undiscountedValue - optionValueForGivenStrike) < absOfValueAtExpiry * MachineConsts.TinyEpsilon)
                {
                    return ImpliedCalculationResultState.ProperResult;
                }
            }
            return ImpliedCalculationResultState.NoProperResult;
        }
        #endregion

        #region private static methods

        /// <summary>Gets the value of the implied Bachelier (=Normal Black) volatility helper function 'h' with
        /// respect to equation (15) in 'Numerical approximation of the implied volatility under arithmetic Brownian motion', 
        /// Jaehyuk Choi, Kwangmoon Kim, and Minsuk Kwak, 2007.
        /// </summary>
        /// <param name="eta">The parameter \eta.</param>
        /// <returns>The value of h(\eta) with respect to equation (15).</returns>
        private static double GetValueOfImpliedStraddleVolaHelperFunction(double eta)
        {
            double nominator = a0 + eta * (a1 + eta * (a2 + eta * (a3 + eta * (a4 + eta * (a5 + eta * (a6 + eta * a7))))));
            double denominator = b0 + eta * (b1 + eta * (b2 + eta * (b3 + eta * (b4 + eta * (b5 + eta * (b6 + eta * (b7 + eta * (b8 + eta * b9))))))));

            return Math.Sqrt(eta) * nominator / denominator;
        }
        #endregion
    }
}