﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.MathLibrary;
using Dodoni.MathLibrary.Basics;
using Dodoni.MathLibrary.ProbabilityTheory.Distributions;

namespace Dodoni.Finance.StandardModels.Black
{
    /// <summary>Represents the implementation of the Black(-Scholes) model for an european straddle option, i.e. the payoff is |spot -strike| and the 
    /// price is given by the price of an european call + put option.
    /// </summary>
    public class BlackEuropeanStraddle : CommonEuropeanBlackOption, IConstantVolatilityStandardEuropeanOption
    {
        #region private static consts

        /// <summary>The maximal number of iterations for the computation of the implied strike.
        /// </summary>
        private static readonly int MaxNumberIterationImpliedStrike = 1000;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="BlackEuropeanStraddle"/> class.
        /// </summary>
        public BlackEuropeanStraddle()
        {
        }

        /// <summary>Initializes a new instance of the <see cref="BlackEuropeanStraddle"/> class.
        /// </summary>
        /// <param name="strike">The strike.</param>
        /// <param name="forward">The forward.</param>
        /// <param name="timeToExpiration">The time span between valuation date and expiry date.</param>
        /// <param name="discountFactor">The discount factor at <paramref name="timeToExpiration"/>.</param>
        public BlackEuropeanStraddle(double strike, double forward, double timeToExpiration, double discountFactor)
            : base(strike, forward, timeToExpiration, discountFactor)
        {
        }
        #endregion

        #region public properties

        #region IConstantVolatilityStandardOption Members

        /// <summary>Gets the type of the option, i.e. a value indicating whether a call or put option is given.
        /// </summary>
        /// <value>The type of the option.</value>
        public OptionType OptionType
        {
            get { return OptionType.Unspecific; }
        }
        #endregion

        #endregion

        #region public methods

        #region IConstantVolatilityStandardOption Members

        /// <summary>Gets the price of the option.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The value of the option.</returns>
        public double GetValue(double volatility)
        {
            return m_DiscountFactor * GetUndiscountedValue(volatility);
        }

        /// <summary>Gets the implied volatility for a specific option price.
        /// </summary>
        /// <param name="optionValue">The value of the option.</param>
        /// <param name="impliedVolatility">The implied volatility (output).</param>
        /// <returns>A value indicating whether <paramref name="impliedVolatility"/> contains valid data.
        /// </returns>
        /// <remarks>This method is the invers function of <see cref="IConstantVolatilityStandardOption.GetValue(double)"/>.</remarks>
        public ImpliedCalculationResultState TryGetImpliedVolatility(double optionValue, out double impliedVolatility)
        {
            return TryGetImpliedVolatilityOfUndiscountedValue(optionValue / m_DiscountFactor, out impliedVolatility);
        }

        /// <summary>Gets the vega of the option, i.e. the partial derivative of the option value formula with respect to the volatility.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The vega of the option, i.e. the partial derivative of the option value formula with respect to the volatility.
        /// </returns>
        public double GetVega(double volatility)
        {
            if ((m_Strike < MachineConsts.Epsilon) || (volatility * volatility * m_TimeToExpiration < MachineConsts.Epsilon))
            {
                return m_DiscountFactor * (m_Strike - m_Forward);
            }
            double dplus = (LogOfMoneyness + 0.5 * volatility * volatility * m_TimeToExpiration) / (volatility * SqrtOfTimeToExpiration);
            return 2.0 * m_DiscountFactor * m_Forward * SqrtOfTimeToExpiration * NormalDistribution.GetStandardDensityValue(dplus);
        }

        /// <summary>Gets the volga of the option, i.e. the second partial derivative of the option value formula with respect to the volatility.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The volga of the option, i.e. the second partial derivative of the option value formula with respect to the volatility.
        /// </returns>
        public double GetVolga(double volatility)
        {
            if ((m_Strike < MachineConsts.Epsilon) || (volatility * volatility * m_TimeToExpiration < MachineConsts.Epsilon))
            {
                return 0.0;
            }
            double dplus = (LogOfMoneyness + 0.5 * volatility * volatility * m_TimeToExpiration) / (volatility * SqrtOfTimeToExpiration);
            double dminus = dplus - volatility * SqrtOfTimeToExpiration;

            return 2.0 * m_DiscountFactor * m_Forward * SqrtOfTimeToExpiration * NormalDistribution.GetStandardDensityValue(dplus) * dplus * dminus / volatility;
        }

        /// <summary>Gets the theta of the option, i.e. the partial derivative of the option value formula with respect to the time to maturity.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The theta of the option, i.e. the partial derivative of the option value formula with respect to the time to expiration.
        /// </returns>
        public double GetTheta(double volatility)
        {
            if ((m_Strike < MachineConsts.Epsilon) || (volatility * volatility * m_TimeToExpiration < MachineConsts.Epsilon))
            {
                return 0.0;
            }
            double dplus = (LogOfMoneyness + 0.5 * volatility * volatility * m_TimeToExpiration) / (volatility * SqrtOfTimeToExpiration);
            double dminus = dplus - volatility * SqrtOfTimeToExpiration;

            double tempTerm1 = 0.25 * volatility / SqrtOfTimeToExpiration;
            double tempTerm2 = LogOfMoneyness / (2.0 * volatility * m_TimeToExpiration * SqrtOfTimeToExpiration);

            return 2.0 * m_DiscountFactor * (m_Forward * NormalDistribution.GetStandardDensityValue(dplus) * (tempTerm1 - tempTerm2) + m_Strike * NormalDistribution.GetStandardDensityValue(dminus) * (tempTerm1 + tempTerm2));
        }
        #endregion

        #region IConstantVolatilityStandardEuropeanOption Members

        /// <summary>Gets the price of the option <c>at time of expiry</c>, i.e. not discounted.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The value of the option <c>at the time of expiry</c>, thus not discounted. To get
        /// the price just multiply the return value with the discount factor.
        /// </returns>
        public double GetUndiscountedValue(double volatility)
        {
            if ((m_Strike < MachineConsts.Epsilon) || (volatility * volatility * m_TimeToExpiration < MachineConsts.Epsilon))
            {
                return Math.Abs(m_Strike - m_Forward);
            }
            double dplus = (LogOfMoneyness + 0.5 * volatility * volatility * m_TimeToExpiration) / (volatility * SqrtOfTimeToExpiration);
            double dminus = dplus - volatility * SqrtOfTimeToExpiration;

            /* the value is given by: (K: strike, f: forward), use N(-t) = 1-N(t)
             *   discfactor * [ f * N(d+) - K * N(d-) + K * N(-d-) - f * N(-d+) ], i.e. call + put value
             *   = discfactor * [ f * N(d+) - K * N(d-) + K* (1-N(d-)) - f * (1-N(d+))
             *   = discfactor * [2 * f * N(d+) - 2 * K * N(d-) + K -f] */
            return (m_Strike - m_Forward + 2.0 * m_Forward * NormalDistribution.GetStandardCdfValue(dplus) - 2.0 * m_Strike * NormalDistribution.GetStandardCdfValue(dminus));
        }

        /// <summary>Gets the intrinsic value of the option.
        /// </summary>
        /// <returns>The intrisic value of the option.</returns>
        public double GetIntrinsicValue()
        {
            return m_DiscountFactor * Math.Abs(m_Strike - m_Forward);
        }

        /// <summary>Gets the implied volatility for a specific (undiscounted) option price.
        /// </summary>
        /// <param name="undiscountedValue">The value of the option at the time of expiry, thus the price but <b>not</b> discounted to time 0.</param>
        /// <param name="impliedVolatility">The implied volatility (output).</param>
        /// <returns>A value indicating whether <paramref name="impliedVolatility"/> contains valid data.
        /// </returns>
        /// <remarks>This method is the invers function of <see cref="IConstantVolatilityStandardEuropeanOption.GetUndiscountedValue(double)"/>.</remarks>
        public ImpliedCalculationResultState TryGetImpliedVolatilityOfUndiscountedValue(double undiscountedValue, out double impliedVolatility)
        {
            /* the straddle, which is a call + put is equivalent to a transformed call option, the value is given by:
             * 2 * [undiscounted] callValue + discfactor * (strike - forward) 
             * but the discounted value is given, thus */
            return TryGetPlainVanillaImpliedVolatility(1, m_Strike, m_Forward, m_TimeToExpiration, undiscountedValue / 2 - 0.5 * (m_Strike - m_Forward), out impliedVolatility);
        }

        /// <summary>Gets the implied strike for a specific option price.
        /// </summary>
        /// <param name="optionValue">The value of the option.</param>
        /// <param name="volatility">The volatility.</param>
        /// <param name="impliedStrike">The implied strike (output).</param>
        /// <returns>A value indicating whether <paramref name="impliedStrike"/> contains valid data.
        /// </returns>
        /// <remarks>The implied strike is in general not unique.</remarks>
        public ImpliedCalculationResultState TryGetImpliedStrike(double optionValue, double volatility, out double impliedStrike)
        {
            return TryGetImpliedStrike(m_Forward, m_TimeToExpiration, optionValue / m_DiscountFactor, volatility, m_Forward, out impliedStrike);
        }

        /// <summary>Gets the forward-delta of the option, i.e. the partial derivative of the option value formula with respect to the forward.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The forward-delta of the option, i.e. the partial derivative of the option value formula with respect to the forward.
        /// </returns>
        public double GetForwardDelta(double volatility)
        {
            if ((m_Strike < MachineConsts.Epsilon) || (volatility * volatility * m_TimeToExpiration < MachineConsts.Epsilon))
            {
                return ((m_Forward >= m_Strike) ? 1 : -1);
            }
            double dplus = (LogOfMoneyness + 0.5 * volatility * volatility * m_TimeToExpiration) / (volatility * SqrtOfTimeToExpiration);
            return m_DiscountFactor * (2.0 * NormalDistribution.GetStandardCdfValue(dplus) - 1.0);
        }

        /// <summary>Gets the forward-gamma of the option, i.e. the second partial derivative of the option value formula with respect to the forward.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The forward-gamma of the option, i.e. the second partial derivative of the option value
        /// formula with respect to the forward.
        /// </returns>
        /// <remarks>The initial value of the underlying is equal to the forward times the discount factor.</remarks>
        public double GetForwardGamma(double volatility)
        {
            if ((m_Strike < MachineConsts.Epsilon) || (volatility * volatility * m_TimeToExpiration < MachineConsts.Epsilon))
            {
                return 0.0;
            }
            double dplus = (LogOfMoneyness + 0.5 * volatility * volatility * m_TimeToExpiration) / (volatility * SqrtOfTimeToExpiration);

            // is just two times the gamma of an call/put option
            return 2.0 * m_DiscountFactor * NormalDistribution.GetStandardDensityValue(dplus) / (volatility * SqrtOfTimeToExpiration * m_Forward);
        }

        /// <summary>Gets the (forward-)vanna of the option, i.e. the partial derivative of the option value formual with respect
        /// to the forward and with respect to the volatility, i.e. '\partial\sigma \partial F'.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The (forward-)vanna of the option, i.e. the partial derivative of the option value formual with respect
        /// to the forward and with respect to the volatility, i.e. '\partial\sigma \partial F'.
        /// </returns>
        public double GetForwardVanna(double volatility)
        {
            if ((m_Strike < MachineConsts.Epsilon) || (volatility * volatility * m_TimeToExpiration < MachineConsts.Epsilon))
            {
                return 0.0;
            }
            double dplus = (LogOfMoneyness + 0.5 * volatility * volatility * m_TimeToExpiration) / (volatility * SqrtOfTimeToExpiration);
            return 2.0 * m_DiscountFactor * NormalDistribution.GetStandardDensityValue(dplus) * (SqrtOfTimeToExpiration - dplus / volatility);
        }

        /// <summary>Gets the strike-delta of the option, i.e. the partial derivative of the option value formula with
        /// respect to the strike.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The strike-delta of the option, i.e. the partial derivative of the option value formula with respect to the strike.
        /// </returns>
        public double GetStrikeDelta(double volatility)
        {
            if ((m_Strike < MachineConsts.Epsilon) || (volatility * volatility * m_TimeToExpiration < MachineConsts.Epsilon))
            {
                return 0.0;
            }
            double dplus = (LogOfMoneyness + 0.5 * volatility * volatility * m_TimeToExpiration) / (volatility * SqrtOfTimeToExpiration);
            double dminus = dplus - volatility * SqrtOfTimeToExpiration;

            return m_DiscountFactor * (-2.0 * m_Forward / (volatility * SqrtOfTimeToExpiration * m_Strike) * NormalDistribution.GetStandardDensityValue(dplus) - (2 * NormalDistribution.GetStandardCdfValue(dminus) - 1) + 2.0 * NormalDistribution.GetStandardDensityValue(dminus) / (volatility * SqrtOfTimeToExpiration));
        }
        #endregion

        /// <summary>Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return String.Format("European Straddle (Black); Strike: {0}; Forward: {1}; Time-To-Expiration: {2}; Discontfactor: {3}.", m_Strike, m_Forward, m_TimeToExpiration, m_DiscountFactor);
        }
        #endregion

        #region protected static methods

        /// <summary>Gets the implied strike for a specific european straddle.
        /// </summary>
        /// <param name="forward">The forward.</param>
        /// <param name="timeToExpiration">The time span between valuation date and expiry date in its <see cref="System.Double"/> representation.</param>
        /// <param name="undiscountedValue">The value of the option at the time of expiry, thus the price but <b>not</b> discounted to time 0.</param>
        /// <param name="blackVolatility">The Black volatility.</param>
        /// <param name="initialStrike">The initial strike used as initial guess for the Newton method.</param>
        /// <param name="impliedStrike">The implied strike (output).</param>
        /// <returns>
        ///    Returns a value indicating whether <paramref name="impliedStrike"/> contains valid data.
        /// </returns>
        /// <remarks>This method checks some academic conditions and is based on the Newton approach.
        ///    <para>The implied strike is not unique.</para>
        /// </remarks>
        protected static ImpliedCalculationResultState TryGetImpliedStrike(double forward, double timeToExpiration, double undiscountedValue, double blackVolatility, double initialStrike, out double impliedStrike)
        {
            /* the value of a straddle is given by K-F + 2 * CallValue(K) thus we do it in a simular way as in the case of a call option */
            impliedStrike = initialStrike;
            double optionValueForGivenStrike = undiscountedValue;
            double absUndiscountedValue = Math.Abs(undiscountedValue);
            double sqrtOfTime = Math.Sqrt(timeToExpiration);

            double dplus, CDFAtdMinus;

            for (int i = 1; i <= MaxNumberIterationImpliedStrike; i++)
            {
                if (impliedStrike < MachineConsts.SuperTinyEpsilon)
                {
                    optionValueForGivenStrike = Math.Abs(forward - impliedStrike);
                    impliedStrike = forward - undiscountedValue;
                }
                else
                {
                    dplus = (Math.Log(forward / impliedStrike) + 0.5 * blackVolatility * blackVolatility * timeToExpiration) / (blackVolatility * sqrtOfTime);
                    CDFAtdMinus = NormalDistribution.GetStandardCdfValue(dplus - blackVolatility * sqrtOfTime);

                    optionValueForGivenStrike = impliedStrike - forward + 2 * (forward * NormalDistribution.GetStandardCdfValue(dplus) - impliedStrike * CDFAtdMinus);
                    impliedStrike += optionValueForGivenStrike * Math.Log(optionValueForGivenStrike / undiscountedValue) / (1 + 2.0 * CDFAtdMinus);
                }
                if (Math.Abs(undiscountedValue - optionValueForGivenStrike) < absUndiscountedValue * MachineConsts.TinyEpsilon)
                {
                    return ImpliedCalculationResultState.ProperResult;
                }
            }
            return ImpliedCalculationResultState.NoProperResult;
        }
        #endregion
    }
}