/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.MathLibrary;
using Dodoni.MathLibrary.Basics;
using Dodoni.MathLibrary.ProbabilityTheory.Distributions;

namespace Dodoni.Finance.StandardModels.Black
{
    /// <summary>Serves as abstract basis class for Black(-Scholes) based options, i.e. the underlying
    /// is assumed to be log-normal distributed with a constant volatility etc.
    /// </summary>
    /// <remarks>Instead of the spot values one has to enter the forward which may
    /// contains some dividend or a different kind of discount factor etc. In the
    /// Black-Scholes framework the forward is defined by 
    /// <para>Forward = exp(T * (r+d)),</para>,
    /// where T represents the exercise date, r is the yield and d may represents a dividend, 
    /// foreign yield etc, thus exp(-(r+d)*T) is equal to the spot value of the underlying.
    /// 'd' is not an input, but the forward is.</remarks>
    public abstract class BlackOption
    {
        #region private consts

        /// <summary>The maximal number of iterations for the computation of the implied volatility.
        /// </summary>
        private const int MaxNumberIterationImpliedVola = 1000;

        /// <summary>The maximal number of iterations for the computation of the implied strike.
        /// </summary>
        private const int MaxNumberIterationImpliedStrike = 1000;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="BlackOption"/> class.
        /// </summary>
        protected BlackOption()
        {
        }
        #endregion

        #region protected static methods

        /// <summary>Gets the implied Black-Scholes volatility for a given value (price) of a specific call or put option.
        /// </summary>
        /// <param name="theta">A value indicating whether a call (=1) or put (=-1) is given.</param>
        /// <param name="strike">The strike.</param>
        /// <param name="forward">The forward.</param>
        /// <param name="timeToExpiration">The time span between valuation date and expiry date in its <see cref="System.Double"/> representation.</param>
        /// <param name="undiscountedValue">The value of the option at the time of expiry, thus the price but <b>not</b> discounted to time 0.</param>
        /// <param name="impliedBlackVolatility">The implied Black volatility (output).</param>
        /// <returns>
        /// A value indicating whether <paramref name="impliedBlackVolatility"/> contains valid data.
        /// </returns>
        /// <remarks>The implementation is based on <para>'By implication', Peter Jckel.</para></remarks>
        protected static ImpliedCalculationResultState TryGetPlainVanillaImpliedVolatility(int theta, double strike, double forward, double timeToExpiration, double undiscountedValue, out double impliedBlackVolatility)
        {
            impliedBlackVolatility = 0.0;

            /* calculate some constants only once: */
            double x = Math.Log(forward / strike);
            if (Double.IsNaN(x))  // strike <= 0?
            {
                return ImpliedCalculationResultState.InputError;
            }

            double beta = undiscountedValue / Math.Sqrt(forward * strike);  // normalized value: the given value is already discounted

            /* 1. case: at-the-money option:
             * Compute the implied vola via inverse commulative distribution function */
            if (Math.Abs(x) < MachineConsts.Epsilon)  // at-the-money option
            {
                impliedBlackVolatility = -2.0 * NormalDistribution.GetStandardInverseCdfValue(0.5 - 0.5 * beta);
                return ImpliedCalculationResultState.ProperResult;
            }

            /* 2. case: in-the-money option: 
             * subtracting the normalized intrinsic value from the normalized value if necessary */

            double expOfXDivTwo = Math.Sqrt(forward / strike);  // =  Math.Exp(x / 2.0);
            double iota = theta * SpecialMathFunctions.HeavisideFunction(theta * x) * (expOfXDivTwo - 1 / expOfXDivTwo);

            /* the normalized value has to be an element of the interval
             *           [iota, exp( \theta * x/2)],
             * otherwise no implied value can be calculated! 
             */
            if (beta < iota)
            {
                return ImpliedCalculationResultState.InputError;  // one may sets 'beta = iota;'
            }
            if (beta > ((theta == 1) ? expOfXDivTwo : 1 / expOfXDivTwo))
            {
                return ImpliedCalculationResultState.InputError;  // one may sets 'beta = ((theta == 1) ? expOfXDivTwo : 1 / expOfXDivTwo);'
            }
            bool isInTheMoneyOption = false;  // indicates if it is a in-the-money option

            if (theta == -1)
            {
                if (x < 0)
                {
                    isInTheMoneyOption = true;
                }
            }
            else
            {
                if (x > 0)  // strike < forward
                {
                    isInTheMoneyOption = true;
                }
            }
            if (isInTheMoneyOption == true)
            {
                beta -= iota;
                theta = 1 - 2 * SpecialMathFunctions.HeavisideFunction(x);

                /* now consider some out-of-the money option and recalculate the intrinsic value */
                iota = theta * SpecialMathFunctions.HeavisideFunction(theta * x) * (expOfXDivTwo - 1 / expOfXDivTwo);
            }

            /* 3.) Set initual guess: */
            double bc = NormalizedCallPutValue(theta, x, expOfXDivTwo, Math.Sqrt(2 * Math.Abs(x)));

            /* 3a.) sigmaLow and sigmaHigh are given by formula (3.6)-(3.7) */
            double tempForSigmaHighExp = (theta == 1) ? expOfXDivTwo : 1 / expOfXDivTwo;            // = Math.Exp(theta * x / 2.0)
            double tempForSigmaHighCDF = NormalDistribution.GetStandardCdfValue(-Math.Sqrt(Math.Abs(x) / 2.0));

            double sigmaLow = Math.Sqrt(2 * x * x / (Math.Abs(x) - 4 * Math.Log((beta - iota) / (bc - iota))));
            double sigmaHigh = -2 * NormalDistribution.GetStandardInverseCdfValue((tempForSigmaHighExp - beta) / (tempForSigmaHighExp - bc) * tempForSigmaHighCDF);

            /* 2b.) calculate the gamma which is given in formula (4.7): */
            double sigmaStar = -2 * NormalDistribution.GetStandardInverseCdfValue(tempForSigmaHighExp / (tempForSigmaHighExp - bc) * tempForSigmaHighCDF);
            double bStar = NormalizedCallPutValue(theta, x, expOfXDivTwo, sigmaStar);

            double sigmaLowStar = Math.Sqrt(2 * x * x / (Math.Abs(x) - 4 * Math.Log((bStar - iota) / (bc - iota))));
            double sigmaHighStar = -2 * NormalDistribution.GetStandardInverseCdfValue((tempForSigmaHighExp - bStar) / (tempForSigmaHighExp - bc) * tempForSigmaHighCDF);

            // the weight w^* given by formula (4.8)
            double w = Math.Pow(Math.Min(Math.Max((sigmaStar - sigmaLowStar) / (sigmaHighStar - sigmaLowStar), 0), 1.0), Math.Log(bc / beta) / Math.Log(bc / bStar));

            /* the code based on the 2006 edition of the paper, i.e. using formula (4.1) in "By implication", 24.11.2010:
            // double gamma = Math.Log((sigmaStar - sigmaLowStar) / (sigmaHighStar - sigmaLowStar)) / Math.Log(bStar / bc);
            // double w = Math.Min(1, Math.Pow(beta / bc, gamma));   // its the weight given by (4.2)

            /* 2c.) now the initial value is given; see formula (4.1):  */
            if (Double.IsNaN(sigmaLow) == false)
            {
                impliedBlackVolatility = sigmaLow * (1 - w) + sigmaHigh * w;
            }
            else
            {
                /* it is not hard to see that if \sigma_low is not defined then 'beta/bc' > 1
                 * which implies w = 1.0 */
                impliedBlackVolatility = sigmaHigh;
            }

            /* 3.) Do the Halley's iteration method: */
            double terminatingCondition = 1.0;
            for (int i = 1; i <= MaxNumberIterationImpliedVola; i++)
            {
                /* A.) compute nu^_n(x,\sigma_n,\Theta) via formula (4.10) & (4.13): */

                // first compute nu_n (without 'hat') via (3.10): 
                double b = NormalizedCallPutValue(theta, x, expOfXDivTwo, impliedBlackVolatility);
                double impliedBSVolaSquare = impliedBlackVolatility * impliedBlackVolatility;

                double bDash = MathConsts.OneOverSqrtTwoPi * Math.Exp(-0.5 * x * x / impliedBSVolaSquare - 0.125 * impliedBSVolaSquare);

                double nuN = 0.0;
                if (beta < bc)
                {
                    nuN = Math.Log((beta - iota) / (b - iota)) * Math.Log(b - iota) / Math.Log(beta - iota) * (b - iota) / bDash;
                }
                else
                {
                    nuN = (beta - b) / bDash;
                }
                double nuHat = Math.Max(nuN, -impliedBlackVolatility / 2.0);

                /* B. compute eta^ via formula (4.14) & (4.15) & (4.11): */
                double etaHat = x * x / (impliedBSVolaSquare * impliedBlackVolatility) - impliedBlackVolatility / 4.0;
                if (beta < bc)
                {
                    double logOfbMinusIota = Math.Log(b - iota);
                    etaHat -= (2 + logOfbMinusIota) / logOfbMinusIota * bDash / (b - iota);
                }
                etaHat = Math.Max(-0.75, etaHat / 2.0 * nuHat);

                /* C. next step and store the terminating condition */
                terminatingCondition = impliedBlackVolatility;  // store the volatility which is used in the step before
                impliedBlackVolatility = impliedBlackVolatility + Math.Max(nuHat / (1 + etaHat), -impliedBlackVolatility / 2.0);

                terminatingCondition = Math.Abs(impliedBlackVolatility / terminatingCondition - 1);

                if (Double.IsNaN(terminatingCondition))
                {
                    impliedBlackVolatility = Double.NaN;
                    return ImpliedCalculationResultState.NoProperResult;
                }
                if (terminatingCondition < MachineConsts.TinyEpsilon)
                {
                    impliedBlackVolatility = impliedBlackVolatility /= Math.Sqrt(timeToExpiration);
                    return ImpliedCalculationResultState.ProperResult;
                }
            }
            impliedBlackVolatility = impliedBlackVolatility / Math.Sqrt(timeToExpiration);
            return ImpliedCalculationResultState.NoProperResult;
        }

        /// <summary>Gets the implied strike for a given value (price) and Black-Scholes volatility of a specific european call or put option.
        /// </summary>
        /// <param name="theta">A value indicating whether a call (=1) or put (=-1) is given.</param>
        /// <param name="forward">The forward.</param>
        /// <param name="timeToExpiration">The time span between valuation date and expiry date in its <see cref="System.Double"/> representation.</param>
        /// <param name="undiscountedValue">The value of the option at the time of expiry, thus the price but <b>not</b> discounted to time 0.</param>
        /// <param name="blackVolatility">The Black volatility.</param>
        /// <param name="sqrtOfTime">The square root of <paramref name="timeToExpiration"/>.</param>
        /// <param name="impliedStrike">The implied strike (output).</param>
        /// <returns>
        /// Returns a value indicating whether <paramref name="impliedStrike"/> contains valid data.
        /// </returns>
        /// <remarks>The implementation is based on the Newton approach.</remarks>
        protected static ImpliedCalculationResultState TryGetPlainVanillaImpliedStrike(int theta, double forward, double timeToExpiration, double undiscountedValue, double blackVolatility, double sqrtOfTime, out double impliedStrike)
        {
            /* we apply Newtons method to the function
             *     H(K) = ln( f(K) / c),
             * where f(K) is the (undiscounted) value of the option and c is the given undiscounted value 
             * of the option. It holds H'(K) = f'(K)/f(K) and Newtons method gives
             * 
             *      K_{n+1} = K_n - H(K_n)/H'(K_n) = K_n - ln(f(K)/c) * f(K) / f'(K).
             */

            impliedStrike = forward;
            double absOfValueAtExpiry = Math.Abs(undiscountedValue);
            double optionValueForGivenStrike = undiscountedValue;

            double dplus, CDFAtThetaTimesdMinus;

            for (int i = 1; i <= MaxNumberIterationImpliedStrike; i++)
            {
                if (impliedStrike < MachineConsts.SuperTinyEpsilon)
                {
                    optionValueForGivenStrike = forward + impliedStrike;
                    impliedStrike = forward - theta * undiscountedValue;
                }
                else
                {
                    dplus = (Math.Log(forward / impliedStrike) + 0.5 * blackVolatility * blackVolatility * timeToExpiration) / (blackVolatility * sqrtOfTime);
                    CDFAtThetaTimesdMinus = NormalDistribution.GetStandardCdfValue(theta * (dplus - blackVolatility * sqrtOfTime));

                    optionValueForGivenStrike = (theta * forward * NormalDistribution.GetStandardCdfValue(theta * dplus) - theta * impliedStrike * CDFAtThetaTimesdMinus);
                    impliedStrike += theta * optionValueForGivenStrike * Math.Log(optionValueForGivenStrike / undiscountedValue) / CDFAtThetaTimesdMinus;
                }
                if (Math.Abs(undiscountedValue - optionValueForGivenStrike) < absOfValueAtExpiry * MachineConsts.TinyEpsilon)
                {
                    return ImpliedCalculationResultState.ProperResult;
                }
            }
            return ImpliedCalculationResultState.NoProperResult;
        }
        #endregion

        #region private static methods

        /// <summary>Gets the normalized value (price) of a call or put option.
        /// </summary>
        /// <param name="theta">Indicates if a call (=1) or put (=-1) is given.</param>
        /// <param name="x">The logarithm of 'forward / strike'.</param>
        /// <param name="expOfXDivTwo"><paramref name="x"/> divided by two and applied to the exponential function. This
        /// argument will be computed once only (performance reason).</param>
        /// <param name="normalisedVolatility">The normalized volatility, i.e. the volatility divided by the 
        /// square root of the time span between valuation and expiry date.</param>
        /// <returns>The normalized value of the call/put option, thus multiplying the result
        /// by the specific discount factor and the square root of forward * strike gives the value of the option.</returns>
        /// <remarks>
        /// The normalized price (value) of the option is given by 
        /// <para>
        ///      value / (discount factor * Sqrt(forward * strike))
        /// </para>
        /// thus multiply the result by the discount factor * Sqrt(forward * strike)
        /// to get the correct price.
        /// <para>The implementation is based on 'By implication', Peter Jckel, formula (2.2)-(2.3).</para>
        /// </remarks>
        private static double NormalizedCallPutValue(int theta, double x, double expOfXDivTwo, double normalisedVolatility)
        {
            if (Math.Abs(x) < MachineConsts.Epsilon)
            {
                return 1 - 2 * NormalDistribution.GetStandardCdfValue(-normalisedVolatility / 2.0);
            }
            return theta * (expOfXDivTwo * NormalDistribution.GetStandardCdfValue(theta * (x / normalisedVolatility + normalisedVolatility / 2.0))
                - 1.0 / expOfXDivTwo * NormalDistribution.GetStandardCdfValue(theta * (x / normalisedVolatility - normalisedVolatility / 2.0)));
        }
        #endregion
    }
}