﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.Finance.StandardModels.Black
{
    /// <summary>Serves as abstract basis class for european options where a strike is involved
    /// and the single underlying is based on the Black model.
    /// </summary>
    public abstract class CommonEuropeanBlackOption : BlackOption
    {
        #region private members

        /// <summary>The logarithm of the moneyness, i.e. the logarithm of forward over strike.
        /// </summary>
        /// <remarks>This value is frequently used and cached for performance reason.</remarks>
        private double m_LogOfMoneyness = Double.NaN;

        /// <summary>The square-root of the time-to-expiration.
        /// </summary>
        /// <remarks>This value is frequently used and cached for performance reason.</remarks>
        private double m_SqrtOfTimeToExpiration = Double.NaN;
        #endregion

        #region protected members

        /// <summary>The strike of the option.
        /// </summary>
        /// <remarks>This member is protected for performance reason only.</remarks>
        protected double m_Strike = Double.NaN;

        /// <summary>The forward.
        /// </summary>
        /// <remarks>This member is protected for performance reason only.</remarks>
        protected double m_Forward = Double.NaN;

        /// <summary>The time to expiration, i.e. the span between valuation date and expiry date in its <see cref="System.Double"/> representation.
        /// </summary>
        /// <remarks>This member is protected for performance reason only.</remarks>
        protected double m_TimeToExpiration = Double.NaN;

        /// <summary>The discount factor.
        /// </summary>
        /// <remarks>This member is protected for performance reason only.</remarks>
        protected double m_DiscountFactor = Double.NaN;
        #endregion

        #region protected constructors

        /// <summary>Initializes a new instance of the <see cref="CommonEuropeanBlackOption"/> class.
        /// </summary>
        protected CommonEuropeanBlackOption()
        {
        }

        /// <summary>Initializes a new instance of the <see cref="CommonEuropeanBlackOption"/> class.
        /// </summary>
        /// <param name="strike">The strike.</param>
        /// <param name="forward">The forward.</param>
        /// <param name="timeToExpiration">The time span between valuation date and expiry date.</param>
        /// <param name="discountFactor">The discount factor at <paramref name="timeToExpiration"/>.</param>
        protected CommonEuropeanBlackOption(double strike, double forward, double timeToExpiration, double discountFactor)
        {
            m_Strike = strike;
            m_Forward = forward;
            m_LogOfMoneyness = Math.Log(forward / strike);
            m_TimeToExpiration = timeToExpiration;
            m_SqrtOfTimeToExpiration = Math.Sqrt(timeToExpiration);
            m_DiscountFactor = discountFactor;
        }
        #endregion

        #region public properties

        /// <summary>Gets or sets the time to expiration, i.e. time span between valuation date and expiry date
        /// in its <see cref="System.Double"/> representation.
        /// </summary>
        /// <value>The time to expiration.</value>
        public double TimeToExpiration
        {
            get { return m_TimeToExpiration; }
            set
            {
                m_TimeToExpiration = value;
                m_SqrtOfTimeToExpiration = Math.Sqrt(value);
            }
        }

        /// <summary>Gets or sets the strike of the option.
        /// </summary>
        /// <value>The strike.</value>
        public double Strike
        {
            get { return m_Strike; }
            set
            {
                m_Strike = value;
                m_LogOfMoneyness = Math.Log(m_Forward / value);
            }
        }

        /// <summary>Gets or sets the forward at <see cref="TimeToExpiration"/>.
        /// </summary>
        /// <value>The forward at <see cref="TimeToExpiration"/>.
        /// </value>
        public double Forward
        {
            get { return m_Forward; }
            set
            {
                m_Forward = value;
                m_LogOfMoneyness = Math.Log(value / m_Strike);
            }
        }

        /// <summary>Gets the discount factor at <see cref="TimeToExpiration"/>.
        /// </summary>
        /// <value>The discount factor at <see cref="TimeToExpiration"/>.
        /// </value>
        public double DiscountFactor
        {
            get { return m_DiscountFactor; }
            set { m_DiscountFactor = value; }
        }
        #endregion

        #region protected properties

        /// <summary>Gets the logarithm of the moneyness, i.e. the logarithm of forward over strike.
        /// </summary>
        /// <value>The log-moneyness.</value>
        protected double LogOfMoneyness
        {
            get { return m_LogOfMoneyness; }
        }

        /// <summary>Gets the square-root of the time-to-expiration.
        /// </summary>
        /// <value>The square-root of the time-to-expiration..</value>
        protected double SqrtOfTimeToExpiration
        {
            get { return m_SqrtOfTimeToExpiration; }
        }
        #endregion
    }
}