﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

namespace Dodoni.Finance.StandardModels
{
    /// <summary>Serves as interface for an european option that assumes a strike and where the underlying 
    /// is specified by a Black-like model, i.e. Black-Scholes, Bachelier (=Normal Black) etc.
    /// </summary>
    public interface IConstantVolatilityStandardEuropeanOption : IConstantVolatilityStandardOption
    {
        /// <summary>Gets or sets the strike of the option.
        /// </summary>
        /// <value>The strike.</value>
        double Strike
        {
            get;
            set;
        }

        /// <summary>Gets or sets the forward at <see cref="IConstantVolatilityStandardOption.TimeToExpiration"/>.
        /// </summary>
        /// <value>The forward at <see cref="IConstantVolatilityStandardOption.TimeToExpiration"/>.</value>
        double Forward
        {
            get;
            set;
        }

        /// <summary>Gets the discount factor at <see cref="IConstantVolatilityStandardOption.TimeToExpiration"/>.
        /// </summary>
        /// <value>The discount factor at <see cref="IConstantVolatilityStandardOption.TimeToExpiration"/>.</value>
        double DiscountFactor
        {
            get;
            set;
        }

        /// <summary>Gets the price of the option <c>at time of expiry</c>, i.e. not discounted.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The value of the option <c>at the time of expiry</c>, thus not discounted. To get 
        /// the price just multiply the return value with the discount factor.</returns>
        double GetUndiscountedValue(double volatility);

        /// <summary>Gets the intrinsic value of the option.
        /// </summary>
        /// <returns>The intrisic value of the option.</returns>
        double GetIntrinsicValue();

        /// <summary>Gets the implied volatility for a specific (undiscounted) option price.
        /// </summary>
        /// <param name="undiscountedValue">The value of the option at the time of expiry, thus the price but <b>not</b> discounted to time 0.</param>
        /// <param name="impliedVolatility">The implied volatility (output).</param>
        /// <returns>A value indicating whether <paramref name="impliedVolatility"/> contains valid data.</returns>
        /// <remarks>This method is the invers function of <see cref="IConstantVolatilityStandardEuropeanOption.GetUndiscountedValue(double)"/>.</remarks>
        ImpliedCalculationResultState TryGetImpliedVolatilityOfUndiscountedValue(double undiscountedValue, out double impliedVolatility);

        /// <summary>Gets the implied strike for a specific option price.
        /// </summary>
        /// <param name="optionValue">The value of the option.</param>
        /// <param name="volatility">The volatility.</param>
        /// <param name="impliedStrike">The implied strike (output).</param>
        /// <returns>A value indicating whether <paramref name="impliedStrike"/> contains valid data.</returns>
        ImpliedCalculationResultState TryGetImpliedStrike(double optionValue, double volatility, out double impliedStrike);

        /// <summary>Gets the forward-delta of the option, i.e. the partial derivative of the option value formula with respect to the forward. 
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The forward-delta of the option, i.e. the partial derivative of the option value formula with respect to the forward.</returns> 
        double GetForwardDelta(double volatility);

        /// <summary>Gets the forward-gamma of the option, i.e. the second partial derivative of the option value formula with respect to the forward.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The forward-gamma of the option, i.e. the second partial derivative of the option value 
        /// formula with respect to the forward.</returns>
        /// <remarks>The initial value of the underlying is equal to the forward times the discount factor.</remarks>
        double GetForwardGamma(double volatility);

        /// <summary>Gets the (forward-)vanna of the option, i.e. the partial derivative of the option value formual with respect 
        /// to the forward and with respect to the volatility, i.e. '\partial\sigma \partial F'.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The (forward-)vanna of the option, i.e. the partial derivative of the option value formual with respect 
        /// to the forward and with respect to the volatility, i.e. '\partial\sigma \partial F'.</returns>
        double GetForwardVanna(double volatility);

        /// <summary>Gets the strike-delta of the option, i.e. the partial derivative of the option value formula with 
        /// respect to the strike.
        /// </summary>
        /// <param name="volatility">The volatility.</param>
        /// <returns>The strike-delta of the option, i.e. the partial derivative of the option value formula with 
        /// respect to the strike.</returns>
        double GetStrikeDelta(double volatility);
    }
}