﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.Finance.DateFactory;

namespace Dodoni.Finance
{
    /// <summary>Extensions for <see cref="TenorTimeSpan"/> objects.
    /// </summary>
    public static class TenorTimeSpanExtensions
    {
        /// <summary>Converts a <see cref="TenorTimeSpan"/> object into its <see cref="IDateScheduleFrequency"/> representation.
        /// </summary>
        /// <param name="tenorTimeSpan">The tenor time span.</param>
        /// <returns>A <see cref="IDateScheduleFrequency"/> object which represent the <paramref name="tenorTimeSpan"/>.</returns>
        public static IDateScheduleFrequency AsFrequency(this TenorTimeSpan tenorTimeSpan)
        {
            return new IndividualDateScheduleFrequency(tenorTimeSpan);
        }

        /// <summary>Adds a specific <see cref="TenorTimeSpan"/> object to the current instance.
        /// </summary>
        /// <param name="tenorTimeSpan">The tenor time span.</param>
        /// <param name="tenorTimeSpanToAdd">The <see cref="TenorTimeSpan"/> to add.</param>
        /// <param name="tenorTimeSpanFactor">A (optional) factor to take into account.</param>
        /// <returns>A <see cref="TenorTimeSpan"/> object which is the result of <paramref name="tenorTimeSpan"/> plus <paramref name="tenorTimeSpanFactor"/> * <paramref name="tenorTimeSpanToAdd"/>.</returns>
        public static TenorTimeSpan AddTimeSpan(this TenorTimeSpan tenorTimeSpan, TenorTimeSpan tenorTimeSpanToAdd, int tenorTimeSpanFactor = 1)
        {
            return new TenorTimeSpan(tenorTimeSpan.Years + tenorTimeSpanFactor * tenorTimeSpanToAdd.Years, tenorTimeSpan.Months + tenorTimeSpanFactor * tenorTimeSpanToAdd.Months, tenorTimeSpan.Days + tenorTimeSpanFactor * tenorTimeSpanToAdd.Days);
        }

        /// <summary>Adds a specific <see cref="TenorTimeSpan"/> object to a specific <see cref="System.DateTime"/> object.
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="tenorTimeSpan">The time span to add in its <see cref="TenorTimeSpan"/> representation.</param>
        /// <returns>The <see cref="System.DateTime"/> which is the result of <paramref name="startDate"/> plus <paramref name="tenorTimeSpan"/>.</returns>
        public static DateTime AddTenorTimeSpan(this DateTime startDate, TenorTimeSpan tenorTimeSpan)
        {
            DateTime date = startDate;

            if (tenorTimeSpan.Years != 0)
            {
                date = date.AddYears(tenorTimeSpan.Years);
            }
            if (tenorTimeSpan.Months != 0)
            {
                date = date.AddMonths(tenorTimeSpan.Months);
            }
            if (tenorTimeSpan.Days != 0)
            {
                date = date.AddDays(tenorTimeSpan.Days);
            }
            return date;
        }

        /// <summary>Adds a specific <see cref="TenorTimeSpan"/> object to a specific <see cref="System.DateTime"/> object.
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="tenorTimeSpan">The time span to add in its <see cref="String"/> representation.</param>
        /// <param name="tenorTimeSpanFactor">A (optional) factor to take into account.</param>
        /// <returns>The <see cref="System.DateTime"/> which is the result of <paramref name="startDate"/> plus <paramref name="tenorTimeSpanFactor"/> * <paramref name="tenorTimeSpan"/>.</returns>
        public static DateTime AddTenorTimeSpan(this DateTime startDate, string tenorTimeSpan, int tenorTimeSpanFactor = 1)
        {
            DateTime date = startDate;
            int years, months, days;
            TenorType tenorType;
            if (TenorTimeSpan.TryParse(tenorTimeSpan, out years, out months, out days, out tenorType) == false)
            {
                throw new ArgumentException(String.Format(Dodoni.BasicComponents.ExceptionMessages.ArgumentIsInvalid, tenorTimeSpan), "tenorTimeSpan");
            }

            if (years != 0)
            {
                date = date.AddYears(tenorTimeSpanFactor * years);
            }
            if (months != 0)
            {
                date = date.AddMonths(tenorTimeSpanFactor * months);
            }
            if (days != 0)
            {
                date = date.AddDays(tenorTimeSpanFactor * days);
            }
            return date;
        }

        /// <summary>Adds a specific <see cref="TenorTimeSpan"/> object to a specific <see cref="System.DateTime"/> object.
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="tenorTimeSpan">The time span to add in some <see cref="TenorTimeSpan"/> representation.</param>
        /// <param name="tenorTimeSpanFactor">A factor to take into account.</param>
        /// <returns>The <see cref="System.DateTime"/> which is the result of <paramref name="startDate"/> plus <paramref name="tenorTimeSpanFactor"/> * <paramref name="tenorTimeSpan"/>.
        /// </returns>
        public static DateTime AddTenorTimeSpan(this DateTime startDate, TenorTimeSpan tenorTimeSpan, int tenorTimeSpanFactor)
        {
            DateTime date = startDate;

            if (tenorTimeSpan.Years != 0)
            {
                date = date.AddYears(tenorTimeSpanFactor * tenorTimeSpan.Years);
            }
            if (tenorTimeSpan.Months != 0)
            {
                date = date.AddMonths(tenorTimeSpanFactor * tenorTimeSpan.Months);
            }
            if (tenorTimeSpan.Days != 0)
            {
                date = date.AddDays(tenorTimeSpanFactor * tenorTimeSpan.Days);
            }
            return date;
        }
    }
}