﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using NUnit.Framework;
using Dodoni.MathLibrary;

namespace Dodoni.Finance.StandardModels
{
    /// <summary>Serves as abstract unit test class for <see cref="IConstantVolatilityStandardEuropeanOption"/> objects.
    /// </summary>
    public abstract class ConstantVolatilityStandardEuropeanOptionTests
    {
        #region protected constructors

        /// <summary>Initializes a new instance of the <see cref="ConstantVolatilityStandardEuropeanOptionTests"/> class.
        /// </summary>
        protected ConstantVolatilityStandardEuropeanOptionTests()
        {
        }
        #endregion

        #region public unit test methods / test case data

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardOption.GetValue(double)"/> method.
        /// </summary>
        /// <param name="strike">The strike.</param>
        /// <param name="forward">The forward.</param>
        /// <param name="timeToExpiration">The time span between valuation date and expiry date.</param>
        /// <param name="discountFactor">The discount factor at <paramref name="timeToExpiration"/>.</param>
        /// <param name="volatiliy">The volatility.</param>
        /// <param name="expectedOptionValue">The expected option value.</param>
        /// <param name="tolerance">The tolerance to apply for the comparison of the <paramref name="expectedOptionValue"/> and the calculated option value. This is
        /// required because many benchmark results in the literature are rounded up to a specific number of digits.</param>
        /// <remarks>Often in the literature the parameters are specified in a different way, for example one
        /// has to compute the forward, discount factor etc.</remarks>
        public virtual void GetOptionValue_TestCase(double strike, double forward, double timeToExpiration, double discountFactor, double volatiliy, double expectedOptionValue, double tolerance)
        {
            var option = CreateOption(strike, forward, timeToExpiration, discountFactor);

            double actualOptionValue = option.GetValue(volatiliy);

            Assert.That(actualOptionValue, Is.EqualTo(expectedOptionValue).Within(tolerance));
        }

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardOption.TryGetImpliedVolatility(double, out double)"/>. First we compute the option value
        /// with respect to a specific volatility, afterwards <see cref="IConstantVolatilityStandardOption.TryGetImpliedVolatility(double, out double)"/> will be called.
        /// </summary>
        /// <param name="optionConfiguration">The option configuration of the option to take into account for the unit test.</param>
        [TestCaseSource("TestCaseDataOptions")]
        public void TryGetImpliedVolatility_ComputedOptionValue_Volatility(ConstantVolatilityStandardEuropeanOptionConfiguration optionConfiguration)
        {
            var optionUnderTest = CreateOption(optionConfiguration.Strike, optionConfiguration.Forward, optionConfiguration.TimeToExpiration, optionConfiguration.DiscountFactor);

            double optionValue = optionUnderTest.GetValue(optionConfiguration.Volatility);
            Assume.That(IsVolatilityInvertibleOptionValue(optionValue, optionUnderTest) == true, String.Format("Option value {0} can not be inverted; intrinsic value {1}", optionValue, optionUnderTest.GetIntrinsicValue()));


            double actualImpliedVolatility;
            Assert.That(optionUnderTest.TryGetImpliedVolatility(optionValue, out actualImpliedVolatility), Is.EqualTo(ImpliedCalculationResultState.ProperResult), String.Format("Option value was: {0}, Intrinsic value: {1}.", optionValue, optionUnderTest.GetIntrinsicValue()));

            Assert.That(actualImpliedVolatility, Is.EqualTo(optionConfiguration.Volatility).Within(1E-9), String.Format("Option value was: {0}, Intrinsic value: {1}", optionValue, optionUnderTest.GetIntrinsicValue()));
        }

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardEuropeanOption.TryGetImpliedVolatilityOfUndiscountedValue(double, out double)"/>. First we compute the undiscounted option value
        /// with respect to a specific volatility, afterwards <see cref="IConstantVolatilityStandardEuropeanOption.TryGetImpliedVolatilityOfUndiscountedValue(double, out double)"/> will be called.
        /// </summary>
        /// <param name="optionConfiguration">The option configuration of the option to take into account for the unit test.</param>
        [TestCaseSource("TestCaseDataOptions")]
        public void TryGetImpliedVolatilityOfUndiscountedValue_ComputedOptionValue_Volatility(ConstantVolatilityStandardEuropeanOptionConfiguration optionConfiguration)
        {
            var optionUnderTest = CreateOption(optionConfiguration.Strike, optionConfiguration.Forward, optionConfiguration.TimeToExpiration, optionConfiguration.DiscountFactor);

            double undiscountedOptionValue = optionUnderTest.GetUndiscountedValue(optionConfiguration.Volatility);
            Assume.That(IsVolatilityInvertibleOptionValue(undiscountedOptionValue * optionUnderTest.DiscountFactor, optionUnderTest) == true, String.Format("Option value {0} can not be inverted; intrinsic value {1}", undiscountedOptionValue * optionUnderTest.DiscountFactor, optionUnderTest.GetIntrinsicValue()));


            double actualImpliedVolatility;
            Assert.That(optionUnderTest.TryGetImpliedVolatilityOfUndiscountedValue(undiscountedOptionValue, out actualImpliedVolatility), Is.EqualTo(ImpliedCalculationResultState.ProperResult), String.Format("Undiscounted Option value was: {0}, undiscounted Intrinsic value: {1}", undiscountedOptionValue, optionUnderTest.GetIntrinsicValue() / optionUnderTest.DiscountFactor));

            Assert.That(actualImpliedVolatility, Is.EqualTo(optionConfiguration.Volatility).Within(1E-9), String.Format("Undiscounted Option value was: {0}, undiscounted Intrinsic value: {1}", undiscountedOptionValue, optionUnderTest.GetIntrinsicValue() / optionUnderTest.DiscountFactor));
        }

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardEuropeanOption.GetForwardDelta(double)"/>. The outcome
        /// is compared to a approximation based on a differential quotient.
        /// </summary>
        /// <param name="optionConfiguration">The option configuration of the option to take into account for the unit test.</param>
        [TestCaseSource("TestCaseDataOptions")]
        public void GetForwardDelta_TestCase_ViaDifferentialQuotientApproximation(ConstantVolatilityStandardEuropeanOptionConfiguration optionConfiguration)
        {
            var optionUnderTest = CreateOption(optionConfiguration.Strike, optionConfiguration.Forward, optionConfiguration.TimeToExpiration, optionConfiguration.DiscountFactor);

            double actualForwardDelta = optionUnderTest.GetForwardDelta(optionConfiguration.Volatility);

            double expectedForwardDelta = DifferentialQuotientGreekApproximation.GetOptionForwardDelta(optionConfiguration, (K, F, t, df, sigma) => CreateOption(K, F, t, df).GetValue(sigma));

            Assert.That(actualForwardDelta, Is.EqualTo(expectedForwardDelta).Within(1E-5), optionUnderTest.ToString());
        }

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardOption.GetTheta(double)"/>. The outcome
        /// is compared to a approximation based on a differential quotient.
        /// </summary>
        /// <param name="optionConfiguration">The option configuration of the option to take into account for the unit test.</param>
        [TestCaseSource("TestCaseDataOptions")]
        public void GetTheta_TestCase_ViaDifferentialQuotientApproximation(ConstantVolatilityStandardEuropeanOptionConfiguration optionConfiguration)
        {
            var optionUnderTest = CreateOption(optionConfiguration.Strike, optionConfiguration.Forward, optionConfiguration.TimeToExpiration, optionConfiguration.DiscountFactor);

            double actualTheta = optionUnderTest.GetTheta(optionConfiguration.Volatility);

            double expectedTheta = DifferentialQuotientGreekApproximation.GetOptionTheta(optionConfiguration, (K, F, t, df, sigma) => CreateOption(K, F, t, df).GetValue(sigma));

            Assert.That(actualTheta, Is.EqualTo(expectedTheta).Within(1E-5), optionUnderTest.ToString());
        }

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardEuropeanOption.GetStrikeDelta(double)"/>. The outcome
        /// is compared to a approximation based on a differential quotient.
        /// </summary>
        /// <param name="optionConfiguration">The option configuration of the option to take into account for the unit test.</param>
        [TestCaseSource("TestCaseDataOptions")]
        public void GetStrikeDelta_TestCase_ViaDifferentialQuotientApproximation(ConstantVolatilityStandardEuropeanOptionConfiguration optionConfiguration)
        {
            var optionUnderTest = CreateOption(optionConfiguration.Strike, optionConfiguration.Forward, optionConfiguration.TimeToExpiration, optionConfiguration.DiscountFactor);

            double actualStrikeDelta = optionUnderTest.GetStrikeDelta(optionConfiguration.Volatility);

            double expectedStrikeDelta = DifferentialQuotientGreekApproximation.GetOptionKappa(optionConfiguration, (K, F, t, df, sigma) => CreateOption(K, F, t, df).GetValue(sigma));

            Assert.That(actualStrikeDelta, Is.EqualTo(expectedStrikeDelta).Within(1E-5), optionUnderTest.ToString());
        }

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardEuropeanOption.GetForwardGamma(double)"/>. The outcome
        /// is compared to a approximation based on a differential quotient.
        /// </summary>
        /// <param name="optionConfiguration">The option configuration of the option to take into account for the unit test.</param>
        [TestCaseSource("TestCaseDataOptions")]
        public void GetForwardGamma_TestCase_ViaDifferentialQuotientApproximation(ConstantVolatilityStandardEuropeanOptionConfiguration optionConfiguration)
        {
            var optionUnderTest = CreateOption(optionConfiguration.Strike, optionConfiguration.Forward, optionConfiguration.TimeToExpiration, optionConfiguration.DiscountFactor);

            double actualForwardGamma = optionUnderTest.GetForwardGamma(optionConfiguration.Volatility);

            double expectedForwardGamma = DifferentialQuotientGreekApproximation.GetOptionForwardGamma(optionConfiguration, (K, F, t, df, sigma) => CreateOption(K, F, t, df).GetValue(sigma));

            Assert.That(actualForwardGamma, Is.EqualTo(expectedForwardGamma).Within(1E-4), String.Format("{0}; option value: {1}", optionUnderTest.ToString(), optionUnderTest.GetValue(optionConfiguration.Volatility)));
        }

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardOption.GetVega(double)"/>. The outcome
        /// is compared to a approximation based on a differential quotient.
        /// </summary>
        /// <param name="optionConfiguration">The option configuration of the option to take into account for the unit test.</param>
        [TestCaseSource("TestCaseDataOptions")]
        public void GetForwardVega_TestCase_ViaDifferentialQuotientApproximation(ConstantVolatilityStandardEuropeanOptionConfiguration optionConfiguration)
        {
            var optionUnderTest = CreateOption(optionConfiguration.Strike, optionConfiguration.Forward, optionConfiguration.TimeToExpiration, optionConfiguration.DiscountFactor);

            double actualVega = optionUnderTest.GetVega(optionConfiguration.Volatility);

            double expectedVega = DifferentialQuotientGreekApproximation.GetOptionVega(optionConfiguration, (K, F, t, df, sigma) => CreateOption(K, F, t, df).GetValue(sigma));

            Assert.That(actualVega, Is.EqualTo(expectedVega).Within(1E-5), optionUnderTest.ToString());
        }

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardOption.GetVolga(double)"/>. The outcome
        /// is compared to a approximation based on a differential quotient.
        /// </summary>
        /// <param name="optionConfiguration">The option configuration of the option to take into account for the unit test.</param>
        [TestCaseSource("TestCaseDataOptions")]
        public void GetForwardVolga_TestCase_ViaDifferentialQuotientApproximation(ConstantVolatilityStandardEuropeanOptionConfiguration optionConfiguration)
        {
            var optionUnderTest = CreateOption(optionConfiguration.Strike, optionConfiguration.Forward, optionConfiguration.TimeToExpiration, optionConfiguration.DiscountFactor);

            double actualVolga = optionUnderTest.GetVolga(optionConfiguration.Volatility);
            double expectedVolga = DifferentialQuotientGreekApproximation.GetOptionVolga(optionConfiguration, (K, F, t, df, sigma) => CreateOption(K, F, t, df).GetValue(sigma));

            Assert.That(actualVolga, Is.EqualTo(expectedVolga).Within(1E-4), optionUnderTest.ToString());
        }

        /// <summary>Serves as unit test for <see cref="IConstantVolatilityStandardEuropeanOption.GetForwardVanna(double)"/>. The outcome
        /// is compared to a approximation based on a differential quotient.
        /// </summary>
        /// <param name="optionConfiguration">The option configuration of the option to take into account for the unit test.</param>
        [TestCaseSource("TestCaseDataOptions")]
        public void GetForwardVanna_TestCase_ViaDifferentialQuotientApproximation(ConstantVolatilityStandardEuropeanOptionConfiguration optionConfiguration)
        {
            var optionUnderTest = CreateOption(optionConfiguration.Strike, optionConfiguration.Forward, optionConfiguration.TimeToExpiration, optionConfiguration.DiscountFactor);

            double actualForwardVanna = optionUnderTest.GetForwardVanna(optionConfiguration.Volatility);

            double expectedForwardVanna = DifferentialQuotientGreekApproximation.GetOptionForwardVanna(optionConfiguration, (K, F, t, df, sigma) => CreateOption(K, F, t, df).GetValue(sigma));

            Assert.That(actualForwardVanna, Is.EqualTo(expectedForwardVanna).Within(1E-4), optionUnderTest.ToString());
        }

        /// <summary>Gets the test case data, i.e. a collection of parameters of the options to take into account for the unit tests.
        /// </summary>
        /// <value>The test case data.</value>
        public IEnumerable<TestCaseData> TestCaseDataOptions
        {
            get
            {
                foreach (var optionParameters in TestOptionParametersForApproximationTests)
                {
                    yield return new TestCaseData(optionParameters);
                }
            }
        }
        #endregion

        #region protected methods/properties

        /// <summary>Determines whether a volatility should be computable that implies a specified option value.
        /// </summary>
        /// <param name="optionValue">The option value.</param>
        /// <param name="option">The option.</param>
        /// <returns>
        /// 	<c>true</c> if there should be an implied volatility for the specified option value be available; otherwise, <c>false</c>.
        /// </returns>
        /// <remarks>The implied volatility of Far-out-of-the-money options can often not be computed. One can design the individual test cases or one may override this
        /// method in an individual manner to avoid questionable results in the unit tests.</remarks>
        protected virtual bool IsVolatilityInvertibleOptionValue(double optionValue, IConstantVolatilityStandardEuropeanOption option)
        {
            return (optionValue > MachineConsts.SuperTinyEpsilon);
        }

        /// <summary>Gets a collection of parameters for the options to take into account for the unit tests, for example for the calculation of the greeks 
        /// based on an approximation by differential quotients etc.
        /// </summary>
        /// <value>The collection of parameters of the options to take into account for the unit tests.</value>
        protected abstract IEnumerable<ConstantVolatilityStandardEuropeanOptionConfiguration> TestOptionParametersForApproximationTests
        {
            get;
        }

        /// <summary>Creates a specific <see cref="IConstantVolatilityStandardEuropeanOption"/> option.
        /// </summary>
        /// <param name="strike">The strike.</param>
        /// <param name="forward">The forward.</param>
        /// <param name="timeToExpiration">The time to expiration.</param>
        /// <param name="discountFactor">The discount factor.</param>
        /// <returns>A <see cref="IConstantVolatilityStandardEuropeanOption"/> object with the desired parameters.</returns>
        protected abstract IConstantVolatilityStandardEuropeanOption CreateOption(double strike, double forward, double timeToExpiration, double discountFactor);
        #endregion
    }
}