﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Linq;
using System.Collections.Generic;

using NUnit.Framework;

namespace Dodoni.Finance.TenorFactory
{
    /// <summary>Serves as class for unit tests with respect to <see cref="TenorTimeSpanExtensions"/>.
    /// </summary>
    [TestFixture]
    public class TenorTimeSpanExtensionsTests
    {
        /// <summary>Serves as unit test for <see cref="TenorTimeSpanExtensions.AddTimeSpan(TenorTimeSpan,TenorTimeSpan,int)"/>. 
        /// </summary>
        /// <param name="tenorTimeSpan">The tenor time span.</param>
        /// <param name="tenorTimeSpanToAdd">The <see cref="TenorTimeSpan"/> to add.</param>
        /// <param name="tenorTimeSpanFactor">A (optional) factor to take into account.</param>
        /// <returns>A <see cref="TenorTimeSpan"/> object which is the result of <paramref name="tenorTimeSpan"/> plus <paramref name="tenorTimeSpanFactor"/> * <paramref name="tenorTimeSpanToAdd"/>.</returns>
        [TestCaseSource("TestCaseData_AddTimeSpan")]
        public TenorTimeSpan AddTimeSpan_TestCase(TenorTimeSpan tenorTimeSpan, TenorTimeSpan tenorTimeSpanToAdd, int tenorTimeSpanFactor)
        {
            return TenorTimeSpanExtensions.AddTimeSpan(tenorTimeSpan, tenorTimeSpanToAdd, tenorTimeSpanFactor);
        }

        /// <summary>Gets the test case data for <see cref="AddTimeSpan_TestCase(TenorTimeSpan, TenorTimeSpan, int)"/>.
        /// </summary>
        /// <value>The test case data for <see cref="AddTimeSpan_TestCase(TenorTimeSpan, TenorTimeSpan, int)"/>.</value>
        public IEnumerable<TestCaseData> TestCaseData_AddTimeSpan
        {
            get
            {
                yield return new TestCaseData(new TenorTimeSpan(1, 6, 0), new TenorTimeSpan(0, 6, 0), 2).Returns(new TenorTimeSpan(2, 6, 0));
                yield return new TestCaseData(new TenorTimeSpan(2, 3, 10), new TenorTimeSpan(0, 3, 0), 3).Returns(new TenorTimeSpan(3, 0, 10));
                yield return new TestCaseData(new TenorTimeSpan(0, 2, 4), new TenorTimeSpan(1, 4, 0), 2).Returns(new TenorTimeSpan(2, 10, 4));
            }
        }

        /// <summary>Serves as unit test for <see cref="TenorTimeSpanExtensions.AddTenorTimeSpan(DateTime,TenorTimeSpan)"/>. 
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="tenorTimeSpan">The tenor time span.</param>
        /// <returns>The <see cref="DateTime"/> object that represents <paramref name="startDate"/> plus <paramref name="tenorTimeSpan"/>.</returns>
        [TestCaseSource("TestCaseData_AddTenorTimeSpan")]
        public DateTime AddTenorTimeSpan_TestCase(DateTime startDate, TenorTimeSpan tenorTimeSpan)
        {
            return TenorTimeSpanExtensions.AddTenorTimeSpan(startDate, tenorTimeSpan);
        }

        /// <summary>Gets the test case data for <see cref="AddTenorTimeSpan_TestCase(DateTime, TenorTimeSpan)"/>.
        /// </summary>
        /// <value>The test case data for <see cref="AddTenorTimeSpan_TestCase(DateTime, TenorTimeSpan)"/>.</value>
        public IEnumerable<TestCaseData> TestCaseData_AddTenorTimeSpan
        {
            get
            {
                yield return new TestCaseData(new DateTime(2010, 2, 18), new TenorTimeSpan(2, 3, 2)).Returns(new DateTime(2012, 5, 20));
                yield return new TestCaseData(new DateTime(2003, 1, 31), new TenorTimeSpan(5, 2, 10)).Returns(new DateTime(2008, 4, 10));
            }
        }

        /// <summary>Serves as unit test for <see cref="TenorTimeSpanExtensions.AddTenorTimeSpan(DateTime,TenorTimeSpan,int)"/>. 
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="tenorTimeSpan">The tenor time span.</param>
        /// <param name="tenorTimeSpanFactor">A factor to take into account.</param>
        /// <returns>The <see cref="DateTime"/> object that represents <paramref name="startDate"/> plus <paramref name="tenorTimeSpanFactor"/> * <paramref name="tenorTimeSpan"/>.</returns>
        [TestCaseSource("TestCaseData_AddTenorTimeSpanWithTenorTimeSpanFactorArgument")]
        public DateTime AddTenorTimeSpan_TestCase(DateTime startDate, TenorTimeSpan tenorTimeSpan, int tenorTimeSpanFactor)
        {
            return TenorTimeSpanExtensions.AddTenorTimeSpan(startDate, tenorTimeSpan, tenorTimeSpanFactor);
        }

        /// <summary>Gets the test case data for <see cref="AddTenorTimeSpan_TestCase(DateTime, TenorTimeSpan,int)"/>.
        /// </summary>
        /// <value>The test case data for <see cref="AddTenorTimeSpan_TestCase(DateTime, TenorTimeSpan,int)"/>.</value>
        public IEnumerable<TestCaseData> TestCaseData_AddTenorTimeSpanWithTenorTimeSpanFactorArgument
        {
            get
            {
                yield return new TestCaseData(new DateTime(2010, 2, 18), new TenorTimeSpan(2, 3, 2), 1).Returns(new DateTime(2012, 5, 20));
                yield return new TestCaseData(new DateTime(2010, 2, 18), new TenorTimeSpan(2, 3, 2), 3).Returns(new DateTime(2016, 11, 24));

                yield return new TestCaseData(new DateTime(2003, 1, 31), new TenorTimeSpan(5, 2, 10), 1).Returns(new DateTime(2008, 4, 10));
                yield return new TestCaseData(new DateTime(2003, 1, 31), new TenorTimeSpan(5, 2, 10), 2).Returns(new DateTime(2013, 6, 20));
            }
        }

        /// <summary>Serves as unit test for <see cref="TenorTimeSpanExtensions.AddTenorTimeSpan(DateTime,string,int)"/>. 
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="tenorTimeSpan">The tenor time span.</param>
        /// <param name="tenorTimeSpanFactor">A factor to take into account.</param>
        /// <returns>The <see cref="DateTime"/> object that represents <paramref name="startDate"/> plus <paramref name="tenorTimeSpanFactor"/> * <paramref name="tenorTimeSpan"/>.</returns>
        [TestCaseSource("TestCaseData_AddTenorTimeSpanWithStringArgument")]
        public DateTime AddTenorTimeSpan_TestCase(DateTime startDate, string tenorTimeSpan, int tenorTimeSpanFactor)
        {
            return TenorTimeSpanExtensions.AddTenorTimeSpan(startDate, tenorTimeSpan, tenorTimeSpanFactor);
        }

        /// <summary>Gets the test case data for <see cref="AddTenorTimeSpan_TestCase(DateTime, string,int)"/>.
        /// </summary>
        /// <value>The test case data for <see cref="AddTenorTimeSpan_TestCase(DateTime, string,int)"/>.</value>
        public IEnumerable<TestCaseData> TestCaseData_AddTenorTimeSpanWithStringArgument
        {
            get
            {
                yield return new TestCaseData(new DateTime(2010, 2, 18), "2Y 3M 2d", 1).Returns(new DateTime(2012, 5, 20));
                yield return new TestCaseData(new DateTime(2010, 2, 18), "2y 3m 2d", 3).Returns(new DateTime(2016, 11, 24));

                yield return new TestCaseData(new DateTime(2003, 1, 31), " 5Y  2m 10D", 1).Returns(new DateTime(2008, 4, 10));
                yield return new TestCaseData(new DateTime(2003, 1, 31), "5Y   2M 10d", 2).Returns(new DateTime(2013, 6, 20));
            }
        }
    }
}