﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.Finance;
using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;
using Dodoni.BasicComponents.LogFileFactory;
using Dodoni.Finance.CommonMarketUsages.BusinessDayConventions;

namespace Dodoni.Finance.CommonMarketUsages
{
    /// <summary>Serves as factory for business day conventions.
    /// </summary>
    public static class BusinessDayConvention
    {
        #region private static members

        /// <summary>The object type ID for logging, i.e. {DD2257D6-F02A-4486-8C50-93B0BC7825CA}.
        /// </summary>
        private static readonly System.Guid ObjectTypeID = new System.Guid(0xdd2257d6, 0xf02a, 0x4486, 0x8c, 0x50, 0x93, 0xb0, 0xbc, 0x78, 0x25, 0xca);

        /// <summary>The internal pool of the business day conventions.
        /// </summary>
        private static IdentifierNameableDictionary<IBusinessDayConvention> sm_Pool;

        /// <summary>The log file.
        /// </summary>
        private static ILogFile sm_LogFile;
        #endregion

        #region public static (readonly) members

        /// <summary>The business day convention 'no adjustment'.
        /// </summary>
        public static readonly IBusinessDayConvention NoAdjustment = new NoAdjustment();

        /// <summary>The business day convention 'following'.
        /// </summary>
        public static readonly IBusinessDayConvention Following = new FollowingAdjustment();

        /// <summary>The business day convention 'modified following'.
        /// </summary>
        public static readonly IBusinessDayConvention ModFollowing = new ModFollowingAdjustment();

        /// <summary>The business day convention 'preceding/previous business day'.
        /// </summary>
        public static readonly IBusinessDayConvention Preceding = new PrecedingAdjustment();

        /// <summary>The business day convention 'end of month' (no adjustment).
        /// </summary>
        public static readonly IBusinessDayConvention EndOfMonth = new EndOfMonthNoAdjustment();

        /// <summary>The business day convention 'end of month, following'.
        /// </summary>
        public static readonly IBusinessDayConvention EndOfMonthFollowing = new EndOfMonthFollowingAdjustment();

        /// <summary>The business day convention 'end of month, preceding'.
        /// </summary>
        public static readonly IBusinessDayConvention EndOfMonthPreceding = new EndOfMonthPrecedingAdjustment();

        /// <summary>The business day convention 'end of month, ignoring leap years'.
        /// </summary>
        public static readonly IBusinessDayConvention EndOfMonthIgnoreLeapYears = new EndOfMonthIgnoreLeapYearsAdjustment();

        /// <summary>The business day convention 'end of quarter' (no adjustment).
        /// </summary>
        public static readonly IBusinessDayConvention EndOfQuarter = new EndOfQuarterNoAdjustment();

        /// <summary>The business day convention 'end of quarter preceding'.
        /// </summary>
        public static readonly IBusinessDayConvention EndOfQuarterPreceding = new EndOfQuarterPrecedingAdjustment();

        /// <summary>The business day convention 'end of quarter following'.
        /// </summary>
        public static readonly IBusinessDayConvention EndOfQuarterFollowing = new EndOfQuarterFollowingAdjustment();

        /// <summary>The business day convention 'third wednesday'.
        /// </summary>
        public static readonly IBusinessDayConvention ThirdWednesday = new ThirdWednesdayAdjustment();

        /// <summary>The business day convention 'two business days prior third wednesday'.
        /// </summary>
        public static readonly IBusinessDayConvention TwoBusinessDaysPriorThirdWednesday = new TwoBusinessDaysPriorThirdWednesdayAdjustment();
        #endregion

        #region static constructors

        /// <summary>Initializes a new instance of the <see cref="BusinessDayConvention"/> class.
        /// </summary>
        static BusinessDayConvention()
        {
            sm_Pool = new IdentifierNameableDictionary<IBusinessDayConvention>(NoAdjustment, Following, ModFollowing, Preceding,
                EndOfMonth, EndOfMonthFollowing, EndOfMonthPreceding, EndOfMonthIgnoreLeapYears, 
                EndOfQuarter, EndOfQuarterPreceding, EndOfQuarterFollowing,
                ThirdWednesday, TwoBusinessDaysPriorThirdWednesday);

            sm_LogFile = LoggingFactory.CreateGlobalLogFile("Business day convention", "Pool", ObjectTypeID);
            sm_LogFile.AddMessage(LoggingFactory.Infos.CreateObject);
        }
        #endregion

        #region public static properties

        /// <summary>Gets the number of business day conventions.
        /// </summary>
        /// <value>The number of business day conventions.</value>
        public static int Count
        {
            get { return sm_Pool.Count; }
        }

        /// <summary>Gets the business day conventions.
        /// </summary>
        /// <value>The business day conventions.</value>
        public static IEnumerable<IBusinessDayConvention> Values
        {
            get { return sm_Pool.Values; }
        }

        /// <summary>Gets the business day convention names.
        /// </summary>
        /// <value>The business day convention names.</value>
        public static IEnumerable<string> Names
        {
            get { return sm_Pool.Names; }
        }
        #endregion

        #region public static methods

        /// <summary>Gets the business day convention names in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <returns>A collection of business day convention names in its <see cref="IdentifierString"/> representation.</returns>
        public static IEnumerable<IdentifierString> GetIdentifierStringNames()
        {
            return sm_Pool.GetNamesAsIdentifierStrings();
        }

        /// <summary>Gets a specified business day convention.
        /// </summary>
        /// <param name="name">The name of the business day convention to search.</param>
        /// <param name="value">The business day convention (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public static bool TryGetValue(IdentifierString name, out IBusinessDayConvention value)
        {
            return sm_Pool.TryGetValue(name, out value);
        }

        /// <summary>Gets a specified business day convention.
        /// </summary>
        /// <param name="name">The name of the business day convention to search.</param>
        /// <param name="value">The business day convention (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public static bool TryGetValue(string name, out IBusinessDayConvention value)
        {
            return sm_Pool.TryGetValue(name, out value);
        }

        /// <summary>Adds a specified business day convention.
        /// </summary>
        /// <param name="value">The business day convention to add.</param>
        /// <returns>A value indicating whether <paramref name="value"/> has been added.</returns>
        public static ItemAddedState Add(IBusinessDayConvention value)
        {
            ItemAddedState state = sm_Pool.Add(value);
            sm_LogFile.AddPoolAddingLogFileMessage(state, value.Name);
            return state;
        }
        #endregion
    }
}