﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.Finance;
using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;
using Dodoni.BasicComponents.LogFileFactory;
using Dodoni.Finance.CommonMarketUsages.DayCountConventions;

namespace Dodoni.Finance.CommonMarketUsages
{
    /// <summary>Serves as factory for day count conventions.
    /// </summary>
    public static class DayCountConvention
    {
        #region private static members

        /// <summary>The internal pool of the day count conventions.
        /// </summary>
        private static IdentifierNameableDictionary<IDayCountConvention> sm_Pool;

        /// <summary>The logfile for the pool.
        /// </summary>
        private static ILogFile sm_LogFile;

        /// <summary>The object type ID, i.e. {159146E8-8ED4-4C8C-8C98-48EE7AAD8E40}.
        /// </summary>
        private static Guid sm_ObjectTypeID = new Guid(0x159146e8, 0x8ed4, 0x4c8c, 0x8c, 0x98, 0x48, 0xee, 0x7a, 0xad, 0x8e, 0x40);
        #endregion

        #region public static (readonly) members

        /// <summary>The 'Actual/Actual' day count conventions.
        /// </summary>
        public static readonly ActualActual ActualActual = new ActualActual();

        /// <summary>The day count convention 'Act/360'.
        /// </summary>
        public static readonly IDayCountConvention Actual360 = new Actual360();

        /// <summary>The day count convention 'Act/365'.
        /// </summary>
        public static readonly IDayCountConvention Actual365 = new Actual365();

        /// <summary>The day count convention 'Act/365.25'.
        /// </summary>
        public static readonly IDayCountConvention Actual365_25 = new Actual365_25();

        /// <summary>The day count convention '1/1'.
        /// </summary>
        public static readonly IDayCountConvention OneOverOne = new OneOverOne();

        /// <summary>The day count convention '30/360'.
        /// </summary>
        public static readonly IDayCountConvention Thirty360 = new Thirty360();

        /// <summary>The day count convention '30E/360'.
        /// </summary>
        public static readonly IDayCountConvention ThirtyE360 = new ThirtyE360();

        /// <summary>Serves as factory for the day count convention 'BU/252' used for Brazilian trades.
        /// </summary>
        public static readonly Bu252 Bu252 = new Bu252();
        #endregion

        #region static constructors

        /// <summary>Initializes a new instance of the <see cref="DayCountConvention"/> class.
        /// </summary>
        static DayCountConvention()
        {
            sm_Pool = new IdentifierNameableDictionary<IDayCountConvention>(Actual360, Actual365, Actual365_25, ActualActual.AFB, ActualActual.ISDA, ActualActual.ISMA,
                OneOverOne, Thirty360, ThirtyE360);

            sm_LogFile = LoggingFactory.CreateGlobalLogFile("Day count convention", "Pool", sm_ObjectTypeID);
            sm_LogFile.AddMessage(LoggingFactory.Infos.CreateObject);
        }
        #endregion

        #region public static properties

        /// <summary>Gets the number of day count conventions.
        /// </summary>
        /// <value>The number of day count conventions.</value>
        public static int Count
        {
            get { return sm_Pool.Count; }
        }

        /// <summary>Gets the day count conventions.
        /// </summary>
        /// <value>The day count conventions.</value>
        public static IEnumerable<IDayCountConvention> Values
        {
            get { return sm_Pool.Values; }
        }

        /// <summary>Gets the day count convention names.
        /// </summary>
        /// <value>The day count convention names.</value>
        public static IEnumerable<string> Names
        {
            get { return sm_Pool.Names; }
        }
        #endregion

        #region public static methods

        /// <summary>Gets the day count convention names in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <returns>A collection of day count convention names its <see cref="IdentifierString"/> representation.</returns>
        public static IEnumerable<IdentifierString> GetIdentifierStringNames()
        {
            return sm_Pool.GetNamesAsIdentifierStrings();
        }

        /// <summary>Gets a specified day count convention.
        /// </summary>
        /// <param name="name">The name of the day count convention.</param>
        /// <param name="value">The day count convention (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public static bool TryGetValue(IdentifierString name, out IDayCountConvention value)
        {
            return sm_Pool.TryGetValue(name, out value);
        }

        /// <summary>Gets a specified day count convention.
        /// </summary>
        /// <param name="name">The name of the day count convention.</param>
        /// <param name="value">The day count convention (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public static bool TryGetValue(string name, out IDayCountConvention value)
        {
            return sm_Pool.TryGetValue(name, out value);
        }

        /// <summary>Adds the specified day count convention.
        /// </summary>
        /// <param name="value">The business day convention to add.</param>
        /// <returns>A value indicating whether <paramref name="value"/> has been added.</returns>
        public static ItemAddedState Add(IDayCountConvention value)
        {
            ItemAddedState state = sm_Pool.Add(value);
            sm_LogFile.AddPoolAddingLogFileMessage(state, (value != null) ? value.Name : null);
            return state;
        }
        #endregion
    }
}