﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

using Dodoni.Finance;

namespace Dodoni.Finance.CommonMarketUsages.HolidayCalendars
{
    /// <summary>Represents an attribute for an enumeration where the enum items represents public holidays.
    /// </summary>
    [AttributeUsage(AttributeTargets.Enum, Inherited = false, AllowMultiple = false)]
    public sealed class CustomHolidayCalendarAttribute : System.Attribute
    {
        #region public (readonly) members

        /// <summary>The name of the holiday calendar.
        /// </summary>
        /// <remarks>The name of the calendar is assumed to be independend of the language. In general the name is some 
        /// shortcut with at most three characters (ISO name).</remarks>
        public readonly string CalendarName;

        /// <summary>The region of the holiday calendar.
        /// </summary>
        public readonly HolidayCalendarRegion Region;

        /// <summary>The resource name (no language dependend suffix) and the corresponding namespace.
        /// </summary>
        public readonly string FullResourceName;

        /// <summary>The language depending description of the holiday calendar with respect to <see cref="FullResourceName"/>, i.e. as some resource property name.
        /// </summary>
        /// <remarks>If set to <c>null</c>, this member will be ignored.</remarks>
        public string Description = null;
        #endregion

        #region private members

        /// <summary>The earliest date for that holiday informations are available.
        /// </summary>
        private DateTime m_FirstDate;

        /// <summary>The latest date for that holiday informations are available.
        /// </summary>
        private DateTime m_LastDate;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="CustomHolidayCalendarAttribute"/> class.
        /// </summary>
        /// <param name="calendarName">The name of the holiday calendar.</param>
        /// <param name="region">The region of the holiday calendar.</param>
        /// <param name="fullResourceName">The name of the resource (including the namespace).</param>
        public CustomHolidayCalendarAttribute(string calendarName, HolidayCalendarRegion region, string fullResourceName)
        {
            CalendarName = calendarName;
            Region = region;
            FullResourceName = fullResourceName;
            m_FirstDate = DateTime.MinValue;
            m_LastDate = DateTime.MaxValue;
        }
        #endregion

        #region public properties

        /// <summary>Gets or sets the earliest date for that holiday informations are available.
        /// </summary>
        /// <value>The first valid date with respect to the given calendar.</value>
        public DateTime FirstDate
        {
            get { return m_FirstDate; }
            set { m_FirstDate = value; }
        }

        /// <summary>Gets or sets the latest date for that holiday informations are available.
        /// </summary>
        /// <value>The last valid date with respect to the given calendar.</value>
        public DateTime LastDate
        {
            get { return m_LastDate; }
            set { m_LastDate = value; }
        }
        #endregion
    }
}