﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.Finance;
using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Utilities;

namespace Dodoni.Finance.CommonMarketUsages.HolidayCalendars
{
    /// <summary>Represents a generic (non-standard) weekend.
    /// </summary>
    internal class GenericWeekend : IWeekendRepresentation
    {
        #region private members

        /// <summary>The days of the weekend.
        /// </summary>
        private ISet<DayOfWeek> m_DaysOfWeekend;

        /// <summary>The name of the weekend type.
        /// </summary>
        private readonly IdentifierString m_Name;

        /// <summary>The annotation.
        /// </summary>
        private string m_Annotation;
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="GenericWeekend"/> class.
        /// </summary>
        /// <param name="dayOfWeekend">The day that represents the weekend.</param>
        internal GenericWeekend(DayOfWeek dayOfWeekend)
        {
            m_DaysOfWeekend = new HashSet<DayOfWeek>();
            m_DaysOfWeekend.Add(dayOfWeekend);
            m_Name = new IdentifierString("{" + dayOfWeekend.ToString() + "}");
            m_Annotation = String.Format(Resources.GenericWeekendType, dayOfWeekend.ToString());
        }

        /// <summary>Initializes a new instance of the <see cref="GenericWeekend"/> class.
        /// </summary>
        /// <param name="daysOfWeekend">The days that represents the weekend.</param>
        internal GenericWeekend(SortedSet<DayOfWeek> daysOfWeekend)
        {
            m_DaysOfWeekend = daysOfWeekend;

            StringBuilder strBuilder = new StringBuilder();
            foreach (DayOfWeek dayOfWeek in daysOfWeekend)
            {
                if (strBuilder.Length > 0)
                {
                    strBuilder.Append(",");
                }
                strBuilder.Append(dayOfWeek.ToString());
            }
            string listOfDayNames = strBuilder.ToString();
            m_Name = new IdentifierString("{" + listOfDayNames + "}");
            m_Annotation = String.Format(Resources.GenericWeekendType, listOfDayNames);
        }
        #endregion

        #region public properties

        #region IIdentifierNameable Member

        /// <summary>Gets the name of the current instance.
        /// </summary>
        /// <value>The language independent name of the current instance.</value>
        public IdentifierString Name
        {
            get { return m_Name; }
        }

        /// <summary>Gets the long name of the current instance.
        /// </summary>
        /// <value>The language dependent long name of the current instance.</value>
        public IdentifierString LongName
        {
            get { return m_Name; }
        }
        #endregion

        #region IAnnotatable Members

        /// <summary>Gets a value indicating whether the annotation is readonly.
        /// </summary>
        /// <value><c>true</c> if the annotation of this instance is readonly; otherwise, <c>false</c>.
        /// </value>
        bool IAnnotatable.HasReadOnlyAnnotation
        {
            get { return true; }
        }

        /// <summary>Gets the annotation of the current instance.
        /// </summary>
        /// <value>The annotation of the current instance.</value>
        public string Annotation
        {
            get { return m_Annotation; }
        }
        #endregion

        #endregion

        #region public methods

        #region IAnnotatable Members

        /// <summary>Sets the annotation of the current instance.
        /// </summary>
        /// <param name="annotation">The annotation.</param>
        /// <returns>A value indicating whether the <see cref="Annotation"/> has been changed.
        /// </returns>
        bool IAnnotatable.TrySetAnnotation(string annotation)
        {
            return false;
        }
        #endregion

        #region IWeekendRepresentation Member

        /// <summary>Determines whether a specified day of week falls on a weekend.
        /// </summary>
        /// <param name="dayOfWeek">The day of week.</param>
        /// <returns><c>true</c> if <paramref name="dayOfWeek"/> falls on a weekend; otherwise, <c>false</c>.
        /// </returns>
        public bool Contains(DayOfWeek dayOfWeek)
        {
            return m_DaysOfWeekend.Contains(dayOfWeek);
        }

        /// <summary>Gets the next working day, more precisely the next date which do not falls on a weekend.
        /// </summary>
        /// <param name="day">The day.</param>
        /// <returns><paramref name="day"/> plus n days where n is at least one and n is choosen in a way
        /// such that the return value represents a non-weekend day; n is choosen minimal with this property.
        /// </returns>
        public DateTime GetNextWorkingDay(DateTime day)
        {
            DateTime runningDay = day.AddDays(1);
            while (m_DaysOfWeekend.Contains(runningDay.DayOfWeek) == true)
            {
                runningDay = runningDay.AddDays(1);
            }
            return runningDay;
        }

        /// <summary>Gets the previous working day, more precisely the previous date which do not falls on a weekend.
        /// </summary>
        /// <param name="day">The day.</param>
        /// <returns><paramref name="day"/> minus n days where n is at least one and n is choosen in a way
        /// such that the return value represents a non-weekend day; n is choosen minimal with this property.
        /// </returns>
        public DateTime GetPreviousWorkingDay(DateTime day)
        {
            DateTime runningDay = day.AddDays(-1);
            while (m_DaysOfWeekend.Contains(runningDay.DayOfWeek) == true)
            {
                runningDay = runningDay.AddDays(-1);
            }
            return runningDay;
        }

        /// <summary>Inserts the days of the week which represents the weekend.
        /// </summary>
        /// <param name="weekendDays">A set of days of the week to add the days which represents the weekend (output).</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="weekendDays"/> is <c>null</c>.</exception>
        /// <remarks>Use this method to collect the days of the week which represents the weekend with respect
        /// to different holiday calendars.</remarks>
        public void AddWeekendDaysTo(ISet<DayOfWeek> weekendDays)
        {
            if (weekendDays == null)
            {
                throw new ArgumentNullException("weekendDays");
            }
            foreach (DayOfWeek dayOfWeekend in m_DaysOfWeekend)
            {
                weekendDays.Add(dayOfWeekend);
            }
        }

        /// <summary>Calculates the intersection of a specific set of days with the set of days which represents a weekend.
        /// </summary>
        /// <param name="weekendDays">A set of days of the week. The output is the intersection with the set of days which 
        /// represents a weekend with respect to the current instance.</param>
        public void IntersectWeekendDaysWith(ISet<DayOfWeek> weekendDays)
        {
            if (weekendDays == null)
            {
                throw new ArgumentNullException("weekendDays");
            }
            weekendDays.IntersectWith(m_DaysOfWeekend);
        }

        /// <summary>Gets the number of days in ]<paramref name="date1"/>, <paramref name="date2"/>]
        /// or ]<paramref name="date2"/>, <paramref name="date1"/>] which are no weekend days.
        /// </summary>
        /// <param name="date1">The date1.</param>
        /// <param name="date2">The date2.</param>
        /// <returns>The number of non-weekend days between <paramref name="date1"/> and <paramref name="date2"/> including the latest date if this is a business day.
        /// </returns>
        public int GetNumberOfWorkingDaysInBetween(DateTime date1, DateTime date2)
        {
            if (date2 < date1)
            {
                DateTime temp = date1;
                date1 = date2;
                date2 = temp;
            }

            date1 = date1.AddDays(1); // we do not count the first date

            int rawNumberOfDaysBetween = date2.Subtract(date1).Days;
            int workingDaysPerWeek = 7 - m_DaysOfWeekend.Count;
            int numberOfBusinessDaysInBetween = workingDaysPerWeek * (rawNumberOfDaysBetween / 7);  // full weeks a 'n' working days

            date1 = date1.AddDays(7 * (rawNumberOfDaysBetween / 7)); // moves the first date full weeks, i.e. the same day of the week 
            if (m_DaysOfWeekend.Contains(date1.DayOfWeek) == true)
            {
                date1 = GetNextWorkingDay(date1);
            }
            while (date1 <= date2)
            {
                numberOfBusinessDaysInBetween++;
                date1 = GetNextWorkingDay(date1);
            }
            return numberOfBusinessDaysInBetween;
        }

        /// <summary>Gets the weekend days for a specific year in some <see cref="DateInfo"/> representation, i.e.
        /// in the case of the 'standard' weekend, add saturday's and sundays into a <see cref="HashSet&lt;DateInfo&gt;"/> instance.
        /// </summary>
        /// <param name="year">The year.</param>
        /// <param name="holidaySet">The holiday table set to add (output).</param>
        /// <param name="format">The standard or custome date and time format string.</param>
        /// <exception cref="NullReferenceException">Thrown, if <paramref name="holidaySet"/> is <c>null</c>.</exception>
        /// <remarks>No element will be remove or change before adding new elements to <paramref name="holidaySet"/>.</remarks>
        public void GetWeekendDateInfos(int year, ISet<DateInfo> holidaySet, string format = "dddd")
        {
            /* here, we do a very simple approach, but this method is use to get informations only, i.e. will not 
             * used for pricing etc. Moreover less holiday calendar have a non-standard weekend.
             */
            DateTime runningDate = new DateTime(year, 1, 1);
            DateTime lastDate = new DateTime(year, 12, 31);

            while (runningDate <= lastDate)
            {
                if (m_DaysOfWeekend.Contains(runningDate.DayOfWeek) == true)
                {
                    holidaySet.Add(new DateInfo(runningDate, runningDate.ToString(format)));
                }
                runningDate = runningDate.AddDays(1);
            }
        }
        #endregion

        #endregion
    }
}