﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.XLBasicComponents
{
    /// <summary>Serves as collection for <see cref="IExcelDataAdvice"/> objects.
    /// </summary>
    public class ExcelDataAdvicePool
    {
        #region internal (readonly) members

        /// <summary>Represents the <see cref="IExcelDataAdvice"/> for Boolean values, i.e. represents 'false' and 'true' with respect to a user configuration.
        /// </summary>
        internal readonly BooleanDataAdvice m_BooleanAdvice = new BooleanDataAdvice();
        #endregion

        #region private members

        /// <summary>The Excel data advice.
        /// </summary>
        private IdentifierNameableDictionary<IExcelDataAdvice> m_ExcelDataAdvice = new IdentifierNameableDictionary<IExcelDataAdvice>(capacity: 50, isReadOnlyExceptAdding: false);

        /// <summary>Occurs quering an element of the pool.
        /// </summary>
        private event Action<ExcelDataAdvice.InitializeEventArgs> m_Initialize;

        /// <summary>A value indicating whether the user has added or removed at least one event to the <see cref="Initialize"/> event handler
        /// since the last call of the event-handler.
        /// </summary>
        /// <remarks>This member is used for performance reason only.</remarks>
        private bool m_DataAdviceInitializeChanged = true;
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="ExcelDataAdvicePool"/> class.
        /// </summary>
        internal ExcelDataAdvicePool()
        {
            m_ExcelDataAdvice.Add(m_BooleanAdvice);
        }
        #endregion

        #region public properties

        /// <summary>Gets the number of elements.
        /// </summary>
        /// <value>The number of elements.</value>
        public int Count
        {
            get { return m_ExcelDataAdvice.Count; }
        }

        /// <summary>Occurs before quering an element of the pool.
        /// </summary>
        /// <value>The initialize event handler.</value>
        public event Action<ExcelDataAdvice.InitializeEventArgs> Initialize
        {
            add
            {
                m_DataAdviceInitializeChanged = true;
                m_Initialize += value;
            }
            remove
            {
                m_DataAdviceInitializeChanged = true;
                m_Initialize -= value;
            }
        }

        /// <summary>Gets the <see cref="IExcelDataAdvice"/> for Boolean values, i.e. represents 'false' and 'true' with respect to a user configuration.
        /// </summary>
        /// <value>The <see cref="IExcelDataAdvice"/> for Boolean values.</value>
        public IExcelDataAdvice Booleans
        {
            get { return m_BooleanAdvice; }
        }
        #endregion

        #region internal methods

        /// <summary>Add a specific <see cref="IExcelDataAdvice"/> object into the <see cref="ExcelDataAdvicePool"/>.
        /// </summary>
        /// <param name="value">The excel data advice to register.</param>
        /// <returns>A value indicating whether <paramref name="value"/> has been inserted.</returns>
        internal ItemAddedState Add(IExcelDataAdvice value)
        {
            return m_ExcelDataAdvice.Add(value);
        }
        #endregion

        #region public methods

        /// <summary>Gets a specific <see cref="IExcelDataAdvice"/>.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="value">The value (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetValue(string name, out IExcelDataAdvice value)
        {
            OnInitialize();
            return m_ExcelDataAdvice.TryGetValue(name, out value);
        }

        /// <summary>Gets a specific <see cref="IExcelDataAdvice"/>.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="value">The value (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public bool TryGetValue(IdentifierString name, out IExcelDataAdvice value)
        {
            OnInitialize();
            return m_ExcelDataAdvice.TryGetValue(name, out value);
        }

        /// <summary>Gets a collection of the pool item names.
        /// </summary>
        /// <returns>A collection of the pool item names.</returns>
        public IEnumerable<string> GetNames()
        {
            OnInitialize();
            return m_ExcelDataAdvice.Names;
        }
        #endregion

        #region private methods

        /// <summary>Raises the <see cref="Initialize"/> event.
        /// </summary>
        /// <remarks>Adds elements to the pool, if needed.</remarks>
        private void OnInitialize()
        {
            if (m_DataAdviceInitializeChanged == true)
            {
                if (m_Initialize != null)
                {
                    ExcelDataAdvice.InitializeEventArgs eventArgs = new ExcelDataAdvice.InitializeEventArgs();
                    m_Initialize(eventArgs);
                }
                m_DataAdviceInitializeChanged = true;
            }
        }
        #endregion
    }
}