﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.XLBasicComponents.IO
{
    /// <summary>Represents the pool of available <see cref="IObjectStreamer"/> instances used
    /// for serialization and deserialization of <see cref="ExcelPool"/> objects.
    /// </summary>
    public static class ObjectStreamerPool
    {
        #region private static members

        /// <summary>The internal pool of <see cref="IObjectStreamer"/> used for serialization/deserialisation.
        /// </summary>
        private static IdentifierNameableDictionary<IObjectStreamer> sm_Pool;
        #endregion

        #region static constructor

        /// <summary>Initializes the <see cref="ObjectStreamerPool"/> class.
        /// </summary>
        static ObjectStreamerPool()
        {
            sm_Pool = new IdentifierNameableDictionary<IObjectStreamer>();

            /* do not allow replacing pool elements: */
            sm_Pool.Adding += (sender, eventArgs) => { eventArgs.Cancel = ((eventArgs.OldItem != null) && (eventArgs.NewItem.Name.IDString == eventArgs.OldItem.Name.IDString)); };
        }
        #endregion

        #region public static properties

        /// <summary>Gets the number of <see cref="IObjectStreamer"/>.
        /// </summary>
        /// <value>The number of <see cref="IObjectStreamer"/>.</value>
        public static int Count
        {
            get { return sm_Pool.Count; }
        }

        /// <summary>Gets the <see cref="IObjectStreamer"/> objects.
        /// </summary>
        /// <value>The <see cref="IObjectStreamer"/> objects.</value>
        public static IEnumerable<IObjectStreamer> ObjectStreamer
        {
            get { return sm_Pool.Values; }
        }
        #endregion

        #region public static methods

        /// <summary>Gets the names of the <see cref="IObjectStreamer"/> objects in the <see cref="ObjectStreamerPool"/>.
        /// </summary>
        /// <value>The names of the <see cref="IObjectStreamer"/> objects.</value>
        public static IEnumerable<IdentifierString> GetNames()
        {
            return sm_Pool.GetNamesAsIdentifierStrings();
        }

        /// <summary>Gets the file extensions of the <see cref="IObjectStreamer"/> objects in the <see cref="ObjectStreamerPool"/> class, in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <returns>The file extensions of the <see cref="IObjectStreamer"/> objects.</returns>
        public static IEnumerable<IdentifierString> GetFileExtensions()
        {
            foreach (var item in sm_Pool.Values)
            {
                yield return item.FileExtension;
            }
        }

        /// <summary>Gets the <see cref="IObjectStreamer"/> object with repsect to a specific name.
        /// </summary>
        /// <param name="objectStreamerName">The (unique) name of the <see cref="IObjectStreamer"/>.</param>
        /// <param name="value">An instance of <see cref="IObjectStreamer"/>, where <see cref="IIdentifierNameable.Name"/> equals to
        /// <paramref name="objectStreamerName"/> (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public static bool TryGetObjectStreamer(IdentifierString objectStreamerName, out IObjectStreamer value)
        {
            return sm_Pool.TryGetValue(objectStreamerName, out value);
        }

        /// <summary>Gets a <see cref="IObjectStreamer"/> object with repsect to a specific name.
        /// </summary>
        /// <param name="objectStreamerName">The (unique) name of the <see cref="IObjectStreamer"/> to search.</param>
        /// <param name="value">An instance of <see cref="IObjectStreamer"/>, where <see cref="IIdentifierNameable.Name"/> equals to
        /// <paramref name="objectStreamerName"/> (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public static bool TryGetObjectStreamer(string objectStreamerName, out IObjectStreamer value)
        {
            return sm_Pool.TryGetValue(objectStreamerName, out value);
        }

        /// <summary>Gets a <see cref="IObjectStreamer"/> object with repsect to a specific file extension.
        /// </summary>
        /// <param name="fileExtension">The (unique) file extension of the <see cref="IObjectStreamer"/> to search.</param>
        /// <param name="value">An instance of <see cref="IObjectStreamer"/>, where <see cref="IObjectStreamer.FileExtension"/> equals to
        /// <paramref name="fileExtension"/> (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data.</returns>
        public static bool TryGetObjectStreamerByFileExtension(string fileExtension, out IObjectStreamer value)
        {
            string idFileExtension = fileExtension.ToIDString();
            foreach (var objectStreamer in sm_Pool.Values)
            {
                if (objectStreamer.FileExtension.IDString == idFileExtension)
                {
                    value = objectStreamer;
                    return true;
                }
            }
            value = null;
            return false;
        }

        /// <summary>Adds a specific <see cref="IObjectStreamer"/>.
        /// </summary>
        /// <param name="objectStreamer">The <see cref="IObjectStreamer"/> to add.</param>
        /// <returns>A value indicating whether <paramref name="objectStreamer"/> has been added.</returns>
        public static ItemAddedState Add(IObjectStreamer objectStreamer)
        {
            return sm_Pool.Add(objectStreamer);
        }
        #endregion
    }
}