﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Windows.Forms;

using ExcelDna.Integration;
using Dodoni.XLBasicComponents.LogFile;
using Dodoni.XLBasicComponents.Utilities;

using Dodoni.BasicComponents.LogFileFactory;

namespace Dodoni.XLBasicComponents
{
    /// <summary>Represents the Excel-AddIn, i.e. the Excel-Menu entries for Dodoni.NET.
    /// </summary>
    /// <remarks>This class, especially the <c>AutoOpen</c> method must be extend to establish the functionallity for reading objects from a specific file.</remarks>
    public class ExcelAddIn : IExcelAddIn
    {
        #region public (readonly) members

        /// <summary>A reference to the Excel object. 
        /// </summary>
        public static readonly dynamic ExcelApplication;

        /// <summary>The configuration of the Excel-AddIn.
        /// </summary>
        public static readonly ExcelConfiguration Configuration;
        #endregion

        #region internal const members

        /// <summary>The name of the Excel menue, i.e. 'Dodoni'.
        /// </summary>
        internal const string MenuName = "Dodoni";

        /// <summary>The name of the Excel-category for general functions.
        /// </summary>
        internal const string GeneralCategoryName = "Dodoni";

        /// <summary>The name of the Excel-category for utility functions.
        /// </summary>
        internal const string UtilityCategoryName = "Dodoni.Utilities";

        /// <summary>The name of the Excel-category of math functions.
        /// </summary>
        internal const string MathCategoryName = "Dodoni.MathLib";

        /// <summary>The name of the Excel-category of basic finance functions.
        /// </summary>
        internal const string FinanceBasicCategoryName = "Dodoni.Finance";
        #endregion

        #region private static members

        /// <summary>The log file generator used by the log file factory.
        /// </summary>
        private static ExcelLogFileGenerator sm_LogFileGenerator;

        /// <summary>The configuration form.
        /// </summary>
        private static ConfigurationForm sm_ConfigForm;

        /// <summary>The windows form which contains the global logfile.
        /// </summary>
        private static GlobalLogFileForm sm_GlobalLoggingForm;

        /// <summary>The pool inspector windows form.
        /// </summary>
        private static ExcelPoolInspectorForm sm_PoolInspector;
        #endregion

        #region constructors

        /// <summary>Initializes the <see cref="ExcelAddIn"/> class.
        /// </summary>
        static ExcelAddIn()
        {
            Configuration = ExcelConfiguration.Create();
            sm_LogFileGenerator = new ExcelLogFileGenerator();
            LoggingFactory.SetLogFileGenerator(sm_LogFileGenerator);

            ExcelApplication = ExcelDnaUtil.Application;
            Application.EnableVisualStyles();  // required for Marquee progress bar style
        }

        /// <summary>Initializes a new instance of the <see cref="ExcelAddIn"/> class.
        /// </summary>
        public ExcelAddIn()
        {
        }
        #endregion

        #region public methods

        #region IExcelAddIn Members

        /// <summary>Initialize the <see cref="IExcelAddIn"/>.
        /// </summary>
        public void AutoOpen()
        {
            try
            {
                ExcelIntegration.RegisterUnhandledExceptionHandler(UnhandledExceptionHandler);  // add a 'standard-output' if a try-catch block is missing

                /* use a Excel specific LogFile and register specific logfile message types: */
                LoggingFactory.RegisterLogFileMsgType(ExcelLogFileMessageTypeExtensions.sm_EmptyExcelCellIgnore);
                LoggingFactory.RegisterLogFileMsgType(ExcelLogFileMessageTypeExtensions.sm_ExcelCellDropdownListCreationFails);
                LoggingFactory.RegisterLogFileMsgType(ExcelLogFileMessageTypeExtensions.sm_ExcelRangeSizeCalculationFails);
            }
            catch (Exception e)
            {
                MessageBox.Show(String.Format("{0} Stack trace: {1}", e.Message, e.StackTrace), "Dodoni.net (XL-BasicComponents): Fatal error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>Performs operations if Excel will be closed.
        /// </summary>
        public void AutoClose()
        {
        }
        #endregion

        #region Menu methods

        /// <summary>Shows the About box.
        /// </summary>
        [ExcelCommand(MenuText = "About", MenuName = ExcelAddIn.MenuName)]
        public static void ShowAboutBox()
        {
            AboutDodoniForm aboutForm = new AboutDodoniForm();
            aboutForm.ShowDialog();
        }

        /// <summary>Shows the configuration Windows form of Dodoni.NET.
        /// </summary>
        [ExcelCommand(MenuText = "Configuration", MenuName = ExcelAddIn.MenuName)]
        public static void ShowConfiguration()
        {
            if (sm_ConfigForm == null)
            {
                sm_ConfigForm = new ConfigurationForm();
            }
            sm_ConfigForm.ShowDialog();
        }

        /// <summary>Shows the global log file.
        /// </summary>
        [ExcelCommand(MenuText = "Global log file", MenuName = ExcelAddIn.MenuName)]
        public static void ShowGlobalLogFile()
        {
            if (sm_GlobalLoggingForm == null)
            {
                sm_GlobalLoggingForm = new GlobalLogFileForm();
                sm_GlobalLoggingForm.SetDataSources(sm_LogFileGenerator.GlobalLogFile);
            }
            sm_GlobalLoggingForm.Show();
        }

        /// <summary>Shows the Pool Inspector Windows form.
        /// </summary>
        [ExcelCommand(MenuText = "Poolinspector", MenuName = ExcelAddIn.MenuName)]
        public static void ShowPoolInspector()
        {
            if (sm_PoolInspector == null)
            {
                sm_PoolInspector = new ExcelPoolInspectorForm();
            }
            sm_PoolInspector.Show();
        }
        #endregion

        #endregion

        #region private static methods

        /// <summary>Serves as method which will be called in a case that an exception is thrown.
        /// </summary>
        /// <param name="exceptionObject">The exception object.</param>
        /// <returns>The <see cref="System.String"/> representation of the exception.</returns>
        private static object UnhandledExceptionHandler(object exceptionObject)
        {
            try
            {
                if (exceptionObject != null)
                {
                    return ExcelDataConverter.GetExcelRangeErrorMessage(exceptionObject.ToString());
                }
                return ExcelDataConverter.GetExcelRangeErrorMessage("Unknown error.");
            }
            catch (Exception exception)
            {
                return exception.Message + ((exceptionObject != null) ? exceptionObject.ToString() : String.Empty);
            }
        }
        #endregion
    }
}