﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.XLBasicComponents.Utilities;

namespace Dodoni.XLBasicComponents
{
    /// <summary>Represents the type of a specific <see cref="ExcelPoolItem"/>, for example a (interest rate) curve etc.
    /// </summary>
    /// <remarks>A mapping beween a collection of <see cref="IExcelDataQuery"/> to a specific object is needed. The type of the object
    /// is described by this class, i.e. a <see cref="IdentifierString"/> representation as well as some <see cref="System.Guid"/>. Both representations must
    /// be unique.</remarks>
    public class ExcelPoolItemType : IIdentifierNameable
    {
        #region public (readonly) members

        /// <summary>The unique identifier of the current instance.
        /// </summary>
        public readonly Guid Identifier;

        /// <summary>The collection of (sub-) categories, for example 'MathLib', 'optimization' etc.; <c>null</c> if no (sub-) category available.
        /// </summary>
        public readonly IEnumerable<IdentifierString> SubCategories = null;
        #endregion

        #region private members

        /// <summary>The identifier string representation of the current instance.
        /// </summary>
        private IdentifierString m_Name;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="ExcelPoolItemType"/> class.
        /// </summary>
        /// <param name="name">The (unique) name.</param>
        /// <param name="identifier">The (unique) identifier.</param>
        public ExcelPoolItemType(string name, Guid identifier)
        {
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            m_Name = new IdentifierString(name);
            Identifier = identifier;
        }

        /// <summary>Initializes a new instance of the <see cref="ExcelPoolItemType"/> class.
        /// </summary>
        /// <param name="name">The (unique) name.</param>
        /// <param name="identifier">The (unique) identifier.</param>
        /// <param name="category">The category of the <see cref="ExcelPoolItemType"/>, for example 'MathLib', 'Finance' etc.</param>
        public ExcelPoolItemType(string name, Guid identifier, string category)
            : this(name, identifier)
        {
            if (category != null)
            {
                SubCategories = new List<IdentifierString>() { new IdentifierString(category) };
            }
        }

        /// <summary>Initializes a new instance of the <see cref="ExcelPoolItemType"/> class.
        /// </summary>
        /// <param name="name">The (unique) name.</param>
        /// <param name="identifier">The (unique) identifier.</param>
        /// <param name="category">The category of the <see cref="ExcelPoolItemType"/>, for example 'MathLib', 'Finance' etc.</param>
        /// <param name="subCategory">The sub-category of the <see cref="ExcelPoolItemType"/>, for example 'Basic', 'Optimization' etc.</param>
        public ExcelPoolItemType(string name, Guid identifier, string category, string subCategory)
            : this(name, identifier)
        {
            if (category != null)
            {
                if (subCategory == null)
                {
                    SubCategories = new List<IdentifierString>() { new IdentifierString(category) };
                }
                else
                {
                    SubCategories = new List<IdentifierString>() { new IdentifierString(category), new IdentifierString(subCategory) };
                }
            }
        }

        /// <summary>Initializes a new instance of the <see cref="ExcelPoolItemType"/> class.
        /// </summary>
        /// <param name="name">The (unique) name.</param>
        /// <param name="identifier">The (unique) identifier.</param>
        /// <param name="subCategories">The sub-categories of the <see cref="ExcelPoolItemType"/>, for example 'MathLib', 'Optimizer', 'N-dim' etc.</param>
        public ExcelPoolItemType(string name, Guid identifier, params string[] subCategories)
            : this(name, identifier)
        {
            if (subCategories != null)
            {
                List<IdentifierString> subCategoryList = new List<IdentifierString>();
                foreach (string subCategory in subCategories)
                {
                    subCategoryList.Add(new IdentifierString(subCategory));
                }
                SubCategories = subCategoryList;
            }
        }
        #endregion

        #region public properties

        #region IIdentifierNameable Members

        /// <summary>Gets the name of the current instance.
        /// </summary>
        /// <value>The language independent name of the current instance.</value>
        public IdentifierString Name
        {
            get { return m_Name; }
        }

        /// <summary>Gets the long name of the current instance.
        /// </summary>
        /// <value>The language dependent long name of the current instance.</value>
        public IdentifierString LongName
        {
            get { return m_Name; }
        }
        #endregion

        #endregion
    }
}