﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Linq;
using System.ComponentModel;
using System.Collections.Generic;

using Dodoni.XLBasicComponents.LogFile;

using Dodoni.BasicComponents.Utilities;
using Dodoni.BasicComponents.LogFileFactory;
using Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes;

namespace Dodoni.XLBasicComponents  // not .LogFile, because it is public, all other are internal!
{
    /// <summary>Represents a (local) logfile implementation.
    /// </summary>
    public class ExcelLogFile : ILogFile
    {
        #region private members

        /// <summary>A reference to the global logfile.
        /// </summary>
        private ExcelLogFileGenerator m_GlobalLogFile;

        /// <summary>The list of messages.
        /// </summary>
        private List<LogFileRow> m_ListOfMessages;

        /// <summary>The (unique) object type ID.
        /// </summary>
        private Guid m_ObjectTypeID;

        /// <summary>The name of the object type.
        /// </summary>
        private string m_ObjectTypeName;

        /// <summary>The unique object ID.
        /// </summary>
        private Guid m_ObjectID;

        /// <summary>The name of the object.
        /// </summary>
        private string m_ObjectName;

        /// <summary>The output type, i.e. a value indicating whether to write into the global/local logfile.
        /// </summary>
        private LogFileOutputType m_OutputType;
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="ExcelLogFile"/> class.
        /// </summary>
        /// <param name="globalLogFile">A reference to the global logfile.</param>
        /// <param name="objectName">The name of the object.</param>
        /// <param name="logFileOutputType">A value indicating whether to write into the global/local logfile.</param>
        /// <param name="objectTypeID">The unique object type ID.</param>
        /// <param name="objectTypeName">The name of the object type.</param>
        internal ExcelLogFile(ExcelLogFileGenerator globalLogFile, string objectName, LogFileOutputType logFileOutputType, Guid objectTypeID, string objectTypeName)
        {
            m_GlobalLogFile = globalLogFile;
            m_ObjectName = objectName;
            m_OutputType = logFileOutputType;
            m_ObjectTypeID = objectTypeID;
            m_ObjectTypeName = objectTypeName;
            m_ObjectID = Guid.NewGuid();

            m_ListOfMessages = new List<LogFileRow>();
        }
        #endregion

        #region public properties

        #region ILogFile Members

        /// <summary>Gets the type of the logfile output.
        /// </summary>
        /// <value>The type of the logfile output.</value>
        public LogFileOutputType LogFileOutputType
        {
            get { return m_OutputType; }
        }

        /// <summary>Gets the unique object type ID.
        /// </summary>
        /// <value>The unique object type ID.</value>
        public Guid ObjectTypeID
        {
            get { return m_ObjectTypeID; }
        }
        #endregion

        /// <summary>Gets the object name.
        /// </summary>
        /// <value>The object name.</value>
        public string ObjectName
        {
            get { return m_ObjectName; }
        }

        /// <summary>Gets the object ID.
        /// </summary>
        /// <value>The object ID.</value>
        public System.Guid ObjectID
        {
            get { return m_ObjectID; }
        }
        #endregion

        #region public methods

        #region ILogFile Members

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        public void AddMessage(InfoLogFileMessageType messageType, string message)
        {
            AddMessage((ILogFileMessageType)messageType, message);
        }

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        public void AddMessage(InfoLogFileMessageType messageType)
        {
            AddMessage((ILogFileMessageType)messageType);
        }

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        public void AddMessage(WarningLogFileMessageType messageType, string message)
        {
            AddMessage((ILogFileMessageType)messageType, message);
        }

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        public void AddMessage(WarningLogFileMessageType messageType)
        {
            AddMessage((ILogFileMessageType)messageType);
        }

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        public void AddMessage(ErrorLogFileMessageType messageType, string message)
        {
            AddMessage((ILogFileMessageType)messageType, message);
        }

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        public void AddMessage(ErrorLogFileMessageType messageType)
        {
            AddMessage((ILogFileMessageType)messageType);
        }

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        public void AddMessage(FatalErrorLogFileMessageType messageType, string message)
        {
            AddMessage((ILogFileMessageType)messageType, message);
        }

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        public void AddMessage(FatalErrorLogFileMessageType messageType)
        {
            AddMessage((ILogFileMessageType)messageType);
        }

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        public void AddMessage(ILogFileMessageType messageType, string message)
        {
            if (m_OutputType != LogFileOutputType.None)
            {
                if (m_OutputType.HasFlag(LogFileOutputType.WriteIntoGlobalLogFile))
                {
                    m_GlobalLogFile.AddGlobal(messageType, message, m_ObjectTypeID, m_ObjectID, m_ObjectName);
                }
                if (m_OutputType.HasFlag(LogFileOutputType.WriteIntoLocalLogFile))
                {
                    m_ListOfMessages.Add(new LogFileRow(messageType, message));
                }
            }
        }

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        public void AddMessage(ILogFileMessageType messageType)
        {
            if (m_OutputType != LogFileOutputType.None)
            {
                if (m_OutputType.HasFlag(LogFileOutputType.WriteIntoGlobalLogFile))
                {
                    m_GlobalLogFile.AddGlobal(messageType, String.Empty, m_ObjectTypeID, m_ObjectID, m_ObjectName);
                }
                if (m_OutputType.HasFlag(LogFileOutputType.WriteIntoLocalLogFile))
                {
                    m_ListOfMessages.Add(new LogFileRow(messageType, String.Empty));
                }
            }
        }
        #endregion

        /// <summary>Gets the logfile in some <see cref="System.String"/> representation.
        /// </summary>
        /// <returns>The logfile in its <see cref="System.String"/> representation.</returns>
        public string GetAsString()
        {
            StringBuilder stringBuilder = new StringBuilder();
            for (int i = 0; i < m_ListOfMessages.Count; i++)
            {
                LogFileRow row = m_ListOfMessages[i];

                stringBuilder.AppendLine(row.TimeStamp + ": <" + row.MessageTypeClassification.ToFormatString() + ": " + row.MessageTypeName + "> [" + m_ObjectName + "; " + m_ObjectTypeName + "] " + row.Message);
            }
            return stringBuilder.ToString();
        }

        /// <summary>Gets the logfile in some <see cref="System.String"/> representation.
        /// </summary>
        /// <param name="messageTypes">The message types to take into account.</param>
        /// <returns>The <see cref="System.String"/> representation of the global logfile, where the first index corresponds to the null-based
        /// row index and the second index corresponds to the type, i.e. date, message type, object name, object type and logfile message.</returns>
        /// <remarks>The output contains some extra row, which represents the header.</remarks>
        public string[][] GetAsStringArray(params LogFileMessageTypeClassification[] messageTypes)
        {
            int filterFlags = 0x00;

            if ((messageTypes != null) && (messageTypes.Length > 0))
            {
                foreach (LogFileMessageTypeClassification logFileMessageType in messageTypes)
                {
                    filterFlags |= (int)logFileMessageType;
                }
            }
            else
            {
                foreach (LogFileMessageTypeClassification logFileMessageType in Enum.GetValues(typeof(LogFileMessageTypeClassification)))
                {
                    filterFlags |= (int)logFileMessageType;
                }
            }
            return GetAsStringArray(filterFlags);
        }
        #endregion

        #region internal methods

        /// <summary>Gets the current instance as some binding list.
        /// </summary>
        /// <returns>The BindingList representation.</returns>
        internal BindingList<LogFileRow> GetAsBindingList()
        {
            return new BindingList<LogFileRow>(m_ListOfMessages);
        }
        #endregion

        #region private methods

        /// <summary>Gets the logfile in some <see cref="System.String"/> representation.
        /// </summary>
        /// <param name="filterFlags">The message types to take into account.</param>
        /// <returns>The <see cref="System.String"/> representation of the global logfile, where the first index corresponds to the null-based
        /// row index and the second index corresponds to the type, i.e. date, message type, object name, object type and logfile message.</returns>
        /// <remarks>The output contains some extra row, which represents the header.</remarks>
        private string[][] GetAsStringArray(int filterFlags)
        {
            List<string[]> messageList = new List<string[]>();

            string[] header = { XLResources.LogFileTimeHeader, XLResources.LogFileClassificationHeader, XLResources.LogFileMessageTypeHeader, XLResources.LogFileObjectNameHeader, XLResources.LogFileObjectTypeHeader, XLResources.LogFileMessageHeader };
            messageList.Add(header);

            for (int i = 0; i < m_ListOfMessages.Count; i++)
            {
                LogFileRow row = m_ListOfMessages[i];
                if (((int)row.MessageTypeClassification & filterFlags) != 0)
                {
                    messageList.Add(new string[] { row.TimeStamp, row.MessageTypeClassification.ToFormatString(), row.MessageTypeName, m_ObjectName, m_ObjectTypeName, row.Message });
                }
            }
            return messageList.ToArray();
        }
        #endregion
    }
}