﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Linq;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Utilities;
using Dodoni.BasicComponents.LogFileFactory;
using Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes;

namespace Dodoni.XLBasicComponents.LogFile
{
    /// <summary>Represents the log file generator, i.e. the representation of Log files with respect to the
    /// Excel interface. Moreover the global logfile will be stored in this class as well.
    /// </summary>
    internal class ExcelLogFileGenerator : ILogFileGenerator
    {
        #region private static members

        /// <summary>The <see cref="System.String"/> representation of the unknown object type.
        /// </summary>
        private const string UnknownObjectTypeName = "[unknown]";

        /// <summary>The <see cref="System.Guid"/> instance which is used for unknown object ID's.
        /// </summary>
        private static readonly System.Guid UnknownObjectIDGuid = Guid.Empty;

        /// <summary>The <see cref="System.String"/> representation of the unknown object name.
        /// </summary>
        private const string UnknownObjectName = "[unknown]";
        #endregion

        #region private members

        /// <summary>The mapping between <see cref="System.Guid"/> objects and the name of the
        /// corresponding object type (for example the Holiday calendar pool, Date schedule, a forward curve etc.).
        /// </summary>
        private Dictionary<System.Guid, string> m_ObjectTypeMapping;

        /// <summary>The global logfile as as a list of <see cref="GlobalLogFileRow"/> objects.
        /// </summary>
        /// <remarks>We will use a Windows form therfore, we use a BindingList instead of a List.</remarks>
        private BindingList<GlobalLogFileRow> m_GlobalLogFile;

        /// <summary>The standard <see cref="LogFileOutputType"/> used to create <see cref="ILogFile"/> objects.
        /// </summary>
        private LogFileOutputType m_StandardLogFileOutputType = LogFileOutputType.WriteIntoLocalLogFile;

        /// <summary>The XML tag in the configuration file for the standard logfile output type.
        /// </summary>
        private const string m_StandardLogFileOutputTypeConfigKey = "StandardLogFileOutputType";

        /// <summary>A value indicating whether the global log file message form will be pop-up if an (fatal) error occur.
        /// </summary>
        private bool m_PopupGlobalLogfileBoxOnError = false;

        /// <summary>The XML tag in the configuration file for the 'popup global logfile Windows form on error' checkbox.
        /// </summary>
        private const string m_PopupGlobalLogfileWindowsFormConfigKey = "PopupGlobalLogfileBoxOnError";
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="ExcelLogFileGenerator"/> class.
        /// </summary>
        public ExcelLogFileGenerator()
        {
            m_ObjectTypeMapping = new Dictionary<Guid, string>();
            m_ObjectTypeMapping.Add(LoggingFactory.UnknownObjectTypeGuid, UnknownObjectTypeName);

            m_GlobalLogFile = new BindingList<GlobalLogFileRow>();
            m_PopupGlobalLogfileBoxOnError = GetPopupGlobalLogfileBoxOnErrorFromConfigFile();
            m_StandardLogFileOutputType = GetStandardLogFileOutputTypeFromConfigFile();
        }
        #endregion

        #region public/internal properties

        #region ILogFileGenerator Members

        /// <summary>Gets a value indicating whether the log file generator is already in use.
        /// </summary>
        /// <value><c>true</c> if at least one <see cref="ILogFile"/> object has been created; otherwise, <c>false</c>.
        /// </value>
        /// <remarks>This flag will be take into account to thrown an exception if at least one <see cref="ILogFile"/> object has been created.</remarks>
        public bool IsInUse
        {
            get { return true; }
        }

        /// <summary>Gets the standard <see cref="LogFileOutputType"/> used for creating <see cref="ILogFile"/> objects.
        /// </summary>
        /// <value>The standard log file output type.</value>
        public LogFileOutputType StandardLogFileOutputType
        {
            get { return m_StandardLogFileOutputType; }
        }
        #endregion

        /// <summary>Gets the global logfile as a BindingList of <see cref="GlobalLogFileRow"/> instances.
        /// </summary>
        /// <value>The global logfile.</value>
        internal BindingList<GlobalLogFileRow> GlobalLogFile
        {
            get { return m_GlobalLogFile; }
        }
        #endregion

        #region public methods

        #region ILogFileGenerator Members

        /// <summary>Gets a new <see cref="ILogFile"/> object.
        /// </summary>
        /// <param name="objectName">The object name.</param>
        /// <param name="objectTypeName">The name of the object type.</param>
        /// <param name="objectTypeID">The unique object type ID.</param>
        /// <param name="logFileOutputType">The type of the logfile output.</param>
        /// <returns>A new <see cref="ILogFile"/> instance with respect to the <paramref name="logFileOutputType"/>.
        /// </returns>
        /// <remarks>Fatal error messages must be written into a global logfile (if available), even if the logfile is some 'local' logfile.</remarks>
        public ILogFile CreateLogFile(string objectName, string objectTypeName, Guid objectTypeID, LogFileOutputType logFileOutputType)
        {
            if (m_ObjectTypeMapping.ContainsKey(objectTypeID) == false)
            {
                m_ObjectTypeMapping.Add(objectTypeID, objectTypeName);
            }
            return new ExcelLogFile(this, objectName, logFileOutputType, objectTypeID, objectTypeName);
        }

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public void AddGlobalLogFileMessage(WarningLogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            AddGlobal(messageType, message, loggingObject);
        }

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public void AddGlobalLogFileMessage(InfoLogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            AddGlobal(messageType, message, loggingObject);
        }

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public void AddGlobalLogFileMessage(ErrorLogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            AddGlobal(messageType, message, loggingObject);

            if (m_PopupGlobalLogfileBoxOnError == true)
            {
                ExcelAddIn.ShowGlobalLogFile();
            }
        }

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public void AddGlobalLogFileMessage(FatalErrorLogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            AddGlobal(messageType, message, loggingObject);
            if (m_PopupGlobalLogfileBoxOnError == true)
            {
                ExcelAddIn.ShowGlobalLogFile();
            }
        }

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        public void AddGlobalLogFileMessage(ILogFileMessageType messageType, string message, ILogging loggingObject = null)
        {
            AddGlobal(messageType, message, loggingObject);
            if (m_PopupGlobalLogfileBoxOnError == true)
            {
                ExcelAddIn.ShowGlobalLogFile();
            }
        }
        #endregion

        #endregion

        #region internal methods

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="objectTypeID">The unique object type ID.</param>
        /// <param name="objectID">The object ID.</param>
        /// <param name="objectName">The name of the object.</param>
        internal void AddGlobal(ILogFileMessageType messageType, string message, Guid objectTypeID, Guid objectID, string objectName)
        {
            string objectTypeName;
            if (m_ObjectTypeMapping.TryGetValue(objectTypeID, out objectTypeName) == false)
            {
                throw new InvalidOperationException("Object type '" + objectTypeID.ToString() + "' has not been registed in the logfile.");
            }
            m_GlobalLogFile.Add(new GlobalLogFileRow(messageType, message, objectTypeID, objectTypeName, objectID, objectName));
        }
        #endregion

        #region internal static methods

        /// <summary>Stores a flag in the config file indicating whether the global logfile Windows form will be pop-up if an (fatal) error occurs.
        /// </summary>
        /// <param name="state">If set to <c>true</c> the global logfile Windows form will be pop-up if an (fatal) error occurs.</param>
        internal static void StorePopupGlobalLogfileBoxOnError(bool state)
        {
            ExcelAddIn.Configuration.GeneralSettings.SetValue(m_PopupGlobalLogfileWindowsFormConfigKey, state);
        }

        /// <summary>Gets a flag from the config file indicating whether the global logfile Windows form will be pop-up if an (fatal) error occurs.
        /// </summary>
        /// <returns>The value of the flag in the config file or some standard value.</returns>
        internal static bool GetPopupGlobalLogfileBoxOnErrorFromConfigFile()
        {
            bool popupGlobalLogfileBoxOnError;

            ExcelAddIn.Configuration.GeneralSettings.TryGetValue(m_PopupGlobalLogfileWindowsFormConfigKey, out popupGlobalLogfileBoxOnError, defaultValue: true);
            return popupGlobalLogfileBoxOnError;
        }

        /// <summary>Stores a <see cref="LogFileOutputType"/> object into the config file which will be use as <see cref="ExcelLogFileGenerator.StandardLogFileOutputType"/> after restarting Dodoni.NET.
        /// </summary>
        /// <param name="standardLogFileOutputType">The standard logfile output type.</param>
        internal static void StoreStandardLogFileOutputType(LogFileOutputType standardLogFileOutputType)
        {
            ExcelAddIn.Configuration.GeneralSettings.SetValue(m_StandardLogFileOutputTypeConfigKey, standardLogFileOutputType.ToString());
        }

        /// <summary>Gets the standard logfile output type with respect to the config file.
        /// </summary>
        /// <returns>The standard logfile output type with respect to the config file or some standard value.</returns>
        internal static LogFileOutputType GetStandardLogFileOutputTypeFromConfigFile()
        {
            string standardLogFileOutputTypeAsString;
            if (ExcelAddIn.Configuration.GeneralSettings.TryGetValue(m_StandardLogFileOutputTypeConfigKey, out standardLogFileOutputTypeAsString) == false)
            {
                return LogFileOutputType.WriteIntoLocalLogFile;
            }
            LogFileOutputType standardLogFileOutputType;
            if (Enum.TryParse<LogFileOutputType>(standardLogFileOutputTypeAsString, out standardLogFileOutputType) == false)
            {
                throw new ConfigurationFileErrorException("Corrupt configuration file. Delete configuration file and configurate the AddIn again.");
            }
            return standardLogFileOutputType;
        }
        #endregion

        #region private methods

        /// <summary>Adds a specific message into the global logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        /// <param name="loggingObject">The logging object if available, i.e. contains perhaps additional informations as object name etc.</param>
        private void AddGlobal(ILogFileMessageType messageType, string message, ILogging loggingObject)
        {
            if (loggingObject != null)
            {
                ExcelLogFile excelLogFile = loggingObject.LogFile as ExcelLogFile;
                if (excelLogFile != null)
                {
                    string objectTypeName;
                    if (m_ObjectTypeMapping.TryGetValue(excelLogFile.ObjectTypeID, out objectTypeName) == false)
                    {
                        throw new InvalidOperationException("Object type '" + excelLogFile.ObjectTypeID.ToString() + "' has not been registed in the logfile.");
                    }
                    m_GlobalLogFile.Add(new GlobalLogFileRow(messageType, message, excelLogFile.ObjectTypeID, objectTypeName, excelLogFile.ObjectID, excelLogFile.ObjectName));
                    return;
                }
            }
            m_GlobalLogFile.Add(new GlobalLogFileRow(messageType, message, LoggingFactory.UnknownObjectTypeGuid, UnknownObjectTypeName, UnknownObjectIDGuid, UnknownObjectName));
        }
        #endregion
    }
}