﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Linq;
using System.Collections.Generic;

using ExcelDna.Integration;
using Dodoni.XLBasicComponents.LogFile;

using Dodoni.BasicComponents.Utilities;
using Dodoni.BasicComponents.LogFileFactory;
using Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes;

namespace Dodoni.XLBasicComponents.UDF
{
    /// <summary>Serves as Excel interface for logging functionality.
    /// </summary>
    public static class ExcelLogging
    {
        #region private consts

        /// <summary>The Excel category for each exported function in this class.
        /// </summary>
        private const string ExcelCategory = ExcelAddIn.GeneralCategoryName;
        #endregion

        #region public (static) methods

        /// <summary>Gets the logfile message types.
        /// </summary>
        /// <returns>The logfile message types.</returns>
        [ExcelFunction(Name = "doListLogFileMsgTypes", Description = "Gets the logfile message types", Category = ExcelCategory)]
        public static object GetLogFileMessageTypes()
        {
            try
            {
                if (ExcelDnaUtil.IsInFunctionWizard())
                {
                    return String.Empty;
                }
                IEnumerable<ILogFileMessageType> msgTypes = LoggingFactory.GetMessageTypes();
                int count = msgTypes.Count();
                object[,] excelOutput = new object[count + 1, 3];
                excelOutput[0, 0] = XLResources.LogFileClassificationHeader;
                excelOutput[0, 1] = XLResources.LogFileNameHeader;
                excelOutput[0, 2] = XLResources.LogFileDescriptionHeader;

                int i = 1;
                foreach (ILogFileMessageType msgType in msgTypes)
                {
                    excelOutput[i, 0] = msgType.Classification.ToFormatString();
                    excelOutput[i, 1] = msgType.Name.String;
                    excelOutput[i, 2] = msgType.LongName.String;
                    i++;
                }
                return ExcelDataConverter.GetExcelRangeOutput(excelOutput);
            }
            catch (Exception e)
            {
                return ExcelDataConverter.GetExcelRangeErrorMessage(e.Message);
            }
        }

        /// <summary>Gets the (local) logfile of a specific element of the pool.
        /// </summary>
        /// <param name="xlObjectName">The name of the object.</param>
        /// <returns>The (local) logfile of the object with name <paramref name="xlObjectName"/>.</returns>
        [ExcelFunction(Name = "doGetLogFile", Description = "Get the log file of a specific element of the pool.", Category = ExcelCategory)]
        public static object GetLocalLogFile(
            [ExcelArgument(Name = "objectName", Description = "The name of the object", AllowReference = true)]
            object xlObjectName)
        {
            try
            {
                IExcelDataQuery objectNameQuery = ExcelDataQuery.Create(xlObjectName);
                string objectName;
                if (objectNameQuery.TryGetValue<string>(out objectName, dataAdvice: ExcelDataAdvice.Create(ExcelPool.GetObjectNames())) != ExcelCellValueState.ProperValue)
                {
                    return ExcelDataConverter.GetExcelRangeErrorMessage("Invalid object name '" + objectNameQuery.ToString(0, 0) + "'.");
                }
                objectNameQuery.QueryCompleted();

                ILogging loggingObject;
                if (ExcelPool.TryGetObject<ILogging>(objectName, out loggingObject) == false)
                {
                    return ExcelDataConverter.GetExcelRangeErrorMessage("No object found of name '" + objectName + "' which supports logging functionality.");
                }
                ExcelLogFile excelLogFile = loggingObject.LogFile as ExcelLogFile;
                if (excelLogFile == null)
                {
                    return ExcelDataConverter.GetExcelRangeErrorMessage("No log file available.");
                }
                return ExcelDataConverter.GetExcelRangeOutput<string>(excelLogFile.GetAsStringArray());
            }
            catch (Exception e)
            {
                return ExcelDataConverter.GetExcelRangeErrorMessage(e.Message);
            }
        }
        #endregion
    }
}