﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.IO;
using System.Text;
using System.Linq;
using System.Collections.Generic;

using ExcelDna.Integration;
using Dodoni.BasicComponents;
using Dodoni.XLBasicComponents.IO;
using Dodoni.XLBasicComponents.Utilities;

namespace Dodoni.XLBasicComponents.UDF
{
    /// <summary>
    /// Excel functions for loading objects from a specific file and saving objects
    /// from the <see cref="ExcelPool"/> into a file with a specific format.
    /// </summary>
    public static class ObjectFactory
    {
        #region private consts

        /// <summary>The Excel category for each exported function in this class.
        /// </summary>
        private const string ExcelCategory = ExcelAddIn.GeneralCategoryName;
        #endregion

        #region public static methods

        /// <summary>Loads objects from a specific file.
        /// </summary>
        /// <param name="xlPath">The path.</param>
        /// <param name="xlFileName">The name of the file.</param>
        /// <param name="xlObjectNames">The (optional) object names to load.</param>
        /// <param name="xlFileFormat">The (optional) file format; the file extension will be used by default.</param>
        /// <returns></returns>
        [ExcelFunction(Name = "doLoadObjects", Description = "Load objects form a specific file", Category = ExcelCategory)]
        public static object LoadObjectsFromFile(
            [ExcelArgument(Name = "path", Description = "The path", AllowReference = true)]
            object xlPath,
            [ExcelArgument(Name = "fileName", Description = "The file name", AllowReference = true)]
            object xlFileName,
            [ExcelArgument(Name = "objectNames", Description = "[Optional] A list of object names to load; all objects will be loaded by default", AllowReference = true)]
            object xlObjectNames,
            [ExcelArgument(Name = "fileFormat", Description = "[Optional] The file format; the file extension will be used by default", AllowReference = true)]
            object xlFileFormat)
        {
            try
            {
                IExcelDataQuery fileFormatDataQuery = ExcelDataQuery.Create(xlFileFormat, "File format");

                string fileName;
                IObjectStreamer objectStreamer;
                if (fileFormatDataQuery.IsEmpty == true)
                {
                    fileName = GetFileName(xlPath, xlFileName, ObjectStreamerPool.GetFileExtensions());

                    if (ObjectStreamerPool.TryGetObjectStreamerByFileExtension(ExtendedPath.GetExtension(fileName), out objectStreamer) == false)
                    {
                        throw new ArgumentException("Invalid file extension '" + Path.GetExtension(fileName) + "', used default file extensions or specify the file format.");
                    }
                }
                else
                {
                    if (fileFormatDataQuery.TryGetPoolValue<IObjectStreamer>(ObjectStreamerPool.TryGetObjectStreamer, out objectStreamer, dataAdvice: ExcelDataAdvice.Create(ObjectStreamerPool.GetNames())) == false)
                    {
                        throw new ArgumentException("Invalid file format " + fileFormatDataQuery.ToString(0, 0) + ".");
                    }
                    fileName = GetFileName(xlPath, xlFileName, objectStreamer.FileExtension);
                    fileFormatDataQuery.QueryCompleted();
                }

                IExcelDataQuery objectNamesDataQuery = ExcelDataQuery.Create(xlObjectNames, "Object names");
                IEnumerable<string> objectNames = objectNamesDataQuery.GetColumnVector<string>();
                objectNamesDataQuery.QueryCompleted();

                StreamReader streamReader = new StreamReader(fileName);
                IObjectStreamReader objectStreamReader = objectStreamer.GetStreamReader(streamReader);
                string infoMessage;
                IEnumerable<ExcelPoolItem> excelPoolItems;
                ExcelPool.TryLoadObjectsByName(objectStreamReader, objectNames, out infoMessage, out excelPoolItems);
                objectStreamReader.Close();

                return infoMessage.ToTimeStampString();
            }
            catch (Exception e)
            {
                return ExcelDataConverter.GetExcelRangeErrorMessage(e);
            }
        }

        /// <summary>List objects in a specific file.
        /// </summary>
        /// <param name="xlPath">The path.</param>
        /// <param name="xlFileName">The name of the file.</param>
        /// <param name="xlFileFormat">The (optional) file format; the file extension will be used by default.</param>
        /// <returns></returns>
        [ExcelFunction(Name = "doListObjects", Description = "List objects in a specific file", Category = ExcelCategory)]
        public static object ListObjectsFromFile(
            [ExcelArgument(Name = "path", Description = "The path", AllowReference = true)]
            object xlPath,
            [ExcelArgument(Name = "fileName", Description = "The file name", AllowReference = true)]
            object xlFileName,
            [ExcelArgument(Name = "fileFormat", Description = "[Optional] The file format; the file extension will be used by default", AllowReference = true)]
            object xlFileFormat)
        {
            try
            {
                IExcelDataQuery fileFormatDataQuery = ExcelDataQuery.Create(xlFileFormat, "File format");

                string fileName;
                IObjectStreamer objectStreamer;

                if (fileFormatDataQuery.IsEmpty == true)
                {
                    fileName = GetFileName(xlPath, xlFileName, ObjectStreamerPool.GetFileExtensions());

                    if (ObjectStreamerPool.TryGetObjectStreamerByFileExtension(ExtendedPath.GetExtension(fileName), out objectStreamer) == false)
                    {
                        throw new ArgumentException("Invalid file extension '" + ExtendedPath.GetExtension(fileName) + "', used default file extensions or specify the file format.");
                    }
                }
                else
                {
                    if (fileFormatDataQuery.TryGetPoolValue<IObjectStreamer>(ObjectStreamerPool.TryGetObjectStreamer, out objectStreamer, dataAdvice: ExcelDataAdvice.Create(ObjectStreamerPool.GetNames())) == false)
                    {
                        throw new ArgumentException("Invalid file format " + fileFormatDataQuery.ToString(0, 0) + ".");
                    }
                    fileName = GetFileName(xlPath, xlFileName, objectStreamer.FileExtension);
                    fileFormatDataQuery.QueryCompleted();
                }

                StreamReader streamReader = new StreamReader(fileName);
                IObjectStreamReader objectStreamReader = objectStreamer.GetStreamReader(streamReader);

                var objectNames = ExcelPool.GetObjectNames(objectStreamReader);
                int n = objectNames.Count();
                object[,] output = new object[n + 1, 2];
                output[0, 0] = "Name";
                output[0, 1] = "Type";
                int j = 1;
                foreach (var obj in objectNames)
                {
                    output[j, 0] = obj.Item2;
                    output[j, 1] = obj.Item1.Name.String;
                    j++;
                }
                objectStreamReader.Close();
                return ExcelDataConverter.GetExcelRangeOutput(output);
            }
            catch (Exception e)
            {
                return ExcelDataConverter.GetExcelRangeErrorMessage(e);
            }
        }

        /// <summary>Write objects into a specific file.
        /// </summary>
        /// <param name="xlPath">The path.</param>
        /// <param name="xlFileName">The name of the file.</param>
        /// <param name="xlObjectNames">The (optional) object names to load.</param>
        /// <param name="xlFileFormat">The (optional) file format; the file extension will be used by default.</param>
        /// <returns></returns>
        [ExcelFunction(Name = "doWriteObjects", Description = "Write objects into a specific file", Category = ExcelCategory)]
        public static object WriteObjectsIntoFile(
            [ExcelArgument(Name = "path", Description = "The path", AllowReference = true)]
            object xlPath,
            [ExcelArgument(Name = "fileName", Description = "The file name", AllowReference = true)]
            object xlFileName,
            [ExcelArgument(Name = "objectNames", Description = "A [optional] list of object names to write.", AllowReference = true)]
            object xlObjectNames,
            [ExcelArgument(Name = "fileFormat", Description = "[Optional] The file format; the file extension will be used by default", AllowReference = true)]
            object xlFileFormat)
        {
            try
            {
                IExcelDataQuery fileFormatDataQuery = ExcelDataQuery.Create(xlFileFormat, "File format");

                string fileName;
                IObjectStreamer objectStreamer;

                if (fileFormatDataQuery.IsEmpty == true)
                {
                    fileName = GetFileName(xlPath, xlFileName, ObjectStreamerPool.GetFileExtensions());

                    if (ObjectStreamerPool.TryGetObjectStreamerByFileExtension(ExtendedPath.GetExtension(fileName), out objectStreamer) == false)
                    {
                        throw new ArgumentException("Invalid file extension '" + ExtendedPath.GetExtension(fileName) + "', used default file extensions or specify the file format.");
                    }
                }
                else
                {
                    if (fileFormatDataQuery.TryGetPoolValue<IObjectStreamer>(ObjectStreamerPool.TryGetObjectStreamer, out objectStreamer, dataAdvice: ExcelDataAdvice.Create(ObjectStreamerPool.GetNames())) == false)
                    {
                        throw new ArgumentException("Invalid file format " + fileFormatDataQuery.ToString(0, 0) + ".");
                    }
                    fileName = GetFileName(xlPath, xlFileName, objectStreamer.FileExtension);
                    fileFormatDataQuery.QueryCompleted();
                }

                IExcelDataQuery objectNamesDataQuery = ExcelDataQuery.Create(xlObjectNames);
                IEnumerable<string> objectNames = objectNamesDataQuery.GetColumnVector<string>();
                objectNamesDataQuery.QueryCompleted();

                StreamWriter streamWriter = new StreamWriter(fileName, false);
                IObjectStreamWriter objectStreamWriter = objectStreamer.GetStreamWriter(streamWriter);
                string infoMessage;
                ExcelPool.TryWriteObjectsByName(objectStreamWriter, objectNames, out infoMessage);
                objectStreamWriter.Close();

                return infoMessage.ToTimeStampString();
            }
            catch (Exception e)
            {
                return ExcelDataConverter.GetExcelRangeErrorMessage(e.Message);
            }
        }
        #endregion

        #region private methods

        /// <summary>Gets the name of the file.
        /// </summary>
        /// <param name="xlPath">The path.</param>
        /// <param name="xlFileName">The name of the file (without path).</param>
        /// <param name="fileExtensionSearchPattern">The file extension to search.</param>
        /// <returns>The file name including path.</returns>
        private static string GetFileName(object xlPath, object xlFileName, IdentifierString fileExtensionSearchPattern)
        {
            IExcelDataQuery pathDataQuery = ExcelDataQuery.Create(xlPath);
            IExcelDataQuery fileNameDataQuery = ExcelDataQuery.Create(xlFileName);

            string path;
            if (pathDataQuery.TryGetValue<string>(out path) != ExcelCellValueState.ProperValue)
            {
                throw new ArgumentException("No valid path!");
            }
            string[] possibleFileNames = Directory.GetFiles(path, "*." + fileExtensionSearchPattern.String, SearchOption.TopDirectoryOnly);

            string fileName;
            if (fileNameDataQuery.TryGetValue<string>(out fileName, dataAdvice: ExcelDataAdvice.Create(possibleFileNames, possibleFileName => Path.GetFileName(possibleFileName))) != ExcelCellValueState.ProperValue)
            {
                throw new ArgumentException("No valid file name!");
            }
            return Path.Combine(path, fileName);
        }

        /// <summary>Gets the name of the file.
        /// </summary>
        /// <param name="xlPath">The path.</param>
        /// <param name="xlFileName">The name of the file (without path).</param>
        /// <param name="fileExtensionSearchPattern">A collection of possible file extensions.</param>
        /// <returns>The file name including path.</returns>
        private static string GetFileName(object xlPath, object xlFileName, IEnumerable<IdentifierString> fileExtensionSearchPattern)
        {
            IExcelDataQuery pathDataQuery = ExcelDataQuery.Create(xlPath);
            IExcelDataQuery fileNameDataQuery = ExcelDataQuery.Create(xlFileName);

            string path;
            if (pathDataQuery.TryGetValue<string>(out path) != ExcelCellValueState.ProperValue)
            {
                throw new ArgumentException("No valid path!");
            }

            List<string> possibleFileNameList = new List<string>();
            foreach (var fileExtension in fileExtensionSearchPattern)
            {
                string[] possibleFileNames = Directory.GetFiles(path, "*." + fileExtension.String, SearchOption.TopDirectoryOnly);
                if ((possibleFileNames != null) && (possibleFileNames.Length > 0))
                {
                    possibleFileNameList.AddRange(possibleFileNames);
                }
            }

            string fileName;
            if (fileNameDataQuery.TryGetValue<string>(out fileName, dataAdvice: ExcelDataAdvice.Create(possibleFileNameList, possibleFileName => Path.GetFileName(possibleFileName))) != ExcelCellValueState.ProperValue)
            {
                throw new ArgumentException("No valid file name!");
            }
            return Path.Combine(path, fileName);
        }
        #endregion
    }
}