﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Utilities;
using Dodoni.BasicComponents.Containers;

using ExcelDna.Integration;
using Dodoni.XLBasicComponents;
using Dodoni.XLIntegration.Utilities;
using Dodoni.XLBasicComponents.Utilities;

namespace Dodoni.XLIntegration.UDF
{
    /// <summary>Represents user defined functions (UDF) used to export diagrams, graphs etc. via gnuplot.
    /// </summary>
    /// <remarks>There is a limitation in the number of arguments for a UDF which depends on the Excel release, for example
    /// 32 arguments in Excel 2003.</remarks>
    public static class PlottingFactory
    {
        #region private consts

        /// <summary>The Excel category for each exported function in this class.
        /// </summary>
        private const string ExcelCategory = GnuPlotExcelAddIn.UtilityCategoryName;
        #endregion

        #region public (static) methods

        /// <summary>Exports a two-dimensional plot with respect to gnuplot.
        /// </summary>
        /// <param name="xlGeneralPropertyNames">A Excel Range (one row or one column) which contains the names of general properties, as 'name' etc.</param>
        /// <param name="xlGeneralPropertyValues">A Excel Range (in the same format as <paramref name="xlGeneralPropertyNames"/>) which contains the values of general properties.</param>
        /// <param name="xlInitCommand">A Excel Range which contains initial gnuplot commands, as for example output size, color setting etc.</param>
        /// <param name="xlXValues0">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues0">A Excel Range which contains y-values.</param>
        /// <param name="xlData0Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlYValues1">A Excel Range which contains y-values.</param>
        /// <param name="xlData1Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlYValues2">A Excel Range which contains y-values.</param>
        /// <param name="xlData2Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlYValues3">A Excel Range which contains y-values.</param>
        /// <param name="xlData3Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlYValues4">A Excel Range which contains y-values.</param>
        /// <param name="xlData4Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlYValues5">A Excel Range which contains y-values.</param>
        /// <param name="xlData5Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlYValues6">A Excel Range which contains y-values.</param>
        /// <param name="xlData6Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlYValues7">A Excel Range which contains y-values.</param>
        /// <param name="xlData7Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlYValues8">A Excel Range which contains y-values.</param>
        /// <param name="xlData8Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <returns></returns>
        [ExcelFunction(Name = "doExportPlot2d", Description = "Exports a specific data plot via gnuplot", Category = ExcelCategory)]
        public static object ExportPlot2d(
            [ExcelArgument(Name = "generalPropertyNameRange", Description = "A single row or column Range which contains the names of general properties", AllowReference = true)]
            object xlGeneralPropertyNames,
            [ExcelArgument(Name = "generalPropertyValueRange", Description = "A single row or column Range which contains the values of general properties", AllowReference = true)]            
            object xlGeneralPropertyValues,
            [ExcelArgument(Name = "initCommands", Description = "A vector of gnuplot commands which will be applied before adding data plots", AllowReference = true)]
            object xlInitCommand,
            [ExcelArgument(Name = "x", Description = "xValues", AllowReference = true)]
            object xlXValues0,
            [ExcelArgument(Name = "y0", Description = "yValues", AllowReference = true)]
            object xlYValues0,
            [ExcelArgument(Name = "commands0", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData0Arguments,
            [ExcelArgument(Name = "y1", Description = "yValues", AllowReference = true)]
            object xlYValues1,
            [ExcelArgument(Name = "commands1", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData1Arguments,
            [ExcelArgument(Name = "y2", Description = "yValues", AllowReference = true)]
            object xlYValues2,
            [ExcelArgument(Name = "commands2", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData2Arguments,
            [ExcelArgument(Name = "y3", Description = "yValues", AllowReference = true)]
            object xlYValues3,
            [ExcelArgument(Name = "commands3", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData3Arguments,
            [ExcelArgument(Name = "y4", Description = "yValues", AllowReference = true)]
            object xlYValues4,
            [ExcelArgument(Name = "commands4", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData4Arguments,
            [ExcelArgument(Name = "y5", Description = "yValues", AllowReference = true)]
            object xlYValues5,
            [ExcelArgument(Name = "commands5", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData5Arguments,
            [ExcelArgument(Name = "y6", Description = "yValues", AllowReference = true)]
            object xlYValues6,
            [ExcelArgument(Name = "commands6", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData6Arguments,
            [ExcelArgument(Name = "y7", Description = "yValues", AllowReference = true)]
            object xlYValues7,
            [ExcelArgument(Name = "commands7", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData7Arguments,
            [ExcelArgument(Name = "y8", Description = "yValues", AllowReference = true)]
            object xlYValues8,
            [ExcelArgument(Name = "commands8", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData8Arguments)
        {
            try
            {
                if (ExcelDnaUtil.IsInFunctionWizard() == true)
                {
                    return String.Empty;
                }
                IExcelDataQuery generalPropertyDataQuery = ExcelDataQuery.Create(xlGeneralPropertyNames, xlGeneralPropertyValues);

                string outputPath = generalPropertyDataQuery.GetRequiredPropertyValue<String>("Output path");
                string outputFileName = generalPropertyDataQuery.GetRequiredPropertyValue<String>("File name");
                GnuplotTerminalType terminalType = generalPropertyDataQuery.GetRequiredPropertyValue<GnuplotTerminalType>("Terminal type", EnumStringRepresentationUsage.StringAttribute);

                bool ignoreEmptyYValues = true;
                generalPropertyDataQuery.TryGetOptionalPropertyValue<bool>("Ignore empty Y-values", ref ignoreEmptyYValues, ExcelDataAdvice.Pool.Booleans);

                IGnuplotGridPointLabelingMachine gridPointLabelingMachine = GnuplotGridPointLabel.NoLabel;
                generalPropertyDataQuery.TryGetOptionalPropertyPoolValue<IGnuplotGridPointLabelingMachine>("Label type", GnuplotGridPointLabel.TryGetValue, ref gridPointLabelingMachine, ExcelDataAdvice.Create(GnuplotGridPointLabel.Names));

                IGnuplotDataProcessing xDataProcessing = new DoubleGnuplotDataProcessing();
                IGnuplotDataProcessing yDataProcessing = new DoubleGnuplotDataProcessing();


                generalPropertyDataQuery.QueryCompleted();

                GnuplotApplication gnuPlotApplication = GnuplotApplication.Create();
                gnuPlotApplication.SetTerminal(terminalType);
                gnuPlotApplication.SetWorkingDirectory(outputPath);
                gnuPlotApplication.SetOutput(outputFileName);

                IExcelDataQuery gnuPlotPreparation = ExcelDataQuery.Create(xlInitCommand);
                if (gnuPlotPreparation.IsEmpty == false)
                {
                    for (int j = 0; j < gnuPlotPreparation.RowCount; j++)
                    {
                        string prepString;
                        if (gnuPlotPreparation.TryGetValue<string>(out prepString, j, 0, null) == ExcelCellValueState.ProperValue)
                        {
                            gnuPlotApplication.Evaluate(prepString);
                        }
                    }
                }

                /* read the data for plotting as well as some additional [optional] arguments */

                IExcelDataQuery xDataFirstQuery = ExcelDataQuery.Create(xlXValues0, "x0");
                if (xDataFirstQuery.IsEmpty == true)
                {
                    throw new ArgumentException("No x-values given " + xDataFirstQuery.GetFormatedDataQueryName() + ".");
                }
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues0, "y0"), yDataProcessing, ExcelDataQuery.Create(xlData0Arguments), gridPointLabelingMachine, ignoreEmptyYValues));
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues1, "y1"), yDataProcessing, ExcelDataQuery.Create(xlData1Arguments), gridPointLabelingMachine, ignoreEmptyYValues));
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues2, "y2"), yDataProcessing, ExcelDataQuery.Create(xlData2Arguments), gridPointLabelingMachine, ignoreEmptyYValues));
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues3, "y3"), yDataProcessing, ExcelDataQuery.Create(xlData3Arguments), gridPointLabelingMachine, ignoreEmptyYValues));
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues4, "y4"), yDataProcessing, ExcelDataQuery.Create(xlData4Arguments), gridPointLabelingMachine, ignoreEmptyYValues));
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues5, "y5"), yDataProcessing, ExcelDataQuery.Create(xlData5Arguments), gridPointLabelingMachine, ignoreEmptyYValues));
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues6, "y6"), yDataProcessing, ExcelDataQuery.Create(xlData6Arguments), gridPointLabelingMachine, ignoreEmptyYValues));
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues7, "y7"), yDataProcessing, ExcelDataQuery.Create(xlData7Arguments), gridPointLabelingMachine, ignoreEmptyYValues));
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues8, "y8"), yDataProcessing, ExcelDataQuery.Create(xlData8Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                string errorMessage;
                if (gnuPlotApplication.Close(out errorMessage) == false)
                {
                    return ExcelDataConverter.GetExcelRangeErrorMessage(errorMessage);
                }
                return ExcelDataConverter.GetExcelRangeMessage("Wrote file " + outputFileName.ToTimeStampString());
            }
            catch (Exception e)
            {
                return ExcelDataConverter.GetExcelRangeErrorMessage(e.Message);
            }
        }

        /// <summary>Exports a two-dimensional (x,y)-plot with respect to gnuplot.
        /// </summary>
        /// <param name="xlGeneralPropertyNames">A Excel Range (one row or one column) which contains the names of general properties, as 'name' etc.</param>
        /// <param name="xlGeneralPropertyValues">A Excel Range (in the same format as <paramref name="xlGeneralPropertyNames"/>) which contains the values of general properties.</param>
        /// <param name="xlInitCommand">A Excel Range which contains initial gnuplot commands, as for example output size, color setting etc.</param>
        /// <param name="xlXValues0">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues0">A Excel Range which contains y-values.</param>
        /// <param name="xlData0Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlXValues1">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues1">A Excel Range which contains y-values.</param>
        /// <param name="xlData1Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlXValues2">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues2">A Excel Range which contains y-values.</param>
        /// <param name="xlData2Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlXValues3">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues3">A Excel Range which contains y-values.</param>
        /// <param name="xlData3Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlXValues4">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues4">A Excel Range which contains y-values.</param>
        /// <param name="xlData4Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlXValues5">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues5">A Excel Range which contains y-values.</param>
        /// <param name="xlData5Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlXValues6">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues6">A Excel Range which contains y-values.</param>
        /// <param name="xlData6Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlXValues7">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues7">A Excel Range which contains y-values.</param>
        /// <param name="xlData7Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <param name="xlXValues8">A Excel Range which contains x-values.</param>
        /// <param name="xlYValues8">A Excel Range which contains y-values.</param>
        /// <param name="xlData8Arguments">[Optional] commands for a specific data set, for example the title etc.</param>
        /// <returns></returns>
        [ExcelFunction(Name = "doExportPlot2d.xy", Description = "Exports a specific data plot via gnuplot", Category = ExcelCategory)]
        public static object ExportPlot2d(
            [ExcelArgument(Name = "generalPropertyNameRange", Description = "A single row or column Range which contains the names of general properties", AllowReference = true)]
            object xlGeneralPropertyNames,
            [ExcelArgument(Name = "generalPropertyValueRange", Description = "A single row or column Range which contains the values of general properties", AllowReference = true)]            
            object xlGeneralPropertyValues,
            [ExcelArgument(Name = "initCommands", Description = "A vector of gnuplot commands which will be applied before adding data plots", AllowReference = true)]
            object xlInitCommand,
            [ExcelArgument(Name = "x0", Description = "xValues", AllowReference = true)]
            object xlXValues0,
            [ExcelArgument(Name = "y0", Description = "yValues", AllowReference = true)]
            object xlYValues0,
            [ExcelArgument(Name = "commands0", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData0Arguments,
            [ExcelArgument(Name = "x1", Description = "xValues", AllowReference = true)]
            object xlXValues1,
            [ExcelArgument(Name = "y1", Description = "yValues", AllowReference = true)]
            object xlYValues1,
            [ExcelArgument(Name = "commands1", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData1Arguments,
            [ExcelArgument(Name = "x2", Description = "xValues", AllowReference = true)]
            object xlXValues2,
            [ExcelArgument(Name = "y2", Description = "yValues", AllowReference = true)]
            object xlYValues2,
            [ExcelArgument(Name = "commands2", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData2Arguments,
            [ExcelArgument(Name = "x3", Description = "xValues", AllowReference = true)]
            object xlXValues3,
            [ExcelArgument(Name = "y3", Description = "yValues", AllowReference = true)]
            object xlYValues3,
            [ExcelArgument(Name = "commands3", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData3Arguments,
            [ExcelArgument(Name = "x4", Description = "xValues", AllowReference = true)]
            object xlXValues4,
            [ExcelArgument(Name = "y4", Description = "yValues", AllowReference = true)]
            object xlYValues4,
            [ExcelArgument(Name = "commands4", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData4Arguments,
            [ExcelArgument(Name = "x5", Description = "xValues", AllowReference = true)]
            object xlXValues5,
            [ExcelArgument(Name = "y5", Description = "yValues", AllowReference = true)]
            object xlYValues5,
            [ExcelArgument(Name = "commands5", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData5Arguments,
            [ExcelArgument(Name = "x6", Description = "xValues", AllowReference = true)]
            object xlXValues6,
            [ExcelArgument(Name = "y6", Description = "yValues", AllowReference = true)]
            object xlYValues6,
            [ExcelArgument(Name = "commands6", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData6Arguments,
            [ExcelArgument(Name = "x7", Description = "xValues", AllowReference = true)]
            object xlXValues7,
            [ExcelArgument(Name = "y7", Description = "yValues", AllowReference = true)]
            object xlYValues7,
            [ExcelArgument(Name = "commands7", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData7Arguments,
            [ExcelArgument(Name = "x8", Description = "xValues", AllowReference = true)]
            object xlXValues8,
            [ExcelArgument(Name = "y8", Description = "yValues", AllowReference = true)]
            object xlYValues8,
            [ExcelArgument(Name = "commands8", Description = "A vector of gnuplot commands", AllowReference = true)]
            object xlData8Arguments)
        {
            try
            {
                if (ExcelDnaUtil.IsInFunctionWizard() == true)
                {
                    return String.Empty;
                }
                IExcelDataQuery generalPropertyDataQuery = ExcelDataQuery.Create(xlGeneralPropertyNames, xlGeneralPropertyValues);

                string outputPath = generalPropertyDataQuery.GetRequiredPropertyValue<String>("Output path");
                string outputFileName = generalPropertyDataQuery.GetRequiredPropertyValue<String>("File name");
                GnuplotTerminalType terminalType = generalPropertyDataQuery.GetRequiredPropertyValue<GnuplotTerminalType>("Terminal type", EnumStringRepresentationUsage.StringAttribute);

                bool ignoreEmptyYValues = true;
                generalPropertyDataQuery.TryGetOptionalPropertyValue<bool>("Ignore empty Y-values", ref ignoreEmptyYValues, ExcelDataAdvice.Pool.Booleans);

                IGnuplotGridPointLabelingMachine gridPointLabelingMachine = GnuplotGridPointLabel.NoLabel;
                generalPropertyDataQuery.TryGetOptionalPropertyPoolValue<IGnuplotGridPointLabelingMachine>("Label type", GnuplotGridPointLabel.TryGetValue, ref gridPointLabelingMachine, ExcelDataAdvice.Create(GnuplotGridPointLabel.Names));

                IGnuplotDataProcessing xDataProcessing = GnuplotDataProcessing.Double;
                generalPropertyDataQuery.TryGetOptionalPropertyPoolValue<IGnuplotDataProcessing>("dataType:x", GnuplotDataProcessing.TryGetValue, ref xDataProcessing, ExcelDataAdvice.Create(GnuplotDataProcessing.Names));

                IGnuplotDataProcessing yDataProcessing = GnuplotDataProcessing.Double;
                generalPropertyDataQuery.TryGetOptionalPropertyPoolValue<IGnuplotDataProcessing>("dataType:y", GnuplotDataProcessing.TryGetValue, ref yDataProcessing, ExcelDataAdvice.Create(GnuplotDataProcessing.Names));

                string gnuplotScriptFileName = null;
                generalPropertyDataQuery.TryGetOptionalPropertyValue<string>("Script file name", ref gnuplotScriptFileName);

                generalPropertyDataQuery.QueryCompleted();

                GnuplotApplication gnuPlotApplication = GnuplotApplication.Create(gnuplotScriptFileName);
                gnuPlotApplication.SetTerminal(terminalType);
                gnuPlotApplication.SetWorkingDirectory(outputPath);
                gnuPlotApplication.SetOutput(outputFileName);

                gnuPlotApplication.Evaluate(xDataProcessing.GetPreliminaryGnuplotCommand(GnuplotAxis.X));

                IExcelDataQuery gnuPlotPreparation = ExcelDataQuery.Create(xlInitCommand);
                if (gnuPlotPreparation.IsEmpty == false)
                {
                    for (int j = 0; j < gnuPlotPreparation.RowCount; j++)
                    {
                        string prepString;
                        if (gnuPlotPreparation.TryGetValue<string>(out prepString, j, 0, null) == ExcelCellValueState.ProperValue)
                        {
                            gnuPlotApplication.Evaluate(prepString);
                        }
                    }
                }

                List<Gnuplot2dData> plottingData = new List<Gnuplot2dData>();

                /* read the data for plotting as well as some additional [optional] arguments */

                IExcelDataQuery xDataFirstQuery = ExcelDataQuery.Create(xlXValues0, "x0");
                if (xDataFirstQuery.IsEmpty == true)
                {
                    throw new ArgumentException("No x-values given " + xDataFirstQuery.GetFormatedDataQueryName() + ".");
                }
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataFirstQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues0, "y0"), yDataProcessing, ExcelDataQuery.Create(xlData0Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                IExcelDataQuery xDataQuery = ExcelDataQuery.Create(xlXValues1, "x1");
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataQuery.IsEmpty ? xDataFirstQuery : xDataQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues1, "y1"), yDataProcessing, ExcelDataQuery.Create(xlData1Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                xDataQuery = ExcelDataQuery.Create(xlXValues2, "x2");
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataQuery.IsEmpty ? xDataFirstQuery : xDataQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues2, "y2"), yDataProcessing, ExcelDataQuery.Create(xlData2Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                xDataQuery = ExcelDataQuery.Create(xlXValues3, "x3");
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataQuery.IsEmpty ? xDataFirstQuery : xDataQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues3, "y3"), yDataProcessing, ExcelDataQuery.Create(xlData3Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                xDataQuery = ExcelDataQuery.Create(xlXValues4, "x4");
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataQuery.IsEmpty ? xDataFirstQuery : xDataQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues4, "y4"), yDataProcessing, ExcelDataQuery.Create(xlData4Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                xDataQuery = ExcelDataQuery.Create(xlXValues5, "x5");
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataQuery.IsEmpty ? xDataFirstQuery : xDataQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues5, "y5"), yDataProcessing, ExcelDataQuery.Create(xlData5Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                xDataQuery = ExcelDataQuery.Create(xlXValues6, "x6");
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataQuery.IsEmpty ? xDataFirstQuery : xDataQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues6, "y6"), yDataProcessing, ExcelDataQuery.Create(xlData6Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                xDataQuery = ExcelDataQuery.Create(xlXValues7, "x7");
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataQuery.IsEmpty ? xDataFirstQuery : xDataQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues7, "y7"), yDataProcessing, ExcelDataQuery.Create(xlData7Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                xDataQuery = ExcelDataQuery.Create(xlXValues8, "x8");
                gnuPlotApplication.AddPlot2D(CreatePlot2dData(xDataQuery.IsEmpty ? xDataFirstQuery : xDataQuery, xDataProcessing, ExcelDataQuery.Create(xlYValues8, "y8"), yDataProcessing, ExcelDataQuery.Create(xlData8Arguments), gridPointLabelingMachine, ignoreEmptyYValues));

                string errorMessage;
                if (gnuPlotApplication.Close(out errorMessage) == false)
                {
                    return ExcelDataConverter.GetExcelRangeErrorMessage(errorMessage);
                }
                return ExcelDataConverter.GetExcelRangeMessage("Wrote file " + outputFileName.ToTimeStampString());
            }
            catch (Exception e)
            {
                return ExcelDataConverter.GetExcelRangeErrorMessage(e.Message);
            }
        }
        #endregion

        #region private (static) methods

        /// <summary>Creates a specific <see cref="Gnuplot2dData"/> object.
        /// </summary>
        /// <param name="xValueRange">The <see cref="IExcelDataQuery"/> object which contains x-values.</param>
        /// <param name="xDataProcessing">The <see cref="IGnuplotDataProcessing"/> object that describes the way how to interpret the values in <paramref name="xValueRange"/>.</param>
        /// <param name="yValueRange">The <see cref="IExcelDataQuery"/> object which contains y-values.</param>
        /// <param name="yDataProcessing">The <see cref="IGnuplotDataProcessing"/> object that describes the way how to interpret the values in <paramref name="yValueRange"/>.</param>
        /// <param name="commandRange">The [optional] GNU plot commands in its <see cref="IExcelDataQuery"/> reprsentation.</param>
        /// <param name="gridPointLabeling">A object used for adding a label into the internal data structure (which does not mean that a label is shown, it just adds an additional column in the data of the plot command).</param>
        /// <param name="ignoreEmptyYValues">If <c>true</c> empty y-values (but with given x-values) will be ignored; otherwise an error occurs if an y-Value is empty but in the
        /// same row a x-value is given.</param>
        private static Gnuplot2dData CreatePlot2dData(IExcelDataQuery xValueRange, IGnuplotDataProcessing xDataProcessing, IExcelDataQuery yValueRange, IGnuplotDataProcessing yDataProcessing, IExcelDataQuery commandRange, IGnuplotGridPointLabelingMachine gridPointLabeling, bool ignoreEmptyYValues)
        {
            Gnuplot2dData gnuplot2dData = new Gnuplot2dData(commandRange.GetColumnVector<string>());

            if ((xValueRange.IsEmpty == false) && (yValueRange.IsEmpty == false))
            {
                int rowCount = Math.Min(xValueRange.RowCount, yValueRange.RowCount);

                for (int j = 0; j < rowCount; j++)
                {
                    string xValue, yValue;
                    ExcelCellValueState xState = xDataProcessing.TryGetDataAsString(xValueRange, j, out xValue);

                    switch (yDataProcessing.TryGetDataAsString(yValueRange, j, out yValue))
                    {
                        case ExcelCellValueState.ProperValue:
                            if (xState == ExcelCellValueState.ProperValue)
                            {
                                gnuplot2dData.Add(xValue, yValue, (gnuplot2dData.HasLabels == false) ? null : gridPointLabeling.GetLabel(xValueRange, xDataProcessing, yValueRange, yDataProcessing, j));
                            }
                            else
                            {
                                throw new ArgumentException("Invalid x-Value '" + xValueRange.ToString(j, 0) + "' " + xValueRange.GetFormatedDataQueryName() + ".");
                            }
                            break;

                        case ExcelCellValueState.EmptyOrMissingExcelCell:
                            if (ignoreEmptyYValues == false)
                            {
                                if (xState == ExcelCellValueState.ProperValue)
                                {
                                    throw new ArgumentException("No y-value for x-value '" + xValueRange.ToString(j, 0) + "' found " + yValueRange.GetFormatedDataQueryName() + ".");
                                }
                                else if (xState == ExcelCellValueState.NoValidValue)
                                {
                                    throw new ArgumentException("Invalid x-value '" + xValueRange.ToString(j, 0) + "' " + xValueRange.GetFormatedDataQueryName() + ".");
                                }
                            }
                            break;

                        case ExcelCellValueState.NoValidValue:
                            throw new ArgumentException("Invalid y-value '" + yValueRange.ToString(j, 0) + "'" + yValueRange.GetFormatedDataQueryName() + ".");

                        default:
                            throw new NotImplementedException();
                    }
                }
            }
            return gnuplot2dData;
        }
        #endregion
    }
}