﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Globalization;
using System.Collections.Generic;

using Dodoni.BasicComponents;

namespace Dodoni.XLIntegration.Utilities
{
    /// <summary>Represents data required for a two-dimensional plot with respect to gnuplot.
    /// </summary>
    public class Gnuplot2dData
    {
        #region private members

        /// <summary>The (optional) additional parameter in its <see cref="System.String"/> representation, perhaps <c>null</c>.
        /// </summary>
        private string m_ParameterAsString;

        /// <summary>Represents the <see cref="System.String"/> representation of the data.
        /// </summary>
        private StringBuilder m_DataStringBuilder = new StringBuilder();

        /// <summary>The number of grid points.
        /// </summary>
        private int m_Count = 0;

        /// <summary>A value whether the current instance contains an additional "column" with labels.
        /// </summary>
        private bool m_HasLabels = false;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="Gnuplot2dData"/> class.
        /// </summary>
        /// <param name="additionalGnuplotArguments">The additional gnuplot arguments, for example the title etc.</param>
        /// <remarks>The <see cref="HasLabels"/> property will be set to <c>true</c> if one of the GnuPlot command contains 'with labels'.</remarks>
        public Gnuplot2dData(params string[] additionalGnuplotArguments)
        {
            if (additionalGnuplotArguments == null)
            {
                m_ParameterAsString = null;
            }
            else
            {
                m_ParameterAsString = String.Join(" ", additionalGnuplotArguments);
            }
            m_HasLabels = GetHasLabels(additionalGnuplotArguments);
        }

        /// <summary>Initializes a new instance of the <see cref="Gnuplot2dData"/> class.
        /// </summary>
        /// <param name="additionalGnuplotArguments">The additional gnuplot arguments, for example the title etc.</param>
        /// <remarks>The <see cref="HasLabels"/> property will be set to <c>true</c> if one of the GnuPlot command contains 'with labels'.</remarks>
        public Gnuplot2dData(IEnumerable<string> additionalGnuplotArguments)
        {
            if (additionalGnuplotArguments == null)
            {
                m_ParameterAsString = null;
            }
            else
            {
                m_ParameterAsString = String.Join(" ", additionalGnuplotArguments);
            }
            m_HasLabels = GetHasLabels(additionalGnuplotArguments);
        }
        #endregion

        #region public properties

        /// <summary>Gets a value indicating whether this instance has (optional) gnuplot command arguments.
        /// </summary>
        /// <value><c>true</c> if this instance has optional gnuplot command arguments; otherwise, <c>false</c>.
        /// </value>
        /// <example>For example the title of graph, the line type, color etc.</example>
        public bool HasParameter
        {
            get { return (m_ParameterAsString != null); }
        }

        /// <summary>Gets the number of grid points.
        /// </summary>
        /// <value>The number of grid points.</value>
        public int Count
        {
            get { return m_Count; }
        }

        /// <summary>Gets a value indicating whether this instance contains labels.
        /// </summary>
        /// <value><c>true</c> if this instance has labels; otherwise, <c>false</c>.
        /// </value>
        public bool HasLabels
        {
            get { return m_HasLabels; }
        }
        #endregion

        #region public static methods

        /// <summary>Creates a new <see cref="Gnuplot2dData"/> object.
        /// </summary>
        /// <param name="additionalGnuplotArguments">The additional gnuplot arguments, for example the title etc.</param>
        /// <returns>A new <see cref="Gnuplot2dData"/> object.</returns>
        public static Gnuplot2dData Create(params string[] additionalGnuplotArguments)
        {
            return new Gnuplot2dData(additionalGnuplotArguments);
        }

        /// <summary>Creates a new <see cref="Gnuplot2dData"/> object.
        /// </summary>
        /// <param name="additionalGnuplotArguments">The additional gnuplot arguments, for example the title etc.</param>
        /// <returns>A new <see cref="Gnuplot2dData"/> object.</returns>
        public static Gnuplot2dData Create(IEnumerable<string> additionalGnuplotArguments)
        {
            return new Gnuplot2dData(additionalGnuplotArguments);
        }
        #endregion

        #region public methods

        /// <summary>Adds the specified grid point.
        /// </summary>
        /// <param name="xValue">The <see cref="System.String"/> representation of the x value.</param>
        /// <param name="yValue">The <see cref="System.String"/> representation of the y value.</param>
        /// <param name="gridPointLabel">The optional label of the grid point.</param>
        /// <returns>A value indicating whether the grid point has been added.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if previous added gridpoints contain a label and <paramref name="gridPointLabel"/> is 
        /// <c>null</c> or empty; or previous added gridpoints do not have an additional label, but <paramref name="gridPointLabel"/> represents a label.</exception>
        public bool Add(string xValue, string yValue, string gridPointLabel = null)
        {
            if ((xValue == null) || (yValue == null) || (xValue.Length == 0) || (yValue.Length == 0))
            {
                return false;
            }
            m_DataStringBuilder.Append(xValue);
            m_DataStringBuilder.Append(" ");
            m_DataStringBuilder.Append(yValue);
            if (gridPointLabel != null)
            {
                m_DataStringBuilder.Append(" ");
                m_DataStringBuilder.Append(gridPointLabel);
            }
            m_DataStringBuilder.AppendLine();
            m_Count++;

            CheckLabelInputConsistency(gridPointLabel);
            return true;
        }

        /// <summary>Adds the specified grid point.
        /// </summary>
        /// <param name="xValue">The <see cref="System.Double"/> representation of the x value.</param>
        /// <param name="yValue">The <see cref="System.Double"/> representation of the y value.</param>
        /// <param name="gridPointLabel">The optional label of the grid point.</param>
        /// <returns>A value indicating whether the grid point has been added.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if previous added gridpoints contain a label and <paramref name="gridPointLabel"/> is 
        /// <c>null</c> or empty; or previous added gridpoints do not have an additional label, but <paramref name="gridPointLabel"/> represents a label.</exception>
        public bool Add(double xValue, double yValue, string gridPointLabel = null)
        {
            if ((Double.IsNaN(xValue) == true) || (Double.IsNaN(yValue) == true) || (Double.IsInfinity(xValue) == true) || (Double.IsInfinity(yValue) == true))
            {
                return false;
            }
            m_DataStringBuilder.Append(xValue.ToString(CultureInfo.InvariantCulture));
            m_DataStringBuilder.Append(" ");
            m_DataStringBuilder.Append(yValue.ToString(CultureInfo.InvariantCulture));
            if (gridPointLabel != null)
            {
                m_DataStringBuilder.Append(" ");
                m_DataStringBuilder.Append(gridPointLabel);
            }
            m_DataStringBuilder.AppendLine();
            m_Count++;

            CheckLabelInputConsistency(gridPointLabel);
            return true;
        }

        /// <summary>Adds the specified grid point.
        /// </summary>
        /// <param name="xValue">The <see cref="System.DateTime"/> representation of the x value.</param>
        /// <param name="yValue">The <see cref="System.Double"/> representation of the y value.</param>
        /// <param name="gridPointLabel">The optional label of the grid point.</param>
        /// <returns>A value indicating whether the grid point has been added.</returns>
        /// <exception cref="InvalidOperationException">Thrown, if previous added gridpoints contain a label and <paramref name="gridPointLabel"/> is 
        /// <c>null</c> or empty; or previous added gridpoints do not have an additional label, but <paramref name="gridPointLabel"/> represents a label.</exception>
        public bool Add(DateTime xValue, double yValue, string gridPointLabel = null)
        {
            if (Double.IsNaN(yValue) == false)
            {
                m_DataStringBuilder.Append(xValue.ToString(CultureInfo.InvariantCulture));
                m_DataStringBuilder.Append(" ");
                m_DataStringBuilder.Append(yValue.ToString(CultureInfo.InvariantCulture));

                if (gridPointLabel != null)
                {
                    m_DataStringBuilder.Append(" ");
                    m_DataStringBuilder.Append(gridPointLabel);
                }
                m_DataStringBuilder.AppendLine();
                m_Count++;

                CheckLabelInputConsistency(gridPointLabel);
                return true;
            }
            return false;
        }

        /// <summary>Adds a collection of grid points.
        /// </summary>
        /// <param name="xValues">The collection x-values in its <see cref="System.Double"/> representation.</param>
        /// <param name="yValues">The collection of y-values in its <see cref="System.Double"/> representation.</param>
        /// <param name="gridPointLabels">The optional label of the grid point.</param>
        /// <returns>A reference to the current object, this allows constructions of the form <code>var obj = Gnuplot2dData.Create(..).Add(xValues,yValues);</code></returns>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="gridPointLabels"/> is not <c>null</c> and the number of elements is 
        /// less than the number of grid points to add.</exception>
        /// <exception cref="InvalidOperationException">Thrown, if some gridpoints contain a label and some not.</exception>
        public Gnuplot2dData Add(IList<double> xValues, IList<double> yValues, params string[] gridPointLabels)
        {
            if (xValues.Count != yValues.Count)
            {
                throw new ArgumentException();
            }
            for (int j = 0; j < xValues.Count; j++)
            {
                if ((gridPointLabels != null) && (gridPointLabels.Length > j))
                {
                    Add(xValues[j], yValues[j], gridPointLabels[j]);
                }
                else
                {
                    Add(xValues[j], yValues[j]);
                }
            }
            return this;
        }

        /// <summary>Adds a collection of grid points.
        /// </summary>
        /// <param name="xValues">The collection x-values in its <see cref="System.DateTime"/> representation.</param>
        /// <param name="yValues">The collection of y-values in its <see cref="System.Double"/> representation.</param>
        /// <param name="gridPointLabels">The optional label of the grid point.</param>
        /// <returns>A reference to the current object, this allows constructions of the form <code>var obj = Gnuplot2dData.Create(..).Add(xValues,yValues);</code></returns>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="gridPointLabels"/> is not <c>null</c> and the number of elements is 
        /// less than the number of grid points to add.</exception>
        /// <exception cref="InvalidOperationException">Thrown, if some gridpoints contain a label and some not.</exception>
        public Gnuplot2dData Add(IList<DateTime> xValues, IList<double> yValues, params string[] gridPointLabels)
        {
            if (xValues.Count != yValues.Count)
            {
                throw new ArgumentException();
            }
            for (int j = 0; j < xValues.Count; j++)
            {
                if ((gridPointLabels != null) && (gridPointLabels.Length > j))
                {
                    Add(xValues[j], yValues[j], gridPointLabels[j]);
                }
                else
                {
                    Add(xValues[j], yValues[j]);
                }
            }
            return this;
        }

        /// <summary>Adds a collection of grid points.
        /// </summary>
        /// <param name="xValues">The collection x-values in its <see cref="System.String"/> representation.</param>
        /// <param name="yValues">The collection of y-values in its <see cref="System.String"/> representation.</param>
        /// <param name="gridPointLabels">The optional label of the grid point.</param>
        /// <returns>A reference to the current object, this allows constructions of the form <code>var obj = Gnuplot2dData.Create(..).Add(xValues,yValues);</code></returns>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="gridPointLabels"/> is not <c>null</c> and the number of elements is 
        /// less than the number of grid points to add.</exception>
        /// <exception cref="InvalidOperationException">Thrown, if some gridpoints contain a label and some not.</exception>
        public Gnuplot2dData Add(IList<string> xValues, IList<string> yValues, params string[] gridPointLabels)
        {
            if (xValues.Count != yValues.Count)
            {
                throw new ArgumentException();
            }
            for (int j = 0; j < xValues.Count; j++)
            {
                if ((gridPointLabels != null) && (gridPointLabels.Length > j))
                {
                    Add(xValues[j], yValues[j], gridPointLabels[j]);
                }
                else
                {
                    Add(xValues[j], yValues[j]);
                }
            }
            return this;
        }

        /// <summary>Removes all grid points from the current <see cref="Gnuplot2dData"/> instance.
        /// </summary>
        public void Clear()
        {
            m_DataStringBuilder.Clear();
            m_Count = 0;
        }

        /// <summary>Gets the data in its <see cref="System.String"/> representation.
        /// </summary>
        /// <returns>The gnuplot <see cref="System.String"/> representation of the data.</returns>
        public string GetDataAsString()
        {
            if (m_Count == 0)
            {
                return String.Empty;
            }
            return m_DataStringBuilder.ToString() + " e ";
        }

        /// <summary>Gets the (optional) additional gnuplot command in its <see cref="System.String"/> representation.
        /// </summary>
        /// <returns>The (optional) gnuplot command in its <see cref="System.String"/> representation; perhaps <c>null</c>.</returns>
        public string GetParameterAsString()
        {
            return m_ParameterAsString;
        }
        #endregion

        #region private/private static methods

        /// <summary>Checks whether the label input is consistent with the previous added grid ponts.
        /// </summary>
        /// <param name="gridPointLabel">The label of a specific grid point.</param>
        /// <exception cref="InvalidOperationException">Thrown, if previous added gridpoints contain a label and <paramref name="gridPointLabel"/> is 
        /// <c>null</c> or empty; or previous added gridpoints do not have an additional label, but <paramref name="gridPointLabel"/> represents a label.</exception>
        private void CheckLabelInputConsistency(string gridPointLabel)
        {
            if (m_Count > 0)
            {
                if ((gridPointLabel != null) && (gridPointLabel.Length > 0))
                {
                    if (m_HasLabels == false)
                    {
                        throw new InvalidOperationException("No label input allowed.");
                    }
                }
                else if (m_HasLabels == true)
                {
                    throw new ArgumentException("Label expected; at least one Gnuplot command contains 'with labels'.");
                }
            }
        }

        /// <summary>Gets a value indicating whether a specified <see cref="Gnuplot2dData"/> expect labels.
        /// </summary>
        /// <param name="gnuplotArguments">The gnuplot arguments.</param>
        /// <returns><c>true</c> if at least one of the command arguments contains 'with labels'; otherwise <c>false</c>.</returns>
        private static bool GetHasLabels(IEnumerable<string> gnuplotArguments)
        {
            if (gnuplotArguments == null)
            {
                return false;
            }
            var labelCommand = new IdentifierString("with labels");
            foreach (var gnuPlotArgument in gnuplotArguments)
            {
                if (IdentifierString.Create(gnuPlotArgument).IDString.Contains(labelCommand.IDString) == true)
                {
                    return true;
                }
            }
            return false;
        }
        #endregion
    }
}