﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.IO;
using System.Text;
using System.Diagnostics;
using System.Collections.Generic;

using Dodoni.XLBasicComponents;
using Dodoni.BasicComponents.LogFileFactory;

namespace Dodoni.XLIntegration.Utilities
{
    /// <summary>Serves as factory for gnuplot instances and as abstract basis class for such wrapper classes.
    /// </summary>
    /// <remarks>Tested with gnuplot 4.4.</remarks>
    public abstract class GnuplotApplication
    {
        #region public static members

        /// <summary>The standard path of <c>gnuplot.exe</c>.
        /// </summary>
        private static string sm_StandardGnuPlotPath = GetGnuPlotPathFromConfigFile();
        #endregion

        #region protected members

        /// <summary>Represents the (guided) input of gnuplot.
        /// </summary>
        protected StreamWriter m_InputStream;

        /// <summary>The data to add into the gnuplot script.
        /// </summary>
        protected List<Gnuplot2dData> m_GnuplotData = new List<Gnuplot2dData>();

        /// <summary>The prefix of the plot command string.
        /// </summary>
        protected StringBuilder m_PlotCommandStrings = new StringBuilder();
        #endregion

        #region protected constructors

        /// <summary>Initializes a new instance of the <see cref="GnuplotApplication"/> class.
        /// </summary>
        protected GnuplotApplication()
        {
        }
        #endregion

        #region public static properties

        /// <summary>Gets the standard path of <c>gnuplot.exe</c>.
        /// </summary>
        /// <value>The standard path of <c>gnuplot.exe</c>.</value>
        public static string StandardGnuPlotPath
        {
            get { return sm_StandardGnuPlotPath; }
        }
        #endregion

        #region public static methods

        /// <summary>Creates a new <see cref="GnuplotApplication"/> object.
        /// </summary>
        /// <param name="scriptFileName">The name of the gnuplot script file to fill with the statements.</param>
        /// <returns>A new <see cref="GnuplotApplication"/> object.</returns>
        public static GnuplotApplication Create(string scriptFileName = null)
        {
            if (scriptFileName == null)
            {
                return new GnuplotApplicationWrapper();
            }
            return new GnuplotApplicationScript(scriptFileName);
        }
        #endregion

        #region public methods

        /// <summary>Evaluate a specific gnuplot expression.
        /// </summary>
        /// <param name="expression">The expression to evaluate.</param>
        public void Evaluate(string expression)
        {
            if (expression != null)
            {
                m_InputStream.WriteLine(expression);
            }
        }

        /// <summary>Evaluate a specific gnuplot expression.
        /// </summary>
        /// <param name="expressions">A collection of expressions to evaluate.</param>
        public void Evaluate(IEnumerable<string> expressions)
        {
            foreach (var expression in expressions)
            {
                Evaluate(expression);
            }
        }

        /// <summary>Sets the terminal, i.e. calls 'set terminal'.
        /// </summary>
        /// <param name="terminalType">The type of the terminal.</param>
        /// <param name="additionalOptions">Additional [Optional] options for the terminal type.</param>
        public void SetTerminal(GnuplotTerminalType terminalType, string additionalOptions = null)
        {
            m_InputStream.WriteLine("set terminal " + terminalType.ToString(), additionalOptions);
        }

        /// <summary>Sets the file output.
        /// </summary>
        /// <param name="fileName">The name of the output file.</param>
        public void SetOutput(string fileName)
        {
            m_InputStream.WriteLine("set output \"" + fileName + "\"");
        }

        /// <summary>Sets the working directory.
        /// </summary>
        /// <param name="path">The path of the working directory.</param>
        public void SetWorkingDirectory(string path)
        {
            m_InputStream.WriteLine("cd '" + path + "'");
        }

        /// <summary>Adds a specific two-dimensional graph.
        /// </summary>
        /// <param name="gnuplot2dData">The two-dimensional data to plot.</param>
        /// <param name="xDataUsingIndex">The one-based index k in the 'using k:x' or 'using k:x:y' statement that represents the x-values.</param>
        /// <param name="yDataUsingIndex">The one-based index 'k' in the 'using x:k' or 'using x:k:y' statement that represents the y-values.</param>
        /// <param name="labelUsingIndex">The one-based index 'k' in the 'using x:y:k' that represents the labels. It will be ignored if the property <see cref="Gnuplot2dData.HasLabels"/> of <paramref name="gnuplot2dData"/> is <c>false</c>.</param>
        public void AddPlot2D(Gnuplot2dData gnuplot2dData, int xDataUsingIndex = 1, int yDataUsingIndex = 2, int labelUsingIndex = 3)
        {
            if (gnuplot2dData.Count > 0)
            {
                if (m_PlotCommandStrings.Length != 0)
                {
                    m_PlotCommandStrings.Append(", ");
                }
                if (gnuplot2dData.HasLabels == false)
                {
                    m_PlotCommandStrings.Append(String.Format("'-' u {0}:{1} ", xDataUsingIndex, yDataUsingIndex));
                }
                else
                {
                    m_PlotCommandStrings.Append(String.Format("'-' u {0}:{1}:{2} ", xDataUsingIndex, yDataUsingIndex, labelUsingIndex));
                }

                if (gnuplot2dData.HasParameter)
                {
                    m_PlotCommandStrings.Append(gnuplot2dData.GetParameterAsString() + " ");
                }
                m_GnuplotData.Add(gnuplot2dData);
            }
        }

        /// <summary>Closes the current instance.
        /// </summary>
        /// <param name="errorMessage">The error message, perhaps <c>null</c> (output).</param>
        /// <returns>If <c>false</c> some error occurs and <paramref name="errorMessage"/> contains a error message; otherwise
        /// <paramref name="errorMessage"/> is <c>null</c>.</returns>
        public abstract bool Close(out string errorMessage);
        #endregion

        #region internal static methods

        /// <summary>Store the configuration, i.e. write a specific configuration file.
        /// </summary>
        /// <param name="gnuPlotPath">The standard path of <c>gnuplot.exe</c>.</param>
        internal static void StoreConfiguration(string gnuPlotPath)
        {
            sm_StandardGnuPlotPath = gnuPlotPath;
            ExcelAddIn.Configuration.GetPropertyCollection("Gnuplot").SetValue("GnuplotPath", gnuPlotPath);
            ExcelAddIn.Configuration.Save();
        }

        /// <summary>Gets the standard path of <c>gnuplot.exe</c> with respect to the config file.
        /// </summary>
        /// <returns>The standard path of <c>gnuplot.exe</c> with respect to the config file or a standard value.</returns>
        internal static string GetGnuPlotPathFromConfigFile()
        {
            string gnuPlotPath;
            if (ExcelAddIn.Configuration.GetPropertyCollection("Gnuplot").TryGetValue("GnuplotPath", out gnuPlotPath) == true)
            {
                return gnuPlotPath;
            }
            return @"C:\Programme\gnuplot\binary\gnuplot.exe";
        }
        #endregion
    }
}