﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

using Dodoni.BasicComponents.Utilities;

namespace Dodoni.XLIntegration.Utilities
{
    /// <summary>Represents the Terminal type of gnuplot.
    /// </summary>
    public enum GnuplotTerminalType
    {
        /// <summary>HTML Canvas object.
        /// </summary>
        [String("Canvas")]
        Canvas,

        /// <summary>Computer Graphics Metafile
        /// </summary>
        [String("Computer Graphics Metafile")]
        cgm,

        /// <summary>EPS format for CorelDRAW
        /// </summary>
        [String("Corel Draw eps")]
        Corel,

        /// <summary>Seiko DPU-414 thermal printer [small medium large]
        /// </summary>
        dpu414,

        /// <summary>ascii art for anything that prints text
        /// </summary>
        dumb,

        /// <summary>dxf-file for AutoCad (default size 120x80)
        /// </summary>
        dxf,

        /// <summary>EEPIC -- extended LaTeX picture environment
        /// </summary>
        [String("LaTeX EEPIC")]
        eepic,

        /// <summary>Enhanced Metafile format
        /// </summary>
        emf,

        /// <summary>LaTeX picture environment with emTeX specials
        /// </summary>
        [String("LaTeX emTeX Picture")]
        emtex,

        /// <summary>LaTeX picture environment using graphicx package
        /// </summary>
        [String("LaTeX graphicx eps")]
        epslatex,

        /// <summary>Epson LQ-style 180-dot per inch (24 pin) printers
        /// </summary>
        epson_180dpi,

        /// <summary>Epson-style 60-dot per inch printers
        /// </summary>
        epson_60dpi,

        /// <summary>Epson LX-800, Star NL-10, NX-1000, PROPRINTER ...
        /// </summary>
        epson_lx800,

        /// <summary>FIG graphics language for XFIG graphics editor
        /// </summary>
        fig,

        /// <summary>GIF images using libgd and TrueType fonts
        /// </summary>
        [String("Gif")]
        gif,

        /// <summary>HP2623A and maybe others
        /// </summary>
        hp2623A,

        /// <summary>HP2648 and HP2647
        /// </summary>
        hp2648,

        /// <summary>HP DeskJet 500c, [75 100 150 300] [rle tiff]
        /// </summary>
        hp500c,

        /// <summary>HP DeskJet 500, [75 100 150 300]
        /// </summary>
        hpdj,

        /// <summary>HP7475 and relatives [number of pens] [eject]
        /// </summary>
        hpgl,

        /// <summary>HP Laserjet series II, [75 100 150 300]
        /// </summary>
        hpljii,

        /// <summary>HP PaintJet and HP3630 [FNT5X9 FNT9X17 FNT13X25]
        /// </summary>
        hppj,

        /// <summary>Imagen laser printer
        /// </summary>
        imagen,

        /// <summary>JPEG images using libgd and TrueType fonts
        /// </summary>
        [String("Jpeg")]
        jpeg,

        /// <summary>LaTeX picture environment
        /// </summary>
        latex,

        /// <summary>Lua generic terminal driver
        /// </summary>
        lua,

        /// <summary>Metafont plotting standard
        /// </summary>
        mf,

        /// <summary>Frame maker MIF 3.00 format
        /// </summary>
        mif,

        /// <summary>MetaPost plotting standard
        /// </summary>
        mp,

        /// <summary>NEC printer CP6, Epson LQ-800 [monocrome color draft]
        /// </summary>
        nec_cp6,

        /// <summary>OKIDATA 320/321 Standard
        /// </summary>
        okidata,

        /// <summary>Portable bitmap [small medium large] [monochrome gray color]
        /// </summary>
        pbm,

        /// <summary>HP Designjet 750C, HP Laserjet III/IV, etc. (many options)
        /// </summary>
        pcl5,

        /// <summary>pdf terminal based on cairo
        /// </summary>
        [String("PDF")]
        pdfcairo,

        /// <summary>PNG images using libgd and TrueType fonts
        /// </summary>
        png,

        /// <summary>png terminal based on cairo
        /// </summary>
        pngcairo,

        /// <summary>PostScript graphics, including EPSF embedded files (*.eps)
        /// </summary>
        [String("Postscript")]
        postscript,

        /// <summary>LaTeX picture environment with PostScript \specials
        /// </summary>
        pslatex,

        /// <summary>plain TeX with PostScript \specials
        /// </summary>
        pstex,

        /// <summary>LaTeX picture environment with PSTricks macros
        /// </summary>
        pstricks,

        /// <summary>QMS/QUIC Laser printer (also Talaris 1200 and others)
        /// </summary>
        qms,

        /// <summary>Star Color Printer
        /// </summary>
        starc,

        /// <summary>W3C Scalable Vector Graphics driver
        /// </summary>
        svg,

        /// <summary>Tandy DMP-130 series 60-dot per inch graphics
        /// </summary>
        tandy_60dpi,

        /// <summary>LaTeX texdraw environment
        /// </summary>
        texdraw,

        /// <summary>TGIF X11 [mode] [x,y] [dashed] ["font" [fontsize]]
        /// </summary>
        tgif,

        /// <summary>TeX TikZ graphics macros via the lua script driver
        /// </summary>
        tikz,

        /// <summary>Tk/Tcl canvas widget [perltk] [interactive]
        /// </summary>
        tkcanvas,

        /// <summary>TPIC -- LaTeX picture environment with tpic \specials
        /// </summary>
        tpic,

        /// <summary>Unknown terminal type - not a plotting device
        /// </summary>
        unknown,

        /// <summary>Microsoft Windows
        /// </summary>
        windows,

        /// <summary>wxWidgets cross-platform windowed terminal
        /// </summary>
        wxt,
    }
}